/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): Gael Harbonnier.

====================================================================*/

package org.objectweb.benchmark.corba.latency;

// Package dependencies.
import java.io.InputStream;
import java.util.Properties;

import org.omg.CORBA.ORB;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;

/**
 * This is the Hello CORBA server.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */
public class Server
{
    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public Server()
    {
        try{
            Properties properties = new Properties();
            InputStream propStream = getClass().getClassLoader().getResourceAsStream("benchmark.properties");
            properties.load(propStream);
        } catch (Exception e) {
            e.printStackTrace();
        }    
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    /**
     * Inits ORB properties.
     */
    private void init_orb()
    {
        try {
            java.io.InputStream propStream = null;

            propStream = getClass().getClassLoader().getResourceAsStream("ORB.properties");
            System.getProperties().load(propStream);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
    /**
     * Runs the Server.
     */
    public void run()
    {
        try{
            // Inits the ORB
            init_orb();       
            ORB orb = ORB.init(new String[0], null);

            // Gets Root POA
            org.omg.CORBA.Object rootobj = orb.resolve_initial_references("RootPOA");
            POA rootpoa = POAHelper.narrow(rootobj);
            rootpoa.the_POAManager().activate();

            // Instanciates new CORBA Object
            PingImpl servant = new PingImpl(orb);
            org.omg.CORBA.Object obj = rootpoa.servant_to_reference(servant);
      
            // Publishes CORBA Object IOR
            String ior = orb.object_to_string(obj);
            IORServer iorServer = new IORServer(ior);
            iorServer.listen();

            // Waits for requests
            System.out.println( "The CORBA server is running ..." );
            orb.run();
        }catch(Exception ex){
            ex.printStackTrace();
        }
    }

    /**
     * The bootstrap method.
     * 
     * @param args - Application arguments
     */
    public static void main(String[] args)
    {
        Server serv = new Server();
        serv.run();
    }
}
