/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Gael Harbonnier, Christophe Demarey.
Contributor(s): _______________.

====================================================================*/


package org.objectweb.benchmark.fractal.latency;

// Package dependencies.
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

import org.objectweb.benchmark.util.ResultPrinter;


/**
 * Fractal Rmi benchmark client application.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */
public class Client
  implements BindingController,
             Runnable
{

    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /** Number of steps*/
    private int nb_step;

    /** Number of invocations per step*/
    private int nb_step_invocation;
    
    /** Beginning of the first step*/
    private int step_start;

    /** Results file name */
    private String result_file;

    /** the host */
    private String host;
    
    /** Ping interface reference */
    private PingItf ping_impl;

   // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public Client()
    {
        java.util.Properties properties = new java.util.Properties();

        try {
            java.io.InputStream propStream = null;
        
            // Bench info
            propStream = getClass().getClassLoader().getResourceAsStream("benchmark.properties");
            properties.load(propStream);
        
            nb_step = Integer.parseInt( properties.getProperty("step.number") );
            nb_step_invocation = Integer.parseInt( properties.getProperty("step.invocation.number") );
            step_start = Integer.parseInt( properties.getProperty("step.start") );
            host = properties.getProperty("com.host");

            // Fractal RMI info
            propStream = getClass().getClassLoader().getResourceAsStream("build.properties");
            properties.load(propStream);
            result_file = properties.getProperty("dest.file");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
    /**
     * Runs the latency benchmark.
     */
    public void test()
    {
        // Don't include the first step_start invocations
        for (int i=0; i<step_start; i++)
            ping_impl.ping();
        
        // Runs test
        System.out.println( "Running Fractal Rmi Benchmark with " + nb_step + " step(s) of "
                            + nb_step_invocation +" method calls." );
        try{
            for(int j=0; j<nb_step; j++)
            {
                long timer = System.currentTimeMillis();
                
                for (int i=0; i<nb_step_invocation; i++)
                    ping_impl.ping();
                timer = (long) ( ( System.currentTimeMillis() - timer ) * 1000 / (float)nb_step_invocation );
             
                // write results
                ResultPrinter p = new ResultPrinter(result_file);
                p.write(timer);
                p.close();
            }
        }catch(Exception ex){
            ex.printStackTrace();
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#listFc()
     */
    public String[] listFc()
    {
        String [] list = new String[0];
        
        if (ping_impl != null)
        {
            list = new String[1];
            list[0] = "PingItf";
        }

        return list;
    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
     */
    public Object lookupFc(final String cItf)
    throws NoSuchInterfaceException
    {
        if (cItf.compareTo("PingItf") == 0)
        {
            return ping_impl;
        }
        throw new NoSuchInterfaceException(cItf);
    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String, java.lang.Object)
     */
    public void bindFc(final String cItf, final Object sItf)
    throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException
    {
        if (cItf.compareTo("PingItf") == 0)
        {
            ping_impl = (PingItf) sItf;
        }
        else
            throw new NoSuchInterfaceException(cItf);
    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
     */
    public void unbindFc(final String cItf)
    throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException
    {
        if (cItf.compareTo("PingItf") == 0)
        {
            ping_impl = null;
        }
        else
            throw new NoSuchInterfaceException(cItf);
    }

    /* (non-Javadoc)
     * @see org.objectweb.benchmark.fractal.latency.Runnable#run()
     */
    public void run()
    {
        test();
    }
}
