/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): Gael Harbonnier.

====================================================================*/

package org.objectweb.benchmark.ws.latency;

// Package dependencies.
import org.objectweb.benchmark.util.ResultPrinter;
import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.apache.axis.encoding.XMLType;


/**
 *  This client tests WS latency by calling several times the ping method.
 *  
 * @author Harbonnier Denimal
 * 
 * @version 1.0
 */
public class Client
{
    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================
    
    /** Number of steps*/
    private int nb_step;

    /** Number of invocations per step*/
    private int nb_step_invocation;
    
    /** Beginning of the first step*/
    private int step_start;

    /** Results file name */
    private String result_file;

    /** host */
    private String host;
    
    /** Ping interface reference */
    private Call ping_ws = null;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     * 
     * @param n - Number of invocations to do.
     * @param name - Results file name.
     */
    public Client(String name)
    {
        result_file = name;
        java.util.Properties properties = new java.util.Properties();

        try {
            java.io.InputStream propStream = null;

            propStream = getClass().getClassLoader().getResourceAsStream("benchmark.properties");
            properties.load(propStream);
        
            nb_step = Integer.parseInt( properties.getProperty("step.number") );
            nb_step_invocation = Integer.parseInt( properties.getProperty("step.invocation.number") );
            step_start = Integer.parseInt( properties.getProperty("step.start") );
            host = properties.getProperty("com.host");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
    /**
     * The bootstrap method.
     * 
     * @param args - Client arguments.
     */
    public static void main(String[] args)
    {
        if (args.length != 1)
        {
            System.out.println("Usage: Client <dest_file>");
            System.exit(0);
        }

        Client c = new Client(args[0]);
        c.run();
    }

    /**
     * Runs n invocations on the CORBA client.
     */
    public void run()
    {
        String endpoint = "http://"+host+":8080/axis/PingWS.jws";

        // Obtains the Ping Itf reference
        try {
            Service service = new Service();
            ping_ws = (Call) service.createCall();
             
            ping_ws.setTargetEndpointAddress( new java.net.URL(endpoint) );
            ping_ws.setOperationName("ping");
            ping_ws.setReturnType( XMLType.AXIS_VOID );
        } catch (Exception e) {
            e.printStackTrace();
        }

        // Runs benchmark
        test();
    }

    /**
     * Runs the latency benchmark.
     */
    public void test()
    {
        Object[] params = new Object[0];
        
        // Don't include the first invocation
        try
        {
            for (int i=0; i<step_start; i++) 
                ping_ws.invoke(params);
        } catch (Exception e) {
            e.printStackTrace();
        }
        
        // Runs test
        System.out.println("Running WS Benchmark with " + nb_step + " step(s) of " + nb_step_invocation +" method calls.");
        try{
            for(int j=0; j<nb_step; j++)
            {
                long timer = System.currentTimeMillis();
                
                for (int i=0; i<nb_step_invocation; i++)
                    ping_ws.invoke(params);
                timer = (long) ( ( System.currentTimeMillis() - timer ) * 1000 / (float)nb_step_invocation );
            
                // write results
                ResultPrinter p = new ResultPrinter(result_file);
                p.write(timer);
                p.close();
            }
        }catch(Exception ex){
            ex.printStackTrace();
        }
    }
}
    
