/*====================================================================

Objectweb Browser framework
Copyright (C) 2000-2003 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.plugin.java;
import java.util.Properties;

import org.objectweb.util.browser.api.MenuItem;
import org.objectweb.util.browser.api.MenuItemTreeView;
import org.objectweb.util.browser.api.TreeView;
import org.objectweb.util.browser.gui.api.DialogAction;
import org.objectweb.util.browser.gui.api.DialogBox;
import org.objectweb.util.browser.gui.lib.DefaultDialogBox;
import org.objectweb.util.browser.gui.lib.LabelBox;

/**
 * This class allows to add a propertie into a <code>java.util.Properties<code> object.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * 
 * @version 0.1
 */
public class AddPropertyAction 
    implements MenuItem, DialogAction {

    //==================================================================
    //
    // No internal state.
    //
    //==================================================================

    /** The Properties object where adding the property */
    protected Properties properties_;
    
    /** The boxes to specify the values */
    protected LabelBox keyBox_, valueBox_;

    //==================================================================
    //
    // No constructor.
    // 
    //==================================================================

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================

    //==================================================================
    //
    // Public methods for MenuItem interface.
    //
    //==================================================================

	/**
	 * @see org.objectweb.util.browser.api.MenuItem#getStatus(org.objectweb.util.browser.api.TreeView)
	 */
	public int getStatus(TreeView treeView) {
        return MenuItem.ENABLED_STATUS;
	}

	/**
	 * @see org.objectweb.util.browser.api.MenuItem#actionPerformed(org.objectweb.util.browser.api.MenuItemTreeView)
	 */
	public void actionPerformed(MenuItemTreeView treeView) throws Exception {
        properties_ = (Properties)treeView.getSelectedObject();
        DialogBox dialog_ = new DefaultDialogBox("Add a new property");
        keyBox_ = new LabelBox("Key");
        valueBox_ = new LabelBox("Value");
        dialog_.addElementBox(keyBox_);
        dialog_.addElementBox(valueBox_);
        dialog_.setValidateAction(this);
        dialog_.show();
	}

    //==================================================================
    //
    // Public methods for DialogAction interface.
    //
    //==================================================================

	/**
	 * Overriding methods
	 * @see org.objectweb.util.browser.gui.api.DialogAction#executeAction()
	 */
	public void executeAction() throws Exception {
        properties_.setProperty(keyBox_.getLabel(),valueBox_.getLabel());
	}

}