/*====================================================================
 
 Objectweb Browser Framework
 Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
 Contact: openccm@objectweb.org
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 USA
 
 Initial developer(s): Jerome Moroy.
 Contributor(s): ______________________________________.
 
 ---------------------------------------------------------------------
 $Id: BindingControllerContext.java,v 1.1 2004/04/20 16:37:26 moroy Exp $
 ====================================================================*/

package org.objectweb.util.browser.plugins.fractal.context;

import java.util.Vector;

import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.core.naming.AbstractContext;
import org.objectweb.util.browser.core.naming.DefaultEntry;
import org.objectweb.util.browser.core.naming.DefaultName;

/**
 * Binding controller context.
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class BindingControllerContext 
     extends AbstractContext
{
    
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    
    /** The binding controller. */
    private BindingController bc_ = null;
    
    // ==================================================================
    //
    // No Constructors.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Public methods for interface Context.
    //
    // ==================================================================
    
    /**
     * Returns an array containing the entries contained by the target context.
     * @return A new array of entry.
     */
    public Entry[] getEntries() {
        String[] itfs = bc_.listFc() ;
        Vector entries = new Vector();
        for (int i=0 ; i < itfs.length ; i++) {
            try {
                Interface itf = (Interface)bc_.lookupFc(itfs[i]);
                entries.add(new DefaultEntry(new ClientInterfaceWrapper(itf), 
                                             new DefaultName(itfs[i]), this));
            } catch (NoSuchInterfaceException e) { }
        }
        return (Entry[]) entries.toArray(new Entry[0]);    
    }
    
    // ==================================================================
    //
    // Additional public methods for Wrapper interface.
    //
    // ==================================================================
    
    /**
     * Fix the wrapped object.
     * @param object The object to wrap
     */
    public void setWrapped(Object object) {        
        bc_ = (BindingController) object;
    }
    
    /**
     * Gets the wrapped object.
     * @return The wrapped object
     */
    public Object getWrapped() {
        return bc_;
    }
}