/*====================================================================
 
 Objectweb Browser Framework
 Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
 Contact: openccm@objectweb.org
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 USA
 
 Initial developer(s): Jerome Moroy.
 Contributor(s): ______________________________________.
 
 ---------------------------------------------------------------------
 $Id: ComponentTable.java,v 1.1 2004/04/20 16:37:27 moroy Exp $
 ====================================================================*/

package org.objectweb.util.browser.plugins.fractal.panel;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.api.Table;
import org.objectweb.util.browser.api.TreeView;
import org.objectweb.util.browser.core.api.ContextContainer;
import org.objectweb.util.browser.core.naming.DefaultEntry;
import org.objectweb.util.browser.core.naming.DefaultName;
import org.objectweb.util.browser.plugins.fractal.FcBrowser;
import org.objectweb.util.browser.plugins.fractal.context.ClientCollectionInterfaceContainer;
import org.objectweb.util.browser.plugins.fractal.context.ClientInterfaceWrapper;
import org.objectweb.util.browser.plugins.fractal.lib.SignatureWrapper;

/**
 * Displays the available business interfaces of a component.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class ComponentTable
  implements Table 
{
    
    //==================================================================
    //
    // Internal states.
    //
    //==================================================================
    
    //==================================================================
    //
    // No constructor.
    //
    //==================================================================
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    /**
     * Return true if the given Interface is a Client interface.
     */
    protected boolean isClientType(Interface ir) {
        InterfaceType it = (InterfaceType) ir.getFcItfType();
        return it.isFcClientItf();
    }
    
    /**
     * Return true if the given Interface is a Collection Client interface.
     */
    protected boolean isClientCollectionType(Interface ir) {
        InterfaceType it = (InterfaceType)ir.getFcItfType();
        return (it.isFcClientItf() && it.isFcCollectionItf());
    }
    
    /**
     * Return true if the given Interface is a Control interface.
     */
    protected boolean isControlInterface(Interface itf) {
        return itf.getFcItfName().endsWith("-controller");
    }
    
    /**
     * Provides the bound interface of the given client interface.
     */
    protected Entry getBoundInterface(Interface itf, BindingController bc){
        if(bc!=null) {
            try {
                Interface bindInterface = (Interface)bc.lookupFc(itf.getFcItfName());
                if(bindInterface!=null)
                    return new DefaultEntry(bindInterface, 
                                            new DefaultName(FcBrowser.getPrefixedName(bindInterface)));
            } catch(NoSuchInterfaceException e) {
                return null;
            }
        }
        return null;
    }
    
    //==================================================================
    //
    // Public methods for Table interface.
    //
    //==================================================================
    
    /**
     * Overriding methods
     * @see org.objectweb.util.browser.api.Table#getHeaders()
     */
    public String[] getHeaders(TreeView treeView) {
        return new String[] {"Interface", "Type", "Connected"};
    }
    
    /**
     * Overriding methods
     * @see org.objectweb.util.browser.api.Table#getRows(org.objectweb.util.browser.api.TreeView)
     */
    public Object[][] getRows(TreeView treeView) {
        Hashtable collectionInterfaces = new Hashtable();
        Vector values = new Vector();
        
        Component comp = (Component)treeView.getSelectedObject();
        BindingController bc = null;
        try{
            bc = FcBrowser.getBindingController(comp);
        } catch(Exception e) {
            System.err.println("Client interface Error : " + e.getMessage());   
        }
        
        Object[] objects = (Object[])comp.getFcInterfaces();
        for (int i = 0; i < objects.length; i++){
            if(!comp.equals(objects[i])){
                Interface itf = (Interface)objects[i];
                if (!isControlInterface(itf)) {
                    String sign = ((InterfaceType)itf.getFcItfType()).getFcItfSignature();
                    InterfaceType it = (InterfaceType)itf.getFcItfType();
                    if (isClientCollectionType(itf)) {                    
                        String name = FcBrowser.getName(it);
                        if(!collectionInterfaces.containsKey(it.getFcItfName())){
                            ContextContainer cc = new ClientCollectionInterfaceContainer(itf);
                            cc.addEntry(name,new TableClientCollectionWrapper(itf));
                            collectionInterfaces.put(it.getFcItfName(),cc);
                        } else {
                            ((ContextContainer)collectionInterfaces.get(it.getFcItfName())).addEntry(name,new TableClientCollectionWrapper(itf));
                        }
                    }  else if(isClientType(itf)) {
                        values.add(new Object[] {new DefaultEntry(new ClientInterfaceWrapper(itf), 
                                                                  new DefaultName(FcBrowser.getName(it))),
                                                 new DefaultEntry(new SignatureWrapper(sign), 
                                                                  new DefaultName(sign)),
                                                 getBoundInterface(itf,bc)});
                    } else {
                        values.add(new Object[] {new DefaultEntry(itf, 
                                                                  new DefaultName(FcBrowser.getName(it))),
                                                 new DefaultEntry(new SignatureWrapper(sign), 
                                                                  new DefaultName(sign)),
                                                 null});
                    }
                }
            }
        }
        // Add the collection interfaces
        Enumeration keys,elements;
        keys = collectionInterfaces.keys();
        elements = collectionInterfaces.elements();
        for (int i = 0 ; i < collectionInterfaces.size() ; i++) {
            ClientCollectionInterfaceContainer itfcc = (ClientCollectionInterfaceContainer)elements.nextElement();
            Entry[] entries = itfcc.getEntries();
            for(int j = 0 ; j < entries.length ; j++) {
                Interface itf = ((TableClientCollectionWrapper)entries[j].getValue()).getItf();
                String sign = ((InterfaceType)itfcc.getItf().getFcItfType()).getFcItfSignature();
                values.add(new Object[]{entries[j],
                        new DefaultEntry(new SignatureWrapper(sign), 
                                new DefaultName(sign)),
                                getBoundInterface(itf,bc)}); 
            } 
        }
        
        Object[][] contenu = new Object[values.size()][2];
        for(int i=0 ; i<values.size() ; i++)
            contenu[i] = (Object[])values.get(i);
        return contenu;
    }
}
