/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.api;

/** To use the Context interface */
import org.objectweb.util.browser.api.Context;

/**
 * Standard interface for JPopupMenu factories.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public interface MenuFactoryConfiguration {

    /**
     * The standard name of this interface to use
     * in the context of the Fractal framework.
     */
    static final String MENU_FACTORY_CONFIGURATION =
        MenuFactoryConfiguration.class.getName();

    /**
     * Fixes the inheritTreeMenu value.
     * @param inheritTreeMenu true if this menu has to inherit menus from parents at the tree level 
     */
    public void setInheritTreeMenu(boolean inheritTreeMenu);

    /**
     * Fixes the inheritTypeMenu value.
     * @param inheritTypeMenu true if this menu has to inherit menus from parents at the java type inheritance level 
     */
    public void setInheritTypeMenu(boolean inheritTypeMenu);

    /**
     * Fixes the context containing the popup menu configuration
     * 
     * @param items The context containing the popup menu configuration
     */
    public void setItems(Context items);

    /**
     * Fixes the context containing the items for tree child
     * @param childItems The context containing the popup menu configuration for tree child
     */
    public void setChildItems(Context childItems);

    /**
     * Gives the context containing the items for this object.
     * @return The context containing this menu configuration
     */
    public Context getItems();

    /**
     * Gives the context containing the items for tree child
     * @return The context containing the tree child configuration
     */
    public Context getChildItems();

    /**
     * Fixes the list of items from which this menu have to inherit at the java type level.
     * @param inheritMenu The list of context
     */
    public void setInheritTypeMenu(Context[] inheritMenu);

    /**
     * Fixes the list of items for tree child.
     * @param inheritMenu The list of context
     */
    public void setInheritTreeMenu(Context[] inheritMenu);

    /**
     * Fixes the name of the class from which this menu has to be instanciate.
     * @param className The name of the class
     */
    public void setClassName(String className);

}
