/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.core.panel;

import org.objectweb.util.browser.api.Panel;
import org.objectweb.util.browser.api.Table;
import org.objectweb.util.browser.core.api.ContextProperty;
import org.objectweb.util.browser.core.api.TableFactory;
import org.objectweb.util.browser.core.common.DynamicTree;

/**
 * 
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * 
 * @version 0.1
 */
public class DefaultTableFactory 
  implements TableFactory {

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    /** The class to instantiate */
    protected Class class_ = null;

    /** The tree */
    protected DynamicTree tree_ = null;

    //==================================================================
    //
    // No constructor.
    // 
    //==================================================================

    public DefaultTableFactory(DynamicTree tree){
        tree_ = tree;
    }

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================

    //==================================================================
    //
    // No public method.
    //
    //==================================================================

    /**
     * Return a Panel associated to the param's object.
     *
     * @param object The object for which the panel is created.
     * @return The associated panel.
     */
    public Panel newPanel(Object object){
        Table table = null;
        if (class_ != null) {
            try {
                table = (Table)class_.newInstance();
            } catch (java.lang.InstantiationException e) {
                System.out.println(class_.getName() + " : Instanciation exception");
            } catch (java.lang.IllegalAccessException e) {
                System.out.println(class_.getName() + " : Illegal access exception");
            }
        }
        if(table!=null && tree_!=null){
            ContextProperty cp = tree_.getContextProperty(); 
            if(cp!=null)
                return new DefaultTablePanel(table, tree_.getAdminCustomization(), cp.getDecoder());
            else
                return new DefaultTablePanel(table, tree_.getAdminCustomization());
        }
        return null;        
    }

    /**
     * Overriding methods
     * @see org.objectweb.util.browser.core.api.TableFactory#setClassName(java.lang.Class)
     */
    public void setClassName(Class theClass) {
        class_ = theClass;
    }

}

