/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.panel;

/** The console's imports */
import java.util.List;
import java.util.Iterator;

import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.core.api.BrowserProperty;
import org.objectweb.util.browser.core.api.PanelFactory;
import org.objectweb.util.browser.core.api.Role;
import org.objectweb.util.browser.core.common.ClassesInheritance;
import org.objectweb.util.browser.core.common.DefaultPropertyContainer;
import org.objectweb.util.browser.core.common.ExtendedBoolean;

/**
 *
 * It contains a cache memory which allow to have only one factory per class type.
 * You can inherit parent panels at the type level but not at the role level.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class PanelPropertyContainer extends DefaultPropertyContainer {

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default contstructor
     * @param loader The object to use to load new configuration
     */
    public PanelPropertyContainer(BrowserProperty loader) {
        super(loader);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Tries to load an element for the associated class name.
     * @param className The name of the class 
     * @return The founded entry or null
     */
    protected Entry getElement(String className, Role role, ExtendedBoolean nodeFound) {
        return loader_.getPanel(className, role, nodeFound);
    }

    /**
     * Adds the contextFactory to the container.
     *
     * @param id The name of the entry to be retrieved.
     * @param object The contextFactory to add
     * 
     */
    protected void addEntry(String id, PanelFactory object) {
        super.addEntry(id, object);
    }

    // ==================================================================
    //
    // Surcharging public methods for DefaultContextContainer class.
    //
    // ==================================================================

    /**
     * Adds the object to the container.
     *
     * This method does not modify the given name and the target context.
     *
     * @param id The name of the entry to be retrieved.
     * @param object The object to add
     * 
     */
    public void addEntry(String id, Object object) {
        if (object != null) {
            addEntry(id, (PanelFactory) object);
        }
    }

    /**
     * Finds the object associated value. 
     * If the key doesn't exist, it looks for in interfaces implemented 
     * by this class and in superclass of the entity
     * In order to have a string representation of the object, 
     * it takes his name (object.getClass().getName())
     * @param object The research key
     * @return The associated value - null is returned if no value exists
     */
    public Entry getRecursiveProperty(Object object) {
        String cKey = getKey(object.getClass());
        // Look for existing association into the cache
        if (cache_.containsKey(cKey)) {
            return getLocalEntry((String) cache_.get(cKey));
        }
        ClassesInheritance ci = new ClassesInheritance(object.getClass());
        List list = ci.getInheritClasses();
        Iterator it = list.iterator();
        Entry returnedEntry = null;
        PanelFactory panelFactory = null;
        boolean isFirst = true;
        while (it.hasNext()) {
            String key = getKey((Class) it.next());;
            Entry entry = getLocalEntry(key);
            if (entry == null) {
                entry = tryToLoad(key);
            }
            if (entry != null) {
                if(isFirst){
                    cache_.put(cKey, key);
                    returnedEntry = entry;
                    panelFactory = (PanelFactory)entry.getValue();
                    isFirst = false;
                } else {
                    panelFactory.addInheritPanelFactory((PanelFactory)entry.getValue());
                }
                PanelFactory currentPanelFactory = (PanelFactory)entry.getValue();
                if(!currentPanelFactory.inheritTypePanel()) break;
            }
        }
        return returnedEntry;
    }

}
