/*===========================================================================

ObjectWeb Browser Framework
Copyright (C) 2004 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jrme Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.popup;

/** The namingContext's imports*/
import org.objectweb.util.browser.api.IconProvider;
import org.objectweb.util.browser.api.MenuItem;
import org.objectweb.util.browser.api.MenuItemTreeView;
import org.objectweb.util.browser.api.TreeView;
import org.objectweb.util.browser.core.common.ClassResolver;
import org.objectweb.util.browser.core.common.DynamicTree;
import org.objectweb.util.browser.core.icon.EmptyIconProvider;

/** The java API's imports*/
import java.awt.Component;
import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

public class AdvancedJPopupMenu extends JPopupMenu {

    /** The associated tree */
    protected DynamicTree tree_;

    /** Indicates if the menu has to inherit of his parent tree menu */
    protected boolean inheritTreeMenu_;

    /** A treeview to obtain the object on which the menu item will act*/
    protected MenuItemTreeView treeView_;

    public AdvancedJPopupMenu(DynamicTree tree, MenuItemTreeView treeView, boolean inheritTreeMenu) {
        tree_ = tree;
        treeView_ = treeView;
        inheritTreeMenu_ = inheritTreeMenu;        
    }

    /**
     * Default constructor
     * @param tree The associated tree
     * @param 
     */
    public AdvancedJPopupMenu(DynamicTree tree, boolean inheritTreeMenu) {
        this(tree, new DefaultMenuItemTreeView(tree), inheritTreeMenu);
    }

    /**
     * Default constructor
     * @param tree The associated tree
     */
    public AdvancedJPopupMenu(DynamicTree tree) {
        this(tree, false);
    }

    public AdvancedJPopupMenu(DynamicTree tree, MenuItemTreeView treeView) {
        this(tree, treeView, false);
    }

    /**
     * Creates a JMenuItem with the given params
     * @param label The label of the item
     * @param action The corresponding action
     * @param itemState Indicates the item states ()
     * @return The created JMenuItem
     */
    protected JMenuItem makeMenuItem(String label, MenuItem action, int itemState, ItemProperty itemProperty) {
        
        IconProvider iconProvider = itemProperty.getIconProvider();
        boolean isUserIcon = iconProvider!=null;
        if(!isUserIcon)
            iconProvider = new EmptyIconProvider();
        Icon icon = iconProvider.newIcon(treeView_.getSelectedObject());
           
        KeyStroke keyStroke = itemProperty.getKeyStroke();
        Action theAction = new DefaultAction(tree_, action, treeView_, label, icon, itemProperty.getKeyStroke(), itemProperty.getMnemonic(), isUserIcon, itemState);
        MyJMenuItem item = new MyJMenuItem(theAction, action.getClass().getName());
        //if(keyStroke!=null){
        //    item.setAccelerator(keyStroke);
        //}
        
        //MyJMenuItem item = null;
        //if(iconProvider!=null)
        //    item = new MyJMenuItem(tree_, label, treeView_, iconProvider.newIcon(treeView_.getSelectedObject()));
        //else
        //    item = new MyJMenuItem(tree_, label, treeView_, null);
        //item.addActionListener(action);
        if (itemState == MenuItem.DISABLED_STATUS)
            item.setEnabled(false);
        return item;
    }

    /**
     * Indicates if the given menuItem already exists into the current menu.
     * @param menuItem The item to test
     * @return true if the given menuItem already exists
     */
    protected boolean contains(MyJMenuItem menuItem) {
        Component myComponent[] = getComponents();
        for (int i = 0; i < myComponent.length; i++)
            if (myComponent[i] instanceof MyJMenuItem
                && ((MyJMenuItem) myComponent[i]).getText().equals(menuItem.getText())
                /*&& ((MyJMenuItem) myComponent[i]).getActionClassName().equals(menuItem.getActionClassName())*/)
                return true;
        return false;
    }

    /**
     * Returns the inheritTreeMenu value.
     * @return true if the menu has to inherit tree menu   
     */
    public boolean hasToInheritTreeMenu() {
        return inheritTreeMenu_;
    }

    /**
     * Overwritten of the container method
     * Avoid to add two separators successively.
     * @param comp The component to add
     * @return The added component
     */
    public Component add(Component comp) {
        if (getComponentCount() > 0) {
            Component c = getComponent(getComponentCount() - 1);
            if (!(c instanceof javax.swing.JPopupMenu.Separator && comp instanceof javax.swing.JPopupMenu.Separator))
                return super.add(comp);
        } else if (!(getComponentCount() == 0 && comp instanceof javax.swing.JPopupMenu.Separator))
            return super.add(comp);
        return comp;
    }

    /**
     * Add all the menu's items into the current menu.
     * If an item already exists, this one is not included
     * @param popupMenu The menu to add
     */
    public void add(JPopupMenu popupMenu) {
        if (popupMenu != null) {
            Component toAdd[] = popupMenu.getComponents();
            if (toAdd.length > 0) {
                addSeparator();
                for (int i = 0; i < toAdd.length; i++) {
                    if (((toAdd[i] instanceof MyJMenuItem) && !contains((MyJMenuItem) toAdd[i])) || !(toAdd[i] instanceof MyJMenuItem))
                        add(toAdd[i]);
                }
                // Remove the last component if it's a Separator instance
                Component c = getComponent(getComponentCount() - 1);
                if (c instanceof javax.swing.JPopupMenu.Separator)
                    remove(c);
            }
        }
    }

    /**
     * Adds an item into the current menu
     * @param label Label of the item
     * @param className The name of the class to instanciate
     * @param tree The associated tree
     */
    public void addMenuItem(String label, ItemProperty itemProperty, DynamicTree tree) {
        if(label!=null && itemProperty!=null){
            String className = itemProperty.getClassName();
            if(label.startsWith("-separator-")){
                addSeparator();        
            } else if (className != null && !className.equals("")) {
                MenuItem action = null;
                try {
                    action = (MenuItem) ClassResolver.resolve(className).newInstance();
                } catch (java.lang.ClassNotFoundException e) {
                    System.out.println("Error : " + className + " : Class Not Found !");
                } catch (java.lang.InstantiationException e) {
                    e.printStackTrace();
                } catch (java.lang.IllegalAccessException e) {
                    e.printStackTrace();
                } catch (java.lang.ClassCastException e) {
                    System.out.println("Error : " + className + " : Not a org.objectweb.util.browser.api.MenuItem implementation");
                }
                if (action != null){
                    int itemState = action.getStatus(treeView_);
                    if(itemState==MenuItem.DISABLED_STATUS || itemState==MenuItem.ENABLED_STATUS)
                        super.add(makeMenuItem(label, action, itemState, itemProperty));
                    else if (itemState==MenuItem.NOT_VISIBLE_STATUS){
                        // Nothing to do
                    }
                }
            }
        }
    }
    
    /**
     * Fixes the treeView to use depending on the context. It can be a treeView provided by a JTree or by a JTable.
     * @param treeView The TreeView to use.
     */
    public void setTreeView(TreeView treeView){
        
    }
}
