/*====================================================================

Objectweb Browser framework
Copyright (C) 2000-2003 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.gui.lib;

import javax.swing.Box;
import javax.swing.BoxLayout;

import org.objectweb.util.browser.gui.api.ElementBox;
import org.objectweb.util.browser.gui.api.ValidateReport;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * 
 * @version 0.1
 */

public abstract class AbstractElementBox 
    extends Box 
    implements ElementBox {

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    /** Indicates if the value is mandatory or optional. The default value is true, so, it's mandatory */
    protected boolean isMandatory_;

    //==================================================================
    //
    // Constructors.
    // 
    //==================================================================

    /**
     * Empty constructor.
     */
    public AbstractElementBox(){
        this(true);
    }

    /**
     * Constructor with params.
     * @param isMandatory If the value is mandatory : TRUE, else : FALSE 
     */
    public AbstractElementBox(boolean isMandatory){
        super(BoxLayout.X_AXIS);
        isMandatory_ = isMandatory;
    }

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================

    //==================================================================
    //
    // Public methods for ElementBox interface.
    //
    //==================================================================

    /** 
     * @see org.objectweb.util.browser.gui.api.ElementBox#validateBox()
     */
    public abstract ValidateReport validateBox();

     /**
      * Returns this.
      * @see org.objectweb.util.browser.gui.api.ElementBox#getBox()
      */
    public Box getBox() {
        return this;
    }

    /**
     * Nothing to do. 
     * @see org.objectweb.util.browser.gui.api.ElementBox#preInitialize()
     */
    public void preInitialize() {
        // Nothing to do
    }

    /** 
     * Nothing to do.
     * @see org.objectweb.util.browser.gui.api.ElementBox#postInitialize()
     */
    public void postInitialize() {
        // Nothing to do
    }

}