/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.gui.lib;

import java.awt.Dimension;

import javax.swing.Box;
import javax.swing.JScrollPane;

import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.api.Wrapper;
import org.objectweb.util.browser.gui.api.ValidateReport;
import org.objectweb.util.browser.core.common.DynamicTree;

/**
 * This class represents the panel which displays a DynamicTree. 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * 
 * @version 0.1
 */
public class TreeBox 
    extends AbstractElementBox {

    //==================================================================
    //
    // Internal state.
    //
    //==================================================================

    /** The tree to display */
    protected DynamicTree tree_;
    
    /** TheJScrollPane associated to the tree */
    protected JScrollPane scrollPane_;

    //==================================================================
    //
    // Constructors.
    // 
    //==================================================================

    /**
     * Default constructor
     * @param tree The DynamicTree to use
     */     
    public TreeBox(DynamicTree tree) {
        this(tree,true);   
    }

    /**
     * 
     * @param tree The DynamicTree to use
     * @param isMandatory If the value is mandatory : TRUE, else : FALSE
     */
    public TreeBox(DynamicTree tree, boolean isMandatory) {
        super(isMandatory);
        tree_ = tree;
        scrollPane_ = new JScrollPane(tree_); 
        add(scrollPane_);
    }

    /**
     * Invokes just before displaying.
     */
    public void preInitialize(){
        if(tree_!=null){
            tree_.setPopupEnabled(false);
            tree_.setDragAndDropEnabled(false);
        }
    }
    
    /**
     * Invokes just before removing.
     */
    public void postInitialize(){
        if(tree_!=null){
            tree_.setPopupEnabled(true);
            tree_.setDragAndDropEnabled(true);
        }
    }

    //==================================================================
    //
    // Internal methods.
    //
    //==================================================================

    // ==================================================================
    //
    //  Public methods for ElementBox.
    //
    // ==================================================================

    /**
     * Validates the content of the ElementBox. Indicates if the different value are acceptables or not.
     * @return The corresponding ValidateReport.
     */
    public ValidateReport validateBox(){
        if(isMandatory_){
            if(getObject()==null)
                return new DefaultValidateReport(false,"You must select a component");
        }
        return new DefaultValidateReport();
    }
    
    /**
     * Returns the Box contains by the ElementBox.
     * @return The Box contains by the ElementBox.
     */
    public Box getBox(){
        return this;
    }

    //==================================================================
    //
    // Public methods.
    //
    //==================================================================

    /** Returns the chosen Object */
    public Object getObject(){
        Object o = null;
        Entry entry = tree_.getSelectedEntry();
            if (entry != null) {
                o = entry.getValue();
                if (o!=null && Wrapper.class.isAssignableFrom(o.getClass()))
                    return ((Wrapper)o).getWrapped();
            }
        return o;        
    }

    /**
     * Sets the preferred dimension of the JScrollPane.
     * @param d The preferred dimension of the JScrollPane
     */
    public void setPreferredSize(Dimension d){
        scrollPane_.setPreferredSize(d);        
    }

}