/*===========================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jrme Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.openccm.explorer.CORBA;

import org.objectweb.util.explorer.api.MenuItem;
import org.objectweb.util.explorer.api.MenuItemTreeView;
import org.objectweb.util.explorer.api.TreeView;
import org.objectweb.util.explorer.swing.gui.api.DialogAction;
import org.objectweb.util.explorer.swing.gui.api.DialogBox;
import org.objectweb.util.explorer.swing.gui.lib.DefaultDialogBox;
import org.objectweb.util.explorer.swing.gui.lib.LabelBox;

/**
 * This action allows us to change the value of an attibute.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * 
 * @version 0.1
 */
public class ChangeAttributeValue
  implements MenuItem, DialogAction
{

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    /** The associated Object. */
    protected org.omg.CORBA.Object object_ = null;

    /** The name of the attribute. */
    protected String name_ = null;

    /** The LabelBox to field the new value. */
    protected LabelBox attributeLabel_ = null;

    //==================================================================
    //
    // No constructor.
    //
    //==================================================================

    //==================================================================
    //
    // Internal method.
    //
    //==================================================================

    /**
     * Must be replaced by a class with the given interface:
     *   public interface TypeWrapper{
     *       public String toString(Object)
     *       public Object fromString(String)
     *   }
     */
    public Object fromString(String value, Class type){
        String typeName = type.getName();
        if(type.equals(Integer.TYPE))
            return new Integer(value);
        else if(type.equals(Character.TYPE))
            return new Character(value.charAt(0));
        else if(type.equals(Double.TYPE))
            return new Double(value);
        else if(type.equals(Boolean.TYPE))
            return new Boolean(value);
        else if(type.equals(Short.TYPE))
            return new Short(value);
        else if(type.equals(Long.TYPE))
            return new Long(value);            
        else if(type.equals(Float.TYPE))
            return new Float(value);            
        return value;
    }
 
    //==================================================================
    //
    // Public methods for MenuItem interface.
    //
    //==================================================================

    /**
     * Enabled if a method which allows us to fix a value exists.
     */
    public int getStatus(TreeView treeView) {
        ViewAttributeWrapper viewAttr = (ViewAttributeWrapper)treeView.getSelectedObject();
        org.omg.CORBA.Object o = viewAttr.getObject();
        String attrName = viewAttr.getName();
        try{
            Class c = o.getClass();
            c.getMethod(attrName,new Class[]{c.getMethod(attrName,null).getReturnType()});
            return MenuItem.ENABLED_STATUS;
        } catch(NoSuchMethodException e) {
            return MenuItem.DISABLED_STATUS;   
        }
    }

    /**
     * Provides a dialog box in order to change the value of the attribute.
     */
    public void actionPerformed(MenuItemTreeView treeView) throws Exception {
        ViewAttributeWrapper viewAttr = (ViewAttributeWrapper)treeView.getSelectedObject();
        object_ = viewAttr.getObject();
        name_ = viewAttr.getName();
        DialogBox dialog = new DefaultDialogBox("Change the value of this attribute");      
        Class c = object_.getClass();
        attributeLabel_ = new LabelBox(name_,c.getMethod(name_,null).invoke(object_,null).toString());
        dialog.addElementBox(attributeLabel_);
        dialog.setValidateAction(this);
        dialog.show();
    }

    //==================================================================
    //
    // Public methods for DialogAction interface.
    //
    //==================================================================

    /**
     * Fixes the value of the attribute.
     */
    public void executeAction() throws Exception {
        String value = attributeLabel_.getLabel();
        Class c = object_.getClass();
        Class type = c.getMethod(name_,null).getReturnType();
        java.lang.reflect.Method method = c.getMethod(name_,new Class[]{type});
        method.invoke(object_,new Object[]{fromString(value,type)});
    }    

}
