/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.explorer.CORBA;

import javax.swing.Timer;

import org.objectweb.openccm.explorer.util.ior.IorPrinter;
import org.objectweb.util.explorer.api.MenuItem;
import org.objectweb.util.explorer.api.MenuItemTreeView;
import org.objectweb.util.explorer.api.TreeView;
import org.objectweb.util.explorer.swing.gui.api.Console;

/**
 * This action checks if a CORBA object already exists.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class PingAction 
  implements MenuItem 
{

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    protected Timer timer_;
    
    //==================================================================
    //
    // Internal methods.
    //
    //==================================================================
    
    //==================================================================
    //
    // Public methods for MenuItem interface.
    //
    //==================================================================

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.MenuItem#getStatus(org.objectweb.util.explorer.api.TreeView)
     */
    public int getStatus(TreeView treeView) {
        return MenuItem.ENABLED_STATUS;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.MenuItem#actionPerformed(org.objectweb.util.explorer.api.MenuItemTreeView)
     */
    public void actionPerformed(MenuItemTreeView e) throws Exception {
        org.omg.CORBA.Object object = (org.omg.CORBA.Object)e.getSelectedObject();
        org.omg.CORBA.ORB orb = org.objectweb.openccm.corba.TheORB.getORB();
        IorPrinter iorPrinter = new IorPrinter(orb.object_to_string(object));
        Console console = ConsoleFactory.getPingConsole();
        console.clear();
        timer_ = new javax.swing.Timer(1000, new PingObjectAction(console, e.getSelectedEntry().getName().toString() + " (" + iorPrinter.getHost() + ":" + iorPrinter.getPort() + ")", object));
        timer_.setInitialDelay(0);
        startTimer();
        console.show();
        stopTimer();
    }

    public void startTimer(){
        timer_.start();
    }

    public void stopTimer(){
        timer_.stop();
    }

    //==================================================================
    //
    // Protected classes.
    //
    //==================================================================

    protected class PingObjectAction
        implements java.awt.event.ActionListener {

        protected Console console_;

        protected String name_;

        protected org.omg.CORBA.Object object_;

        public PingObjectAction(Console console, String name, org.omg.CORBA.Object object){
            console_ = console;
            name_ = name;
            object_ = object;
        }

        public void actionPerformed(java.awt.event.ActionEvent e){
            console_.add("Pinging " + name_ + ": ");
            try {
                long timeBefore = System.currentTimeMillis();
                object_._non_existent();
                long timeAfter = System.currentTimeMillis();
                console_.add("time=" + (timeAfter - timeBefore) + " ms\n");
            }catch(Exception ex){
                console_.add("Error: " + ex.getClass().getName());
                stopTimer();
            }
        }
        
    }
        
}
