/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.explorer.Components;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;

import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.objectweb.openccm.explorer.menu.TreeDialogSingleton;
import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.explorer.api.MenuItem;
import org.objectweb.util.explorer.api.MenuItemTreeView;
import org.objectweb.util.explorer.api.Tree;
import org.objectweb.util.explorer.api.TreeView;
import org.objectweb.util.explorer.swing.api.Explorer;

import org.omg.Components.CCMObject;
import org.omg.Components.ConsumerDescription;
import org.omg.Components.EmitterDescription;

/**
 * This items allows to subscribe a consumer to a publisher 
 * (<code>::Components::CCMObject::subscribe</code> called).
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class ConnectConsumerAction 
  implements MenuItem 
{

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.MenuItem#getStatus(org.objectweb.util.explorer.api.TreeView)
     */
    public int getStatus(TreeView arg0){
        return MenuItem.ENABLED_STATUS;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.MenuItem#actionPerformed(org.objectweb.util.explorer.api.MenuItemTreeView)
     */
    public void actionPerformed(MenuItemTreeView e) throws Exception {
        EmitterContainer ec = (EmitterContainer) e.getSelectedObject();
        EmitterDescription emitter = ec.getEmitter();
        CCMObject component = ec.getComponent();

        org.objectweb.fractal.api.Component tree = TreeDialogSingleton.getInstance();
        Explorer explorerItf = (Explorer)tree.getFcInterface(Explorer.EXPLORER);
        Tree treeItf = (Tree)tree.getFcInterface(Tree.TREE);
        
        JPanel treePanel = new JPanel();
        treePanel.setLayout(new GridLayout(1, 0));
        treePanel.setPreferredSize(new Dimension(450, 350));
        treePanel.add(new JScrollPane(explorerItf.getTree()));
        ActionEvent ae = (ActionEvent)e.getEvent();
        int result = JOptionPane.showOptionDialog((Component) ae.getSource(), treePanel, "Select a consumer", JOptionPane.OK_CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE, null, null, null);
        if (result == 0) {
            Entry entry = treeItf.getSelectedEntry();
            if (entry != null) {
                Object object = entry.getValue();
                if (object != null) {
                    System.err.println(object);
                    ConsumerDescription consumer = (ConsumerDescription) object;
                    if (component != null && emitter != null && consumer != null) {
                        component.connect_consumer(emitter.name, consumer.consumer);
                    }
                }
            }
        }
    }

}
