/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/

package org.objectweb.openccm.explorer.Components;

import java.util.List;
import java.util.Vector;
import java.util.Iterator;

import org.omg.Components.CCMObject;
import org.omg.Components.ComponentPortDescription;
import org.omg.Components.ReceptacleDescription;
import org.omg.Components.EmitterDescription;
import org.omg.Components.PublisherDescription;
import org.omg.Components.ConnectionDescription;
import org.objectweb.openccm.explorer.CORBA.TypageCORBA;
import org.objectweb.openccm.explorer.util.ior.IorPrinter;
import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.explorer.api.Table;
import org.objectweb.util.explorer.api.TreeView;
import org.objectweb.util.explorer.core.naming.lib.DefaultEntry;
import org.omg.Components.SubscriberDescription;

/**
 * Displays ports of the given <code>CCMObject</code> component.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class RequiredInterfacesTable 
    implements Table 
{

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================
    
    //==================================================================
    //
    // No constructor.
    //
    //==================================================================

    //==================================================================
    //
    // Internal methods.
    //
    //==================================================================  

    protected String getName(org.omg.CORBA.Object o) {
        TypageCORBA tc = new TypageCORBA(o, org.objectweb.openccm.corba.TheORB.getORB());
        return tc.getTypeID();
    }

    //==================================================================
    //
    // Public methods for Table interface.
    //
    //==================================================================

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Table#getHeaders(org.objectweb.util.explorer.api.TreeView)
     */
    public String[] getHeaders(TreeView treeView) {
        return new String[]{"Port","Connection","Connected"};
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Table#getRows(org.objectweb.util.explorer.api.TreeView)
     */
    public Object[][] getRows(TreeView treeView) {
        org.omg.CORBA.ORB orb = org.objectweb.openccm.corba.TheORB.getORB();
        IorPrinter iorPrinter = null;
        List l = new Vector();

        CCMObject component = (CCMObject) treeView.getSelectedObject();
        ComponentPortDescription cpd = component.get_all_ports();

        // List of receptacles
        ReceptacleDescription[] rd = cpd.receptacles;
        for (int i = 0; i < rd.length; i++) {
            ReceptacleContainer rc = new ReceptacleContainer();
            rc.setComponent(component);
            rc.setReceptacle(rd[i]);
            Entry rdEntry = new DefaultEntry(rd[i].name, rc);

            ConnectionDescription[] cd = rd[i].connections;
            if(cd.length > 0){
                for (int j = 0; j < cd.length; j++) {
                    ConnectionContainer cc = new ConnectionContainer();
                    cc.setConnection(cd[j]);
                    cc.setReceptacleContainer(rc);
                    Entry ccEntry = new DefaultEntry("Connection (" + j + ")", cc);
         
                    Entry cEntry = new DefaultEntry(getName(cd[j].objref), cd[j].objref);
           
                    l.add(new Object[]{rdEntry, ccEntry, cEntry});
                }
            } else {
                l.add(new Object[]{rdEntry, null, null});
            }
        }

        // List of emitters
        EmitterDescription[] ed = cpd.emitters;
        for (int i = 0; i < ed.length; i++) {
            EmitterContainer ec = new EmitterContainer();
            ec.setComponent(component);
            ec.setEmitter(ed[i]);
            Entry edEntry = new DefaultEntry(ed[i].name, ec);

            if(ed[i].consumer!=null){                
                Entry cEntry = new DefaultEntry(getName(ed[i].consumer), ed[i].consumer);
                l.add(new Object[]{edEntry, null, cEntry});
            } else {
                l.add(new Object[]{edEntry, null, null});
            }
        }

        // List of publishers
        PublisherDescription[] pd = cpd.publishers;
        for (int i = 0; i < pd.length; i++) {
            PublisherContainer pc = new PublisherContainer();
            pc.setComponent(component);
            pc.setPublisher(pd[i]);
            Entry pdEntry = new DefaultEntry(pd[i].name, pc);

            SubscriberDescription[] sd = pd[i].consumers;
            if(sd.length > 0){
                for (int j = 0; j < sd.length; j++) {
                    SubscriberContainer sc = new SubscriberContainer();
                    sc.setSubscriber_(sd[j]);
                    sc.setPublisherContainer_(pc);
                    Entry scEntry = new DefaultEntry("Connection (" + j + ")", sc);

                    Entry sEntry = new DefaultEntry(getName(sd[j].consumer), sd[j].consumer);
                    
                    l.add(new Object[]{pdEntry, scEntry, sEntry});
                }  
            } else {
                l.add(new Object[]{pdEntry, null, null});
            }
            
        }

        Object[][] contenu = new Object[l.size()][3];
        Iterator it = l.iterator();
        int cpt = 0;
        while(it.hasNext()){
            contenu[cpt] = (Object[])it.next();
            cpt++;
        }

        return contenu;
 
   }
}
