/*===========================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s):Sylvain Leblanc.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.openccm.explorer.CosTrading.gui;

import java.awt.Color;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import org.objectweb.openccm.explorer.menu.LabelBox;
import org.omg.CosTrading.Admin;
import org.omg.CosTrading.FollowOption;

/**
 * OMG CosTradingRepos TypeStruct Panel. 
 *
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 * @version 0.1
 */
public class AdminPanel 
     extends DefaultNodeViewPanel 
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The CosTrading Admin reference. */
    protected Admin admin_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default constructor. */
    public AdminPanel() {
        setBackground(Color.white);
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Returns a new management panel for attributes described in the
     * SupportAttributes OMG IDL interface.
     *
     * @return A new JPanel with all to manage SupportAttributes of a
     * trader.
     */
    protected JPanel addSupportAttributes() {
        JPanel supportPanel = new JPanel();
        supportPanel.setBackground(Color.white);
        supportPanel.setBorder(new TitledBorder(null," Support attributes ",TitledBorder.CENTER,TitledBorder.TOP));

        Box box = Box.createVerticalBox();

        // supports_modifiable_properties
		BooleanBox supports_modifiable_properties = new BooleanBox("Supports of modifiable properties: ", 
                                                                   admin_.supports_modifiable_properties());
        supports_modifiable_properties.addItemListener(new AbstractItemListener(admin_) 
            {
                protected void doAdminAction(Object modified_item) {       
                    adm_.set_supports_modifiable_properties(new Boolean((String)modified_item).booleanValue());
                }
            });
        supports_modifiable_properties.setToolTipText("Whether the trader supports property modification.");

        // supports_dynamic_properties
		BooleanBox supports_dynamic_properties = new BooleanBox("Supports of dynamic properties: ", 
                                                                admin_.supports_dynamic_properties());
        supports_dynamic_properties.addItemListener(new AbstractItemListener(admin_) 
            {
                protected void doAdminAction(Object modified_item) {       
                    adm_.set_supports_dynamic_properties(new Boolean((String)modified_item).booleanValue());
                }
            });
        supports_dynamic_properties.setToolTipText("Whether the trader supports dynamic properties.");

        // supports_proxy_offers
		BooleanBox supports_proxy_offers = new BooleanBox("Supports of proxy offers: ", 
                                                          admin_.supports_proxy_offers());	
        supports_proxy_offers.addItemListener(new AbstractItemListener(admin_) 
            {
                protected void doAdminAction(Object modified_item) {       
                    adm_.set_supports_proxy_offers(new Boolean((String)modified_item).booleanValue());
                }
            });
        supports_proxy_offers.setToolTipText("Whether the trader supports proxy offers.");

        box.add(supports_modifiable_properties);
        box.add(Box.createVerticalStrut(10));
        box.add(supports_dynamic_properties);
        box.add(Box.createVerticalStrut(10));
        box.add(supports_proxy_offers);

        supportPanel.add(box);

        return supportPanel;
    }

    /**
     * Returns a new management panel for attributes described in the
     * LinkAttributes OMG IDL interface.
     *
     * @return A new JPanel with all to manage LinkAttributes of a
     * trader.
     */
    protected JPanel addLinkAttributes() {
        JPanel linkAttrPanel = new JPanel();
        linkAttrPanel.setBackground(Color.white);
        linkAttrPanel.setBorder(new TitledBorder(null," Link attributes ",TitledBorder.CENTER,TitledBorder.TOP));

        // max_link_follow_policy
		FollowOptionBox max_link_follow_policy = new FollowOptionBox("Maximum link follow policy allowed: ", 
                                                                     admin_.max_link_follow_policy());
        max_link_follow_policy.addItemListener(new AbstractItemListener(admin_) 
            {
                protected void doAdminAction(Object modified_item) {
                    FollowOption opt;
                    String value = (String)modified_item;
                    if (value.equals("local_only")) opt = FollowOption.local_only;
                    else if (value.equals("if_no_local")) opt = FollowOption.if_no_local;
                    else opt = FollowOption.always;
                    adm_.set_max_link_follow_policy(opt);
                }
            });
        max_link_follow_policy.setToolTipText("Most permissive follow policy " + 
                                              "allowed when creating new links.");

        Box box = Box.createVerticalBox();
        box.add(max_link_follow_policy);

        linkAttrPanel.add(box);

        return linkAttrPanel;
    }

    /**
     * Returns a new management panel for attributes described in the
     * ImportAttributes OMG IDL interface.
     *
     * @return A new JPanel with all to manage ImportAttributes of a
     * trader.
     */
    protected JPanel addImportAttributes() {
        JPanel importAttrPanel = new JPanel();
        importAttrPanel.setBackground(Color.white);
        importAttrPanel.setBorder(new TitledBorder(null," Import attributes ",TitledBorder.CENTER,TitledBorder.TOP));

        // def_search_card
		LabelBox def_search_card = new LabelBox("Default search cardinality: ", String.valueOf(admin_.def_search_card()));
        def_search_card.addActionListener(new AbstractTextActionListener(admin_) 
            {
                protected void doAdminAction(java.awt.event.ActionEvent e) {
                    adm_.set_def_search_card(Integer.parseInt(e.getActionCommand()));
                }
            });
        def_search_card.setToolTipText("Default upper bound of offers to be searched for a query operation.");

        // max_search_card
		LabelBox max_search_card = new LabelBox("Maximum search cardinality: ", String.valueOf(admin_.max_search_card()));
        max_search_card.addActionListener(new AbstractTextActionListener(admin_) 
            {
                protected void doAdminAction(java.awt.event.ActionEvent e) {
                    adm_.set_max_search_card(Integer.parseInt(e.getActionCommand()));
                }
            });
        max_search_card.setToolTipText("Maximum upper bound of offers to be searched for a query operation.");

        // def_match_card
		LabelBox def_match_card = new LabelBox("Default match cardinality: ", String.valueOf(admin_.def_match_card()));
        def_match_card.addActionListener(new AbstractTextActionListener(admin_) 
            {
                protected void doAdminAction(java.awt.event.ActionEvent e) {
                    adm_.set_def_match_card(Integer.parseInt(e.getActionCommand()));
                }
            });
        def_match_card.setToolTipText("Default upper bound of matched offers to be ordered in applying a preference criteria.");

        // max_match_card
		LabelBox max_match_card = new LabelBox("Maximum match cardinality: ", String.valueOf(admin_.max_match_card()));
        max_match_card.addActionListener(new AbstractTextActionListener(admin_) 
            {
                protected void doAdminAction(java.awt.event.ActionEvent e) {
                    adm_.set_max_match_card(Integer.parseInt(e.getActionCommand()));
                }
            });
        max_match_card.setToolTipText("Maximum upper bound of matched offers to be ordered in applying a preference criteria.");

        // def_return_card
		LabelBox def_return_card = new LabelBox("Default return cardinality: ", String.valueOf(admin_.def_return_card()));
        def_return_card.addActionListener(new AbstractTextActionListener(admin_) 
            {
                protected void doAdminAction(java.awt.event.ActionEvent e) {
                    adm_.set_def_return_card(Integer.parseInt(e.getActionCommand()));
                }
            });
        def_return_card.setToolTipText("Default upper bound of ordered offers to be returned to an importer.");

        // max_return_card
		LabelBox max_return_card = new LabelBox("Maximum return cardinality: ", String.valueOf(admin_.max_return_card()));
        max_return_card.addActionListener(new AbstractTextActionListener(admin_) 
            {
                protected void doAdminAction(java.awt.event.ActionEvent e) {
                    adm_.set_max_return_card(Integer.parseInt(e.getActionCommand()));
                }
            });
        max_return_card.setToolTipText("Maximum upper bound of ordered offers to be returned to an importer.");

        // max_list
		LabelBox max_list = new LabelBox("Maximum list size: ", String.valueOf(admin_.max_list()));
        max_list.addActionListener(new AbstractTextActionListener(admin_) 
            {
                protected void doAdminAction(java.awt.event.ActionEvent e) {
                    adm_.set_max_list(Integer.parseInt(e.getActionCommand()));
                }
            });
        max_list.setToolTipText("The upper bound on the size of any list returned by the trader.");

        // def_hop_count
		LabelBox def_hop_count = new LabelBox("Default hop count: ", String.valueOf(admin_.def_hop_count()));
        def_hop_count.addActionListener(new AbstractTextActionListener(admin_) 
            {
                protected void doAdminAction(java.awt.event.ActionEvent e) {
                    adm_.set_def_hop_count(Integer.parseInt(e.getActionCommand()));
                }
            });
        def_hop_count.setToolTipText("Default upper bound of depth of links to be traversed.");

        // max_hop_count
		LabelBox max_hop_count = new LabelBox("Maximum hop count: ", String.valueOf(admin_.max_hop_count()));
        def_hop_count.addActionListener(new AbstractTextActionListener(admin_) 
            {
                protected void doAdminAction(java.awt.event.ActionEvent e) {
                    adm_.set_def_hop_count(Integer.parseInt(e.getActionCommand()));
                }
            });
        max_hop_count.setToolTipText("Maximum upper bound of depth of links to be traversed.");

        // def_follow_policy
		FollowOptionBox def_follow_policy = new FollowOptionBox("Default follow policy: ", 
                                                                admin_.def_follow_policy());
        def_follow_policy.addItemListener(new AbstractItemListener(admin_) 
            {
                protected void doAdminAction(Object modified_item) {
                    FollowOption opt;
                    String value = (String)modified_item;
                    if (value.equals("local_only")) opt = FollowOption.local_only;
                    else if (value.equals("if_no_local")) opt = FollowOption.if_no_local;
                    else opt = FollowOption.always;
                    adm_.set_def_follow_policy(opt);
                }
            });
        def_follow_policy.setToolTipText("Default link follow behavior for a particular trader.");

        // max_follow_policy
		FollowOptionBox max_follow_policy = new FollowOptionBox("Maximum follow policy: ", 
                                                                admin_.max_follow_policy());
        max_follow_policy.addItemListener(new AbstractItemListener(admin_) 
            {
                protected void doAdminAction(Object modified_item) {
                    FollowOption opt;
                    String value = (String)modified_item;
                    if (value.equals("local_only")) opt = FollowOption.local_only;
                    else if (value.equals("if_no_local")) opt = FollowOption.if_no_local;
                    else opt = FollowOption.always;
                    adm_.set_max_follow_policy(opt);
                }
            });
        max_follow_policy.setToolTipText("Limiting link follow policy for all links of the trader - overrides both link and importer policies.");


        // Adding them to the box
        Box box = Box.createVerticalBox();
        box.add(def_search_card);
        box.add(Box.createVerticalStrut(10));
        box.add(max_search_card);
        box.add(Box.createVerticalStrut(10));
        box.add(def_match_card);
        box.add(Box.createVerticalStrut(10));
        box.add(max_match_card);
        box.add(Box.createVerticalStrut(10));
        box.add(def_return_card);
        box.add(Box.createVerticalStrut(10));
        box.add(max_return_card);
        box.add(Box.createVerticalStrut(10));
        box.add(max_list);
        box.add(Box.createVerticalStrut(10));
        box.add(def_hop_count);
        box.add(Box.createVerticalStrut(10));
        box.add(max_hop_count);
        box.add(Box.createVerticalStrut(10));
        box.add(def_follow_policy);
        box.add(Box.createVerticalStrut(10));
        box.add(max_follow_policy);
        importAttrPanel.add(box);

        return importAttrPanel;
    }

    /**
     * Returns a new management panel for other configurable things of
     * the Admin OMG IDL interface.
     *
     * @return A new JPanel with all to manage other things
     * configurable via the Admin OMG IDL interface.
     */
    protected JPanel addOtherAttributes() {
        JPanel otherAttrPanel = new JPanel();
        otherAttrPanel.setBackground(Color.white);
        otherAttrPanel.setBorder(new TitledBorder(null," Other attributes ",TitledBorder.CENTER,TitledBorder.TOP));

        // request_id_stem
		LabelBox request_id_stem = new LabelBox("request_id_stem: ", new String(admin_.request_id_stem()));
        request_id_stem.addActionListener(new AbstractTextActionListener(admin_) 
            {
                protected void doAdminAction(java.awt.event.ActionEvent e) {
                    adm_.set_request_id_stem(e.getActionCommand().getBytes());
                }
            });
        request_id_stem.setToolTipText("Identification of the trader, to be \n" + 
                                       "used as the stem for the production of \n" + 
                                       "an id for a query request from one trader \n" + 
                                       "to another.");

        Box box = Box.createVerticalBox();
        box.add(request_id_stem);
        otherAttrPanel.add(box);

        return otherAttrPanel;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Creates the content of the Panel.
     */
    public void onSetTreeView() {
        if (getTreeView() == null) return;
        admin_ = (Admin)getTreeView().getSelectedObject();
        if (admin_ != null) {
            add(addSupportAttributes());
            add(addLinkAttributes());
            add(addImportAttributes());
            add(addOtherAttributes());
        }
    }
}














