/*===========================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s):Sylvain Leblanc.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.openccm.explorer.CosTrading.gui;

import javax.swing.table.AbstractTableModel;

import org.omg.CORBA.TCKind;
import org.omg.CosTrading.Property;

/**
 * A Table model for service type properties.
 *
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 * @version 0.1
 */
public class OfferPropsTableModel 
     extends AbstractTableModel 
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Names of the cols: "property", "value". */
    protected final String[] names = new String[] {"property", "value"};

    /** Rows values in a list. */
    protected java.util.List[] rows;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default constructor. 
     *
     * @param props The properties to display in the table.
     */
    public OfferPropsTableModel(Property[] props) {
        rows = new java.util.List[props.length];
            
        for (int i=0 ; i<props.length ; i++) {
            rows[i] = new java.util.LinkedList();
            rows[i].add(props[i].name);
            rows[i].add(extractFromAny(props[i].value).toString());
        }
    } 

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Internal method to extract something from an any.
     *
     * @param a The any containing the value to extract.
     *
     * @return An object related to the any value that can be
     * displayed.
     */
    protected Object extractFromAny(org.omg.CORBA.Any a) {
		
		org.omg.CORBA.TypeCode tc = a.type();
        while (tc.kind() == TCKind.tk_alias) {
            try {
                tc = tc.content_type();
            } catch (org.omg.CORBA.TypeCodePackage.BadKind ex) {}
        }
        int kind = tc.kind().value();
        switch (kind) {
            case org.omg.CORBA.TCKind._tk_boolean : return new Boolean(a.extract_boolean());
            case org.omg.CORBA.TCKind._tk_long : return new Integer(a.extract_long());
            case org.omg.CORBA.TCKind._tk_float : return new Float(a.extract_float());
            case org.omg.CORBA.TCKind._tk_double : return new Double(a.extract_double());
            case org.omg.CORBA.TCKind._tk_short : return new Short(a.extract_short());
            case org.omg.CORBA.TCKind._tk_char : return new Character(a.extract_char());
            case org.omg.CORBA.TCKind._tk_octet : return new Byte(a.extract_octet());
            case org.omg.CORBA.TCKind._tk_wchar : return new Character(a.extract_char());
            case org.omg.CORBA.TCKind._tk_string : return a.extract_string();
            case org.omg.CORBA.TCKind._tk_wstring : return a.extract_string();
            case org.omg.CORBA.TCKind._tk_ulong : return new Integer(a.extract_ulong());
            case org.omg.CORBA.TCKind._tk_ushort : return new Short(a.extract_ushort());
            case org.omg.CORBA.TCKind._tk_sequence : {                
                // Here we make the assumption of values inserted in "()" and separeted by ","
                // e.g. (10, 5, 54, 67) is a sequence of long.
                try {

                    String image = "(";

                    org.omg.CORBA.TypeCode content_tc = null;
                    try {
                        content_tc = tc.content_type();
                    } catch (org.omg.CORBA.TypeCodePackage.BadKind ex) {
                        // cannot happened here
                    }
            
                    org.omg.DynamicAny.DynAny dyn_any = org.objectweb.openccm.corba.TheDynamicAnyFactory.getFactory().create_dyn_any(a);
                    org.omg.DynamicAny.DynSequence dyn_seq = org.omg.DynamicAny.DynSequenceHelper.narrow(dyn_any);                    
                    dyn_seq.rewind();
                    int length = dyn_seq.get_length();
                    switch (content_tc.kind().value()) {
                        case TCKind._tk_short : {
                            for (int i=0 ; i<length ; dyn_seq.next(), i++) {
                                image = image + dyn_seq.current_component().get_short() + ", ";
                            }
                            break;
                        }
                        case TCKind._tk_ushort : {
                            for (int i=0 ; i<length ; dyn_seq.next(), i++) {
                                image = image + dyn_seq.current_component().get_ushort() + ", ";
                            }
                            break;
                        }
                        case TCKind._tk_long : {
                            for (int i=0 ; i<length ; dyn_seq.next(), i++) {
                                image = image + dyn_seq.current_component().get_long() + ", ";
                            }
                            break;
                        }
                        case TCKind._tk_ulong : {
                            for (int i=0 ; i<length ; dyn_seq.next(), i++) {
                                image = image + dyn_seq.current_component().get_ulong() + ", ";
                            }
                            break;
                        }
                        case TCKind._tk_longlong : {
                            for (int i=0 ; i<length ; dyn_seq.next(), i++) {
                                image = image + dyn_seq.current_component().get_longlong() + ", ";
                            }
                            break;
                        }
                        case TCKind._tk_ulonglong : {
                            for (int i=0 ; i<length ; dyn_seq.next(), i++) {
                                image = image + dyn_seq.current_component().get_ulonglong() + ", ";
                            }
                            break;
                        }
                        case TCKind._tk_float : {
                            for (int i=0 ; i<length ; dyn_seq.next(), i++) {
                                image = image + dyn_seq.current_component().get_float() + ", ";
                            }
                            break;
                        }
                        case TCKind._tk_double : {
                            for (int i=0 ; i<length ; dyn_seq.next(), i++) {
                                image = image + dyn_seq.current_component().get_double() + ", ";
                            }
                            break;
                        }
                        case TCKind._tk_wchar : {
                            for (int i=0 ; i<length ; dyn_seq.next(), i++) {
                                image = image + "'" + dyn_seq.current_component().get_wchar() + "', ";
                            }
                            break;
                        }
                        case TCKind._tk_char : {
                            for (int i=0 ; i<length ; dyn_seq.next(), i++) {
                                image = image + "'" + dyn_seq.current_component().get_char() + "', ";
                            }
                            break;
                        }
                        case TCKind._tk_wstring :  {
                            for (int i=0 ; i<length ; dyn_seq.next(), i++) {
                                image = image + "\"" + dyn_seq.current_component().get_wstring() + "\", ";
                            }
                            break;
                        }
                        case TCKind._tk_string : {
                            for (int i=0 ; i<length ; dyn_seq.next(), i++) {
                                image = image + "\"" + dyn_seq.current_component().get_string() + "\", ";
                            }
                            break;
                        }
                        case TCKind._tk_octet : {
                            for (int i=0 ; i<length ; dyn_seq.next(), i++) {
                                image = image + dyn_seq.current_component().get_octet() + ", ";
                            }
                            break;
                        }
                        case TCKind._tk_boolean : {
                            for (int i=0 ; i<length ; dyn_seq.next(), i++) {
                                image = image + dyn_seq.current_component().get_boolean() + ", ";
                            }
                            break;
                        }
                    }
                    image = image.substring(0, image.length()-2) + ")";
                    return image;
                } catch (org.omg.DynamicAny.DynAnyPackage.TypeMismatch ex) {
                    // should not happen
                } catch (org.omg.DynamicAny.DynAnyPackage.InvalidValue ew) {
                    // should not happen
                } catch (org.omg.DynamicAny.DynAnyFactoryPackage.InconsistentTypeCode ex) {
                    // should not happen
                }
            }
        }
        return "";
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // some overrides
    public int getColumnCount() { return names.length; }
    public String getColumnName(int col) { return names[col]; }
    public int getRowCount() { return rows.length;}
    public Object getValueAt(int row, int col) { return rows[row].get(col); }
};
