/*===========================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s):Sylvain Leblanc.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.openccm.explorer.CosTrading.gui;

import javax.swing.table.AbstractTableModel;

import org.objectweb.openccm.corba.TypeCodeUtils;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.PropStruct;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.PropertyMode;

/**
 * A Table model for service type properties.
 *
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 * @version 0.1
 */
public class ServiceTypePropsTableModel extends AbstractTableModel {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Names of the cols: "name", "value type", "mode". */
    protected final String[] names = new String[] {"name", "value type", "mode"};

    /** Rows values in a list. */
    protected java.util.List[] rows;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default constructor. 
     *
     * @param props The properties to display in the table.
     */
    public ServiceTypePropsTableModel(PropStruct[] props) {
        rows = new java.util.List[props.length];
            
        for (int i=0 ; i<props.length ; i++) {
            rows[i] = new java.util.LinkedList();
            rows[i].add(props[i].name);
            rows[i].add(TypeCodeUtils.toStringValue(props[i].value_type));
            rows[i].add(modeImage(props[i].mode));
        }
    } 

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Return the string image to display for a property mode.
     *
     * @param mode The property mode to display.
     *
     * @return The string to display for the given property mode.
     */
    protected String modeImage(PropertyMode mode) {
        if (mode.equals(PropertyMode.PROP_NORMAL)) return "default";
        if (mode.equals(PropertyMode.PROP_MANDATORY)) return "mandatory"; 
        if (mode.equals(PropertyMode.PROP_READONLY)) return "readonly"; 
        if (mode.equals(PropertyMode.PROP_MANDATORY_READONLY)) return "mandatory readonly"; 
        return "";
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // some overrides
    public int getColumnCount() { return names.length; }
    public String getColumnName(int col) { return names[col]; }
    public int getRowCount() { return rows.length;}
    public Object getValueAt(int row, int col) { return rows[row].get(col); }
};
