/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________. 

====================================================================*/
package org.objectweb.openccm.explorer.commandLine;

import org.objectweb.openccm.corba.TheORB;
import org.objectweb.openccm.corba.TheRootPOA;
import org.objectweb.util.cmdline.lib.ApplicationBase;
import org.objectweb.util.cmdline.lib.DefaultCommandLine;
import org.objectweb.util.misc.api.ExceptionWrapper;
import org.objectweb.openccm.explorer.Main;
import org.objectweb.openccm.explorer.MainPDA;
import org.objectweb.openccm.explorer.CORBA.ConsoleFactory;

import java.util.Vector;
import java.util.List;

/**
 * This class manages the different options and launch the corresponding command.
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */
public class ExplorerApplication 
     extends ApplicationBase 
  implements DefaultApplication, ConfigApplication, ViewApplication {

    /* The default browser configuration files or directory. */
    protected String[] browserConfigFiles_ = null;

    /* The default context config file or directory. */
    protected String context_ = "";

    /* The browser config file or directory fix by the user. */
    protected String config_ = null;

    /* The view of the main frame to use. */
    protected String view_ = "basic";

    /**
     * Default Constructor.
     */
    public ExplorerApplication() {
        super(new DefaultCommandLine(new String[] { "ccm_explorer" }, new String[0], new String[] { "Display the OpenCCM explorer console" }, true));
        // Creates the ClientApplication options
        getCommandLine().addOption(new DefaultContextOption(this));
        getCommandLine().addOption(new DefaultBrowserOption(this));
        getCommandLine().addOption(new ConfigOption(this));
        getCommandLine().addOption(new ViewOption(this));
    }

    /**
     * Remove the ORB arguments.
     * @param args The command line arguments.
     * @return The new args without ORB arguments.
     */
    private String[] removeORBArgument(String[] args) {
        int nb = 0;
        String[] tmp = new String[args.length];
        for (int i = 0; i < args.length; i++) {
            if (!args[i].startsWith("-ORB")) {
                tmp[nb] = args[i];
                nb++;
            } else {
                i++;
            }
        }
        String[] tmp2 = new String[nb];
        for (int i = 0; i < nb; i++)
            tmp2[i] = tmp[i];
        return tmp2;
    }

    /**
     * Fixes the Configuration file to use
     * @param file The configuration file to use
     */
    public void setConfigFile(String file) {
        config_ = file;
    }

    /**
     * Fixes the Default Context Configuration file to use
     * @param file The configuration file to use
     */
    public void setDefaultContextConfigFile(String file) {
        context_ = file;
    }

    /**
     * Fixes the Default Browser Configuration file to use
     * @param file The configuration file to use
     */
    public void setDefaultBrowserConfigFile(String[] file) {
        browserConfigFiles_ = file;
    }

    /**
     * Fixes the view to use
     * @param file The view to use
     */
    public void setView(String view){
        view_ = view;    
    }
    
    /**
     * Executes the main function.
     * @param args The command line arguments.
     */
    public void runMain(String[] args) {
        // Initializes the CORBA::ORB singleton.
        org.omg.CORBA.ORB orb = org.objectweb.openccm.ccm.Components.Runtime.init(args);
        //org.objectweb.ccm.CORBA.TheORB.initialize(args);

        // Surcharges the ApplicationBase.runMain method.
        try {
            String[] arguments = getCommandLine().parse(removeORBArgument(args));
            int ret = start(arguments);
        } catch (ExceptionWrapper exc) {
            report_exception(exc.getException());
        } catch (Exception exc) {
            report_exception(exc);
        }
    }

    /**
     * Starts the main function.
     * @param args The command line arguments.
     */
    public int start(String[] args) {
        List fileList = new Vector();
        if(browserConfigFiles_!=null){
            for(int i=0 ; i < browserConfigFiles_.length ; i++) {
                fileList.add(browserConfigFiles_[i]);
            }
		}
        if (config_ != null) {
            fileList.add(config_);
        }
        if(view_.equals("tabs")) {
            new MainPDA(context_, (String[]) fileList.toArray(new String[0]));
        } else if(view_.equals("basic")) {
            new Main(context_, (String[]) fileList.toArray(new String[0]));
        }
        return 0;
    }

    /**
     * The main method.
     * @param args The command line arguments.
     */
    public static void main(String[] args) {
        ExplorerApplication application = new ExplorerApplication();
        application.runMain(args);
        ConsoleFactory.getDebugConsole().add("Running the ORB \n");
        
        // Activates the POA
        try
        {
            ConsoleFactory.getDebugConsole().add("Activating the RootPOA\n");
            TheRootPOA.getRootPOA().the_POAManager().activate();
        }
        catch ( java.lang.Exception ex )
        {
            System.err.println( "An exception has been intercepted" );
            ex.printStackTrace();
        }        
        
        TheORB.run();
    }

}
