/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.explorer.menu;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.openccm.explorer.CORBA.ConsoleFactory;
import org.objectweb.util.explorer.api.Tree;
import org.objectweb.util.explorer.swing.api.Explorer;

/**
 * This class provides a DynamicTree.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class TreeDialogSingleton {

    protected static Component tree_ = null;

    protected static Tree treeItf_ = null;
    
    protected static Explorer explorerItf_ = null;
    
    /**
     * Provides an instance of a DynamicTree.
     * @return A DynamicTree instance.
     */
    public static Component getInstance() {
        if (treeItf_ != null) {
            treeItf_.refreshAll();
        }
        return tree_;
    }

    /**
     * Provides the Explorer interface of the current Tree component.
     * @return The explorer interface to use.
     */
    public static Explorer getExplorerInterface(){
        if (treeItf_ != null) {
            treeItf_.refreshAll();
        }
        return explorerItf_;
    }
    
    /**
     * Provides the Tree interface of the current Tree component.
     * @return The tree interface to use.
     */
    public static Tree getTreeInterface(){
        if (treeItf_ != null) {
            treeItf_.refreshAll();
        }
        return treeItf_;
    }
        
    /**
     * Fixes the Component instance.
     * @param instance The instance to use.
     */
    public static void setInstance(Component instance) {
        tree_ = instance;
        if(tree_!=null){
            try {
                treeItf_ = (Tree)tree_.getFcInterface(Tree.TREE);
                explorerItf_ = (Explorer)tree_.getFcInterface(Explorer.EXPLORER);
            } catch (NoSuchInterfaceException e) {
                ConsoleFactory.getDebugConsole().add(e.getMessage());
            }
        }
    }

}
