/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.corba.idl2java;

// Package dependencies
import java.util.Map;

/**
 * Common Interface for an IDL to Java generator.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */
public interface IdlToJavaOperations
{
    /**
     * Set the IDL file to parse.
     * 
     * @param file - The input file.
     */
    public void
    setInputFile(String file);
    
    /**
     * Get the IDL file to parse.
     * 
     * @return The input file.
     */
    public String
    getInputFile();
    
    /**
     * Set the destination directory for generated java files.
     * 
     * @param destdir - The destination directory.
     */
    public void
    setDestinationDirectory(String destdir);
    
    /**
     * Get the destination directory for generated java files.
     * 
     * @return The destination directory.
     */
    public String
    getDestinationDirectory();
    
    /**
     * Add a directory to the include list.
     * (Preprocessor -I option)
     * 
     * @param directory - The directory to add.
     */
    public void
    addIncludeDirectory(String directory);

    /**
     * Get directories added to the include list.
     * (Preprocessor -I option)
     * 
     * @return The list of included directories.
     */
    public String[]
    getIncludeDirectories();

    /**
     * Add a package mapping definition.
     * 
     * @param idl_package - The idl package name.
     * @param java_package - The java package name.
     */
    public void
    addPackageMapping(String idl_package, String java_package);

    /**
     * Get package mapping definitions.
     * 
     * @return The map of package mappings.
     */
    public Map
    getPackageMappings();
}