/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.corba.idl2java;

// Package dependencies.

/**
 * This is an implementation for -I preprocessor option, e.g. -IDIR.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */
public class OptionIDIR
     extends PreprocessorOptionBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the Idl2Java Application. */
    private IdlToJavaOperations idl2java_;
    
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor.
     *
     * @param preprocessor The associated preprocessor.
     */
    public
    OptionIDIR(IdlToJavaOperations idl2java)
    {
        setLabels(new String[] { "-IDIR" });
        setDescription(new String[] { "Put DIR in the include file search path"});
        
        // Init internal state
        idl2java_ = idl2java;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.Option
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cpp.api.PreprocessorHolder
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cpp.api.PreprocessorOption
    //
    // ==================================================================

    /**
     * Gets the base option label.
     *
     * @return The base option label.
     */
    public String
    getBaseOptionLabel()
    {
        return "-I";
    }

    /**
     * Consumes the option value.
     *
     * @param current The option value.
     */
    public void
    consumeOption(String current)
    {
        // Adds the DIR to the include directories of the preprocessor.
        idl2java_.addIncludeDirectory(current);
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

}
