/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ---------------------------------------------

====================================================================*/

package org.objectweb.openccm.OpenCCM_DCI.cif;

import org.objectweb.openccm.OpenCCM_DCI.*;

/**
 * This is the implementation of the OpenCCM_DCI::AssemblyManagerMoedl
 * component type.
 *
 * This class inherits from the skeleton
 * generated by the OpenCCM's CIF to Java mapping generator.
 *
 * @author <a href="mailto:Frederic.Briclet@lifl.fr">Briclet Frederic</A>
 */

public class AssemblyManagerModelImpl
     extends org.objectweb.openccm.OpenCCM_DCI.
             AssemblyManagerModelSessionComposition.AssemblyManagerModelImpl
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    //The XML descriptor ressource 
    private String xml_descriptor;
    // The assembly uuid
    private String assemblyUUID;
    
    private byte[] componentArchive;
    
    //Private boolean state
    private boolean removed=false;
    
    private org.omg.Components.Cookie cookie;
    
    private org.objectweb.openccm.OpenCCM_DCI
        .AssemblyManagerModelRegistration AMMR; 
    
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================
    public AssemblyManagerModelImpl(String xml_descriptor){
        this.xml_descriptor=xml_descriptor;   
        //Build an assembly uuid
        assemblyUUID=System.currentTimeMillis()+"";
        assemblyUUID.substring(assemblyUUID.length()-5,assemblyUUID.length());
        assemblyUUID="AAM:"+assemblyUUID;           
    }
    
    public AssemblyManagerModelImpl(byte[] componentArchive){
            this.componentArchive=componentArchive;   
            //Build an assembly uuid
            assemblyUUID=System.currentTimeMillis()+"";
            assemblyUUID.substring(assemblyUUID.length()-5,assemblyUUID.length());
            assemblyUUID="AAM:"+assemblyUUID;           
        }
    // ==================================================================
    //
    // Accessor methods
    //
    // ==================================================================
    public String xml_descriptor(){       
        if(xml_descriptor==null)
            return "Not Component assembly description available";
        return xml_descriptor;
        
    }
    
    public String assemblyUUID(){
        return assemblyUUID;
    }
    // ==================================================================
    //
    // Common component methods.
    //
    // ==================================================================
    public  void 
    configuration_complete()
    {        
        try{
            if(xml_descriptor!=null)
                ((AssemblyInstanceManagementSegImpl)
                 obtain_executor("assembly_instance_management"))
                    .setupXML(xml_descriptor);        
            else  {
                    ((AssemblyInstanceManagementSegImpl)
                    obtain_executor("assembly_instance_management"))
                       .setupComponentArchive(componentArchive);
                       assemblyUUID=((AssemblyInstanceManagementSegImpl)
                                    obtain_executor("assembly_instance_management")).getModelName()
                                    +":"+assemblyUUID;
                    }  
                    
           org.omg.Components.SessionContext sc=
                (org.omg.Components.SessionContext)get_context();
                
            AMMR=get_context()
            .get_connection_assembly_manager_model_registration();
            
            cookie= AMMR.register_assembly_manager_model
                (org.objectweb.openccm.
                 OpenCCM_DCI.AssemblyManagerModelHelper         
                 .narrow(sc.get_CCM_object())); 
            ((AssemblyInstanceManagementSegImpl)
                    obtain_executor("assembly_instance_management"))
                    .setupCookie(cookie);
                
        }
        catch(Exception e){
            e.printStackTrace();
        }                
    }
    
    /**
     * Implementation of the removing method.
     * A removinf event will be sent soon.
     */
    public void
    ccm_remove()
    throws org.omg.Components.CCMException
    {
        System.err.println("Assembly manager removed invoke:boolean value:"+removed);
        
        if(removed) return;
            
        try{
            //The disconnection processing is done by the facet
            //AMMR.unregister_assembly_manager_model(cookie);   

                    
            removed=true;
            
            ((AssemblyInstanceManagementSegImpl)
                     obtain_executor("assembly_instance_management"))
                        .uninstall();
    
    
                   
            }
            catch(Exception e){
                    e.printStackTrace();
                }
            
            
    }

    public void
    push(AssemblyManagerRemoved amr)
    {

        try
            {
                //  System.err.println("AMMI >> Disconnect received");
                ((AssemblyInstanceManagementSegImpl)
                 obtain_executor("assembly_instance_management"))
                    .disconnectRemovedAssembly(amr.assembly_manager_ref);
            }
        catch(Exception e){
            e.printStackTrace();
        }
    }
}
