/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): Philippe Merle.

====================================================================*/

package org.objectweb.openccm.OpenCCM_DCI.cif;

import org.objectweb.openccm.OpenCCM_DCI.*;
import org.objectweb.openccm.ccm.Deployment.ContainerLocalImpl;
import org.objectweb.openccm.ccm.Deployment.listener.ContainerListenerDefault;
import org.objectweb.openccm.corba.*;


/**
 * This is the implementation of the Container facet provided by the
 * ContainerManager component.
 * This class inherits from the skeleton
 * generated by the OpenCCM's CIF to Java mapping generator.
 *
 * @author <a href="mailto:Frederic.Briclet@lifl.fr">Briclet Frederic</A>
 */

public class ContainerSegImpl 
       extends  org.objectweb.openccm.OpenCCM_DCI.
                ContainerManagerSessionComposition.ContainerSeg
       implements  org.objectweb.openccm.ccm.Deployment.ContainerHomeInformation
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    // reference on a container
    private org.objectweb.openccm.ccm.Deployment.ContainerImpl container ;
    //the container configuration
    private org.omg.Components.ConfigValue[] configuration;
    //Cookie to the parent ComponentServer
    private org.omg.Components.Cookie cookie;
    //Reference on the CCM object
    private org.objectweb.openccm.OpenCCM_DCI.ContainerManagerCCM cont;
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    private org.objectweb.openccm.OpenCCM_DCI.ContainerManagerCCM
    getCont()
    {
        if(cont==null)
            cont=(org.objectweb.openccm.OpenCCM_DCI.ContainerManagerCCM)
              get_context(); 
        return cont;
    }
    // ==================================================================
    //
    // constructor.
    //
    // ==================================================================
    public ContainerSegImpl(){
        super();
    }   
   
    // ==================================================================
    //
    // public methods.
    //
    // ==================================================================
    /**
     * Method used by the container manager main segment to configure
     * the container.
     * @param configuration the container configuration
     */
    public void 
    configuration(org.omg.Components.ConfigValue[] configuration)
    {
        try{
          
            //process container configuration

            container= new org.objectweb.openccm.ccm.Deployment
                            .ContainerImpl(configuration,
                                           null,
                                           get_context()
                                           .get_connection_component_installation());
           container._delegate().set_container_home_info(this);
           /*container.add_listener
                (new ContainerListenerDefault
                (new ContainerLocalImpl(container._delegate())));*/
                
/* Contributor: Philippe Merle
 *
 * Following does not work on JacORB 2.0.
 
           org.omg.CORBA.Object ref = 
                container._this_object(org.objectweb
                                       .ccm.CORBA.TheORB.getORB());

 * Replaced by the following
 */

          
            org.omg.CORBA.Object ref = null;
            try
            {
                byte[] oid = org.objectweb.openccm.corba.TheRootPOA.getRootPOA().activate_object(container);
                ref = org.objectweb.openccm.corba.TheRootPOA.getRootPOA().id_to_reference(oid);
            }
            catch(Exception ex)
            {
                ex.printStackTrace();
            }

/* End of the correction. */

             
                
              //set the configuration
            this.configuration=configuration;
            //create a deleguate container
            
                                               
            org.omg.Components.Deployment.Container cont_ref = 
                org.omg.Components.Deployment.ContainerHelper.narrow(ref);
            container.the_container_ref(cont_ref);     

            //set a internal attribute
            
        }
        catch(Exception e){
            e.printStackTrace();
        }
    }
    /**
     * Setting method used by the ContainerManager main segment
     * to set the registration cookie with the parent componentserver.
     * @param cookie The cookie to memorize
     */
    public void
    cookie(org.omg.Components.Cookie cookie)
    {
      this.cookie=cookie;
    }
    
    /**
     * Unregister the home reference from the home list
     */
    public void
    unregister_home(org.omg.Components.CCMHome ref)
    throws org.omg.Components.RemoveFailure
    {
          try{
            // Retrieves all ContainerManager connection
            org.objectweb.openccm.OpenCCM_DCI.
                ContainerManagerPackage.homes_createdConnection 
                connect[]=getCont().get_connections_homes_created();
            //compare all home reference
            for(int i=0;i<connect.length;i++)
                if(connect[i].objref._is_equivalent(ref))
                    {                        
                        getCont().disconnect_homes_created(connect[i].ck);
                        return ;
                    }
            }
            catch(org.omg.Components.ExceededConnectionLimit ecl){
                ecl.printStackTrace();
            }
        throw new org.omg.Components.RemoveFailure();
    }
    // ==================================================================
    //
    //  Components::Deployment::Container  methods.
    //
    // ==================================================================

    /**
     * This methods returned the parent componentserver
     * @return The parent componentserver
     */
    public 
    org.omg.Components.Deployment.ComponentServer
    get_component_server(){
        return getCont().get_connection_component_server();
    }

    /**
     * This method allows Home installation.
     * @param uuid The archive uuid
     * @param entrypt The component home entrypoint
     * @param config Configvalue to configure the home
     */
    public org.omg.Components.CCMHome
        install_home(java.lang.String uuid,
                     java.lang.String entrypt,
                     org.omg.Components.ConfigValue[] config)
        throws org.omg.Components.Deployment.UnknownImplId,
               org.omg.Components.Deployment.ImplEntryPointNotFound,
               org.omg.Components.Deployment.InstallationFailure,
               org.omg.Components.Deployment.InvalidConfiguration    
    {
        // deleguate the installation to the container attribute
        org.omg.Components.CCMHome home=
            container.install_home(uuid,entrypt,config);
        
        try{
            // connect the created home to the ContainerManager component
            getCont().connect_homes_created(home);        
        }
        catch(Exception e)
            {
                System.err.println(e.getMessage());
                throw new org.omg.Components.Deployment.InstallationFailure();
            }
        //return the home created
        return home;
    }

    /**
     * This method remove a home.
     * TODO: Disconnect the home form ContaierManager
     * @param homeref The reference of the home to remove
     */
    public
    void
    remove_home(org.omg.Components.CCMHome homeref)
        throws org.omg.Components.RemoveFailure
    {
        try{
            // Retrieves all ContainerManager connection
            org.objectweb.openccm.OpenCCM_DCI.
                ContainerManagerPackage.homes_createdConnection 
                connect[]=getCont().get_connections_homes_created();
            //compare all home reference
            for(int i=0;i<connect.length;i++)
                if(connect[i].objref._is_equivalent(homeref))
                    {
                        //remove the home reference will be done by the callback
                        //method unregister_home
                        //connect[i].objref.remove_home();
                        homeref.remove_home();            
                        return ;
                    }
        }
        catch(Exception ecl){
            ecl.printStackTrace();
        }
        throw new org.omg.Components.RemoveFailure();
        
    }

    /**
     * This method return an array of all the running home
     * created by this container.
     */     
    public org.omg.Components.CCMHome[]
    get_homes()
    {
        //Retrieve all the ContainerManager connection
        org.objectweb.openccm.OpenCCM_DCI.
          ContainerManagerPackage.homes_createdConnection 
          connect[]=getCont().get_connections_homes_created();
        //Create a home array
        org.omg.Components.CCMHome[] homes=
          new org.omg.Components.CCMHome[connect.length];
        // Fill the home array
        for(int i=0;i<connect.length;i++)
          homes[i]=connect[i].objref;
        //return the array
        return homes;       
    }

    /**
     * Remove the current container and all the home created.
     */
    public  void
    remove()
    throws org.omg.Components.RemoveFailure
    {
        try
            {
                //unregister the container from its parent componentserver
                get_context().get_connection_container_manager_registration()
                    .unregister_container_manager(cookie);     
                //retrieve all the home connection
                org.objectweb.openccm.OpenCCM_DCI.
                  ContainerManagerPackage.homes_createdConnection 
                  connect[]=getCont().get_connections_homes_created();
                //remove all the home
                for(int i=0;i<connect.length;i++){
                    //cont.disconnect_homes_created(connect[i].ck);                    
                     connect[i].objref.remove_home();
                    
                }
                //Remove the deleguation container
                container.remove();
                getCont().remove();                            
            }
        catch(Exception e )
            {
                System.err.println("Cannot disconnect Container/ComponentServer/Node");
                e.printStackTrace();
                throw new org.omg.Components.RemoveFailure();
            }
       
    }
  
    // ==================================================================
    //
    // Accessor methods.
    //
    // ==================================================================

    public org.omg.Components.ConfigValue[]
    configuration()
    {
        return this.configuration;
    }


}
