/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/



package org.objectweb.openccm.OpenCCM_DCI.cif;

import org.objectweb.openccm.OpenCCM_DCI.*;
import org.objectweb.openccm.corba.*;

/**
 * This is the implementation of the ServerActivator facet
 * provided by the node component.
 *
 * This class inherits from the skeleton
 * generated by the OpenCCM's CIF to Java mapping generator.
 *
 * @author <a href="mailto:Frederic.Briclet@lifl.fr">Briclet Frederic</A>
 */
public class ServerActivatorImpl 
       extends  org.objectweb.openccm
                .OpenCCM_DCI.NodeManagerSessionComposition
                .ServerActivatorSeg
{
   // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /*Last attributed component server number */        
    private int last_cs_num;
    
    /*Table for storage of cs*/
    private java.util.Hashtable c_servers;
    
    /*Table for storage of each component server*/
    private java.util.Hashtable process;
    
     /* The parent NodeManager component which keep all connection*/
    private org.objectweb.openccm.OpenCCM_DCI.NodeManagerCCM node;

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    private org.objectweb.openccm.OpenCCM_DCI.NodeManagerCCM
    getNode()
    {
        if(node==null)
             this.node=(org.objectweb.openccm
                       .OpenCCM_DCI.NodeManagerCCM)get_context();
        return node;
    }
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================
    /**
     * Main constructor of ServerActivator
     */ 
    public ServerActivatorImpl(){
        //super();
        last_cs_num=0;
        c_servers= new java.util.Hashtable();
        process=new java.util.Hashtable(); 
          
     }
    // ==================================================================
    //
    // Public methods.
    //
    // ================================================================== 

    /**
     * Operation create_component_server :
     * Three main steps  build are lead in this method :
     *      - Build arguments for the command to execute
     *      - Build a new launcher (with this argument) 
     *        whitch start the Component Server
     *      - Retrieve information ComponentServer reference and process
     *
     * @param config The configvalue to use to configure the component server
     * @return The componentserver created
     *
     * TODO: Consider the config value
     * WARNING : TODO change the connect operation with node manager using 
     *           the context with 
     *            node.get_context().get_CCM_object().connect(.....)
     */
    public 
    org.omg.Components.Deployment.ComponentServer 
    create_component_server(org.omg.Components.ConfigValue[] config)
    throws org.omg.Components.CreateFailure,
           org.omg.Components.Deployment.InvalidConfiguration
    {
        org.omg.Components.Deployment.ComponentServer server=null;
        try{
                      
            //Build the name of new Component Server base on a number 
            String server_name="ComponentServer"+last_cs_num++;
            
            org.objectweb.openccm.ccm.Components.ConfigValueImpl[] config1= 
                new org.objectweb.openccm.ccm.Components.ConfigValueImpl[1];
            
            org.omg.CORBA.Any any= 
                org.objectweb.openccm.corba.TheORB.getORB().create_any();
            
            //store the component server name in config parameter
            any.insert_string(server_name); 
            config1[0]= new org.objectweb.openccm.ccm.Components.ConfigValueImpl("ServerName",any);
            //Retrieve the ComponentServerManagerHome
            ComponentServerManagerHome cshome=
                getNode().get_connection_component_server_manager_home();
            //create a ComponentServerManager
            org.objectweb.openccm.OpenCCM_DCI.ComponentServerManager csm=
                cshome.create();
            //configure the componentServer
            csm.config(config1);
            
            //connect the registration interface
            csm.connect_component_server_manager_registration
                (getNode().provide_component_server_manager_registration());
            //connect the parent server activator interface
            csm.connect_parent_server_activator
                (getNode().provide_server_activator());
            //connect the ContainerManagerHome
            csm.connect_container_manager_home
                (getNode().get_connection_container_manager_home());
            //connect the ComponentInstallation to use
            csm.connect_component_installation
                (getNode().provide_component_installation());
            //configure complete the ComponentServerManager
            csm.configuration_complete();
            //return the componentserver interface
            return csm.provide_component_server();
            }
       catch(Exception e)
           {
               e.printStackTrace();
               throw new org.omg.Components.CreateFailure();  
           }
    }

    /**
     * Operation remove_component_server, by stoping him and killing 
     * his process.
     * The component server give in parameter must be created by 
     * this Server Activator 
     * else a RemoveFailure exception will be raise.
     * Call the remove() function on the server pass to parameter.
     *
     * @param server The component server to remove
     */
    public void 
    remove_component_server(
                            org.omg.Components
                            .Deployment.ComponentServer server)
        throws org.omg.Components.RemoveFailure
    {
            
      /*  //Retrive configuration informations
        org.omg.Components.ConfigValue[] config =server.configuration();    
        // the first config value of component server must be component server name         
        if(!config[0].name.equals("ServerName"))
            throw new org.omg.Components.RemoveFailure();   
        // test if this Component server was start by this Server Activator 
        if(!c_servers.containsKey(config[0].value.extract_string()))
            throw new org.omg.Components.RemoveFailure();   
        
        try{
            //Start the removing processing for the component server
            server.remove();        
            //remove the reference of the process
            //Process p=(Process)process.remove(config[0].value.extract_string());
            //Remove the reference of the component server 
            //c_servers.remove(config[0].value.extract_string());
            //Kill the process
            //p.destroy();
           }
        catch(Exception e){e.printStackTrace();
                          System.out.println(e.getMessage());
                          }*/

        org.objectweb.openccm.OpenCCM_DCI.
            NodeManagerPackage.component_server_managersConnection 
            connect[]=getNode().get_connections_component_server_managers();
        
        for(int i=0;i<connect.length;i++)
            if(connect[i].objref.
               provide_component_server().
               _is_equivalent(server))
                {                    
                    connect[i].objref.remove();
                    return ;
                }
        throw new org.omg.Components.RemoveFailure();
    }
    
    
    /**
     * Operation get_component_servers: Return a set of all active 
     * components servers
     * created by this Server Activator
     *
     *@return An array of the running componentServer.
     */
    public org.omg.Components.Deployment.ComponentServer[]
    get_component_servers()
    {
        
        /*         c_servers.values().toArray(
                   new org.omg.Components.Deployment.
                   ComponentServer[c_servers.size()]);*/
        
        org.objectweb.openccm.OpenCCM_DCI.
            NodeManagerPackage.component_server_managersConnection 
            connect[]=getNode().get_connections_component_server_managers();
       
        org.omg.Components.Deployment.ComponentServer[] cs=
            new org.omg.Components.Deployment.ComponentServer[connect.length];
        
        for(int i=0;i<connect.length;i++)
            cs[i]=connect[i].objref.provide_component_server();
        
        return cs;                                             
        //  return null;
    }

	/**
	 * Remove the ServerActivator including all the started component servers
	 */
	 public void
	 remove()
	 throws org.omg.Components.RemoveFailure
	 {
	 	org.objectweb.openccm.OpenCCM_DCI.
            NodeManagerPackage.component_server_managersConnection 
            connect[]=getNode().get_connections_component_server_managers();
        
        for(int i=0;i<connect.length;i++)
        {                    
            connect[i].objref.remove();         
        }
	 }
}
