/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/

package org.objectweb.openccm.browser.DCI;

/** The Browser's imports */
import org.objectweb.util.browser.api.TreeView;
import org.objectweb.util.browser.api.Table;
import org.objectweb.util.browser.core.naming.DefaultName;
import org.objectweb.util.browser.core.naming.DefaultEntry;


/** The CCM's imports */
import org.objectweb.openccm.browser.CORBA.TypageCORBA;
import org.ist.coach.DCI.NodeManager;
import org.ist.coach.DCI.ExtComponentInstallation;

/**
 * Displays the installed archives available for this <code>Node</code>.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class ComponentInstallationTable
  implements Table 
{

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================
    
    //==================================================================
    //
    // No constructor.
    //
    //==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    protected String getName(org.omg.CORBA.Object o) {
        TypageCORBA tc = new TypageCORBA(o, org.objectweb.openccm.corba.TheORB.getORB());
        return tc.getTypeID();
    }

    //==================================================================
    //
    // Public methods for Table interface.
    //
    //==================================================================

    /**
     * Overriding methods
     * @see org.objectweb.util.browser.api.Table#getHeaders()
     */
    public String[] getHeaders(TreeView treeView) {
        return new String[]{"Archive"};
    }
    
    /**
     * Overriding methods
     * @see org.objectweb.util.browser.api.Table#getRows(org.objectweb.util.browser.api.TreeView)
     */
    public Object[][] getRows(TreeView treeView) {
        NodeManager nodeManager = (NodeManager)treeView.getSelectedObject();
        ExtComponentInstallation componentInstallation = nodeManager.provide_component_installation();
        if(componentInstallation!=null){
            String[] uuids = componentInstallation.get_all_implUUIDs();
            Object[][] contenu = new Object[uuids.length][1];
            for (int i = 0; i < uuids.length; i++) {
                 contenu[i][0] = new DefaultEntry(new ArchiveUUID(uuids[i]), new DefaultName(uuids[i]));
             }
            return contenu;
        }
        return null;
    }
}
