/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.browser.DCI;

/** The consoles's imports */
import org.objectweb.openccm.browser.CORBA.TypageCORBA;
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.api.Context;
import org.objectweb.util.browser.api.Wrapper;
import org.objectweb.util.browser.core.naming.DefaultEntry;
import org.objectweb.util.browser.core.naming.DefaultName;
import org.objectweb.util.browser.core.api.ContextContainer;
import org.omg.Components.Deployment.Assembly;

import org.ist.coach.DCI.AssemblyManager;
import org.ist.coach.DCI.DCIDeployment;
import org.ist.coach.DCI.DCIManager;
import org.ist.coach.DCI.DCIInformation;
import org.ist.coach.DCI.NodeManager;
import org.ist.coach.DCI.UnknownAssembly;

/**
 * This context displays:
 * <ul>
 *   <li>a sequence of all NodeManager instances registered with the DCIManager.</li>
 *   <li>the assemblies installed on the DCIManager.</li>
 *   <li>the assemblies runnning under the DCIManager.</li>
 *   <li>the available homes.</li>
 * </ul>
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class DCIManagerContext 
    implements Context, Wrapper {

    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /** The DCIManager to browse */
    protected DCIManager DCIManager_ = null;

    // ==================================================================
    //
    // No constructor.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    protected String getName(org.omg.CORBA.Object o) {
        TypageCORBA tc = new TypageCORBA(o, org.objectweb.openccm.corba.TheORB.getORB());
        return tc.getTypeID();
    }

    /**
     * Provides a name depending on the AssemblyManager id.
     */
    protected String getName(String assemblyUUID){
        String returnedValue = "Assembly ";
        if(assemblyUUID!=null && assemblyUUID.indexOf(":")!=-1)
            returnedValue = assemblyUUID.substring(0,assemblyUUID.indexOf(":")) + " Assembly"; 
        return returnedValue;
    }
   
    /**
     * Provides a name depending on the AssemblyManager id.
     */
    protected String getName(AssemblyManager assemblyManager){
        String returnedValue = "Assembly ";
        String assemblyId = assemblyManager.ass_id();
        if(assemblyId.indexOf(":")!=-1)
            returnedValue = assemblyId.substring(0,assemblyId.indexOf(":"));
        return returnedValue;
    }    
    
    protected boolean isStarted(AssemblyManager assemblyManager){
        Assembly assembly = assemblyManager.provide_assembly();
        return assembly.get_state().equals(org.omg.Components.Deployment.AssemblyState.INSERVICE);
    }
    
    // ==================================================================
    //
    // Public methods for interface Context.
    //
    // ==================================================================

    public Entry[] getEntries() {
        java.util.List v = new java.util.Vector();

        DCIInformation DCIInfo = DCIManager_.provide_dci_information();
        
        ContextContainer nodeContainer = new NodeContextContainer();        
        NodeManager[] nodeManagerList = DCIInfo.get_node_managers();
        for(int i=0 ; i<nodeManagerList.length ; i++)
            nodeContainer.addEntry(nodeManagerList[i].node_name(), nodeManagerList[i]);
        v.add(new DefaultEntry(nodeContainer, new DefaultName("Nodes")));
        
        DCIDeployment DCIDeploy = DCIManager_.provide_dci_deployment();
        
        // Completing both the "Installed Assemblies" context and the "Running Assemblies" context
        ContextContainer installedAssembliesContainer = new InstalledAssembliesContextContainer(DCIDeploy);
        ContextContainer runningAssembliesContainer = new RunningAssembliesContextContainer();
        String[] assemblyUUIDs = DCIDeploy.get_assemblies();
        for (int i = 0; i < assemblyUUIDs.length; i++) {
            String assemblyName = getName(assemblyUUIDs[i]) + " (" + (i+1) + ")";
            installedAssembliesContainer.addEntry(assemblyName,new AssemblyUUIDWrapper(assemblyUUIDs[i],DCIDeploy));
            try{
                String[] instanceUUID = DCIDeploy.get_assembly_instances(assemblyUUIDs[i]);
                for (int j = 0; j < instanceUUID.length; j++) {
                    AssemblyManager assemblyManager = DCIDeploy.get_assembly_manager(instanceUUID[j]);
                    AssemblyManagerWrapperWithDCIDeployment am = new AssemblyManagerWrapperWithDCIDeployment();
                    am.setAssemblyManager(assemblyManager);
                    am.setDCIDeployment(DCIDeploy);
                    am.setUUID(instanceUUID[j]);
                    if(isStarted(assemblyManager))  
                        runningAssembliesContainer.addEntry(getName(assemblyManager) + " (" + (j+1) + ")",am);
                }
            } catch (UnknownAssembly e) {
                e.printStackTrace();
            }
        }
        v.add(new DefaultEntry(installedAssembliesContainer, new DefaultName("Installed Assemblies")));
        v.add(new DefaultEntry(runningAssembliesContainer, new DefaultName("Running Assemblies")));
        
        v.add(new DefaultEntry(DCIManager_.provide_home_finder(), new DefaultName("Homes")));        
        
        // Return the array of entry
        return (Entry[]) v.toArray(new Entry[0]);
    }

    // ==================================================================
    //
    // Additional public methods for Wrapper interface.
    //
    // ==================================================================

    /**
     * Fix the wrapped object
     * @param object The object to wrap
     */
    public void setWrapped(Object object) {
        DCIManager_ = (DCIManager) object;
    }

    /**
     * Gets the wrapped object
     * @return The wrapped object
     */
    public Object getWrapped() {
        return DCIManager_;
    }

}