/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.openccm.ccm.Deployment;

public class ComponentServerMain  extends org.objectweb.openccm.corba.ApplicationServer
{
    
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     ** To store the component server name.
     **/
    protected String name_ ="ComponentServer";

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **/
    public
    ComponentServerMain()
    {
    }


    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  The bootstrap main method.
     **
     **  @param args The command-line arguments.
     **/
    public static void
    main(String args[])
    {
    ComponentServerMain server = new ComponentServerMain();
    int status = server.start(args);
    System.exit(status);
    }

    // ==================================================================
    //
    // Methods for the org.objectweb.openccm.corba.ApplicationServer.
    //
    // ==================================================================

    /**
     **  Before running the CORBA::ORB.
     **
     **  @param args The command line arguments.
     **
     **  @return The status.
     **/
    public int
    before_run(String[] args)
    {
    // Init the OpenCCM Components runtime.
    try{
        org.objectweb.openccm.ccm.Components.Runtime.init();
        //org.objectweb.openccm.corba.TheORB.initialize(args);
    }
    catch(Exception e){ System.out.println(e.getMessage());}
    
    try{
        //setup componentinstallation for this JVM 
        org.omg.CosNaming.NameComponent[] ncomp
        = new org.omg.CosNaming.NameComponent[1];
        
        ncomp[0]= new org.omg.CosNaming.NameComponent("ComponentInstallation"+args[1],"");
        
        org.omg.CORBA.Object obj = org.objectweb.openccm.corba.TheNameService
        .getNamingContext()
        .getNamingContext()
        .resolve(ncomp);
        
        org.omg.Components.Deployment.ComponentInstallation server_inst=
        org.omg.Components.Deployment.ComponentInstallationHelper.narrow(obj);
        
        TheComponentInstallation.setComponentInstallation(server_inst);        
        
    }
    catch(Exception e)
        {
        System.out.println("Cannot retrieves the componentInstallation "+
                   "ComponentInstallation"+args[1]+e.getMessage());
    
        e.printStackTrace();
        }
    // Create the server servant.
    
    org.objectweb.openccm.ccm.Components.ConfigValueImpl[] config= 
        new org.objectweb.openccm.ccm.Components.ConfigValueImpl[1];
    org.omg.CORBA.Any any= org.objectweb.openccm.corba.TheORB.getORB().create_any();
    
    //store the component server name in config parameter
    any.insert_string(args[0]+args[1]); 
    config[0]= new org.objectweb.openccm.ccm.Components.ConfigValueImpl("ServerName",any);
    
    ComponentServerImpl csi = new ComponentServerImpl(config);  
    
    // Activate it.
    org.omg.CORBA.Object ref =
            csi._this_object(org.objectweb.openccm.corba.TheORB.getORB());
    
    org.omg.Components.Deployment.ComponentServer serverRef = 
        org.omg.Components.Deployment.ComponentServerHelper.narrow(ref);
    
    /* args [0] nom de la machine locale
     * args[1] non du fichier a ecrire
     */ 
    // Obtain the Name Service.
    org.objectweb.openccm.corba.NamingContext ns =
        org.objectweb.openccm.corba.TheNameService.getNamingContext();
    
    // Bind the server object in the name service.
    ns.rebind(args[0]+args[1], serverRef);
    
    
    // Output the IOR of the ComponentServer
    //org.objectweb.openccm.corba.TheORB.save_IOR(serverRef, args[0]+args[1]);
    
        
    System.out.println("The OpenCCM Component Server "+args[0]+" is ready.");
    System.out.println(org.objectweb.openccm.corba.TheORB.getORB().object_to_string(ref));
    return 0;
    }
    
    /**
     **  After running the CORBA::ORB.
     **/
    public void
    after_run()
    {
    // Obtain the Name Service.
    org.objectweb.openccm.corba.NamingContext ns =
        org.objectweb.openccm.corba.TheNameService.getNamingContext();
    
    // Unbind the server object from the name service.
    ns.unbind(name_);    
    }    
    
}
