/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet.
Contributor(s): Sylvain Leblanc,
                Romain Rouvoy,
                Philippe Merle,
                Briclet Fr?d?ric,
                Christophe Demarey,
                Hameau Fabien.

====================================================================*/

package org.objectweb.openccm.ccm.Deployment;

import org.objectweb.openccm.ccm.Containers.MetaInformation.HomeType;
import org.objectweb.openccm.ccm.Containers.MetaInformation.HomeInstance;

/**
 * This class implements the OMG IDL
 * OpenCCM::ComponentServer::Container interface.
 *
 * @author <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 * @author <a href="mailto:Fabien.Hameau@lifl.fr">Hameau Fabien</a>
 * 
 * @version 0.5
 */

public class ContainerBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     ** Fred Modification :
     *  Change the_component_server type from ComponentServerBase to
     * component_server_impl. Because of split of ComponentServer and
     * Component Installation in two JVM the Server object isn't required
     * anymore. So a componentServerBase cannot retrieve the ComponentServer
     * facette.
     **/
     protected ComponentServerImpl the_component_server_;
    //protected ComponentServerBase the_component_server_;

    /**
     **
     **/
    protected org.omg.Components.ConfigValue[] configuration_;

    /**
     **
     **/
    protected org.objectweb.ccm.util.Vector homes_;

    /**
     **
     **/
    protected fr.lifl.cim.jidlscript.JOSInterpreter josh_;

    /**
     **
     **/
    protected org.objectweb.openccm.ccm.Containers.PCA rootPCA_;

    /**
     **
     **/
    protected org.objectweb.openccm.ccm.Containers.SystemConfiguration container_configuration_;

    /**
     **
     **/
    protected org.objectweb.openccm.ccm.Containers.SystemConfiguration home_configuration_;

    /*fred adding*/
    protected org.omg.Components.Deployment.ComponentInstallation the_component_installation;
    
    /**
     **
     **/
    protected org.objectweb.openccm.ccm.Containers.SystemConfiguration component_configuration_;

    /**
     **
     **/
    protected SystemHomeManagerImpl home_manager_;
    
    /**
     **
     **/
    protected org.objectweb.openccm.ccm.Deployment.listener.ContainerListener listener_;

    /**
     **
     **/
    protected org.omg.Components.Deployment.Container the_container_ref_;


    protected ContainerHomeInformation containerHomeInfo;
                
    // Adding a reference to the home type MI managed by 
    // this container 
    
    protected HomeType home_type_;
    
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **  @param classLoader The class loader used to load home classes.
     **  @param script The base configuration of the container.
     **
     **  @throws BadConfiguration If the base configuration script is not valid.
     **/
    public
    ContainerBase(org.omg.Components.ConfigValue[] config,
                    /*---> Fred modification:
                     * component_server type change from ComponentServerBase
                     * to ComponentServerImpl*/
                  ComponentServerImpl /*ComponentServerBase*/ component_server,
                  org.omg.Components.Deployment.ComponentInstallation the_component_installation)
    
    throws org.omg.Components.CreateFailure,
           org.omg.Components.Deployment.InvalidConfiguration
    {
        the_component_server_ = component_server;
        this.the_component_installation=the_component_installation;
        configuration_ = config;
        homes_ = new org.objectweb.ccm.util.Vector();
        home_manager_ = new SystemHomeManagerImpl();
        listener_  = null;
        the_container_ref_ = null;
        
        home_type_ = null; // no home type MI available at this point
       
        
        org.objectweb.openccm.ccm.Deployment.listener
                                    .TheContainerConfigurator
                                    .get_instance()
                                    .configure_container(new ContainerLocalImpl(this));

        container_configuration_ = null;
        home_configuration_ = null;
        component_configuration_ = null;
        fr.lifl.cim.jidlscript.lang.JISScope scope = null;

        // initialize the container.
        try
        {
            // create the interpreter
            josh_ = new fr.lifl.cim.jidlscript.JOSInterpreter();

            // create the root PCA.
            rootPCA_ = new org.objectweb.openccm.ccm.Containers.RootPCAImpl(home_manager_);
            scope = new fr.lifl.cim.jidlscript.lang.JISScope("CONTAINER");
            fr.lifl.cim.jidlscript.lang.JISRootScope.root().insertScoped(scope);
            create_alias(scope, "org.objectweb.openccm.ccm.Deployment.ContainerBase", this, "container");
            //
            //  TODO : other alias needed ???
            //
        }
        catch(fr.lifl.cim.jidlscript.lang.JISError ex)
        {
            throw new org.omg.Components.CreateFailure();
        }

        // retrieve the script from the config values.
        java.lang.String script = "";
        for (int i=0;i<config.length;i++)
        {
            if (config[i].name.equals("container_script"))
            {
                script = config[i].value.extract_string();
                break;
            }
        }

        // executes the specific configuration script.
        try
        {
            josh_.eval(script);
        }
        catch(fr.lifl.cim.jidlscript.lang.JISError ex)
        {
            //
            //  TODO : is it the good exception ???
            //
            //            throw new org.omg.Components.Deployment.InvalidConfiguration(-1, "container_script");
            throw new java.lang.Error("### script failure : \n"+ex.exception.toString());
        }
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Trace an error.
     */
    protected void
    trace_error(String method,
                String msg,
                Throwable exc)
    {
        System.err.println("Components::Deployment::Container::" + method + ": " + msg);
        if(exc != null)
            exc.printStackTrace();
    }

    /**
     **
     **/
    protected void
    create_alias(fr.lifl.cim.jidlscript.lang.JISScope scope,
                 java.lang.String clazz,
                 java.lang.Object instance,
                 java.lang.String alias)
    {
        try
        {
            fr.lifl.cim.jidlscript.java.JISJavaClass jc = 
                fr.lifl.cim.jidlscript.java.JISJavaClass.importClass(new fr.lifl.cim.jidlscript.lang.JISString(clazz));
            fr.lifl.cim.jidlscript.java.JISJavaInstance ji = new fr.lifl.cim.jidlscript.java.JISJavaInstance(jc, instance);
            scope.insertScoped(new fr.lifl.cim.jidlscript.lang.JISAlias(alias, ji));
        }
        catch(fr.lifl.cim.jidlscript.lang.JISError ex)
        {
            //
            //  TODO : something else ???
            //
        }
    }

    /**
     * Creates a home executor.
     *
     * @param entrypt The entry point in the archive.
     *
     * @return The created home executor.
     *
     * @exception org.omg.Components.Deployment.ImplEntryPointNotFound
     *            Thrown if the home executor entry point is invalid.
     */
    protected org.omg.Components.HomeExecutorBase
    create_home_executor(java.lang.String entrypt)
    throws org.omg.Components.Deployment.ImplEntryPointNotFound,
           org.omg.Components.Deployment.InstallationFailure
    {
        try
        {
            // Call the static home executor 'entrypt' method.
            java.lang.Object object =
                TheURLClassLoader.getURLClassLoader()
                .callStaticClassMethod(entrypt, new Object[0]);

            // Check that the result is a CCMHome executor.
            return (org.omg.Components.HomeExecutorBase)object;
        }
        catch(java.lang.ClassNotFoundException exc)
        {
            trace_error("install_home",
                        "Class '" + exc.getMessage() + "' not found for home executor entry point '"
                        + entrypt + "'!!!",
                        null);
            throw new org.omg.Components.Deployment.ImplEntryPointNotFound();
        }
        catch(java.lang.NoSuchMethodException exc)
        {
            trace_error("install_home",
                        "Home executor entry point 'public static org.omg.Components.HomeExecutorBase "
                        + entrypt + "();' not found, check the home executor entry point signature!!!",
                         null);
            throw new org.omg.Components.Deployment.ImplEntryPointNotFound();
        }
        catch(java.lang.reflect.InvocationTargetException exc)
        {
            trace_error("install_home",
                        "Exception thrown by home executor entry point '" + entrypt + "'!!!",
                        exc.getTargetException());
            throw new org.omg.Components.Deployment.InstallationFailure(
                          org.objectweb.openccm.ccm.Deployment.ExceptionThrownByHomeExecutorEntryPoint.value
                  );
        }
        catch(java.lang.IllegalAccessException exc)
        {
            trace_error("install_home",
                        "Illegal access by home executor entry point '" + entrypt + "'!!!",
                        exc);
            throw new org.omg.Components.Deployment.InstallationFailure(
                          org.objectweb.openccm.ccm.Deployment.IllegalAccessByHomeExecutorEntryPoint.value
                  );
        }
        catch(java.lang.ClassCastException exc)
        {
            trace_error("install_home",
                        "Home executor entry point '" + entrypt +
                        "' does not return org.omg.Components.HomeExecutorBase," +
                        " check the home executor entry point signature!!!",
                        null);
            throw new org.omg.Components.Deployment.ImplEntryPointNotFound();
        }
    }

    /**
     * Used by the computeFullHomeServantClassName method in order
     * to compute full home servant class names.
     */
    static private org.objectweb.openccm.generator.translator.idl2java.api.CommonTranslator
        javaTranslator = new org.objectweb.openccm.generator.translator.idl2java.lib.CommonTranslator();

    /**
     * Compute the full home servant class name associated
     * to a home executor class.
     *
     * @param homeExecutorClass The home executor class.
     *
     * @return The full home servant class name, or null if not found.
     */
    protected static String
    computeFullHomeServantClassName(Class homeExecutorClass)
    {
        // If homeExecutorClass is null then returns null.
        if(homeExecutorClass == null)
            return null;

        // The full class name with the package name.
        String fullClassName = homeExecutorClass.getName();

        // The short class name without the package name.
        int idx = fullClassName.lastIndexOf(".") + 1;
        String shortClassName = fullClassName.substring(idx);

        // If the short class name is prefixed CCM_ then
        // returns the full home servant class name.
        if(shortClassName.startsWith("CCM_"))
            return fullClassName.substring(0, idx)
                 + javaTranslator.checkKeywords(shortClassName.substring(4))
                 + "CCM";

        // Find the CCM home executor interface into implemented interfaces.
        //
        Class[] itfs = homeExecutorClass.getInterfaces();

    // Required to navigate in the reverse order
        // else interface CCM_*Operations will be found
        // before interface CCM_*
        for(int i=itfs.length-1; i>=0; i--)
        {
             String className = computeFullHomeServantClassName(itfs[i]);
             if(className != null)
                 return className;
        }

    // Find the CCM home executor interface into the super class.
        String className = computeFullHomeServantClassName(homeExecutorClass.getSuperclass());
        if(className != null)
            return className;

        // Returns null as the home servant class was not found.
        return null;
    }

    /**
     * Load an already installed package in the container class
     * loader. 
     *
     * @param uuid The identifier used when the package has been
     * installed. The corresponding package will be loaded in the
     * container class loader. 
     *
     * @exception org.omg.Components.Deployment.UnknownImplId If the
     * given identifier is unknown in this container.
     * @exception org.omg.Components.Deployment.InstallationFailure If
     * the package cannot be retrieved or loaded.
     */
    protected void load_lib_dependency(String uuid) 
        throws org.omg.Components.Deployment.UnknownImplId,
        org.omg.Components.Deployment.InstallationFailure 
    {
        try{
            TheURLClassLoader.getURLClassLoader()
                .addURL(new java.net.URL(the_component_installation
                                         .get_implementation(uuid)));
            
        }
        catch(java.net.MalformedURLException ef)
        {
            trace_error("load_lib_dependency", 
                        "Installation failuer for package related to " + 
                        uuid + ".", null);
            throw new org.omg.Components.Deployment.InstallationFailure();
        }
    }

    protected String retrieve_MI_name(String className)
    {
      
      try 
      {
        // isolate names
        String tmpName = className.substring(0, className.lastIndexOf("Impl."));
        // get the IDL name of home
        String IDLName = tmpName.substring(tmpName.lastIndexOf(".") + 1);
        // get the prefix for concerned nale    
        String prefix = tmpName.substring(0,tmpName.lastIndexOf("."));
        prefix = tmpName.substring(0,prefix.lastIndexOf(".") + 1);
        // compute the full name
        if (IDLName != null)
        {
          return prefix + IDLName;
        }
      }
      // problem during name computation
      catch(Exception e)
      {
        trace_error("retrieve_MI_name","error when computing IDL name for MetaInformation", null);
      }
      return null;
    }
    
    /**
     **
     **/
    public void
    unregister_home(org.omg.Components.CCMHome ref)
    throws org.omg.Components.RemoveFailure
    {        
        listener_.on_remove(ref);
        for(java.util.Iterator it=homes_.iterator();it.hasNext(); )
          if(ref._is_equivalent(((org.omg.Components.CCMHome)it.next()))){  
            it.remove();
            if(containerHomeInfo!=null)
                get_container_home_info().unregister_home(ref);
            return;
            
        }
       // The reference was not foound throw a Remove Exception
       throw new org.omg.Components.RemoveFailure();
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     ** Add a new plugin to the deployment process
     **
     ** @param listener the deployment listener to add
     **/
    public void
    add_listener (org.objectweb.openccm.ccm.Deployment.listener.ContainerListener listener)
    {
        if (listener_ == null)
            listener_ = listener;
        else
        listener_.set_delegate(listener);
    }

    /**
     **
     **/
    public void 
    the_container_ref(org.omg.Components.Deployment.Container container)
    {
        the_container_ref_ = container;
    }

    /**
     **
     **/
    public org.omg.Components.Deployment.Container
    the_container_ref()
    {
        return the_container_ref_;
    }

    /**
     **
     **/
    public org.objectweb.openccm.ccm.Containers.PCA 
    get_rootPCA()
    {
        return rootPCA_;
    }

    /**
     **
     **/
    public org.objectweb.openccm.ccm.Containers.SystemHome
    create_system_home(java.lang.String implUUID,
                       java.lang.String alias,
                       java.lang.String entrypt)
    throws org.objectweb.openccm.ccm.Containers.CreationFailed
    {
        return home_manager_.create_system_home(implUUID, alias, entrypt);
    }

    /**
     **
     **/
    public org.objectweb.openccm.ccm.Containers.SystemHome
    find_system_home(java.lang.String alias)
    throws org.objectweb.openccm.ccm.Containers.HomeNotFound
    {
        return home_manager_.find_system_home(alias);
    }

    /**
     **
     **/
    public void
    set_container_configuration(org.objectweb.openccm.ccm.Containers.SystemConfiguration config)
    {
        container_configuration_ = config;
    }

    /**
     **
     **/
    public org.objectweb.openccm.ccm.Containers.SystemConfiguration
    get_container_configuration()
    {
        return container_configuration_;
    }

    /**
     **
     **/
    public void
    set_home_configuration(org.objectweb.openccm.ccm.Containers.SystemConfiguration config)
    {
        home_configuration_ = config;
    }

    /**
     **
     **/
    public org.objectweb.openccm.ccm.Containers.SystemConfiguration
    get_home_configuration()
    {
        return home_configuration_;
    }

    /**
     **
     **/
    public void
    set_component_configuration(org.objectweb.openccm.ccm.Containers.SystemConfiguration config)
    {
        component_configuration_ = config;
    }

    /**
     **
     **/
    public org.objectweb.openccm.ccm.Containers.SystemConfiguration
    get_component_configuration()
    {
        return component_configuration_;
    }

    /**
     ** Add the home to the container inner list
     **
     ** @param ref the reference of the home to add
     **/
    public void
    add_home(org.omg.Components.CCMHome ref)
    {
        homes_.add(ref);
        listener_.on_add(ref); 
    }
    
    /**
     ** Add a reference on the parent ContainerSeg
     **/
    public void
    set_container_home_info(ContainerHomeInformation containerHomeInfo)
    {
         this.containerHomeInfo=containerHomeInfo;   
    }
                    
    /**
     ** Accessor to the containerSeg
     **/
    public ContainerHomeInformation
    get_container_home_info()
    {
        return containerHomeInfo;
    }
     
    
    
    // ==================================================================
    //
    // Methods for the Components::Deployment::Container interface.
    //
    // ==================================================================

    //
    //  IDL:omg.org/Components/Deployment/ComponentServer/configuration:1.0
    //
    /**
     **  To obtain the container configuration.
     **
     **/
    public org.omg.Components.ConfigValue[]
    configuration()
    {
        return configuration_;
    }

    //
    //  IDL:omg.org/Components/Deployment/Container/get_component_server:1.0
    //
    /**
     **  To obtain a reference on the component server that has created this container.
     **/
    public org.omg.Components.Deployment.ComponentServer
    get_component_server()
    {
        return the_component_server_.asComponentServer();
    }

    //
    //  IDL:omg.org/Components/Deployment/Container/install_home:1.0
    //
    /**
     * Install a home in the container.
     * This can be done more than once.
     *
     * @param uuid Implementation identifier.
     * @param entrypt The entry point method.
     * @param config The configuration values.
     *
     * @return The created home.
     *
     * @exception UnknownImplId Invalid implementation identifier.
     * @exception ImplEntryPointNotFound Invalid entry point.
     * @exception InstallationFailure Installation failure.
     * @exception InvalidConfiguration Invalid configuration values.
     */
    public org.omg.Components.CCMHome
    install_home(java.lang.String uuid,
                 java.lang.String entrypt,
                 org.omg.Components.ConfigValue[] config)
    throws org.omg.Components.Deployment.UnknownImplId,
           org.omg.Components.Deployment.ImplEntryPointNotFound,
           org.omg.Components.Deployment.InstallationFailure,
           org.omg.Components.Deployment.InvalidConfiguration
    {
        try {
       
            // Retrieve the archive and load it into the class loader.
            // Here the ComponentInstallation is used to retrieve the archive
            // local URL.
            load_lib_dependency(uuid);
            
            // load lib dependencies in the container class loader.
            for (int i=0; i<config.length; i++)
             {
                 if (config[i].name.equals("load_lib_dependency")) 
                 {
                     load_lib_dependency(config[i].value.extract_string());
                 } 
             }

            // retrieve the script from the config values.
            java.lang.String script = "";
            for (int i=0; i<config.length; i++)
            {
                if (config[i].name.equals("home_script")) 
                {
                    try
                    {
                        script = config[i].value.extract_string();
                    }
                    catch(org.omg.CORBA.SystemException exc)
                    {
                        trace_error("install_home",
                                    "Invalid type for the 'home_script' configuration value, a string is expected!!!",
                                    null);
                        throw new org.omg.Components.Deployment.InvalidConfiguration(
                                      org.omg.Components.Deployment.InvalidConfigValueType.value,
                                      "home_script");
                    }
                    break;
                }
            }

            // executes the specific configuration script.
            try
            {
                josh_.eval(script);
            }
            catch(fr.lifl.cim.jidlscript.lang.JISError exc)
            {
                trace_error("install_home",
                            "Invalid value for the 'home_script' configuration value!!!\n"
                            + script,
                            null);
                throw new org.omg.Components.Deployment.InvalidConfiguration(
                              org.objectweb.openccm.ccm.Deployment.InvalidConfigValue.value,
                              "home_script");
            }
        // begin MI modif
        /////////////////////////////////////////////
        // create the home type MI for the given home
        /////////////////////////////////////////////
        //  retrieving IDL Declaration name
        String IDL_name = retrieve_MI_name(entrypt);
        if (IDL_name != null)
        {
          //  setting the class and method name
          String methodName = IDL_name + "_homeMI.create_type_MI";
          // call the computed name method (static)
          try
          {
          home_type_ =
            (HomeType) TheURLClassLoader
              .getURLClassLoader()
              .callStaticClassMethod(
              methodName,
              new Object[0]);
          }
          catch(Exception e)
              {
                  // the home type entrypt is not available and 
                  // the home_type is not created
                  home_type_ = null;
              }
        }
        // End of MI modif
            // create the home executor.
            org.omg.Components.HomeExecutorBase home =
                create_home_executor(entrypt);           

            String clazz_name = computeFullHomeServantClassName(home.getClass());
            if (clazz_name == null) 
            {
                trace_error("install_home",
                            "Home servant class for " + home.getClass().getName() + " not found!!!",
                             null);
                throw new org.omg.Components.Deployment.InstallationFailure(
                              org.objectweb.openccm.ccm.Deployment.HomeServantClassNotFound.value
                          );
            }

            // Instantiate the executor.
            org.objectweb.openccm.ccm.Containers.HomeExecutorBase home_exec =
                (org.objectweb.openccm.ccm.Containers.HomeExecutorBase)
                create_home_executor(clazz_name + "._create_home_executor");

            home_exec._delegate(home);

            // create the home servant and set infos.
            org.objectweb.openccm.ccm.Containers.HomeServantImpl servant = 
                new org.objectweb.openccm.ccm.Containers.HomeServantImpl();
            servant.the_container_base(this);
            home_exec._home_servant(servant);
            servant.the_home_executor(home_exec);
            servant.the_home_configuration(get_home_configuration());
            servant.the_component_configuration(get_component_configuration());
            servant.the_home_finder(home_manager_);
        ///////////////////////////// MODIF MetaInformation
        if (home_type_ != null)
        { 
          HomeInstance _tmp_hi = home_type_.create_home_instance();
          if (_tmp_hi == null)
          {
            trace_error("install_home", "home instance is null ", null);
          }
          servant.the_home_instance(_tmp_hi);
          
        }
        else
        {
          trace_error(
            "retrieve_MI_name",
            "Instance MetaInformation for HomeFider not supported ",
            null);
        }
            // create and set the home reference.
            org.omg.Components.CCMHome ref = rootPCA_.activate_home(servant);
            

            int h_s=homes_.size();            
            listener_.on_create(ref);            
            if(homes_.size()==h_s){
                 homes_.add(ref);              
              }
            return ref;
        }
        catch(org.omg.Components.Deployment.UnknownImplId exc)
        {
            trace_error("install_home", "Throw exception " + exc.getMessage(), null);
            throw exc;
        }
        catch(org.omg.Components.Deployment.ImplEntryPointNotFound exc)
        {
            trace_error("install_home", "Throw exception " + exc.getMessage(), null);
            throw exc;
        }
        catch(org.omg.Components.Deployment.InstallationFailure exc)
        {
            trace_error("install_home", "Throw exception " + exc.getMessage(), null);
            throw exc;
        }
        catch(org.omg.Components.Deployment.InvalidConfiguration exc)
        {
            trace_error("install_home", "Throw exception " + exc.getMessage(), null);
            throw exc;
        }
        catch(Throwable exc)
        {
            trace_error("install_home", "Unexpected Java exception!", exc);
            trace_error("install_home", "Throw exception Components::Deployment::InstallationFailure", null);
            throw new org.omg.Components.Deployment.InstallationFailure(
                          org.objectweb.openccm.ccm.Deployment.UnknownInstallationFailureReason.value
                      );
        }
    }

    //
    //  IDL:omg.org/Components/Deployment/Container/remove_home:1.0
    //
    /**
     **
     **/
    public void
    remove_home(org.omg.Components.CCMHome ref)
    throws org.omg.Components.RemoveFailure
    {
/* Bug #306
   OLD CODE

        listener_.on_remove(ref);
        boolean b = homes_.remove(ref);
        if (!b)
            throw new org.omg.Components.RemoveFailure();

   NEW CODE
*/

      // Tests if ref is a home of this containers.
      for(int i=0; i<homes_.size(); i++)
      {
          org.omg.Components.CCMHome h =
              (org.omg.Components.CCMHome)homes_.get(i);

          // If ref is a home of this container then removes it.
          if(ref._is_equivalent(h))
      {
              ref.remove_home();
              return;
          }
      }

      // Else ref is not a home of this container.
      throw new org.omg.Components.RemoveFailure();
    }

    //
    //  IDL:omg.org/Components/Deployment/Container/get_homes:1.0
    //
    /**
     **
     **/
    public org.omg.Components.CCMHome[]
    get_homes()
    {
        org.omg.Components.CCMHome[] result =
            new org.omg.Components.CCMHome[homes_.size()];

        for (int i=0;i<result.length;i++)
            result[i] = (org.omg.Components.CCMHome)homes_.get(i);

        return result;
    }

    //
    //  IDL:omg.org/Components/Deployment/Container/remove:1.0
    //
    /**
     **
     **/
    public void
    remove()
    throws org.omg.Components.RemoveFailure
    {
/* Bug #306
   NEW CODE
*/

        // Removes all its homes.
        org.omg.Components.CCMHome[] homes = get_homes();
        for(int i=0; i<homes.length; i++)
        {
            homes[i].remove_home();
        }
        if(listener_!=null)
            listener_.on_destroy();
        
        // Unregister the container from its component server.
        if(the_component_server_!=null)
            the_component_server_.unregister_container(the_container_ref());
          
        try {
            org.objectweb.openccm.corba.TheRootPOA.getRootPOA().deactivate_object(org.objectweb.openccm.corba.TheRootPOA.getRootPOA().reference_to_id(the_container_ref()));
            
        } catch (org.omg.PortableServer.POAPackage.WrongAdapter ex) {
            throw new org.omg.Components.RemoveFailure();
        } catch (org.omg.PortableServer.POAPackage.WrongPolicy ex) {
            throw new org.omg.Components.RemoveFailure();
        } catch (org.omg.PortableServer.POAPackage.ObjectNotActive ex) {
            throw new org.omg.Components.RemoveFailure();
        }
    }
} 
