/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): 

====================================================================*/

package org.objectweb.openccm.ccm.Deployment;

public class ParameterConverter{
 
    /**
     * All the different kind of parameter used
     */
    private ParameterType parameterTypes[] = 
                            new ParameterType[]{new ORBParameter(),
                                                new IORParameter(),
                                                new SimpleParameter()};
    /**
     * The definition of parameter type
     */
    private abstract class ParameterType {
      
     public abstract boolean is_a(String param);       
     
     public abstract String convert(String param);      
     
     public abstract boolean implyNextConvertion();
    }

    /**
     * Basis parameter type
     */
    private class SimpleParameter extends ParameterType{
 
         public boolean is_a(String param)
         {
            return param!=null;
         }       
         
         public String convert(String param)
         {
            return param;
         }
      
         public boolean 
        implyNextConvertion()
        {
            return false;
        }
        
        public String
        toString()
        {
            return "SimpleType";
        }
    }



    class TestAuthenticator extends java.net.Authenticator {
        
           private String username = "titi";
        private String password = "titi";
        
        public TestAuthenticator() {}
        
        public java.net.PasswordAuthentication getPasswordAuthentication()
        {
            return new java.net.PasswordAuthentication(username,(password.toCharArray()));
            }
    }

    /**
     * IOR parameter type containing containing only URL. Stringified
     * IOR are considerer as simple parameter
     */
    private class IORParameter extends SimpleParameter {
 
         public boolean is_a(String param)
         {
            return param.endsWith(".IOR")||param.endsWith(".ior");
         }       
         
         public String convert(String param)
         {

           String ch=param.substring(param.indexOf("=")+1);
                    try{
                        
                        //java.net.Authenticator.setDefault(new TestAuthenticator());
                        java.net.URL url=new java.net.URL(ch);
                    
                        //String userPassword = "anonymous:anonymous";
                    
                        // Encode String
                        //String encoding = new sun.misc.BASE64Encoder().encode (userPassword.getBytes());
                    
                         
                        // Need to work with URLConnection to set request property
                       // java.net.URLConnection uc = url.openConnection();
                        //uc.setRequestProperty  ("Authorization", "Basic " + encoding);
                        //System.err.println("Authorization fixed");   
                                                
                        java.io.InputStream in=url.openStream(); //uc.getInputStream();
                        byte []b=new byte[in.available()];
                        
                        in.read(b,0,in.available());
                        ch=param.substring(0,param.indexOf("=")+1)+(new String(b)).trim();
                        //System.out.println(args[i]);
                        }
            catch(Exception e)
                   {
                    e.printStackTrace();
                    System.err.println("Cannot read the URL contents.");
                    return param ;
                    
                    } 
            return ch;
         }
         
        public String
        toString()
        {
            return "IORType";
        }       
    }   
    
    /**
     * The ORB parameter can require convertion of the following parameter
     * if it denote an IOR with an URL
     */
    public class ORBParameter extends SimpleParameter{
      
    
         public boolean is_a(String param)
         {
            return param.startsWith("-ORB");
         }       
         
      
        public boolean 
        implyNextConvertion()
        {
            return true;
        }
        
        public String
        toString()
        {
            return "ORBType";
        }
    }
    
    /**
     * The basis parameter witch do all the treatment
     */
    private class Parameter{
 
     
     private String arg=null;
     private ParameterType type;
     private Parameter next;
     private int index;
     
     public Parameter(String args[],int index ){
        this.arg=args[index];
        this.index=index;
        
        for(int i=0;i<parameterTypes.length ;i++)
            {
                if(parameterTypes[i].is_a(arg)){
                    this.type=parameterTypes[i];
                    break;
                }
            }
        if(args.length-1>index)
            next= new Parameter(args,index+1);
        
     }
 
     public void convert(){
        arg=type.convert(arg);
    }
     
     public java.util.LinkedList
     processTreatment()
     {     
          
        if(next==null){
            java.util.LinkedList ll= new java.util.LinkedList();     
            ll.addFirst(arg);
            return ll;
          }
          
        if(type.implyNextConvertion())
            next.convert();
        
        java.util.LinkedList ll=next.processTreatment();
        ll.addFirst(arg);
        return ll;           
     }          

     public ParameterType
     getParameterType()
        {
            return type;
        }
           
    }
 
    /**
     * Static method to convert arguments
     */
    public String[] 
    convertParameter(String args[])
    {
       java.util.LinkedList ll=(new Parameter(args,0)).processTreatment();
       
       String [] ch=(String[])ll.toArray(new String[ll.size()]);
             
           
           return ch;
    }
    
    /*
    public static void main(String args[]){
        ParameterConverter pc=new ParameterConverter();
        String []arg=pc.convertParameter(new String[]{"-ORBInitRef","0214541121","-ORBInitRef","TITI=http://127.0.0.1/dci_toto.IOR" });    
        for(int i=0;i<arg.length;i++)
               System.err.println(arg[i]);  
    }*/
           
}
