/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base;

/**
 * This Exception is thrown to signal that a fatal error has occured during 
 * deployment
 * 
 * @author <a href="mailto:briclet@lifl.fr">Frederic Briclet</a>
 *
 * @version 0.1
 */
public class FatalDeploymentException
    extends DeployerExceptionWrapper 
{
    //  ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    private DeployerContext deployer;
    //  ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    
    /**
     * Basic constructor taking in parameter the initial exception
     * which make failed the deployment.
     */
    public 
    FatalDeploymentException(Exception e)
    {
        super(e, "Fatal error occur during the deployment :\n" + e.getMessage());
    }
    
    /**
     * Basic constructor taking in parameter the initial exception
     * which make failed the deployment and an explaination.
     */
    public FatalDeploymentException(Exception e, String message)
    {
        super(e,"Fatal error occur during the deployment \n" + message + ":\n");
    }
    
    /**
     * Basic constructor taking in parameter the initial exception
     * which make failed the deployment and an explaination.
     */
    public FatalDeploymentException(DeployerContext dc ,Exception e, String message)
    {
        super(e,"Fatal error occur during the deployment \n" + message + ":\n");
        this.deployer=dc;
    }
    //  ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    //  ==================================================================
    //
    //  Public methods 
    //
    // ==================================================================
    public String getMessage()
    {
        if(deployer==null)
            return super.getMessage();
        try{
            return super.getMessage()+"\n Check your CAD file it must contains a error\n"
                                     +deployer.getStringifiedDeployerDescription();
        }
        catch(Exception e){
            return super.getMessage()+"\n Check your CAD file it must contains a error";
        }

    }

}
