/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.binder;


// import all required packages
/**
 * The connect event deployer is in charge to build an asynchronous connection
 * defined by the element connectevent.
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */


public class ConnecteventDeployer 
       extends ConnecteventDeployerContext
    {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    /* Internal exception used to manage the connection*/
    private class CannotResolveReferenceException extends Exception{};

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================


    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================   

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
   
    
    /**
     * Establishe the connection between the provided interface and the 
     * required interface
     * 
     * @param ConnectEventEstablishmentFailureException is thrown if     
     * a problem occured during establisment
     */
    public void 
    deploy()
    throws ConnectEventEstablishmentFailureException
    {
        try{            
            // Get the consumes object
            org.omg.Components.EventConsumerBase consumedObject=null ;
            
            if(getConsumesportDeployer()!=null)
                consumedObject=getConsumesportDeployer().getConsumer();
            else consumedObject=getExistinginterfaceDeployer().getEventConsumerBase();
            
            if(getEmitsportDeployer()!=null)
                getEmitsportDeployer().connectEmiterToConsumer(consumedObject);
            else getPublishesportDeployer().connectPublisherToConsumer(consumedObject);   
        }
        catch(Exception e){ 
            throw new ConnectEventEstablishmentFailureException(this,e);
        }
        getLifeCycleManager().stepEndedSuccessFully();
    }




    /**
     *Tear down connection : unsubscrive the publisher     
     *
     * @param deployTable the internal table use to store information.
     * @param cei the connectevent to tear_down
     * @return Return the corba object found
     */
    public void 
    tearDown()
    throws ConnectEventBreakingFailureException
    {
        try{                       
            if(getEmitsportDeployer()!=null)
                getEmitsportDeployer().disconnectConsumer();
            else getPublishesportDeployer().disconnectConsumer();
        }
        catch(Exception e)
            {
                throw new ConnectEventBreakingFailureException(this,e);
            }

        getLifeCycleManager().stepEndedSuccessFully();    
    }

}
