/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.binder;

import org.objectweb.openccm.descriptor.componentassembly.Publishesport;
import org.objectweb.openccm.descriptor.componentassembly.beans.PublishesportBean;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base.ChildDeployerContext;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base.InitializationError;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.installer.ComponentinstantiationDeployer;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.util.FindbyDeployer;

/**
 * The publishes deployer context defines all the connection requires for
 * publishesport resolution.
 * 
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class PublishesportDeployerContext 
       extends ChildDeployerContext 
    {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    private PublishesportBean publishesport;
    private FindbyDeployer findbyDeployer;
    private ComponentinstantiationDeployer componentinstantiationDeployer;
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================  
    
    /**
     * @return
     */
    public FindbyDeployer 
    getFindbyDeployer() 
    {
        return findbyDeployer;
    }

    /**
     * @return
     */
    public Publishesport 
    getPublishesport() 
    {
        return publishesport;
    }

    /**
     * @param deployer
     */
    public void 
    connectFindbyDeployer(FindbyDeployer deployer) 
    {
        findbyDeployer = deployer;
    }

    /**
     * @param publishesport
     */
    public void 
    connectPublishesport(PublishesportBean publishesport) 
    {
        this.publishesport = publishesport;
        connectDeployerDescription(publishesport);
    }
    /**
     * @return
     */
    public ComponentinstantiationDeployer 
    getComponentinstantiationDeployer() 
    {
        return componentinstantiationDeployer;
    }

    /**
     * @param deployer
     */
    public void 
    connectComponentinstantiationDeployer(ComponentinstantiationDeployer deployer) 
    {
        componentinstantiationDeployer = deployer;
    }
    
    /**
     * This initialization method is used to check that all the services and 
     * information required to deploy are presents. If all is OK the deployer 
     * is put in inactive deployer state waiting for the deployment processing.
     * 
     * @exception InitializationError is thrown if a problem is detected 
     */
    public void 
    initialize() 
    throws InitializationError
    {
        if(getPublishesport()==null)
         getErrorManager()
            .submitException(new InitializationError(this,"Connection on Publishesport bean is required"+
                                          "check the handler code it must specify this connection"));
                                  
        if(getPublishesport().getComponentinstantiationref()!=null&&(
            getPublishesport().getComponentinstantiationref().getIdref()==null ||
            getPublishesport().getComponentinstantiationref().getIdref().length()==0)) 
             getErrorManager()
              .submitException( new InitializationError(this,"The idref attribute of the componentinstantiationref element"+
                                           "must be fullfilled, check thePublishesport element"+
                                           "description\n."+getStringifiedDeployerDescription()));
        if(getPublishesport().getComponentinstantiationref()==null&&getFindbyDeployer()==null)
           getErrorManager()
           .submitException( new InitializationError(this,"One of componentinstantiationref/findby children must be"+
                                           "specified check the Publishesport element"+
                                           "description\n."+getStringifiedDeployerDescription()));                                                              
        getLifeCycleManager().stepEndedSuccessFully();
    }


}
