/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): Jacek Cala, Philippe Merle.

====================================================================*/


package org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.installer;

//Import all the required packages
import java.util.Iterator;
import java.util.StringTokenizer;
import org.objectweb.openccm.corba.TheORB;
import org.objectweb.openccm.corba.TheORBSingleton;
import org.objectweb.openccm.descriptor.properties.beans.*;
import org.omg.CORBA.*;
import org.omg.Components.CCMObject;
import org.omg.DynamicAny.DynAny;
import org.omg.DynamicAny.DynAnyFactory;
import org.omg.DynamicAny.DynAnyFactoryHelper;
import org.omg.DynamicAny.DynSequence;
import org.omg.DynamicAny.DynSequenceHelper;

/**
 * ComponentpropertiesDeployer is used setup all the components properties.
 *
 * @author <a href="mailto:frederic.briclet@lifl.fr">Briclet Frederic</a>
 *
 * @version 0.1
 */
public class ComponentpropertiesDeployer
    extends ComponentpropertiesDeployerContext {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    // ==================================================================
    //
    // Internals methods.
    //
    // ==================================================================
    /**
     * Internal method to setup the simple component properties. This method simply
     * browse simple properties list and setup its
     *
     * @param propertiesObject The properties object to apply to component
     * @param new_component    The component to configure
     */
    private void 
    setupSimpleProperty(
        PropertiesBean propertiesObject,
        org.omg.Components.CCMObject new_component)
    throws Exception 
    {
        //browse all the component simple properties
        for (java.util.Iterator i = propertiesObject.getSimpleList().iterator();
            i.hasNext();
            ) {
            SimpleBean property = (SimpleBean) i.next();
            // build a set request
            org.omg.CORBA.Request request =
                new_component._request("_set_" + property.getName());
            //Process the simple property
            processSimple(property, request.add_in_arg());
            // invocation of the set request
            request.invoke();
        }
    }

    /**
     * Internal method to setup the sequence property. This methods browse the sequence
     * list property and recursivly setup the imbricated properties
     * 
     * @param new_component THe component to setup
     * @param propertiesObject The component properties to apply
     */
    private void 
    setupSequenceListProperty(
        CCMObject new_component,
        PropertiesBean propertiesObject)
    throws Exception 
    {
        for (java.util.Iterator i =
            propertiesObject.getSequenceList().iterator();
            i.hasNext();)
           {
            SequenceBean property = (SequenceBean) i.next();

            org.omg.CORBA.Request request =
                new_component._request("_set_" + property.getName());

            NVList nvlist = request.arguments();
            nvlist.add_value(
                property.getName(),
                processSequence(property).to_any(),
                ARG_IN.value);
            request.invoke();
        }
    }

    /**
     * Retrieve the TCKind object relative to the typeCodeID
     *
     * @param typeCodeId the stringfied typecode to process
     * @return the binded TCKind 
     */
    private TCKind 
    getTCKing(String typeCodeId) 
    {

        //System.out.println("Type rechercher :"+typeCodeId);
        if (typeCodeId.equals("boolean"))
            return org.omg.CORBA.TCKind.tk_boolean;
        else if (typeCodeId.equals("char"))
            return org.omg.CORBA.TCKind.tk_char;
        else if (typeCodeId.equals("double"))
            return org.omg.CORBA.TCKind.tk_double;
        else if (typeCodeId.equals("float"))
            return org.omg.CORBA.TCKind.tk_float;
        else if (typeCodeId.equals("short"))
            return org.omg.CORBA.TCKind.tk_short;
        else if (typeCodeId.equals("long"))
            return org.omg.CORBA.TCKind.tk_long;
        else if (typeCodeId.equals("objref"))
            return org.omg.CORBA.TCKind.tk_objref;
        else if (typeCodeId.equals("octet"))
            return org.omg.CORBA.TCKind.tk_octet;
        else if (typeCodeId.equals("string"))
            return org.omg.CORBA.TCKind.tk_string;
        else if (typeCodeId.equals("ulong"))
            return org.omg.CORBA.TCKind.tk_ulong;
        else if (typeCodeId.equals("ushort"))
            return org.omg.CORBA.TCKind.tk_ushort;
        else if (typeCodeId.equals("sequence"))
            return org.omg.CORBA.TCKind.tk_sequence;
        else if (typeCodeId.equals("struct"))
            return org.omg.CORBA.TCKind.tk_struct;
        return null; // UNKNOWN type

    }
    
    /**
     * This method return the TypeCode relative to the given string
     * representation.
     * 
     * @param type_identifier The type to found  
     * @return The right TypeCode
     */
    private TypeCode 
    getTypeCode(String type_identifier)
    {
        return TheORB.getORB().get_primitive_tc(getTCKing(type_identifier));
    }

    /**
     * This method provide the TypeCode of a sequence relative to a stringified 
     * type representation.
     * The format expected of the parameter is sequence:sequence:..:primitive_type
     * which represent a sequence of sequence of primitive type. 
     * 
     * @param type The well formated sequence type in a string format
     * @return The relative TypeCode to the given type
     */
    private TypeCode 
    getSequenceTypeCode(String type) 
    {
        return getSequenceType(new StringTokenizer(type, ":"));
    }

    /**
     * Rcursive method call to build imbricated sequence TypeCode.
     * 
     * @param tok The token to analyse
     * @return The TypeCode relative to the current token
     */
    private TypeCode 
    getSequenceType(StringTokenizer tok) 
    {
        String ch = tok.nextToken();
        if (ch.equals("sequence"))
            return TheORB.getORB().create_sequence_tc(0, getSequenceType(tok));
        else
            return getTypeCode(ch);
    }
    
    /**
     * This method setup the given @param ds DynSequence with
     * the given @param si SequenceBean. This method process 
     * recursively the imbricated sequence.
     * 
     * @param si The sequencebean to analyse
     * @param ds The DynSequence to setup
     * @throws Exception is thrown in problem case
     */
    private void 
    processSequence(SequenceBean si, DynSequence ds)
    throws Exception 
    {
        if (si.getSimpleList() != null && si.getSimpleList().size() != 0) {

            ds.set_length(si.getSimpleList().size());

            for (java.util.Iterator i = si.getSimpleList().iterator();
                i.hasNext();
                ds.next()) {
                SimpleBean property = (SimpleBean) i.next();
                DynAny any = ds.current_component(); //TheORB.create_any();
                processSimple(property, any);
            }
            return;
        }
        ds.set_length(si.getSequenceList().size());

        for (java.util.Iterator i = si.getSequenceList().iterator();
            i.hasNext();
            ds.next()) {
            SequenceBean seqBean = (SequenceBean) i.next();
            processSequence(
                seqBean,
                DynSequenceHelper.narrow(ds.current_component()));
        }
        ds.rewind();
    }
    
    /**
     * This method return the DynSequence relative to the given @param si
     * SequenceBean configuration.
     *      
     * @param si The configuration information
     * @return The DynSequence to use for configuring a CORBA object
     * @throws Exception is thrown in problem case
     */
    private DynSequence 
    processSequence(SequenceBean si) 
    throws Exception 
    {
        isValid(si,si.getType());
        
        DynAnyFactory daf =
            DynAnyFactoryHelper.narrow(
                TheORBSingleton.resolve_initial_reference("DynAnyFactory"));

        DynAny da =
            daf.create_dyn_any_from_type_code(
                getSequenceTypeCode(si.getType().trim()));
        DynSequence ds = DynSequenceHelper.narrow(da);

        processSequence(si, ds);
        return ds;
    }

    /*private void processStruct(SimpleBean property ){
      }*/

    /**
     * Internal method to fix a simple simple property
     *
     * @param property The simple property to fix
     * @param any The any object to configure
     * @return the any object configurated
     **/
    private Any 
    processSimple(SimpleBean property, Any any) 
    throws Exception 
    {
    
        if (property.getType().equals("double"))
            any.insert_longlong(Long.parseLong(property.getValue().getValue()));
        //else if(property.getType().equals( "objref"))
        //     any.insert_Object(property.getValue().getValue());
        else if (property.getType().equals("boolean"))
            any.insert_boolean(
//
// Bug #300826
// Contributors: Jacek Cala - Philippe Merle 
//
// Previous code:
//
//                Boolean.getBoolean(property.getValue().getValue()));
//
// New code:

                Boolean.valueOf(property.getValue().getValue()).booleanValue());

//
        else if (property.getType().equals("char"))
            any.insert_char(property.getValue().getValue().charAt(0));
        else if (property.getType().equals("float"))
            any.insert_float(Integer.parseInt(property.getValue().getValue()));
        else if (property.getType().equals("short"))
            any.insert_short(Short.parseShort(property.getValue().getValue()));
        else if (property.getType().equals("long"))
            any.insert_long(Integer.parseInt(property.getValue().getValue()));
        else if (property.getType().equals("octet"))
            any.insert_octet(Byte.parseByte(property.getValue().getValue()));
        else if (property.getType().equals("string"))
            any.insert_string(property.getValue().getValue());
        else if (property.getType().equals("ulong"))
            any.insert_ulong(Integer.parseInt(property.getValue().getValue()));
        else if (property.getType().equals("ushort"))
            any.insert_ushort(Short.parseShort(property.getValue().getValue()));

        return any;
    }

    /**
       * Internal method to fix a simple simple property in a DynAny
       *
       * @param property The simple property to fix
       * @param any The any object to configure
       * @return the any object configurated
       **/
    private void 
    processSimple(SimpleBean property, DynAny any)
    throws Exception 
    {
        //Any any=org.objectweb.openccm.corba.TheORB.getORB().create_any();
        if (property.getType().equals("double"))
            any.insert_longlong(Long.parseLong(property.getValue().getValue()));
        //else if(property.getType().equals( "objref"))
        //     any.insert_Object(property.getValue().getValue());
        else if (property.getType().equals("boolean"))
            any.insert_boolean(
//
// Bug #300826
// Contributors: Jacek Cala - Philippe Merle 
//
// Previous code:
//
//                Boolean.getBoolean(property.getValue().getValue()));
//
// New code:

                Boolean.valueOf(property.getValue().getValue()).booleanValue());

//
        else if (property.getType().equals("char"))
            any.insert_char(property.getValue().getValue().charAt(0));
        else if (property.getType().equals("float"))
            any.insert_float(Integer.parseInt(property.getValue().getValue()));
        else if (property.getType().equals("short"))
            any.insert_short(Short.parseShort(property.getValue().getValue()));
        else if (property.getType().equals("long"))
            any.insert_long(Integer.parseInt(property.getValue().getValue()));
        else if (property.getType().equals("octet"))
            any.insert_octet(Byte.parseByte(property.getValue().getValue()));
        else if (property.getType().equals("string"))
            any.insert_string(property.getValue().getValue());
        else if (property.getType().equals("ulong"))
            any.insert_ulong(Integer.parseInt(property.getValue().getValue()));
        else if (property.getType().equals("ushort"))
            any.insert_ushort(Short.parseShort(property.getValue().getValue()));

        //return any;
    }
    
    private void
    isValid(SequenceBean sb,String typeToRespect)
    throws ComponentpropertiesMalFormedException
    {

        if(!sb.getType().equals(typeToRespect))
            throw new ComponentpropertiesMalFormedException(typeToRespect,sb.getType());
            
        String following=typeToRespect.substring(typeToRespect.indexOf(":")+1);
        
       /* if(sb.getSequenceList()!=null&&sb.getSimpleList()!=null)
            return false;*/
        if(sb.getSequenceList()!=null)        
            for(Iterator i=sb.getSequenceList().iterator();i.hasNext();)
               isValid((SequenceBean)i.next(),following);
                    
        else if(sb.getSimpleList()!=null)
            for(Iterator it=sb.getSimpleList().iterator();it.hasNext();)
               isValid((SimpleBean)it.next(),following);               
        
    }
    
    private void
    isValid(SimpleBean sb,String typeToRespect)
    throws ComponentpropertiesMalFormedException
    {
        if(!sb.getType().equals(typeToRespect))
          throw new ComponentpropertiesMalFormedException(typeToRespect,sb.getType());
    }
    
    //  ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
    public class ComponentpropertiesMalFormedException extends Exception{
        
        public 
        ComponentpropertiesMalFormedException(String excepted, String found){
            super("The componentproperties seem to be corrupted, imbricated"+
                  " properties were expected to be ["+excepted+"] and it was found"+
                  " a ["+found+"]");
        }
    }
    /**
     * This method  setup the component properties.
     * The first part process the simple properties, the second part process
     * the sequence properties.
     * TODO: structure setup process     
     *    
     * @param new_component the component to setup   
     */
    public void 
    setupComponentProperties(
        org.omg.Components.CCMObject new_component)
    throws ComponentPropertiesSetupFailureException 
    {
        try {
            java.io.InputStreamReader reader =
                getFileinarchiveDeployer().getInputStreamReader();

            // retrieving the properties object descriptor
            PropertiesBean propertiesObject =
                PropertiesBeanImpl.unmarshalBean(reader, true);
            //setup the simple properties
            setupSimpleProperty(propertiesObject, new_component);
            //setup the sequence properties
            setupSequenceListProperty(new_component, propertiesObject);
        } catch (Exception e) {
            e.printStackTrace();
            throw new ComponentPropertiesSetupFailureException(this, e);
        }
    }

}