/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Fr?d?ric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.installer;

import java.util.LinkedList;

import org.objectweb.openccm.descriptor.softpkg.Dependency;
import org.objectweb.openccm.descriptor.softpkg.ccm.CodeDeployer;
import org.objectweb.openccm.descriptor.softpkg.ccm.CodeInstallationFailureException;
import org.objectweb.openccm.descriptor.softpkg.ccm.DependencyDeployer;
import org.objectweb.openccm.descriptor.softpkg.ccm.DependencyResolutionFailureException;
import org.omg.Components.ConfigValue;
import org.omg.Components.ValueFactoryEntrypoint;
import org.omg.Components.valueFactoryEntrypointSeqHelper;

/**
 * The NodeDependencyVisitorDeployer implements a visitor able to process
 * dependency for node target and ECM taget. It also implements a visitor
 * for the codeDeployer.
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Fr?d?ric</a>
 *
 * @version 0.1
 */
public class NodeDependencyVisitorDeployer
       implements DependencyDeployerVisitor, 
                  CodeDeployerVisitor 
    {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    private java.util.List config_values;
    private String entryPoint;
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================       
    public NodeDependencyVisitorDeployer()
    {
        config_values=new LinkedList(); 
    }
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================
    /**
     * Visitor method for a dependency deployer.
     * 
     * @param dependencyDeployer the dependency to solve
     * @throws DependencyResolutionFailureException if the dependency 
     * cannot be solve.
     */
    public void 
    visit(DependencyDeployer dependencyDeployer)
    throws DependencyResolutionFailureException 
    {
        
        org.omg.Components.ConfigValue config = null;
        Dependency dep=dependencyDeployer.getDependency();
        if ((dep.getAction().compareToIgnoreCase("install") == 0)
               && (dep.getFileinarchive()!=null)
               && ((dep.getFileinarchive().getName()!=null)||dep.getFileinarchive().getLink()!=null)
               /*&& (dep.getName()!=null)
               && (dep.getName().getValue()!=null)*/
               )
               {  
                   // Set up the dependency as config value for home
                   // installation if the dependency is java classes
                   if (dep.getType().trim().
                       compareToIgnoreCase("Java Class") == 0)
                   {
                       config = new org.objectweb.openccm.ccm.Components.ConfigValueImpl();
                       config.name = "load_lib_dependency";
                       config.value = org.objectweb.openccm.corba.TheORB.getORB().create_any();
                       config.value.insert_string(/*dep.getName().getValue()*/dep.getFileinarchive().getName());
                       config_values.add(config);
                   }
               }
            if((dep.getAction().compareToIgnoreCase("install") == 0)
               &&(dep.getType().compareToIgnoreCase("HOME_CONFIG_VALUE")==0)
               &&(dep.getName()!=null)
               &&(dep.getName().getValue()!=null)
               &&(dep.getName().getValue().indexOf(":")!=-1))
               {
                   config = new org.objectweb.openccm.ccm.Components.ConfigValueImpl();
                   String configName=dep.getName()
                                       .getValue()
                                       .substring(0,dep.getName()
                                                       .getValue()
                                                       .indexOf(":"));

                   String configValue=dep.getName()
                                         .getValue()
                                         .substring(dep.getName()
                                                        .getValue()
                                                        .indexOf(":")+1,
                                                        dep.getName().getValue().length());

                   config.name = configName;
                   config.value = org.objectweb.openccm.corba.TheORB.getORB().create_any();
                   config.value.insert_string(configValue);
                   config_values.add(config);
               }
               
               if(dep.getValuetypefactory()!=null&&
                  dep.getValuetypefactory().getRepid()!=null&&
                  dep.getValuetypefactory().getValueentrypoint()!=null&&
                  dep.getValuetypefactory().getFactoryentrypoint()!=null&&
                  dep.getValuetypefactory().getFileinarchive()!=null)
                  {
                   config=new org.objectweb.openccm.ccm.Components.ConfigValueImpl();
                   config.name=org.omg.Components.VALUE_FACTORY_FEATURE_NAME.value;
                   config.value = org.objectweb.openccm.corba.TheORB.getORB().create_any();
                   ValueFactoryEntrypoint vtfe=new ValueFactoryEntrypoint();
                
                   vtfe.code_location="fileinarchive:"+dep.getValuetypefactory().getFileinarchive().getName();
                   vtfe.factory_entrypoint=dep.getValuetypefactory().getFactoryentrypoint();
                   vtfe.value_entrypoint=dep.getValuetypefactory().getValueentrypoint();
                   vtfe.repid=dep.getValuetypefactory().getRepid();
                   
                   ValueFactoryEntrypoint [] vts=new ValueFactoryEntrypoint[1];
                   vts[0]=vtfe;                                             
                   valueFactoryEntrypointSeqHelper.insert(config.value,vts);
                               
                   config_values.add(config);                                         
                  }                  
    }
    /**
     * Build an return an array of config value relative
     * to the dependency processing.
     * 
     * @return the built configvalues
     */
    public ConfigValue[]
    getConfigValue()
    {
        return (ConfigValue[])
                config_values
                .toArray(new ConfigValue[config_values.size()]);
    }
    
    /**
     * Code visitor to found entrypoint
     * 
     * @param CodeDeployer the codeDeploye to visit
     * @throws CodeInstallationFailureException if entrypoint not found
     */
    public void
    visit(CodeDeployer codeDeployer)
    throws CodeInstallationFailureException        
    {
        try{
            entryPoint=codeDeployer.getCode().getEntrypoint().getValue();
        }
        catch(Exception e){
            throw new CodeInstallationFailureException(null,e);
        }   
    }
    /**
     * Return the entryPoint found by the code visitor method.
     * 
     * @return The entrypoint found
     */    
    public String 
    getEntryPoint()
    {
        return entryPoint;
    }


}
