/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.installer;
//Import all required packages

import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base.DeployerActiveState;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base.DeployerInactiveState;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base.FatalDeploymentException;

/**
 * Process the processcollocation deployment.
 *
 * @author <a href="mailto:frederic.briclet@lifl.fr">Briclet Frederic</a>
 *
 * @version 0.1
 */
public class ProcesscollocationDeployer extends ProcesscollocationDeployerContext{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
     // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================


    /**
     * Deploy the processcollocation 
     * @throws FatalDeploymentException if fatal problem occured
     **/
    public void 
    deploy()
    throws FatalDeploymentException
    {
   
        traverse(this,getHomeplacementDeployers());       
    }
    
    /**
     * HomeplacementDeployer visitor to install a CCMHome
     * 
     * @param home The homeplacement to install
     * @param status The processcollocation current status 
     * @throws FatalDeploymentException if fatal problem occured
     */
    public void 
    visit(HomeplacementDeployer home,DeployerInactiveState status)
    throws FatalDeploymentException
    {
         home.deploy();
     }
   
   
    /**
     * TearDown method browse its homeplacement children. 
     * @throws FatalDeploymentException if fatal problem occured
     **/     
    public void 
    tearDown()
    throws FatalDeploymentException
    {
        traverse(this,getHomeplacementDeployers());
    }    
    /**
     * HomeplacementDeployer visitor to uninstall a CCMHome
     * 
     * @param home The homeplacement to install
     * @param status The processcollocation current status 
     * @throws FatalDeploymentException if fatal problem occured
     */    
    public void 
    visit(HomeplacementDeployer home,DeployerActiveState status)
    throws FatalDeploymentException
    {
        home.tearDown();
    }
}
