/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.registrar;

/**
 * Specific deployer allowing to register a corba object and to unbind it
 * @author <a href="mailto:frederic.briclet@lifl.fr">Briclet Frederic</a>
 *
 * @version 0.1
 */

public class RegisterwithnamingDeployer 
       extends RegisterwithnamingDeployerContext
    {
    // ==================================================================
    //
    // Internal state
    //
    // ==================================================================

    /** Namecomponent use to found object*/
    private org.omg.CosNaming.NameComponent [] ncomp;    
    /* Boolean indocating if the referencement have failed*/
    private boolean failed;
    /*reference on naming context*/
    private org.omg.CosNaming.NamingContextExt rootContext;
      
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Rebind method in charge to register the object give in parameter
     * with the name service. 
     *
     * @param objectToRebind the object to register with the name service
     * @throws NameserviceRegistrationFailureException if registration failed
     */
    public void
    rebind(org.omg.CORBA.Object objectToRebind)
    throws NameserviceRegistrationFailureException
    {

        /*Retrieve a NamingContextExt*/
        rootContext=getNamingcontextExt();
        try{
            //Invoque the rebind
            ncomp=rebind(rootContext,getRegisterwithnaming().getName(),objectToRebind);    
        }
        catch(Exception e)
            {
                failed=true;
                throw new NameserviceRegistrationFailureException(this,e," TO complete");
            }
        getLifeCycleManager().stepEndedSuccessFully();
    }

    /**
     * Static method with parameter available out the component assembly.
     * This method can be invoke by another object that deployer
     * 
     * @param rootContextExt the naming context to use for rebinding
     * @param name The rebinding name
     * @param objectToRebind The corba object to rebind
     */
    public static  org.omg.CosNaming.NameComponent []
    rebind(org.omg.CosNaming.NamingContextExt rootContextExt,
           String name,
           org.omg.CORBA.Object objectToRebind)
    throws Exception
    {
        String pathError="" ;
        org.omg.CosNaming.NameComponent [] nameComp=null;
        
        nameComp=rootContextExt.to_name(name);
        String nameTmp="";
        //ADD a loop to find a valid name based on the given name
        try{
            rootContextExt.resolve(nameComp);
            for(int i=0;true;i++)
            {
                nameTmp=name+"("+i+")";
                nameComp=rootContextExt.to_name(nameTmp);
                rootContextExt.resolve(nameComp);
            }
                
        }
        catch(org.omg.CosNaming.NamingContextPackage.NotFound nf )
        {
            
        }
        
        try{
            
            rootContextExt.rebind(nameComp,objectToRebind);            
        }
        catch(org.omg.CosNaming.NamingContextPackage.NotFound nf )
            {
                org.omg.CosNaming.NamingContext context=rootContextExt;

                org.omg.CosNaming.NameComponent [] ncompTmp= 
                    new org.omg.CosNaming.NameComponent[1];
               
                //All the context required doesn't exist   
                
                for(int i=0; i<nameComp.length-1; i++)
                    {
                        pathError+=nameComp[i].id+"/";
                        ncompTmp[0]=nameComp[i];
                    
                    try{
                        context=org.omg.CosNaming.
                            NamingContextHelper.narrow(context.resolve(ncompTmp));
                    }
                    catch(org.omg.CORBA.BAD_PARAM bp)
                        {
                            System.err.println("Invalid registeration name,"+pathError+
                                               " is already used as full path to register a corba object ");
                            System.err.println
                                ("The corba object cannot be registered, take care some connection can fail");
                            throw new org.omg.CosNaming.NamingContextPackage.CannotProceed();
                        }
                    catch(Exception e ){
                        context=context.bind_new_context(ncompTmp);                        
                    }
                            
                }
                ncompTmp[0]=nameComp[nameComp.length-1];
                context.rebind(ncompTmp,objectToRebind);
            }
        return nameComp;
    }

    /**
     * Unbind method which unregister from the name service the previous
     * registered object
     */
    public void
    unbind()
    throws NameserviceRegistrationFailureException
    {
        if(failed)
            return;
        try{
            org.objectweb.openccm.corba
                .TheNameService.getNamingContext()
                .getNamingContext().unbind(ncomp);    
        }
        catch(Exception e){
                throw new NameserviceRegistrationFailureException(this,e,"Error occur during unbinding");
        }
        getLifeCycleManager().stepEndedSuccessFully();             
    }
}
    
