/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.root;

import org.objectweb.openccm.descriptor.componentassembly.beans.ComponentassemblyBean;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base.*;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.binder.*;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.installer.*;

/**
 * The componentassembly deployer context defines all the connection requires for
 * the componentassembly processing. 
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class ComponentassemblyDeployerContext 
       extends RootDeployerContext 
    {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    /*All the deployer that can be accessed by the componentplacement deployer*/
    private ComponentfilesDeployer cfd;
    protected ComponentassemblyBean cad;
    private PartitioningDeployer part;
    private ConnectionsDeployer connect;
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    // ==================================================================
    //
    //  Public accessors methods.
    //
    // ==================================================================
    public void 
    connectComponentassembly(ComponentassemblyBean cad) 
    {
        this.cad = cad;
        connectDeployerDescription(cad);
    }
    
    protected ComponentassemblyBean    
    getComponentassembly(){
        return cad;
    }
    
    protected ComponentfilesDeployer 
    getComponentfilesDeployer() 
    {
        return cfd;
    }

    public void 
    connectComponentfilesDeployer(ComponentfilesDeployer cfd) 
    {
        this.cfd = cfd;
    }

    public void 
    connectPartitioningDeployer(PartitioningDeployer part) 
    {
        this.part = part;
    }
    
    public PartitioningDeployer 
    getPartitioningDeployer()
    {
        return part;
    }

    public void 
    connectConnectionsDeployer(ConnectionsDeployer connect) 
    {
        this.connect = connect;
    }

    public ConnectionsDeployer 
    getConnectionsDeployer() 
    {
        return connect;
    }
    
    /**
     * This initialization method is used to check that all the services and 
     * information required to deploy are presents. If all is OK the deployer 
     * is put in inactive deployer state waiting for the deployment processing.
     * 
     * @exception InitializationError is thrown if a problem is detected 
     */    
    public void 
    initialize() 
    throws InitializationError 
    {
        if(getComponentassembly()==null)       
            getErrorManager()
            .submitException(new InitializationError(this,"Connection on componentassembly bean is required"+
                             "check the handler code it must specify this connection"));   
        
        if(getPartitioningDeployer()==null)
            getErrorManager()
            .submitException(new InitializationError(this,"A partitioning child must be specified in componentassembly element,"+
                                           "check you componentassembly element description.\n"+
                                           getStringifiedDeployerDescription()));
                                           
        getLifeCycleManager().stepEndedSuccessFully();
    }
}
