/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.descriptor.softpkg.ccm;

import java.io.IOException;
import java.io.InputStream;
import java.util.LinkedList;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base.DeployerInactiveState;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.installer.DependencyDeployerVisitor;
import org.omg.Components.ConfigValue;
import org.omg.Components.Deployment.ComponentInstallation;

/**
 * The softpkg deployer root.
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class SoftpkgDeployer 
       extends SoftpkgDeployerContext 
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    private LinkedList configValues;
    private ComponentInstallation componentInstallation;
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================    
    public SoftpkgDeployer(){
        configValues=new LinkedList();
    }
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================
    /**
     * resolve all the sub dependencies and return the produce config value
     * @return the config value produce
     * @throws DependencyResolutionFailureException if the resolution failed
     */    
    public ConfigValue[] 
    processDependencies(ComponentInstallation componentInstallation)
    throws DependencyResolutionFailureException
    {   
     
        try{
            this.componentInstallation=componentInstallation;
            traverse(this,getDependencyDeployers());    
            return (ConfigValue[])configValues.toArray(new ConfigValue[configValues.size()]);
        }
        catch(Exception e){
           throw (DependencyResolutionFailureException)e;
        }
    }
    /**
     * The dependency visitor to resolve a dependency.
     * 
     * @param deployer The dependency deployer to execute
     * @param status The softpkg current status
     * @throws DependencyResolutionFailureException
     */
    public void 
    visit(DependencyDeployer deployer, DeployerInactiveState status)
    throws DependencyResolutionFailureException
    {        
        configValues.add(deployer.resolveDependency(componentInstallation));
    }
    /**
     * Return an input stream on the current component archive.
     * 
     * @return the input stream on the current archive
     * @throws IOException
     */
    public InputStream
    getComponentArchiveStream()
    throws IOException
    {
        return getZipEntryRetriever().getNewInputStream();            
    }
    
    /**
     * The dependency visitor method taking in parameter the visitor 
     * to execute on all the dependencies. 
     * @param depVisitor the visitor to use.
     * @throws DependencyResolutionFailureException
     */
    public void 
    visitDependencyDeployers(DependencyDeployerVisitor depVisitor)
    throws DependencyResolutionFailureException
    {
        DependencyDeployer depDepl[]=getDependencyDeployers();
        for(int i=0;i<depDepl.length;i++)
            depVisitor.visit(depDepl[i]);
    }
}
