/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.explorer.DCI;

import org.objectweb.openccm.OpenCCM_DCI.AssemblyManager;
import org.objectweb.openccm.OpenCCM_DCI.AssemblyFactoryManager;
import org.objectweb.openccm.OpenCCM_DCI.AssemblyFactoryManagerPackage.assembly_managersConnection;
import org.objectweb.util.explorer.api.Context;
import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.explorer.core.naming.lib.DefaultEntry;

/**
 * This context displays the <code>AssemblyFactory</code> and the <code>ComponentInstallation</code> registered in the given NodeManager.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class AssemblyFactoryManagerContext 
  implements Context 
{

    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    // ==================================================================
    //
    // No constructor.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Provides a name depending on the AssemblyManager id.
     */
    protected String getName(AssemblyManager assemblyManager){
        String returnedValue = "Assembly ";
        String assemblyId = assemblyManager.ass_id();
        if(assemblyId.indexOf(":")!=-1)
            returnedValue = assemblyId.substring(0,assemblyId.indexOf(":"));
        return returnedValue;
    }

    // ==================================================================
    //
    // Public methods for interface Context.
    //
    // ==================================================================

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Context#getEntries(java.lang.Object)
     */
    public Entry[] getEntries(Object object) {
        AssemblyFactoryManager assemblyFactoryManager = (AssemblyFactoryManager)object;
        
        java.util.List v = new java.util.Vector();

        assembly_managersConnection[] assemblyManagerList = assemblyFactoryManager.get_connections_assembly_managers();
        for (int i=0 ; i<assemblyManagerList.length ; i++){
            assembly_managersConnection assemblyManagerConnection = assemblyManagerList[i];
            AssemblyManagerWrapperWithCookie assemblyManagerWrapper = new AssemblyManagerWrapperWithCookie();
            assemblyManagerWrapper.setAssemblyFactory(assemblyFactoryManager.provide_assembly_factory());
            assemblyManagerWrapper.setCookie(assemblyManagerConnection.ck);
            assemblyManagerWrapper.setAssemblyManager(assemblyManagerConnection.objref);
            v.add(new DefaultEntry(getName(assemblyManagerConnection.objref)+ " (" + (i+1) + ")", assemblyManagerWrapper));
        }
        
        // Return the array of entry
        return (Entry[]) v.toArray(new Entry[0]);
    }

}
