@echo off
rem # ====================================================================
rem #
rem # OpenCCM: The Open CORBA Component Model Platform
rem # Copyright (C) 2000-2002 USTL - LIFL - GOAL
rem # Contact: openccm-team@objectweb.org
rem #
rem # This library is free software; you can redistribute it and/or
rem # modify it under the terms of the GNU Lesser General Public
rem # License as published by the Free Software Foundation; either
rem # version 2.1 of the License, or any later version.
rem #
rem # This library is distributed in the hope that it will be useful,
rem # but WITHOUT ANY WARRANTY; without even the implied warranty of
rem # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
rem # Lesser General Public License for more details.
rem #
rem # You should have received a copy of the GNU Lesser General Public
rem # License along with this library; if not, write to the Free Software
rem # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
rem # USA
rem #
rem # Initial developer(s): Briclet Frederic
rem # Contributor(s): Jerome Moroy------------------------.
rem #
rem # ====================================================================
rem #
rem #  Start the OpenCCM Java node.
rem #
rem # ====================================================================

setlocal

rem #  =====================
rem #  Compute Command Line
rem #  =====================

set /a nb_args=0
for %%i in (%*) do set /a nb_args=nb_args+1

if %nb_args% == 0 goto help

set /a nb=0
set with_npd_file="false"
set with_tracing="false"
set services_list=

:initial
if "%1"=="-h" goto :help
if "%1"=="?" goto :help
if "%1"=="-I" goto setNPD
if "%1" == "--tracing" goto tracing
if %nb% == %nb_args% (goto end_command_line) else (goto setName)


:setNPD
shift
set npdFilePath="-I"%1
set with_npd_file="true"
set /a nb=nb+1
shift
goto initial

:tracing
set with_tracing="true"
set services_list=%services_list% Tracing
shift
set /a nb=nb+1
goto initial

:setName
set node_name=%1
shift
set /a nb=nb+1
goto initial
    
:help
echo "usage: node_start  [-I npdFilePath] [--tracing] <node_name>"    
goto :eof

:end_command_line

:start
set WORK_DIR=%OpenCCM_CONFIG_DIR%\NodeManagers\%node_name%
set NODE_PATH=%OpenCCM_CONFIG_DIR%\NodeManagers\%node_name%\%node_name%



rem #  ============================
rem #  OpenCCM Platform installed ?
rem #  ============================

:platform_installed
call %OpenCCM_HOMEDIR%\bin\ccm_installed
if %CCM_INSTALLED%==0 goto :eof

rem #  =============
rem #  Node started ?
rem #  =============

rem # Check if this Java Component Server is already started.
if not exist %NODE_PATH%.PID goto :node_not_started

echo Error: The Component NodeManager %node_name% is already started!
goto :eof

:node_not_started

mkdir %NODE_PATH%\

rem #  ======================
rem #  Name Service started ?
rem #  ======================

rem # Obtain the IOR of the started Name Service.
call ns_ior > %NODE_PATH%.tmp
for /f %%i in (%NODE_PATH%.tmp) do set ior_NS=%%i
del %NODE_PATH%.tmp

rem # Check if a Name Service is started.
if defined ior_NS goto test_dci

echo Error: The Name Service is not started!
goto :eof


:test_dci
rem #  ======================
rem #  DCI started ?
rem #  ======================


rem # Obtain the IOR of the started DCI.
call dci_ior > %NODE_PATH%.tmp
for /f %%i in (%NODE_PATH%.tmp) do set ior_DCI=%%i
del %NODE_PATH%.tmp

rem # Check if a Name Service is started.
if defined ior_DCI goto test_node

echo Error: The DCI is not started!
goto :eof


:test_node



set ci_dir=%NODE_PATH%\ComponentInstallation
mkdir %ci_dir%


if %with_npd_file% == "false" goto start_without_npd

echo "User Node Properties File defined"
set jvm_args="-Dior_ns=%ior_NS% -Dior_dci=%ior_DCI% -Dwork_dir=%WORK_DIR%"
set run_id=node_start_user
goto service_tracing

:start_without_npd

echo No User Node Properties File defined, using default properties
copy  %OpenCCM_HOMEDIR%\xml\descriptor\dci\NodeProperties.npd %NODE_PATH%.npd
set npdFilePath=-I%OpenCCM_CONFIG_DIR%\NodeManagers\%node_name%\%node_name%.npd
set jvm_args="-Dior_ns=%ior_NS% -Dior_dci=%ior_DCI% -Dnode_name=%node_name% -Dci_dir=%ci_dir% -Dwork_dir=%WORK_DIR%"
set run_id=node_start_default


:service_tracing
if %with_tracing% == "false" goto run
set run_id=%run_id%_tracing

:run
set args="--runid %run_id% -arg NPDFILE=%npdFilePath% -arg NSREGISTER=OpenCCM/NodeManagers/%node_name% -arg NODENAME=%node_name%  file:%OpenCCM_HOMEDIR%\xml\launcher\node_server.xml"

set LAUNCHER_JVM_ARGS="%jvm_args%"
set LAUNCHER_ARGS="%args%"
call openccm_launcher fork %NODE_PATH%.output %NODE_PATH%.PID

echo The NodeManager %node_name% is started

