/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Offroy.
Contributor(s): ______________________________________.

====================================================================*/
/*
 * Created on 23 juin 2003 by jerome OFFROY (offroy@lifl.fr)
 *
 */
package org.omg.lifl.eclipse.plugin.utils;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.omg.lifl.eclipse.plugin.project.OpenCCM.MainPlugin;

/**
 * @author offroy
 *
 */
public class WorkBenchHelper {
    public static void log() {
        Shell shell = new Shell();
    
        MessageDialog.openInformation(
            shell,
            "PopUpMenu Plug-in",
            "Launch Shell was executed.\nOn ressource : \n"
                + "filename     = "
                + WorkBenchHelper.getSelectedRessourceFileName()
                + "\n"
                + "full filename = "
                + WorkBenchHelper.getSelectedRessourceFullFileName()
                + "\n"
                + "path          = "
                + WorkBenchHelper.getSelectedRessourcePath()
                + "\n"
                + "full path     = "
                + WorkBenchHelper.getSelectedRessourceFullPath()
                + "\n"
                + "extension    = "
                + WorkBenchHelper.getSelectedRessourceExtension()
                + "\n"
                );
    }

    protected static IResource getRessource() {
        Object selection = null;
        ISelection currentSelection =
                MainPlugin
                .getDefault()
                .getWorkbench()
                .getActiveWorkbenchWindow()
                .getSelectionService()
                .getSelection();
        if ((currentSelection != null)
            && (currentSelection instanceof IStructuredSelection))
            selection =
                ((IStructuredSelection) currentSelection).getFirstElement();
        IResource resource = null;
        if (selection instanceof IAdaptable)
            resource =
                (IResource) ((IAdaptable) selection).getAdapter(
                    IResource.class);
        return resource;
    }

    /**
     * @return a String representing the selected ressource 's 
     * in the current view of the Eclipse WorkBench
     * filename
     */
    public static String getSelectedRessourceFileName() {
        return getRessource().getFullPath().lastSegment();
    }

    /**
     * @return a String representing the selected ressource
     * in the current view of the Eclipse WorkBench
     * full filename in the File System
     */
    public static String getSelectedRessourceFullFileName() {
        return getRessource().getLocation().toOSString();
    }

    /**
     * @return a String representing the selected ressource 
     * in the current view of the Eclipse WorkBench
     * path relative to Workspace 
     */
    public static String getSelectedRessourcePath() {
        return getRessource().getFullPath().toOSString();
    }

    /**
     * @return a String representing the selected ressource
     * in the current view of the Eclipse WorkBench
     * full path in the File System
     */
    public static String getSelectedRessourceFullPath() {
        return getRessource().getLocation().removeLastSegments(1).toOSString();
    }

    /**
     * @return a String representing the selected ressource
     * in the current view of the Eclipse WorkBench
     * extension
     */
    public static String getSelectedRessourceExtension() {        
        return getRessource().getFileExtension();
    }

    /**
     * @return a String representing the selected ressource
     * in the current view of the Eclipse WorkBench
     * project name
     */
    public static String getSelectedRessourceProjectName() {        
        return getRessource().getProject().getName();
    }
    
    

}
