package ist.coach.coachEmfClientComponents.gui;

import javax.swing.table.AbstractTableModel;
import java.util.Vector;
import ist.coach.coachEmfServices.EmfBasicLog.LogRecord;
import ist.coach.coachEmfServices.EmfBasicLog.LogSeverity;
import java.text.DateFormat;
import org.omg.CORBA.Any;

class LogTableModel extends AbstractTableModel {

        final String[] columnNames = {" Time            ",
					" Log Type ",
					" Source Type     ",
					" Source          ",
					" Log Id      ",
					" Details "
		};

		Vector data;
		protected static int NUM_COLUMNS = 6;
		protected static int TIME_C		 = 0;

		protected static int EVENT_C	 = 1;
		protected static int SOURCE_T_C	 = 2;
		protected static int SOURCE_C	 = 3;
		protected static int ID_C 		 = 4;
		protected static int DETAILS_C	 = 5;

		private int nextEmptyRow = 0;
		private static int START_NUM_ROWS = 30; //20
		int numRows = 0;

        LogTableModel() {

			data = new Vector();
        }

        public int getColumnCount() {
            return columnNames.length;
        }

        public int getRowCount() {

			if (numRows < START_NUM_ROWS)
				return START_NUM_ROWS;
			else
            	return numRows;
        }

        public String getColumnName(int col) {

            return columnNames[col];
        }

        public Object getValueAt(int row, int col) {

			try {
				LogDisplayRecord logR = (LogDisplayRecord) data.elementAt(row);

				switch(col) {
				case 0:
					return logR.timeR;
				case 1:
					return logR.eventType;
				case 2:
					return logR.sourceClass;
				case 3:
					return logR.source;
				case 4:
					return logR.notifId;
				case 5:
					return logR.details;
				}
			}
			catch(Exception e) {}

            return new String();

        }

        /*
         * JTable uses this method to determine the default renderer/
         * editor for each cell.  If we didn't implement this method,
         * then the last column would contain text ("true"/"false"),
         * rather than a check box.
         */
        public Class getColumnClass(int c) {
            return getValueAt(0, c).getClass();
        }

        /*
         * Don't need to implement this method unless your table's
         * editable.
         */
        public boolean isCellEditable(int row, int col) {
                return false;
        }

	public synchronized void updateLogTable(String dateR, String eventType,
						String source, String sourceClass,
						String notifId, String details) {

		 // System.err.println("LogTableModel> updated called!!!!!");
			LogDisplayRecord logR = new LogDisplayRecord(dateR, eventType, source,
											sourceClass,notifId, details);

			data.addElement(logR);
			if (data.size() == data.capacity())
				data.ensureCapacity(data.size() + 100);

			numRows++;
			fireTableRowsInserted(nextEmptyRow, nextEmptyRow);
			nextEmptyRow++;
			// System.err.println("LogTableModel> updated returns....");
        }

		public synchronized void updateLogArchives(LogRecord[] records) {

		  DateFormat df = DateFormat.getDateTimeInstance(DateFormat.DEFAULT, DateFormat.DEFAULT);

		  java.util.Date log_record_date;
		  String log_record_date_str = new String();
		  String log_record_id = new String();
		  String log_record_source= new String();
		  String log_record_class= new String();
		  String log_record_type= new String();
		  String log_record_details= new String();

		  for (int i = 0; i < records.length; i++) {
			log_record_date = new java.util.Date(records[i].time);
			log_record_date_str = df.format(log_record_date);
			log_record_id = String.valueOf(records[i].id);
			log_record_details = records[i].info.extract_string();

			for(int j = 0; j < records[i].attr_list.length; j++) {

			  if (records[i].attr_list[j].name.equals("Source"))
			   log_record_source = records[i].attr_list[j].value.extract_string();
			  else
			  if (records[i].attr_list[j].name.equals("SourceClass"))
			    log_record_class = records[i].attr_list[j].value.extract_string();
			  else
			  if (records[i].attr_list[j].name.equals("EventType")) {

				log_record_type = GuiMessages.info_log;

				switch(records[i].attr_list[j].value.extract_short()) {
				 case LogSeverity._Error :
					log_record_type = GuiMessages.error_log;
					break;
			     case LogSeverity._FatalError :
					log_record_type = GuiMessages.fatal_error_log;
					break;
			    case LogSeverity._Warning :
					log_record_type = GuiMessages.warning_log;
					break;
			  };

			}
		  }
		  updateLogTable(log_record_date_str, log_record_type,
						log_record_source, log_record_class,
						log_record_id, log_record_details);
		}
	  }

		public synchronized void clear() {

			int oldNumRows = numRows;

			numRows = START_NUM_ROWS;
			data.removeAllElements();
			nextEmptyRow = 0;

			if (oldNumRows > START_NUM_ROWS)
				fireTableRowsDeleted(START_NUM_ROWS, oldNumRows - 1);

			fireTableRowsUpdated(0, START_NUM_ROWS - 1);
		}

        public void printDebugData() {
            int numRows = getRowCount();

			LogDisplayRecord record ;
            for (int i=0; i < numRows; i++) {
				System.err.println("------> LogDisplayRecord" + i + " <-----------");
				record = (LogDisplayRecord) data.get(i);
				record.print();
			}
        }

	private class LogDisplayRecord {

		String timeR = null;
		String notifId = null;
		String eventType = null;
		String source = null;
		String sourceClass = null;
		String details = null;

		LogDisplayRecord() {}
		LogDisplayRecord(String timeR, String eventType, String source,
					String sourceClass, String notifId, String details) {


			this.timeR = timeR;
			this.notifId = notifId;
			this.eventType = eventType;
			this.source = source;
			this.sourceClass = sourceClass;
			this.details = details;
		}

		void print() {
			System.err.println("Time =" + timeR +
								"\tEvent Type =" + eventType +
								"\tSource = " + source +
								"\tSourceClass = " + sourceClass +
								"\tId = " + notifId +
								"\tDetails = " + details);
		}

	}
}

