/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.explorer.plugin.java;

import org.objectweb.util.explorer.api.MenuItem;
import org.objectweb.util.explorer.api.MenuItemTreeView;
import org.objectweb.util.explorer.api.TreeView;
import org.objectweb.util.explorer.swing.gui.api.DialogAction;
import org.objectweb.util.explorer.swing.gui.api.DialogBox;
import org.objectweb.util.explorer.swing.gui.lib.DefaultDialogBox;
import org.objectweb.util.explorer.swing.gui.lib.LabelBox;

/**
 * This action's goal is to rename an Entry.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class RenameInitialEntryAction 
  implements MenuItem, DialogAction 
{

    //==================================================================
    //
    // Internal state.
    //
    //==================================================================

    /** The name of the entry. */
    protected LabelBox labelBox_ = null;
    
    /** The selected TreeView. */
    protected TreeView treeView_ = null;
    
    /** The current name. */
    protected String currentName_ = null;

    //==================================================================
    //
    // No constructor.
    // 
    //==================================================================

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================

    //==================================================================
    //
    // Public methods for MenuItem interface.
    //
    //==================================================================
    
    /* (non-Javadoc)
     * @see org.objectweb.util.browser.api.MenuItem#getStatus(org.objectweb.util.browser.api.TreeView)
     */
    public int getStatus(TreeView treeView) {
        return MenuItem.ENABLED_STATUS;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.browser.api.MenuItem#actionPerformed(org.objectweb.util.browser.api.MenuItemTreeView)
     */
    public void actionPerformed(MenuItemTreeView treeView) throws Exception {
        treeView_ = treeView;
        currentName_ = treeView.getSelectedEntry().getName().toString();
        
        DialogBox box = new DefaultDialogBox("Renames the entry");
        labelBox_ = new LabelBox("Entry name", currentName_);
        box.addElementBox(labelBox_);
        box.setValidateAction(this);
        box.show();
    }

    //==================================================================
    //
    // Public methods for DialogAction interface.
    //
    //==================================================================    

    /* (non-Javadoc)
     * @see org.objectweb.util.browser.gui.api.DialogAction#executeAction()
     */
    public void executeAction() throws Exception {
        treeView_.getTree().renameInitialEntry(currentName_, labelBox_.getLabel());
    }
}
