/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.explorer.plugin.java.map;

import org.objectweb.util.explorer.api.MenuItem;
import org.objectweb.util.explorer.api.MenuItemTreeView;
import org.objectweb.util.explorer.api.TreeView;

/**
 * This action clears all the elements of a <code>java.util.Map</code> object.
 * A <code>UnsupportedOperationException</code> may be thrown if the implementation 
 * of the map doesn't supported the requested operation. 
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * 
 * @version 0.1
 */
public class MapClearAction 
  implements MenuItem 
{

    //==================================================================
    //
    // No internal state.
    //
    //==================================================================

    //==================================================================
    //
    // No constructor.
    // 
    //==================================================================

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================

    //==================================================================
    //
    // Public methods for MenuItem.
    //
    //==================================================================

    /**
     * This item is enabled only if the map contains at least one entry. 
     */
    public int getStatus(TreeView treeView) 
    {
        // Obtains the selected map.
        java.util.Map map = (java.util.Map)treeView.getSelectedObject();

        // Item enabled if this map contains at least one entry.
        if (map.size()>0)
            return MenuItem.ENABLED_STATUS;
        return MenuItem.DISABLED_STATUS;
    }

    /**
     * Removes all the mappings from this map.
     */
    public void actionPerformed(MenuItemTreeView treeView) 
    throws Exception 
    {
        // Obtains the selected map.
        java.util.Map map = (java.util.Map)treeView.getSelectedObject();
        
        // Removes all entries from this map.
        map.clear();    
    }

}