/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.explorer.plugin.java.map;

import org.objectweb.util.explorer.api.Context;
import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.explorer.core.naming.lib.DefaultEntry;

/**
 * This class allows to browse a <code>java.util.Map</code> object.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * 
 * @version 0.1
 */
public class MapContext 
  implements Context 
{

    //==================================================================
    //
    // No internal state.
    //
    //==================================================================

    //==================================================================
    //
    // No constructor.
    // 
    //==================================================================

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================

    //==================================================================
    //
    // Public methods for Context interface.
    //
    //==================================================================

    /**
     * Returns an array of Entry representing the mappings contained in this map.
     * @return The entries of the context.
     */
    public Entry[] getEntries(Object value) 
    {
        java.util.Map map = (java.util.Map)value;

        // Creates the array of entries.
        Entry[] entries = new Entry[map.size()];
        
        // Populates the array with the mappings contained in the map.
        Object[] keys = map.keySet().toArray();
        Object[] elements = map.values().toArray();
        for (int i = 0; i < entries.length; i++) 
        {
            entries[i] = new DefaultEntry(keys[i].toString(), elements[i]);
        }
        return entries;
    }

}