/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.explorer.plugin.java.reflect;

import java.util.Iterator;
import java.util.Vector;

/**
 * 
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * 
 * @version 0.1
 */
public class ClassVisibilityConfig 
    extends Vector {

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    /** The <code>String</code> value representing an inner class. */
    public static final String INNER_CLASS = "inner_class";

    /** The <code>String</code> value representing an <code>attribute</code>. */
    public static final String ATTRIBUTE = "attribute";

    /** The <code>String</code> value representing a <code>method</code>. */
    public static final String METHOD = "method";
    
    /** The <code>String</code> value representing the <code>private</code> modifier. */
    public static final String PRIVATE = "private";
    
    /** The <code>String</code> value representing the <code>protected</code> modifier. */
    public static final String PROTECTED = "protected";
    
    /** The <code>String</code> value representing the <code>public</code> modifier. */
    public static final String PUBLIC = "public";
    
    /** The <code>String</code> value representing the <code>static</code> modifier. */
    public static final String STATIC = "static";
    
    /** The <code>String</code> value representing the <code>final</code> modifier. */
    public static final String FINAL = "final";
    
    /** The <code>String</code> value representing the <code>abstract</code> modifier. */
    public static final String ABSTRACT = "abstract";
    
    /** The <code>String</code> value representing the default modifier (neither public, nor protected, nor private). */
    public static final String PACKAGE = "package";
    
    /** The <code>String</code> value representing the tree inheritance visibility. */
    public static final String TREE_INHERITANCE = "tree-inheritance";

    /** The <code>String</code> value representing the tree inheritance view. */
    public static final String FLAT_VIEW = "flat-view";
    
    /** The <code>String</code> value representing the type of a field. */
    public static final String TYPE = "type";
    
    /** The <code>String</code> value indicating that the params of a method have to be displayed. */
    public static final String PARAMS = "params";
    
    /** The <code>String</code> value indicating that the return type of a method have to be displayed. */
    public static final String RETURN_TYPE = "return-type";
    
    /** The <code>String</code> value indicating that the declaring class of a method have to be displayed. */
    public static final String DECLARING_CLASS = "declaring-class";
    
    //==================================================================
    //
    // No constructor.
    // 
    //==================================================================

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================

    //==================================================================
    //
    // Surcharging Object methods.
    //
    //==================================================================

    public Object clone(){
        ClassVisibilityConfig classVisibilityConfig = new ClassVisibilityConfig();
        Iterator it = iterator();
        while (it.hasNext()) {
	        String element = (String) it.next();
            classVisibilityConfig.put(new String(element), true);
	    }
        return classVisibilityConfig; 
    }

    //==================================================================
    //
    // Public methods.
    //
    //==================================================================

    /**
     * Returns the value to which the specified key is mapped in this identity ClassVisibilityMap,
     * or false if the map contains no mapping for this key.
     * 
     * @param key The key whose associated value is to be returned.
     * 
     * @return The corresponding value. 
     */
    public boolean get(String key){
        return contains(key);
    }

    /**
     * Associates the specified value with the specified key in this map. 
     * If the map previously contained a mapping for this key, the old value is replaced.
     * 
     * @param key Key with which the specified value is to be associated.
     * @param value value to be associated with the specified key.
     */
    public void put(String key, boolean value){
        if(!value)
            remove(key);
        else if(!contains(key))
            add(key);    
    }
    
}

 