/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.explorer.plugin.java.reflect;

import java.lang.reflect.Field;

/**
 * Provides a representation of a <code>Field</code> depending on the user configuration.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * 
 * @version 0.1
 */
public class FieldUtil 
    extends MemberUtil {

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    /** The Field to manipulate */
    protected Field field_;

    //==================================================================
    //
    // Constructors.
    // 
    //==================================================================

    public FieldUtil(Field field){
        super();
        field_ = field;
    }

    public FieldUtil(Field field, ClassVisibilityConfig config) {
        super(config);
        field_ = field;
    }

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================

    //==================================================================
    //
    // Public methods.
    //
    //==================================================================

    /**
     * Provides an HTML representation of a Field
     */
    public String toHTML(){
        boolean displayType = config_.get(ClassVisibilityConfig.ATTRIBUTE + "." + ClassVisibilityConfig.TYPE);
        String visibility = getVisibility(field_.getModifiers());
        StringBuffer name = new StringBuffer((visibility!=null?"<i>"+visibility+"</i>" + " ":"") + "<b>" + field_.getName() + "</b>");  
        if(displayType)
            name.append(": " + ClassVisibility.getTypeName(field_.getType()));   
        return name.toString();    
    }

}


    