/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.explorer.api;

/**
 * <p>
 * Interface which must be implemented by plug-ins in order to 
 * execute action on a node during a Drag & Drop operation. <br>
 * In order to define a Drag&Drop action, you can have a look to the corresponding
 * <a href="explorer-dtd.html#drop-action">DTD element</a> or to 
 * an <a href="package-summary.html#xml">XML example.</a>
 * </p>
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * 
 * @version 0.2
 */
public interface DropAction 
{

    /**
     * Executes the action.
     * 
     * @param dropTreeView The current view on the tree.
     *   
     * @exception Exception The invocation may thrown every kind of exception.
     */
    public void 
    execute(DropTreeView dropTreeView) 
    throws Exception;
    
}