/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.core.code.lib;

import org.objectweb.util.explorer.ExplorerUtils;
import org.objectweb.util.explorer.core.code.api.CodeDescription;
import org.objectweb.util.explorer.core.common.api.Description;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class BasicCodeDescription 
  implements CodeDescription 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    /** The language in which the code has been written. */
    public String language_;
    
    /** The code to interpret. */
    public String code_;
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // No internal method.
    //
    // ==================================================================

    protected boolean equals(BasicCodeDescription codeDesc){
        return ExplorerUtils.compareObjects(language_, codeDesc.language_)
            && ExplorerUtils.compareObjects(code_, codeDesc.code_);
    }
    
    // ==================================================================
    //
    // Public methods for Description interface.
    //
    // ==================================================================        

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.api.Description#getDescriptionType()
     */
    public String getDescriptionType() {
        return Description.CODE_DESCRIPTION;
    } 

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.api.Description#isEmpty()
     */
    public boolean isEmpty() {
        return language_==null || language_.equals("") || code_==null || code_.equals("");
    }
    
    // ==================================================================
    //
    // Public methods for CodeDescription interface.
    //
    // ==================================================================    
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.code.api.CodeDescription#setLanguage(java.lang.String)
     */
    public void setLanguage(String language) {
        language_ = language;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.code.api.CodeDescription#getLanguage()
     */
    public String getLanguage() {
        return language_;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.code.api.CodeDescription#setCode(java.lang.String)
     */
    public void setCode(String code) {
        code_ = code;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.code.api.CodeDescription#getCode()
     */
    public String getCode() {
        return code_;
    }
    
    // ==================================================================
    //
    // Public methods surcharging Object class.
    //
    // ==================================================================    
    
    /**
     * Surcharging the Object.equals method.
     * @see Object#equals(java.lang.Object)
     */
    public boolean equals(Object o){
        if(o!=null && o instanceof BasicCodeDescription){
            return equals((BasicCodeDescription)o);
        }
        return false;
    }
   
    /**
     * Return a String representation of a BasicImageIcon 
     * @see Object#toString() 
     */    
    public String toString(){
        return "BasicCodeDescription["
            + "language=" + ExplorerUtils.toString(language_)  
            + ", code=" + ExplorerUtils.toString(code_)
            + "]";
    }        
    
}