/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.core.code.lib;

import org.objectweb.util.explorer.ExplorerUtils;
import org.objectweb.util.explorer.core.code.api.Code;
import org.objectweb.util.explorer.core.code.api.CodeConfiguration;
import org.objectweb.util.explorer.core.common.lib.ClassResolver;
import org.objectweb.util.trace.TraceSystem;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class JavaCode
  implements CodeConfiguration, Code 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    /** The class to instanciate. */
    protected Class theClass_ = null;
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // No internal method.
    //
    // ==================================================================
    
    protected boolean equals(JavaCode code){
        if(code!=null){
            Class codeClass = code.theClass_;
            if(theClass_!=null && codeClass!=null){
                return theClass_.getName().equals(codeClass.getName());
            }
        }
        return false;
    }
    
    // ==================================================================
    //
    // Public methods for CodeConfiguration interface.
    //
    // ==================================================================

    public void setCode(String code) {
        try {
            theClass_ = ClassResolver.resolve(code);
        } catch (ClassNotFoundException e) {
            TraceSystem.get("explorer").info(code + ": Class not found!");
        }
    }

    // ==================================================================
    //
    // Public methods for Code interface.
    //
    // ==================================================================

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.code.api.Code#isAssignableFrom(java.lang.Class)
     */
    public boolean isInstanceOf(Class c){
        if(c!=null && theClass_!=null)
            return c.isAssignableFrom(theClass_);
        return false;
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.code.api.Code#createInstance()
     */
    public Object createInstance() {
        if(theClass_!=null){
            try {
                return theClass_.newInstance();
            } catch (InstantiationException e) {
                TraceSystem.get("explorer").info(e.getMessage());
            } catch (IllegalAccessException e) {
                TraceSystem.get("explorer").info(e.getMessage());
            }
        }
        return null;
    }

    // ==================================================================
    //
    // Public methods surcharging Object class.
    //
    // ==================================================================    
    
    /**
     * Surcharging the Object.equals method.
     * @see Object#equals(java.lang.Object)
     */
    public boolean equals(Object o){
        if(o!=null && o instanceof JavaCode){
            return equals((JavaCode)o);
        }
        return false;
    }
   
    /**
     * Return a String representation of a BasicImageIcon 
     * @see Object#toString() 
     */    
    public String toString(){
        return "JavaCode[class=" + 
            ExplorerUtils.toString(theClass_) + "]";
    }    
        
}
