/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.core.common.api;


/**
 * This interface is used to compute a key according to several parameters.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public interface KeyProvider {

    /** The name of the fractal interface reference. */
    public static String KEY_PROVIDER = "key-provider";
        
    /**
     * This method is used to compute a key.
     * The expected parameter is an array of String elements wich are the following:
     * <ul>
     *   <li>The ID of the type system (i.e. Java).</li>
     *   <li>The ID of the type name (i.e. java.lang.Object).</li>
     *   <li>The ID of the role.</li>
     * </ul>
     * @param key The elements of the key.
     * @return The computed key.
     * @throws BadParamException When the given key parameter is not an array of String.
     */
    public Object computeKey(Object key) throws BadParamException;
    
}
