/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.core.common.lib;

import java.io.FileFilter;

/**
 * This class represents an Abstract file filter.
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public abstract class AbstractFileFilter 
           implements FileFilter 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================
    
    /** The array of file extensions. */
    protected String[] extensions_ = new String[0];

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default constructor
     */
    public AbstractFileFilter(String extension){
        if(extension!=null)
            extensions_ = new String[]{extension};
    }

    /**
     * Default constructor
     */
    public AbstractFileFilter(String[] extensions){
        if(extensions!=null)
            extensions_ = extensions;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Indicates if the given extension is contained in the given array of extensions
     */
    protected boolean contains(String[] extensions, String extension){
        if(extensions!=null && extension!=null){
            for(int i=0 ; i < extensions.length ; i++)
                if(extension.equalsIgnoreCase(extensions[i]))
                    return true;
        }
        return false;
    }

    // ==================================================================
    //
    // Public methods inherits from FileFilter class.
    //
    // ==================================================================
    
    /**
     * Whether the given file is accepted by this filter.
     */
    public boolean accept(java.io.File f) {
        if(extensions_!=null){
            String extension = f.getName().substring(f.getName().lastIndexOf('.') + 1);
            return contains(extensions_, extension) || f.isDirectory();
        }
        return false;
    }

    /**
     * The description of this filter. For example: "JPG and GIF Images"
     */
    public abstract String getDescription();

}
