/*====================================================================

GoTM: GoTM is an open Transaction Monitor
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: gotm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): .

---------------------------------------------------------------------
$Id: BindingFeature.java,v 1.1 2004/11/08 16:13:50 moroy Exp $
====================================================================*/

package org.objectweb.util.explorer.core.common.lib;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;


/**
 * Abstract implementation of a feature using client interfaces.
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 * @version 0.1
 */
public abstract class BindingFeature
              extends BasicLoggable
           implements BindingController 
{
    /** the list of bindings. */
    private Map fcBindings ;
    
    /**
     * @return Returns the fcBindings.
     */
    private Map getFcBindings() {
        return fcBindings;
    }
    
    /**
     * @param fcBindings The fcBindings to set.
     */
    private void setFcBindings(Map fcBindings) {
        this.fcBindings = fcBindings;
    }
    
    public BindingFeature() {
        setFcBindings(new HashMap());
    }
    
    /**
     * Checks if the interface name is allowed to be binded. 
     * @param name the interface to check.
     * @return the name of the allowed interface.
     * @throws NoSuchInterfaceException if the interface is unknown.
     */
    private String checkBinding(String name) 
        throws NoSuchInterfaceException 
    {
        String[] list = clientFc();
        for (int i=0 ; i < list.length ; i++)
            if (name.startsWith(list[i]))
                return name ;
        throw new NoSuchInterfaceException(name);
    }
    
    protected Object lookupFc(int index) {
        try {
            return lookupFc(listFc()[index]);
        } catch (NoSuchInterfaceException e) {
            return null;
        } catch (ArrayIndexOutOfBoundsException e2) {
            return null;
        }
    }
    
    public abstract String[] clientFc();
    
    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#listFc()
     */
    public String[] listFc() {
        return (String[]) getFcBindings().keySet().toArray(new String[0]);
    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
     */
    public Object lookupFc(String arg0) 
        throws NoSuchInterfaceException 
    {
        return getFcBindings().get(checkBinding(arg0));
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String, java.lang.Object)
     */
    public void bindFc(String arg0, Object arg1)
        throws NoSuchInterfaceException, 
               IllegalBindingException,
               IllegalLifeCycleException 
    {
        getFcBindings().put(checkBinding(arg0),arg1);
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
     */
    public void unbindFc(String arg0) 
        throws NoSuchInterfaceException,
               IllegalBindingException, 
               IllegalLifeCycleException 
    {
        getFcBindings().remove(checkBinding(arg0));
    }
}