/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.core.dnd.lib;

import org.objectweb.util.explorer.ExplorerUtils;
import org.objectweb.util.explorer.core.code.api.CodeDescription;
import org.objectweb.util.explorer.core.common.api.Description;
import org.objectweb.util.explorer.core.dnd.api.DnDDescription;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class BasicDnDDescription 
  implements DnDDescription 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================
    
    /** The type of the action (MOVE, COPY or LINK). */
    protected String actionType_;
    
    /** The label of the action. */
    protected String label_;
    
    /** The code of the action. */
    protected CodeDescription codeDesc_;
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Two BasicDnDDescription are considered as equals if and only if 
     * they have the same label and the same CodeDescription.
     */     
    protected boolean equals(BasicDnDDescription dndDesc){
        return ExplorerUtils.compareObjects(label_, dndDesc.label_)
            && ExplorerUtils.compareObjects(codeDesc_, dndDesc.codeDesc_);
    }
    
    // ==================================================================
    //
    // Public methods for Description interface. 
    //
    // ==================================================================    
        
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.api.Description#getDescriptionType()
     */
    public String getDescriptionType() {
        return Description.DND_DESCRIPTION;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.api.Description#isEmpty()
     */
    public boolean isEmpty() {
        return label_==null || label_.equals("") || codeDesc_==null || codeDesc_.isEmpty();
    }
        
    // ==================================================================
    //
    // Public methods for DnDDescription interface. 
    //
    // ==================================================================    
        
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.dnd.api.DnDDescription#setType(int)
     */
    public void setType(String actionType) {
        actionType_ = actionType;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.dnd.api.DnDDescription#getType()
     */
    public String getType() {
        return actionType_;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.dnd.api.DnDDescription#setLabel(java.lang.String)
     */
    public void setLabel(String label) {
        label_ = label;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.dnd.api.DnDDescription#getLabel()
     */
    public String getLabel() {
        return label_;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.dnd.api.DnDDescription#setCodeDescription(org.objectweb.util.explorer.core.code.api.CodeDescription)
     */
    public void setCodeDescription(CodeDescription codeDesc) {
        codeDesc_ = codeDesc;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.dnd.api.DnDDescription#getCodeDescription()
     */
    public CodeDescription getCodeDescription() {
        return codeDesc_;
    } 
   
    // ==================================================================
    //
    // Public methods surcharging Object class.
    //
    // ==================================================================    
   
    /**
     * Surcharging the Object.equals method.
     * @see DefaultKey#equals(DefaultKey)
     */    
    public boolean equals(Object o){
        if(o!=null && o instanceof BasicDnDDescription)
            return equals((BasicDnDDescription)o);
        return false;
    }
    
    /**
     * Return a String representation of a BasicItemDescription  
     */ 
    public String toString(){
        return "BasicDnDDescription[" +
                "type=" + actionType_ +
                ", label=" + ExplorerUtils.toString(label_) +
                ", codeDesc=" + ExplorerUtils.toString(codeDesc_) +
                "]";
    }  

}
