/*====================================================================

Objectweb Browser framework
Copyright (C) 2000-2003 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.explorer.core.dnd.lib;

import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.trace.TraceSystem;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * 
 * @version 0.1
 */
public class EntryTransferable 
    implements Transferable, ClipboardOwner {

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    /** The flavors representing the TreeView class */
    // 
    // new DataFlavor(TreeView.class,"treeView")
    public static final DataFlavor[] flavors_ =  getDataFlavor();
    
    /** A list representing the flavors of the TreeView class */
    protected final List flavorList_ = Arrays.asList(flavors_);
    
    /** The Entry to transfer */
    protected Entry entryView_;

    //==================================================================
    //
    // No constructor.
    // 
    //==================================================================

    public EntryTransferable(Entry entryView){
        entryView_ = entryView;
    }

    //==================================================================
    //
    // Internal methods.
    //
    //==================================================================

    protected static DataFlavor[] getDataFlavor(){
        try{
            return new DataFlavor[]{new DataFlavor(DataFlavor.javaJVMLocalObjectMimeType + ";class=org.objectweb.util.explorer.api.Entry")};
        } catch(ClassNotFoundException e){
            TraceSystem.get("explorer").warn("[EntryTransferable] org.objectweb.util.explorer.api.Entry: Class Not Found!");
            return null;
        }
    }

    //==================================================================
    //
    // Public methods for Transferable.
    //
    //==================================================================

    /* 
     * @see java.awt.datatransfer.Transferable#getTransferDataFlavors()
     */
    public DataFlavor[] getTransferDataFlavors() {
        return flavors_;
    }

    /* 
     * @see java.awt.datatransfer.Transferable#isDataFlavorSupported(java.awt.datatransfer.DataFlavor)
     */
    public boolean isDataFlavorSupported(DataFlavor flavor) {
        return flavorList_.contains(flavor);
    }

    /* 
     * @see java.awt.datatransfer.Transferable#getTransferData(java.awt.datatransfer.DataFlavor)
     */
    public Object getTransferData(DataFlavor flavor) throws UnsupportedFlavorException, IOException {
        return entryView_;
    }

    //==================================================================
    //
    // Public methods for ClipboardOwner.
    //
    //==================================================================

    /* 
     * @see java.awt.datatransfer.ClipboardOwner#lostOwnership(java.awt.datatransfer.Clipboard, java.awt.datatransfer.Transferable)
     */
    public void lostOwnership(Clipboard clipboard, Transferable contents) {
        System.out.println ("EntryTransferable losts ownership of "  + clipboard.getName());
        System.out.println ("data: " + contents); 
    }

}