/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.core.icon.lib;

import java.net.URL;

import org.objectweb.util.explorer.ExplorerUtils;
import org.objectweb.util.explorer.core.icon.api.IconFileDescription;

/**
 * Basic implementation of the <code>IconFileDescription</code> interface.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class BasicIconFileDescription 
     extends BasicIconDescription
  implements IconFileDescription 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    /** The URL of the image file. */
    protected URL url_;
        
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Compares two BasicIconFileDescription objects. 
     * Two BasicIconFileDescription objects are considered as equal 
     * only if they have the same URL (not null).
     */
    protected boolean equals(BasicIconFileDescription iconFileDescription){
        return ExplorerUtils.compareObjects(url_, iconFileDescription.url_);
    }    

    // ==================================================================
    //
    // Public methods for Description interface.
    //
    // ==================================================================
    
    /* (non-Javadoc)
     * @see boolean#isNull()
     */
    public boolean isEmpty() {
        return url_==null;
    }    
     
    // ==================================================================
    //
    // Public methods for ... interface.
    //
    // ==================================================================
        
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.icon.api.IconFileDescription#setURL(java.net.URL)
     */
    public void setURL(URL url) {
        url_ = url;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.icon.api.IconFileDescription#getURL()
     */
    public URL getURL() {
        return url_;
    }

    // ==================================================================
    //
    // Public methods surcharging Object class.
    //
    // ==================================================================    
    
    /**
     * Surcharging the Object.equals method.
     * @see Object#equals(java.lang.Object)
     */
    public boolean equals(Object o){
        if(o!=null && o instanceof BasicIconFileDescription){
            return equals((BasicIconFileDescription)o);
        }
        return false;
    }
   
    /**
     * Return a String representation of a BasicImageIcon 
     * @see Object#toString() 
     */    
    public String toString(){
        return "BasicIconFileDescription[url=" + 
            ExplorerUtils.toString(url_) + "]";
    }    
    
}


