/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.core.panel.lib;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.objectweb.util.explorer.core.panel.api.CompositePanelDescription;
import org.objectweb.util.explorer.core.panel.api.PanelDescription;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class BasicCompositePanelDescription 
     extends AbstractPanelDescription
  implements CompositePanelDescription 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    protected List panels_ = null;
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    public BasicCompositePanelDescription(){
        panels_ = new Vector();
    }
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for CompositePanelDescription interface. 
    //
    // ==================================================================    
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.panel.api.PanelDescription#getPanelType()
     */
    public String getPanelType() {
        return COMPOSITE_PANEL;
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.panel.api.CompositePanelDescription#addPanelDescription(org.objectweb.util.explorer.core.panel.api.PanelDescription)
     */
    public void addPanelDescription(PanelDescription panelDesc) {
        if(panelDesc!=null && !panelDesc.isEmpty()){
            if(panelDesc.inheritTypePanel())
                inheritTypePanel_ = true;
            try{
                CompositePanelDescription source = (CompositePanelDescription)panelDesc;
                addPanelDescriptions(source.getPanelDescriptions());
            } catch(ClassCastException e){
                panels_.add(panelDesc);
            }
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.panel.api.CompositePanelDescription#addPanelDescriptions(org.objectweb.util.explorer.core.panel.api.PanelDescription[])
     */
    public void addPanelDescriptions(PanelDescription[] panelDescs) {
        if(panelDescs != null){
            for (int i = 0; i < panelDescs.length; i++) {
                addPanelDescription(panelDescs[i]);
            }
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.panel.api.CompositePanelDescription#getPanelDescriptions()
     */
    public PanelDescription[] getPanelDescriptions() {
        return (PanelDescription[])panels_.toArray(new PanelDescription[panels_.size()]);
    }

    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.api.Description#isEmpty()
     */
    public boolean isEmpty() {
        return panels_.isEmpty();
    }
    
    /**
     * Return a String representation of a BasicPanelDescription  
     */ 
    public String toString(){
        StringBuffer buf = new StringBuffer();
        buf.append("BasicCompositePanelDescription[panels=");
        Iterator it = panels_.iterator();
        while (it.hasNext()) {
            buf.append(it.next() + ", ");
        }
        buf.append("]");
        return buf.toString();
    }     
}


