/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.interpreter.lib.swt;

import java.net.URL;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.util.explorer.api.IconProvider;
import org.objectweb.util.explorer.api.MenuItemTreeView;
import org.objectweb.util.explorer.api.TreeView;
import org.objectweb.util.explorer.core.code.api.Code;
import org.objectweb.util.explorer.core.code.api.CodeProvider;
import org.objectweb.util.explorer.core.common.api.Description;
import org.objectweb.util.explorer.core.icon.api.IconDescription;
import org.objectweb.util.explorer.core.menu.api.AcceleratorDescription;
import org.objectweb.util.explorer.core.menu.api.ItemDescription;
import org.objectweb.util.explorer.core.menu.api.MenuDescription;
import org.objectweb.util.explorer.core.menu.api.MenuElement;
import org.objectweb.util.explorer.core.menu.api.MenuSeparator;
import org.objectweb.util.explorer.interpreter.api.DescriptionInterpreter;
import org.objectweb.util.explorer.interpreter.lib.AbstractDescriptionInterpreter;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public abstract class AbstractMenuInterpreter 
     extends AbstractDescriptionInterpreter 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    protected final String ICON_INTERPRETER = "icon-interpreter";
        
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Provides the icon for the given icon description 
     * applied to the given object.
     */
    protected Image getIcon(IconDescription iconDesc, TreeView treeView){
        if(iconDesc!=null){
            IconProvider iconProvider = (IconProvider) getIconInterpreter().interprete(iconDesc, treeView);
            if(iconProvider!=null){
                Object object = iconProvider.newIcon(treeView.getSelectedObject());
                if(object instanceof Image){
                    return (Image)object;
                } else if(object instanceof String) {
                    return (Image)(new IconFileProvider((String)object)).newIcon(treeView.getSelectedObject());
                } else if (object instanceof URL) {
                    return (Image)(new IconFileProvider((URL)object)).newIcon(treeView.getSelectedObject());
                }
            }
        }
        return null;
    }

    protected int getAccelerator(AcceleratorDescription accDesc){
        int acc = 0;
        if(accDesc!=null && !accDesc.isEmpty()){
            if (accDesc.getCtrlKey()) acc+=SWT.CTRL;
            if (accDesc.getAltKey()) acc+=SWT.ALT;
            if (accDesc.getShiftKey()) acc+=SWT.SHIFT;
            acc += accDesc.getAcceleratorCharacter();    
        }
        return acc;
    }
    
    protected String getAcceleratorName(AcceleratorDescription accDesc){
        if(accDesc!=null && !accDesc.isEmpty()){
            return accDesc.getName();
        }
        return null;
    }
    
    /**
     * Provides the Java/SWT item defined by the given item description.
     */
    protected void addMenuItem(Menu menu, ItemDescription itemDesc, MenuItemTreeView treeView){
        if(itemDesc!=null && !itemDesc.isEmpty()){
            Code actionCode = getCodeProvider().getCode(itemDesc.getCodeDescription());
            if(actionCode!=null && actionCode.isInstanceOf(org.objectweb.util.explorer.api.MenuItem.class)){
                org.objectweb.util.explorer.api.MenuItem action = (org.objectweb.util.explorer.api.MenuItem)actionCode.createInstance();
                int actionStatus = action.getStatus(treeView);
                if(actionStatus!=org.objectweb.util.explorer.api.MenuItem.NOT_VISIBLE_STATUS){
                    MenuItem menuItem = new MenuItem(menu, SWT.PUSH);
                    String acceleratorName = getAcceleratorName(itemDesc.getAcceleratorDescription());
                    if(acceleratorName!=null && !acceleratorName.equals("")){
                        menuItem.setText(itemDesc.getLabel() + "\t" + acceleratorName);
                        menuItem.setAccelerator(getAccelerator(itemDesc.getAcceleratorDescription()));
                    } else {
                        menuItem.setText(itemDesc.getLabel());
                    }
                    menuItem.addListener(SWT.Selection, new GenericAction(action, treeView));
                    menuItem.setEnabled(actionStatus!=org.objectweb.util.explorer.api.MenuItem.DISABLED_STATUS);
                    menuItem.setImage(getIcon(itemDesc.getIconDescription(), treeView));
                }
            } else {
                getTrace().warn(itemDesc.getCodeDescription().getCode() + " is not a MenuItem instance!");
            }
        }
    }
    
    /**
     * Provides a Java/SWT menu separator 
     */
    protected void addMenuSeparator(Menu menu){
        MenuItem separator = new MenuItem(menu, SWT.SEPARATOR);
    }
    
    protected Menu createMenu(MenuDescription menuDesc, MenuItemTreeView treeView){
        Menu menu = null;
        if(menuDesc!=null && !menuDesc.isEmpty()){
            menu = getMenu();
            MenuElement[] menuElement = menuDesc.getAllMenuElements();
            for (int i = 0; i < menuElement.length; i++) {
                MenuElement element = menuElement[i];
                if(element instanceof ItemDescription){
                    addMenuItem(menu, (ItemDescription)element, treeView); 
                } else if(element instanceof MenuSeparator){
                    addMenuSeparator(menu);
                }
            }
        }
        return menu;
    }

    protected DescriptionInterpreter getIconInterpreter(){
        try {
            return (DescriptionInterpreter)lookupFc(ICON_INTERPRETER);
        } catch (NoSuchInterfaceException e) {
            getTrace().warn(ICON_INTERPRETER + ": interface not found!");
            return null;
        }
    }    
    
    // ==================================================================
    //
    // Public methods for BindingFeature abstract class. 
    //
    // ==================================================================     
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.lib.BindingFeature#clientFc()
     */
    public String[] clientFc() {
        return new String[]{CodeProvider.CODE_PROVIDER, ICON_INTERPRETER};
    }

    // ==================================================================
    //
    // Public methods for DescriptionInterpreter interface. 
    //
    // ==================================================================    
        
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.interpreter.api.DescriptionInterpreter#interprete(org.objectweb.util.explorer.core.common.api.Description, org.objectweb.util.explorer.api.TreeView)
     */
    public Object interprete(Description description, TreeView treeView) {
        MenuDescription menuDesc = (MenuDescription)description;
        return createMenu(menuDesc, (MenuItemTreeView)treeView);
    }

    // ==================================================================
    //
    // Public abstract methods. 
    //
    // ==================================================================    
    
    public abstract Menu getMenu();
}
