/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.parser.lib;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.util.explorer.core.common.api.ExplorerConstants;
import org.objectweb.util.explorer.core.common.lib.BindingFeature;
import org.objectweb.util.explorer.explorerConfig.Icon;
import org.objectweb.util.explorer.explorerConfig.Node;
import org.objectweb.util.explorer.parser.api.NodeParser;
import org.objectweb.util.explorer.property.api.PropertyFeeder;

/**
 * This component manages the "icon" XML element.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class IconManager 
     extends BindingFeature 
  implements NodeParser 
{
    
    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    /** The feeder to use. */
    protected PropertyFeeder feeder_ = null;
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // No internal method.
    //
    // ==================================================================

    protected PropertyFeeder getFeeder(){
        if(feeder_==null) {
            try {
                feeder_ = (PropertyFeeder)lookupFc(PropertyFeeder.PROPERTY_FEEDER);
            } catch (NoSuchInterfaceException e) {
                e.printStackTrace();
            }
        }
        return feeder_;
    }
    
    // ==================================================================
    //
    // Public methods for BindingFeature abstract class.
    //
    // ==================================================================
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.lib.BindingFeature#clientFc()
     */
    public String[] clientFc() {
        return new String[]{PropertyFeeder.PROPERTY_FEEDER};
    }

    // ==================================================================
    //
    // Public methods for NodeParser interface.
    //
    // ==================================================================
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.parser.api.NodeParser#parseNode(java.lang.Object, org.objectweb.util.explorer.explorerConfig.Node)
     */
    public void parseNode(Object key, Node node) {
        Icon icon = node.getIcon();
        if(icon!=null){
            getFeeder().feed(ExplorerConstants.ICON_PROPERTY, key, ParserUtils.getIconDescription(icon));
        }
    }

}
