/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.parser.lib;

import java.util.Iterator;
import java.util.List;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.util.explorer.core.common.lib.BindingFeature;
import org.objectweb.util.explorer.explorerConfig.Include;
import org.objectweb.util.explorer.explorerConfig.beans.ExplorerBean;
import org.objectweb.util.explorer.parser.api.ExplorerParser;
import org.objectweb.util.explorer.parser.api.ParserConfiguration;

/**
 * This component manages the "include" XML element.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class IncludeManager 
     extends BindingFeature 
  implements ExplorerParser 
{

    //==================================================================
    //
    // No Internal State.
    //
    // ==================================================================

    // ==================================================================
    //
    // No Constructor.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /** Returns the ParserConfiguration interface. */
    protected ParserConfiguration getParserConfiguration(){  
        try {
            return (ParserConfiguration)lookupFc(ParserConfiguration.PARSER_CONFIGURATION);
        } catch (NoSuchInterfaceException e) {
            return null;
        }
    }
    
    // ==================================================================
    //
    // Public methods surcharging BindingFeature class.
    //
    // ==================================================================    

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.lib.BindingFeature#clientFc()
     */
    public String[] clientFc() {
        return new String[]{ParserConfiguration.PARSER_CONFIGURATION};
    }
    
    // ==================================================================
    //
    // Public methods for BrowserParser interface.
    //
    // ==================================================================    

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.parser.api.ExplorerParser#parseExplorer(org.objectweb.util.explorer.explorerConfig.beans.ExplorerBean)
     */
    public void parseExplorer(ExplorerBean explorer) {
        List theIncludes = explorer.getIncludeList();
        Iterator it = theIncludes.iterator();
        while (it.hasNext()) {
            Include element = (Include) it.next();
            getParserConfiguration().addPropertyFile(element.getUrl());
        }
    }

}