/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.parser.lib;

import java.util.Iterator;
import java.util.List;

import org.objectweb.util.explorer.core.common.api.ExplorerConstants;
import org.objectweb.util.explorer.core.panel.api.CompositePanelDescription;
import org.objectweb.util.explorer.core.panel.api.PanelDescription;
import org.objectweb.util.explorer.core.panel.api.TableDescription;
import org.objectweb.util.explorer.core.panel.lib.BasicCompositePanelDescription;
import org.objectweb.util.explorer.core.panel.lib.BasicPanelDescription;
import org.objectweb.util.explorer.core.panel.lib.BasicTableDescription;
import org.objectweb.util.explorer.explorerConfig.Node;
import org.objectweb.util.explorer.explorerConfig.Panel;
import org.objectweb.util.explorer.explorerConfig.Table;
import org.objectweb.util.explorer.explorerConfig.beans.PanelBean;

/**
 * This component manages the "panel" XML element.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class PanelManager 
     extends AbstractNodeParser 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    protected PanelDescription getPanelDescription(Table table){
       TableDescription tableDescription = null;
       if(table!=null && table.getCode()!=null){
           tableDescription = new BasicTableDescription();
           tableDescription.setCodeDescription(ParserUtils.getCodeDescription(table.getCode()));
        }
       return tableDescription;
    }
    
    protected PanelDescription getPanelDescription(Panel panel){
        PanelDescription panelDesc = null;
        if(panel!=null){
            if(panel.getCode()!=null){
                panelDesc = new BasicPanelDescription();
                panelDesc.setCodeDescription(ParserUtils.getCodeDescription(panel.getCode()));
            } else {
                panelDesc = new BasicCompositePanelDescription();
                CompositePanelDescription compositePanel = (CompositePanelDescription)panelDesc;
                List subPanels = ((PanelBean)panel).getChildrenList();
                Iterator it = subPanels.iterator();
                while (it.hasNext()) {
                    Object element = it.next();
                    if(Panel.class.isAssignableFrom(element.getClass())){
                        compositePanel.addPanelDescription(getPanelDescription((Panel)element));
                    } else if (Table.class.isAssignableFrom(element.getClass())){
                        compositePanel.addPanelDescription(getPanelDescription((Table)element));
                    }
                }
            }
            panelDesc.setInheritTypePanel(panel.getInheritTypePanel().equals("false")?false:true);
        }
        return panelDesc;
    }
    
    // ==================================================================
    //
    // Public methods for NodeParser interface. 
    //
    // ==================================================================    
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.parser.api.NodeParser#parseNode(java.lang.Object, org.objectweb.util.explorer.explorerConfig.Node)
     */
    public void parseNode(Object key, Node node) {
        PanelDescription panelDesc = getPanelDescription(node.getPanel());
        if(panelDesc!=null && !panelDesc.isEmpty()){
            getFeeder().feed(ExplorerConstants.PANEL_PROPERTY, key, panelDesc);
        }
    }

}


