/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.property.lib;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.util.explorer.api.IconProvider;
import org.objectweb.util.explorer.core.common.api.Description;
import org.objectweb.util.explorer.core.icon.api.IconCodeDescription;
import org.objectweb.util.explorer.core.icon.api.IconDescription;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class IconProperty 
     extends AbstractProperty 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    /** The data structure containing the mappings. */
    protected Map properties_ = null;    
        
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Empty constructor
     */
    public IconProperty(){
        properties_ = new HashMap();
    }
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for PropertyFeeder interface. 
    //
    // ==================================================================        
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.property.api.PropertyFeeder#feed(java.lang.String, java.lang.Object, org.objectweb.util.explorer.core.common.api.Description)
     */
    public void feed(String propertyType, Object key, Description value) {
        IconDescription iconDesc = (IconDescription)value;
        if(iconDesc!=null && !iconDesc.isEmpty()){
            if(iconDesc instanceof IconCodeDescription){
                IconCodeDescription iconCodeDesc = (IconCodeDescription)iconDesc;
                if(getCodeProvider().getCode(iconCodeDesc.getCode()).isInstanceOf(IconProvider.class)){
                    properties_.put(key,value);
                } else {
                    getTrace().warn(iconCodeDesc.getCode() + " is not assignable from IconProvider!");
                }
            } else {
                properties_.put(key,value);
            }
        }
    }
    
    // ==================================================================
    //
    // Public methods for PropertyProvider interface. 
    //
    // ==================================================================        
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.property.api.PropertyProvider#getPropertyDescription(java.lang.String, java.lang.Object)
     */
    public Description getPropertyDescription(String propertyType, Object key) {
        return (Description)properties_.get(key);
    }

}


