/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.property.lib;

import org.objectweb.util.explorer.core.common.api.Description;
import org.objectweb.util.explorer.core.menu.api.MenuDescription;
import org.objectweb.util.explorer.core.menu.api.MenuElement;

/**
 * Implementation of the MenuProperty fractal component.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class MenuProperty
     extends AbstractProperty
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Internal method.
    //
    // ==================================================================

    /**
     * Merges the two menu definition
     */
    protected void mergeMenuDescription(MenuDescription source, MenuDescription target) {
        MenuElement[] elements = target.getAllMenuElements();
        for (int i = 0; i < elements.length; i++) {
            source.addMenuElement(elements[i]);
        }
    }
    
    // ==================================================================
    //
    // Public methods ExplorerPropertyFeeder interface. 
    //
    // ==================================================================    

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.api.ExplorerPropertyFeeder#feed(int, java.lang.Object, java.lang.Object)
     */
    public void feed(String propertyType, Object key, Description value) {
        if(value!=null && value instanceof MenuDescription){
            if(properties_.containsKey(key)){
                Description description = (Description)properties_.get(key);
                mergeMenuDescription((MenuDescription)description, (MenuDescription)value);
            } else {
                properties_.put(key,value);
            }
        }
    }

    // ==================================================================
    //
    // Public methods PropertyProvider interface. 
    //
    // ==================================================================    
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.api.PropertyProvider#getPropertyDescription(int, java.lang.Object)
     */
    public Description getPropertyDescription(String propertyType, Object key) {
        return (Description)properties_.get(key);
    }

}

