/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.resolver.api;

import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.explorer.core.common.api.Description;

/**
 * This interface is used to resolve a property for an entry.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public interface PropertyResolver {

    /** The name of the fractal interface reference. */
    public static String PROPERTY_RESOLVER = "property-resolver";    

    /**
     * This interface is used to provide a description of the given property type
     * for the given entry.
     * 
     * This method doesn't modify the given parameters. 
     * 
     * @param propertyType The type of property. The available types are:
     * <ul>
     *   <li>ExplorerConstants.ICON_PROPERTY for defining icon.</li>
     *   <li>ExplorerConstants.MENU_PROPERTY for defining menu.</li>
     *   <li>ExplorerConstants.INFO_PROPERTY for defining info.</li>
     * </ul>
     * @param currentEntry The current entry.
     * @param parentEntry The parent entry of the given entry.
     *
     * @return The associated description or null if no description is found.
     */
    public Description resolve(String propertyType, Entry currentEntry, Entry parentEntry);
        
    /**
     * This interface is used to provide a description of the given property type
     * for the given entry.
     * 
     * This method doesn't modify the given parameters. 
     * 
     * @param propertyType The type of property. The available types are:
     * <ul>
     *   <li>ExplorerConstants.ICON_PROPERTY for defining icon.</li>
     *   <li>ExplorerConstants.MENU_PROPERTY for defining menu.</li>
     *   <li>ExplorerConstants.INFO_PROPERTY for defining info.</li>
     * </ul>
     * @param currentEntry The current entry.
     *
     * @return The associated description or null if no description is found.
     */
    public Description resolve(String propertyType, Entry currentEntry);
    
}
