/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.resolver.lib;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.explorer.core.common.api.Description;
import org.objectweb.util.explorer.core.common.lib.BindingFeature;
import org.objectweb.util.explorer.resolver.api.PropertyResolver;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class PropertyResolverDispatcher 
     extends BindingFeature 
  implements PropertyResolver 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal method.
    //
    // ==================================================================

    /**
     * Provides the interface compliant with the given interface name suffixe.
     * The name of the binding must be the following:<br>
     * <code>&#160;&#160;PropertyResolver.PROPERTY_RESOLVER + "-collection-" + itfSuffixe</code>
     * @return The associated PropertyResolver
     */
    protected PropertyResolver getPropertyResolver(String itfSuffixe){
        try {
            return (PropertyResolver)lookupFc(PropertyResolver.PROPERTY_RESOLVER + "-collection-" + itfSuffixe);
        } catch (NoSuchInterfaceException e) {
            getTrace().info(PropertyResolver.PROPERTY_RESOLVER + "-collection-" + itfSuffixe + ": interface not found!");
            return null;
        }
    }
    
    // ==================================================================
    //
    // Public methods for BindingFeature abstract class. 
    //
    // ==================================================================    
        
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.lib.BindingFeature#clientFc()
     */
    public String[] clientFc() {
        return new String[]{PropertyResolver.PROPERTY_RESOLVER + "-collection"};
    }

    // ==================================================================
    //
    // Public methods for PropertyProvider interface. 
    //
    // ==================================================================    

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.resolver.api.PropertyResolver#resolve(java.lang.String, org.objectweb.util.explorer.api.Entry, org.objectweb.util.explorer.api.Entry)
     */
    public Description resolve(String propertyType, Entry currentEntry, Entry parentEntry) {
        PropertyResolver propertyResolver = getPropertyResolver(propertyType);
        if (propertyResolver!=null){
            return propertyResolver.resolve(propertyType,currentEntry,parentEntry);
        }
        return null;
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.resolver.api.PropertyResolver#resolve(java.lang.String, org.objectweb.util.explorer.api.Entry)
     */
    public Description resolve(String propertyType, Entry currentEntry) {
        return resolve(propertyType, currentEntry, null);
    }
    
}


