/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.resolver.lib;

import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.explorer.core.code.api.CodeDescription;
import org.objectweb.util.explorer.core.common.api.Description;
import org.objectweb.util.explorer.core.common.api.ExplorerConstants;
import org.objectweb.util.explorer.core.root.api.RootDescriptions;
import org.objectweb.util.explorer.core.root.lib.BasicRootDescriptions;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class RootResolver 
     extends AbstractPropertyResolver 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.resolver.lib.AbstractPropertyResolver#getPropertyType()
     */
    protected String getPropertyType() {
        return ExplorerConstants.ROOT_PROPERTY;
    }

    /**
     * Tries to find a root description for the given key.
     * @param key The storage key
     * @return The associated description (or null if no description is found).
     */
    protected CodeDescription getRootDesc(Object key){
        return (CodeDescription)getPropertyProvider().getPropertyDescription(getPropertyType(), key);
    }    

    protected RootDescriptions getRootDescription(){
        RootDescriptions rootDesc = new BasicRootDescriptions();
        String[] roleIds = getRoleProvider().getInheritedRoleIds(getRoleManager().getCurrentRoleIds());
        for (int i = 0; i < roleIds.length; i++) {
            CodeDescription codeDesc = getRootDesc(roleIds[i]);
            if(codeDesc!=null && !codeDesc.isEmpty()){
                rootDesc.addRootDescription(codeDesc);
            }
        }
        return rootDesc;
    }
    
    // ==================================================================
    //
    // Public methods for PropertyResolver interface. 
    //
    // ==================================================================    
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.resolver.api.PropertyResolver#resolve(java.lang.String, org.objectweb.util.explorer.api.Entry, org.objectweb.util.explorer.api.Entry)
     */
    public Description resolve(String propertyType, Entry currentEntry, Entry parentEntry) {
        return getRootDescription();
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.resolver.api.PropertyResolver#resolve(java.lang.String, org.objectweb.util.explorer.api.Entry)
     */
    public Description resolve(String propertyType, Entry currentEntry) {
        return resolve(getPropertyType(), currentEntry, null);
    }

}
