/*
 * ====================================================================
 * 
 * Objectweb Explorer Framework Copyright (C) 2000-2004 INRIA - USTL - LIFL -
 * GOAL Contact: openccm@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * Initial developer(s): Latitia Lafeuille. Contributor(s):
 * ______________________________________.
 * 
 * ====================================================================
 */
package org.objectweb.util.explorer.swing.graph;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
//import java.awt.event.ActionEvent;
//import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;
//import java.util.HashMap;
import java.util.Map;

//import javax.swing.AbstractAction;
//import javax.swing.Action;
//import javax.swing.JMenuItem;
//import javax.swing.JPopupMenu;
//import javax.swing.SwingUtilities;

import org.jgraph.JGraph;
import org.jgraph.graph.CellHandle;
import org.jgraph.graph.CellViewRenderer;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.GraphContext;
import org.jgraph.graph.VertexRenderer;
import org.jgraph.graph.VertexView;

public class CompositeVertexView extends VertexView {

    /** Boolean that indicates if it's the first passage into the view */
    private static boolean firstPaint = true;

    /** The renderer of the view */
    protected static MyVertexRenderer renderer = new MyVertexRenderer();

    public CompositeVertexView(JGraph graph, org.jgraph.graph.CellMapper cm,
            Object cell) {
        super(cell, graph, cm);
    }

    public CellViewRenderer getRenderer() {
        return renderer;
    }

    /**
     * Returns the bounds of the compositeVertexView.
     */
    public Rectangle2D getBounds() {
        CompositeVertex vertex = (CompositeVertex) getCell();
        Rectangle bounds = super.getBounds().getBounds();
        if (vertex.isOpen()) {
            Point origin = VertexGraphicsInterface.SUPER_COMPOSITE_ORIGIN;
            bounds.x = bounds.x + origin.x;
            bounds.y = bounds.y + origin.y;
            ((CompositeVertex) vertex).setOrigin(new Point(bounds.x, bounds.y));
        }
        return bounds;
    }

    /**
     * Returns the handle of the CompositeVertexView. In the handle, we can
     * change the mouse actions
     */
    public CellHandle getHandle(GraphContext context) {
        if (GraphConstants.isSizeable(getAllAttributes())
                && context.getGraph().isSizeable())
            return new MySizeHandle(this, context);
        return null;
    }

    public static class MyVertexRenderer extends VertexRenderer {

        /** The interface that determinate the graphics methods of a port */
        private PortGraphicsInterface pg = Graph.getPortGraphics();

        /** The interface that determinate the graphics methods of a vertex */
        private VertexGraphicsInterface vg = Graph.getVertexGraphics();

        private boolean isFirstPaint;

        public MyVertexRenderer() {
            super();
            isFirstPaint = true;
        }

        /**
         * The method serves to resize a vertex
         * @param size, the new size of the vertex
         */
        public void resizeVertex(Dimension size) {
            CompositeVertex vertex = (CompositeVertex) view.getCell();
            vertex.setSize(size);
            Map vAttributes = vertex.getAttributes();
            Rectangle r = GraphConstants.getBounds(vAttributes).getBounds();
            r.height = size.height;
            r.width = size.width;
            GraphConstants.setBounds(vAttributes, r);
            graph.getGraphLayoutCache().update(view);
        }

        /**
         * Draw a port on an closed composite vertex
         * @param g,
         *            the graphics
         * @param port,
         *            the port to paint
         * @param size,
         *            the size of the closed composite vertex
         */
        public void drawClosedCompositePortLabel(Graphics g, MyPort port,
                Dimension size) {
            CompositeVertex vertex = (CompositeVertex) view.getCell();
            int portsServer = vertex.getListServerPorts().size();
            int portsClients = vertex.getListClientPorts().size();
            int portsControl = vertex.getListControllerPorts().size();
            Point pos;
            String portType = port.getType();
            int portLabelWidth = vertex.getPortLabelWidth(port);

            //if the port is a server port
            if (portType.equals(PortType.SERVER_PORT)) {
                int interval = (size.height - PortGraphicsInterface.FIRST_PORT_Y)
                        / portsServer;
                pos = new Point(
                        VertexGraphicsInterface.PRIMITIVE_MEMBRANE_SIZE + 2,
                        interval * (port.getNumber() - 1)
                                + PortGraphicsInterface.FIRST_PORT_Y
                                + pg.getPortSize(portType, false).height);
                if (portLabelWidth > vertex.getSize().width / 2) {
                    int newPortLabelSize = (getBounds().width / 2 - ((vertex
                            .getName().length() * VertexGraphicsInterface.PRIMITIVE_NAME_WIDTH)/2))
                            / PortGraphicsInterface.PORT_NAME_WIDTH;
                    if ((port.getName().length() > newPortLabelSize)
                            && (newPortLabelSize > 3))
                        g.drawString(port.getName().substring(0,
                                newPortLabelSize - 3)
                                + "...", pos.x, pos.y);
                    else {
                        g.drawString(port.getName(), pos.x, pos.y);
                    }
                } else
                    g.drawString(port.getName(), pos.x, pos.y);
            }

            //if the port is a client port
            else if ((portType.equals(PortType.CLIENT_PORT))
                    || (portType.equals(PortType.COLLECTION_PORT))) {
                int interval = (size.height - PortGraphicsInterface.FIRST_PORT_Y)
                        / portsClients;
                pos = new Point(size.width - portLabelWidth
                        - VertexGraphicsInterface.PRIMITIVE_MEMBRANE_SIZE - 3,
                        interval * (port.getNumber() - 1)
                                + PortGraphicsInterface.FIRST_PORT_Y
                                + pg.getPortSize(portType, false).height);
                if (portLabelWidth > vertex.getSize().width / 2) {
                    int vertexLabelSize = vertex.getName().length()
                            * VertexGraphicsInterface.PRIMITIVE_NAME_WIDTH;
                    int newPortLabelSize = (getBounds().width / 2 - ((vertex
                            .getName().length() * VertexGraphicsInterface.PRIMITIVE_NAME_WIDTH)/2))
                            / PortGraphicsInterface.PORT_NAME_WIDTH;
                    pos.x = getBounds().width
                            - (newPortLabelSize * PortGraphicsInterface.PORT_NAME_WIDTH);
                    if ((port.getName().length() > newPortLabelSize)
                            && (newPortLabelSize > 3))
                        g.drawString(port.getName().substring(0,
                                newPortLabelSize - 3)
                                + "...", pos.x, pos.y);
                    else {
                        g.drawString(port.getName(), pos.x, pos.y);
                    }
                } else
                    g.drawString(port.getName(), pos.x, pos.y);
            }

            //else, if the prot is a controller
            else {
                int interval = size.width / portsControl;
                pos = new Point(2 + interval * (port.getNumber() - 1),
                        PortGraphicsInterface.CONTROLLER_PORT_Y);
                g.drawString(port.getName(), pos.x, pos.y);
            }

        }

        /**
         * Draw a port on an open composite vertex
         * 
         * @param g,
         *            the graphics
         * @param port,
         *            the port to paint
         * @param size,
         *            the size of the open composite vertex
         */
        public void drawOpenCompositePortLabel(Graphics g, MyPort port,
                Dimension size) {
            CompositeVertex vertex = (CompositeVertex) view.getCell();
            int portsInternalServer = ((CompositeVertex) vertex)
                    .getListInternalServerPorts().size();
            int portsInternalClient = ((CompositeVertex) vertex)
                    .getListInternalClientPorts().size();

            //The name of the port generated isn't draw
            //(the port generated has the same name that the internal port)
            if (!port.isGenerated()) {
                Point pos;
                String portType = port.getType();
                int portLabelWidth = vertex.getPortLabelWidth(port);

                //if the port is a server port
                if (portType.equals(PortType.SERVER_PORT)) {
                    //interval that separate two server ports
                    int interval = (size.height - PortGraphicsInterface.FIRST_PORT_Y)
                            / portsInternalServer;
                    pos = new Point(
                            VertexGraphicsInterface.COMPOSITE_MEMBRANE_SIZE + 3,
                            interval * (port.getNumber() - 1)
                                    + PortGraphicsInterface.FIRST_PORT_Y);
                    g.drawString(port.getName(), pos.x, pos.y
                            + PortGraphicsInterface.FIRST_PORT_Y);
                }

                //if the port is a client port
                else if ((portType.equals(PortType.CLIENT_PORT))
                        || ((portType.equals(PortType.COLLECTION_PORT)))) {
                    //interval that separate two client ports
                    int interval = (size.height - PortGraphicsInterface.FIRST_PORT_Y)
                            / portsInternalClient;
                    pos = new Point(size.width - portLabelWidth
                            - VertexGraphicsInterface.COMPOSITE_MEMBRANE_SIZE
                            - 4, interval * (port.getNumber() - 1)
                            + PortGraphicsInterface.FIRST_PORT_Y);
                    g.drawString(port.getName(), pos.x, pos.y
                            + PortGraphicsInterface.FIRST_PORT_Y);
                }
            }

        }

        /**
         * Paint an open composite vertex
         * @param g,
         *            the graphics
         */
        public void paintOpenComposite(Graphics g) {
            CompositeVertex vertex = (CompositeVertex) view.getCell();
            Rectangle gBounds = g.getClipBounds();
            Dimension compositeSize = vg.getSuperCompositeSize();
            Dimension size = new Dimension(view.getBounds().getBounds().width,
                    view.getBounds().getBounds().height);
            gBounds.height = size.height;
            gBounds.width = size.width;
            if (size.width < compositeSize.width)
                size.width = compositeSize.width;
            if (size.height < compositeSize.height)
                size.height = compositeSize.height;
            g.setClip(gBounds.x, gBounds.y, size.width, size.height);
            resizeVertex(size);
            vg.drawVertex(g, vertex.getType(), size, vertex);
            //draw label
            g.setColor(Color.black);
            Font defaultFont = g.getFont();
            g.setFont(new Font("TimesRoman", Font.PLAIN, 18));
            g.drawString(vertex.getName(),
                    VertexGraphicsInterface.SUPER_COMPOSITE_NAME_LOCATION.x,
                    VertexGraphicsInterface.SUPER_COMPOSITE_NAME_LOCATION.y);
            g.setFont(defaultFont);
            //draw ports label
            for (int i = 0; i < vertex.getChildCount(); i++) {
                MyPort port = (MyPort) vertex.getChildAt(i);
                drawOpenCompositePortLabel(g, port, size);
            }
            if (firstPaint)
                firstPaint = false;
            else {
                /** *********************************************************************** */
                // A ce niveau, il faudrait pouvoir appliquer l'algorithm de
                // placement au composant composite (composite vertex) afin
                // ques les
                // sous-composants se replacent correctement
                // J'ai essay :
                //  -> Appliquer l'algorithm  partir du deuxime passage
                // dans cette mthode.
                // Problme rencontr :
                //  -> Une boucle dans l'algoritm dans la mthode
                // refreshGraph au niveau de la mthode
                // graph.getGraphLayoutCache().edit(...)
                // Cette mthode rafraichit tout le model, c'est pourquoi la
                // mthode paint tait rappelle
                // et relanait l'algo.
                // Solution apporte :
                //  1 -> Utiliser une mthode diffrente de la mthode "edit"
                // :
                // refresh de toutes les cellules, refresh de la cellule
                // concerne, update, validate, revalidate,...
                //  2 -> Ajout d'un boolen dans cette classe :
                // "isRefreshed", et appliquer l'algo seulement lorsque ce
                // boolen est  "true". Avant d'appliquer l'algo, on met ce
                // boolen
                //  false afin d'viter la boucle.
                // Au final :
                //  1 -> Aucune de ces mthodes utilises seules ou  la
                // suite apportent un rsultat aussi satisfaisant que celui
                // qu'apporte la mthode edit
                // (mauvais rafraichissement des ports ou des vertex,
                // impossibilit de bouger les vertex, ...)
                //  2 -> la boucle presiste toujours ...
                /** *********************************************************************** */
                //                    if(isRefreshed){
                //                    LayoutAlgorithm algo = ((MyCompositeVertex) vertex)
                //                            .getAlgorithm();
                //                    MyCompositeVertex cv = (MyCompositeVertex) vertex;
                //                    //isRefreshed = false;
                //                    algo.applyLayout(graph, new Point(getBounds().x,
                //                           getBounds().y), cv.getSize());
                //                    isRefreshed = false;
                //                    }
            }

        }

        /**
         * Paint an closed composite vertex
         * @param g,
         *            the graphics
         */
        public void paintClosedComposite(Graphics g) {
            CompositeVertex vertex = (CompositeVertex) view.getCell();
            //draw vertex
            Dimension defaultSize = vg.getDefaultSize();
            Dimension size;
            if (isFirstPaint) {
                size = vertex.getSize();
                isFirstPaint = false;
            } else
                size = new Dimension(view.getBounds().getBounds().width, view
                        .getBounds().getBounds().height);
            //the size of the vertex must be larger than the default size
            if (size.width < defaultSize.width)
                size.width = defaultSize.width;
            if (size.height < defaultSize.height)
                size.height = defaultSize.height;
            Map vAttributes = vertex.getAttributes();
            Rectangle gBounds = g.getClipBounds();
            gBounds.height = size.height;
            gBounds.width = size.width;
            //resize the graphics
            g.setClip(gBounds.x, gBounds.y, gBounds.width, gBounds.height);
            resizeVertex(size);
            vg.drawVertex(g, vertex.getType(), size, vertex);

            //draw vertex label
            g.setColor(Color.black);
            if (!vertex.isStarted())
                g.setColor(new Color(149, 0, 0));
            int vertexLabelLocation = size.width / 2
            - (vertex.getName().length()
            * VertexGraphicsInterface.PRIMITIVE_NAME_WIDTH +5)/2;
            Font defaultFont = g.getFont();
            g.setFont(new Font("TimesRoman", Font.PLAIN, 18));
            g.drawString(vertex.getName(), vertexLabelLocation, size.height / 2
                    + VertexGraphicsInterface.PRIMITIVE_NAME_HEIGHT);
            g.setFont(defaultFont);

            //draw ports labels
            for (int i = 0; i < vertex.getChildCount(); i++) {
                MyPort port = (MyPort) vertex.getChildAt(i);
                drawClosedCompositePortLabel(g, port, size);
            }

        }

        /**
         * The method is used to paint the vertex
         */
        public void paint(Graphics g) {
            CompositeVertex vertex = (CompositeVertex) view.getCell();
            if (vertex.isOpen()) {
                paintOpenComposite(g);
            } else {
                paintClosedComposite(g);
            }
        }
    }

    public static class MySizeHandle extends SizeHandle {

        public MySizeHandle(VertexView vertexview, GraphContext ctx) {
            super(vertexview, ctx);
        }

        /**
         * In this method, we have to add the same popup that there is in the
         * tree
         */
        /*
        public void mousePressed(MouseEvent event) {
            super.mousePressed(event);
            if (SwingUtilities.isRightMouseButton(event)) {
                int x = event.getX();
                int y = event.getY();
                final PrimitiveVertex v = (PrimitiveVertex) vertex.getCell();
                //create a jpopup
                final JPopupMenu popup = new JPopupMenu();

                //create actions
                Action demarrerAction = new AbstractAction("DEMARRER") {
                    public void actionPerformed(ActionEvent e) {
                        v.setStarted(true);
                        HashMap nested = new HashMap();
                        Map vAttributes = vertex.getAttributes();
                        nested.put(vertex, vAttributes);
                        graph.getGraphLayoutCache().edit(nested, null, null,
                                null);
                        graph.getGraphLayoutCache().update(vertex);
                    }
                };

                Action arreterAction = new AbstractAction("ARRETER") {
                    public void actionPerformed(ActionEvent e) {
                        v.setStarted(false);
                        HashMap nested = new HashMap();
                        Map vAttributes = vertex.getAttributes();
                        nested.put(vertex, vAttributes);
                        graph.getGraphLayoutCache().edit(nested, null, null,
                                null);
                        graph.getGraphLayoutCache().update(vertex);
                    }
                };

                //create jmenus
                JMenuItem demarrer = new JMenuItem(demarrerAction);
                JMenuItem arreter = new JMenuItem(arreterAction);

                //insert the menus inside the popup
                popup.add(demarrer);
                popup.addSeparator();
                popup.add(arreter);
                popup.show(graph, x, y);
            }
        }*/
    }
}