/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.swing.lib;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.util.explorer.api.Panel;
import org.objectweb.util.explorer.api.Tree;
import org.objectweb.util.explorer.api.TreeView;
import org.objectweb.util.explorer.core.common.api.Description;
import org.objectweb.util.explorer.core.common.api.ExplorerConstants;
import org.objectweb.util.explorer.core.common.api.Synchronization;
import org.objectweb.util.explorer.core.common.lib.BindingFeature;
import org.objectweb.util.explorer.interpreter.api.DescriptionInterpreter;
import org.objectweb.util.explorer.resolver.api.PropertyResolver;
import org.objectweb.util.explorer.swing.api.ViewPanel;
import org.objectweb.util.explorer.swing.panel.WhitePanel;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class DefaultViewPanel 
     extends BindingFeature 
  implements Synchronization, ViewPanel 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================
    
    protected JPanel viewPanel_ = null;

    protected Panel userPanel_ = null, whitePanel_ = null;
    
    protected TreeView currentTreeView_ = null;
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    public DefaultViewPanel(){
        whitePanel_ = new WhitePanel();
        viewPanel_ = new JPanel();
        viewPanel_.setLayout(new GridLayout(1, 0));
        viewPanel_.setPreferredSize(new Dimension(400, 500));
        //viewPanel_.add(new JScrollPane((JPanel)whitePanel_.getPanel()));
        viewPanel_.add((JPanel)whitePanel_.getPanel());
    }
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Provides the PropertyResolver to use.
     */
    protected PropertyResolver getPropertyResolver() {
        try {
            return (PropertyResolver)lookupFc(PropertyResolver.PROPERTY_RESOLVER);
        } catch (NoSuchInterfaceException e) {
            getTrace().warn(PropertyResolver.PROPERTY_RESOLVER + ": interface not found!");
            return null;
        }
    }

    /**
     * Provides the descriptionInterpreter to use.
     */
    protected DescriptionInterpreter getDescriptionInterpreter() {
        try {
            return (DescriptionInterpreter)lookupFc(DescriptionInterpreter.DESCRIPTION_INTERPRETER);
        } catch (NoSuchInterfaceException e) {
            getTrace().warn(DescriptionInterpreter.DESCRIPTION_INTERPRETER + ": interface not found!");
            return null;
        }
    }    

    /**
     * Provides the tree interface to use.
     */
    protected Tree getTree() {
        try {
            return (Tree)lookupFc(Tree.TREE);
        } catch (NoSuchInterfaceException e) {
            getTrace().warn(Tree.TREE + ": interface not found!");
            return null;
        }
    }    
        
    /**
     * Fixes the panel associated to the selected node
     */
    protected void setViewPanel(Panel newPanel) {
        viewPanel_.removeAll();
        userPanel_ = newPanel;
        viewPanel_.add(new JScrollPane((JPanel)userPanel_.getPanel()));
        viewPanel_.revalidate();
    }    
    
    // ==================================================================
    //
    // Public methods for BindingFeature abstract class. 
    //
    // ==================================================================    
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.lib.BindingFeature#clientFc()
     */
    public String[] clientFc() {
        return new String[]{Tree.TREE, PropertyResolver.PROPERTY_RESOLVER,DescriptionInterpreter.DESCRIPTION_INTERPRETER};
    }

    // ==================================================================
    //
    // Public methods for Synchronization interface. 
    //
    // ==================================================================    

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.code.api.Synchronization#refresh(org.objectweb.util.explorer.api.Entry)
     */
    public void refresh(TreeView treeView) {
        if(userPanel_!=null && currentTreeView_!=null){
            userPanel_.unselected(currentTreeView_);
        }
        if(treeView!=null){
            Description desc = getPropertyResolver().resolve(ExplorerConstants.PANEL_PROPERTY, treeView.getSelectedEntry());
            // TODO: Temporary methods until a dynamic fractal assembly is returned by the "interprete" method.
            Map map = new HashMap();
            map.put(PropertyResolver.PROPERTY_RESOLVER, getPropertyResolver());
            map.put(DescriptionInterpreter.DESCRIPTION_INTERPRETER, getDescriptionInterpreter());
            map.put(Tree.TREE, getTree());
            Panel p = (Panel)getDescriptionInterpreter().interprete(desc, map);
            if(p!=null){
                currentTreeView_ = treeView;
                p.selected(treeView);
                setViewPanel(p);
            } else {
                userPanel_ = null;
                setViewPanel(whitePanel_);
            }
        }
    }

    // ==================================================================
    //
    // Public methods for ViewPanel interface. 
    //
    // ==================================================================    
    
    /**
     * Provides the view panel
     * @return
     */
    public JPanel getViewPanel(){
        return viewPanel_;
    }
    
}

