/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.swing.panel;

import java.awt.Color;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.JPanel;

import org.objectweb.util.explorer.api.Panel;
import org.objectweb.util.explorer.api.TreeView;
import org.objectweb.util.explorer.swing.api.CompositePanel;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class BasicCompositePanel 
     extends JPanel 
  implements CompositePanel, Panel 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================
    
    /** The list containing the inner panels */
    protected Vector panelList_;
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    public BasicCompositePanel(){
        panelList_ = new Vector();
        setBackground(Color.white);
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    }
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for CompositePanel interface. 
    //
    // ==================================================================    
        
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.swing.api.CompositePanel#add(org.objectweb.util.explorer.api.Panel)
     */
    public void add(Panel panel) {
        if(panel!=null){
            panelList_.add(panel);
        }
    }

    // ==================================================================
    //
    // Public methods for Panel interface. 
    //
    // ==================================================================        
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Panel#selected(org.objectweb.util.explorer.api.TreeView)
     */
    public void selected(TreeView treeView) {
        Iterator it = panelList_.iterator();
        while (it.hasNext()) {
            Panel element = (Panel) it.next();
            element.selected(treeView);
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Panel#getPanel()
     */
    public Object getPanel() {
        Iterator it = panelList_.iterator();
        while (it.hasNext()) {
            Panel element = (Panel) it.next();
            add((JPanel)element.getPanel());
        }
        return this;    
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Panel#unselected(org.objectweb.util.explorer.api.TreeView)
     */
    public void unselected(TreeView treeView) {
        Iterator it = panelList_.iterator();
        while (it.hasNext()) {
            Panel element = (Panel) it.next();
            element.unselected(treeView);
        }        
    }

}
