/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.swt.lib;

import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSource;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DragSourceListener;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.TreeEvent;
import org.eclipse.swt.events.TreeListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TreeItem;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.util.explorer.api.Context;
import org.objectweb.util.explorer.api.DropAction;
import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.explorer.api.IconProvider;
import org.objectweb.util.explorer.api.Info;
import org.objectweb.util.explorer.api.MenuItemTreeView;
import org.objectweb.util.explorer.api.Tree;
import org.objectweb.util.explorer.api.TreeView;
import org.objectweb.util.explorer.core.common.api.Description;
import org.objectweb.util.explorer.core.common.api.ExplorerConstants;
import org.objectweb.util.explorer.core.common.api.TextComponent;
import org.objectweb.util.explorer.core.common.lib.BindingFeature;
import org.objectweb.util.explorer.core.common.lib.DefaultTreeView;
import org.objectweb.util.explorer.core.common.lib.TreeBindingFeature;
import org.objectweb.util.explorer.core.dnd.api.DnDDescription;
import org.objectweb.util.explorer.core.dnd.api.DnDDescriptions;
import org.objectweb.util.explorer.core.dnd.lib.DefaultDropTreeView;
import org.objectweb.util.explorer.core.menu.api.MenuDescription;
import org.objectweb.util.explorer.core.menu.lib.DefaultMenuItemTreeView;
import org.objectweb.util.explorer.core.naming.lib.DefaultEntry;
import org.objectweb.util.explorer.interpreter.api.DescriptionInterpreter;
import org.objectweb.util.explorer.interpreter.lib.swt.IconFileProvider;
import org.objectweb.util.explorer.resolver.api.PropertyResolver;
import org.objectweb.util.explorer.swt.api.Explorer;
import org.objectweb.util.trace.Trace;
import org.objectweb.util.trace.TraceSystem;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class DynamicTree 
     extends org.eclipse.swt.widgets.Tree 
  implements Tree, BindingController, Explorer 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    /** The binding feature to use. */
    protected BindingFeature bindingFeature_ = null;

    /** The display to use. */
    protected Display display_ = null;
    
    /** The shell to use. */
    public static Shell shell_ = null;

    /** Registers the state of the node. */
    protected Map nodeStates_ = null;

    /** The context containing the initial entries. */
    protected Vector initialContext_ = null;

    /** The menu bar to use. */
    protected Menu menuBar_ = null;

    /** The action menu item. */
    protected MenuItem actionMenu_ = null;
        
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Calls by the constructor
     */
    private void initDragAndDrop(){
        Transfer[] types = new Transfer[] {EntryTransfer.getInstance()};
        int operations = DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_LINK;
        
        final DragSource source = new DragSource (this, operations);
        source.setTransfer(types);
        source.addDragListener (new MyDragSourceListener ());
        
        DropTarget target = new DropTarget(this, operations);
        target.setTransfer(types);
        target.addDropListener (new MyDropTargetAdapter());
    }
    
    /**
     * @param composite
     * @param type
     */
    public DynamicTree() {
        super(new Shell(), SWT.BORDER);
        setSize(400,500);
        
        bindingFeature_ = new TreeBindingFeature();
        nodeStates_ = new HashMap();
        initialContext_ = new Vector();
        
        addListener (SWT.Expand, new MyExpensionListener());
        addListener (SWT.MenuDetect, new MyMenuDetectListener());
        addListener (SWT.MouseDown, new MyMouseDownListener());
        addListener (SWT.Selection, new MySelectionListener());
        addTreeListener(new MyTreeListener());
        
        initDragAndDrop();
    }    
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    protected Trace getTrace(){
        return TraceSystem.get("explorer");
    }

    /**
     * Returns the selected entry
     */
    protected Entry getTheSelectedEntry() {
        TreeItem[] items = getSelection();
        if(items != null && items.length>0){
            return (Entry)items[0].getData();
        }
        return null;
    }

    /**
     * Returns the parent of the selected entry
     */
    protected Entry getParentEntry() {
        TreeItem[] items = getSelection();
        if(items[0].getParentItem()!=null){
            return (Entry)items[0].getParentItem().getData();
        }
        return null;
    }    
    
    /**
     * Provides a view on the tree
     * 
     * @return
     */
    protected TreeView getTreeView() {
        return new DefaultTreeView((Tree)this, getTheSelectedEntry(), getParentEntry());
    }    

    /**
     * Provides a menu item view.
     * @return
     */
    protected MenuItemTreeView getMenuItemTreeView() {
        return new DefaultMenuItemTreeView((Tree)this, getTheSelectedEntry(), getParentEntry());
    }

    /**
     * Adds the given entry into the tree.
     * @param entry
     */
    protected void addInitialEntry(Entry entry){
        TreeItem item = new TreeItem(this,0);
        item.setText(entry.getName().toString());
        item.setData(entry);
        completeTreeItem(item);
        item.setExpanded(true);
    }
    
    /**
     * Computes if an the given TreeItem is a leaf or not.
     * A tree item is a leaf if the value of the contained entry is a context
     * instance or if a context is defined for the value of the contained entry.
     * This method may modify the value of the treeItem. More precisely, it may
     * add a wrapper into the contained entry. 
     */
    protected void completeTreeItem(TreeItem treeItem){
        Entry entry = (Entry)treeItem.getData();
        
        // Computes the icon
        Image image = (Image)getImageIcon(entry);
        if(image!=null){
            treeItem.setImage(image);
        }
        
        // Computes the wrapper
        if(entry.getWrapper()!=null){
            // The entry value is a context instance.
            new TreeItem(treeItem,0);
        } else {
            Description desc = getPropertyResolver().resolve(ExplorerConstants.WRAPPER_PROPERTY, entry);
            if(desc!=null){
                Context childContext = (Context) getDescriptionInterpreter().interprete(desc, null);
                if (childContext != null) {
                    entry.setWrapper(childContext);
                    new TreeItem(treeItem,0);
                }
            }
        }
    }

    /**
     * Gets a string representation of the given item.
     * 
     * @param item The tree item.
     * @return The path from the root to the given item.
     * 
     */
    protected String pathToString(TreeItem item) {
        if(item!=null){
            TreeItem current = item;
            String value = current.getText();
            while((current = current.getParentItem())!=null){
                value = current.getText() + "/" + value;
            }
            return value;
        }
        return "";
    }
        
    // ==================================================================
    // Access to Fractal interfaces
    // ==================================================================
    
    /**
     * Provides the PropertyResolver to use.
     */
    protected PropertyResolver getPropertyResolver() {
        try {
            return (PropertyResolver) bindingFeature_.lookupFc(PropertyResolver.PROPERTY_RESOLVER);
        } catch (NoSuchInterfaceException e) {
            getTrace().warn(PropertyResolver.PROPERTY_RESOLVER + ": interface not found!");
            return null;
        }
    }

    /**
     * Provides the descriptionInterpreter to use.
     */
    protected DescriptionInterpreter getDescriptionInterpreter() {
        try {
            return (DescriptionInterpreter) bindingFeature_.lookupFc(DescriptionInterpreter.DESCRIPTION_INTERPRETER);
        } catch (NoSuchInterfaceException e) {
            getTrace().warn(DescriptionInterpreter.DESCRIPTION_INTERPRETER + ": interface not found!");
            return null;
        }
    }

    /**
     * Provides the TextComponent to use.
     */    
    protected TextComponent getTextComponent(){
        try {
            return (TextComponent) bindingFeature_.lookupFc(TextComponent.TEXT_COMPONENT);
        } catch (NoSuchInterfaceException e) {
            getTrace().warn(TextComponent.TEXT_COMPONENT + ": interface not found!");
            return null;
        }        
    }
    
    // ==================================================================
    // Icon Management
    // ==================================================================
    
    protected Image getImageIcon(Entry entry){
        Description desc = getPropertyResolver().resolve(ExplorerConstants.ICON_PROPERTY, entry);
        IconProvider iconProvider = (IconProvider) getDescriptionInterpreter().interprete(desc, null);
        if(iconProvider!=null){
            Object object = iconProvider.newIcon(entry.getValue());
            if(object instanceof Image){
                return (Image)object;
            } else if(object instanceof String) {
                return (Image)(new IconFileProvider((String)object)).newIcon(entry.getValue()); 
            } else if (object instanceof URL) {
                return (Image)(new IconFileProvider((URL)object)).newIcon(entry.getValue());
            }
        }
        return null;
    }
    
    // ==================================================================
    // Info Management
    // ==================================================================
    
    protected void displayInfo(Entry entry){
        if (getTextComponent() != null){
            String message = "";
            Description desc = getPropertyResolver().resolve(ExplorerConstants.INFO_PROPERTY, entry);
            if(desc!=null){
                Info info = (Info) getDescriptionInterpreter().interprete(desc, null);
                if (info != null) {
                    message = info.getInfo(getTreeView());
                }
            }            
            getTextComponent().setText(message);
        }
    }

    // ==================================================================
    // Refresh Management
    // ==================================================================

    /**
     * Refreshes all the tree. 
     */
    protected void refreshAllPathes(){
        System.out.println("---- BEGIN refreshAllPathes ----");
        removeAll();
        Iterator it = initialContext_.iterator();
        while (it.hasNext()) {
            Entry entry = (Entry) it.next();
            System.out.println("=> Adds " + entry.getName());
            addInitialEntry(entry);
        }
        System.out.println("---- END refreshAllPathes ----");
    }
    
    /**
     * Refreshes the action menu depending on the selected object.
     * @param entry The selected entry
     */
    protected void refreshActionItem(Entry entry){
        if(actionMenu_!=null && entry!=null){
            Description desc = getPropertyResolver().resolve(ExplorerConstants.MENU_PROPERTY, entry, getParentEntry());
            if(desc!=null && ! desc.isEmpty()){
                ((MenuDescription)desc).setDescriptionType("bar");
                Menu menu = (Menu) getDescriptionInterpreter().interprete(desc, getMenuItemTreeView());
                actionMenu_.setMenu(menu);
                actionMenu_.setEnabled(true);
            } else {
                actionMenu_.setEnabled(false);
            }
        }
    }
    
    protected void refreshActionItem(){
        refreshActionItem(getTheSelectedEntry());
    }
    
    // ==================================================================
    //
    // Public methods for Tree interface.
    //
    // ==================================================================

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Tree#refreshAll()
     */
    public void refreshAll() {
        refreshAllPathes();
    }    
    
    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.util.explorer.api.Tree#addEntry(java.lang.Object,
     *      java.lang.Object)
     */
    public void addEntry(Object name, Object value) {
        if(name!=null && value!=null){
            Entry entry = new DefaultEntry(name, value);
            addInitialEntry(entry);
            initialContext_.add(entry);
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Tree#addEntry(java.lang.Object, java.lang.Object, int)
     */
    public void addEntry(Object name, Object value, int level) {
        // TODO: Not implemented
        throw new RuntimeException("NOT IMPLEMENTED YET!");
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Tree#selectPath(java.lang.String)
     */
    public void selectPath(String path) {
        // TODO: Not implemented
        throw new RuntimeException("NOT IMPLEMENTED YET!");
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Tree#clear()
     */
    public void clear() {
        // TODO: Not implemented
        throw new RuntimeException("NOT IMPLEMENTED YET!");
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Tree#getInitialContextSize()
     */
    public int getInitialContextSize() {
        // TODO: Not implemented
        throw new RuntimeException("NOT IMPLEMENTED YET!");
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Tree#removeEntry(java.lang.Object)
     */
    public void removeEntry(Object name) {
        // TODO: Not implemented
        throw new RuntimeException("NOT IMPLEMENTED YET!");
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Tree#renameInitialEntry(java.lang.Object, java.lang.Object)
     */
    public void renameInitialEntry(Object currentName, Object newName) {
        // TODO: Not implemented
        throw new RuntimeException("NOT IMPLEMENTED YET!");
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Tree#duplicate()
     */
    public Component duplicate() {
        // TODO: Not implemented
        throw new RuntimeException("NOT IMPLEMENTED YET!");
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Tree#duplicate(boolean)
     */
    public Component duplicate(boolean withData) {
        // TODO: Not implemented
        throw new RuntimeException("NOT IMPLEMENTED YET!");
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Tree#getSelectedEntry()
     */
    public Entry getSelectedEntry() {
        // TODO: Not implemented
        throw new RuntimeException("NOT IMPLEMENTED YET!");
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Tree#renameSelectedNode(java.lang.Object, java.lang.Object)
     */
    public void renameSelectedNode(Object odlName, Object newName) {
        // TODO: Not implemented
        throw new RuntimeException("NOT IMPLEMENTED YET!");
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Tree#close()
     */
    public void close() {
        // TODO: Not implemented
        throw new RuntimeException("NOT IMPLEMENTED YET!");
    }

    // ==================================================================
    //
    // Public methods for Explorer interface.
    //
    // ==================================================================
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.swt.api.Explorer#getTree()
     */
    public org.eclipse.swt.widgets.Tree getTree(){
        return this;
    }

    /**
     * Fixes the display to use.
     * @param display The display to use.
     */
    public void setDisplay(Display display){
        display_ = display;
    }
    
    /**
     * Fixes the shell to use.
     * @param shell The shell to use.
     */
    public void setShell(Shell shell){
        shell_ = shell;
    }

    /**
     * Fixes the menu to use
     * @param menu The menu to use.
     */
    public void setMenuBar(Menu menu){
        if(menu!=null){
            menuBar_ = menu;
            actionMenu_ = new MenuItem(menu, SWT.CASCADE);
            actionMenu_.setText("Actions");
            actionMenu_.setEnabled(false);
        }
    }
    
    // ==================================================================
    //
    // Public methods for BindingController interface.
    //
    // ==================================================================

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String,
     *      java.lang.Object)
     */
    public void bindFc(String itfName, Object itfValue)
            throws NoSuchInterfaceException, IllegalBindingException,
            IllegalLifeCycleException {
        bindingFeature_.bindFc(itfName, itfValue);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.fractal.api.control.BindingController#listFc()
     */
    public String[] listFc() {
        return bindingFeature_.listFc();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
     */
    public Object lookupFc(String itfName) throws NoSuchInterfaceException {
        return bindingFeature_.lookupFc(itfName);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
     */
    public void unbindFc(String itfName) throws NoSuchInterfaceException,
            IllegalBindingException, IllegalLifeCycleException {
        bindingFeature_.unbindFc(itfName);
    }
        
    // ==================================================================
    //
    // Internal class for Tree management.
    //
    // ==================================================================
    
    final class MyExpensionListener implements Listener {
        public void handleEvent (Event e) {
            TreeItem currentItem = (TreeItem)e.item;
            getTrace().debug("Expension listener: " + currentItem.getText());
            
            // Removes the children
            TreeItem[] itemChildren = currentItem.getItems();
            for (int i = 0; i < itemChildren.length; i++) {
                if (itemChildren[i].getData() != null)
                    return;
                itemChildren[i].dispose();
            }
            
            // Computes the children
            Entry currentEntry = (Entry)currentItem.getData();
            Context context = (Context)currentEntry.getWrapper();
            Entry[] children = context.getEntries(currentEntry.getValue());
            for (int i = 0; i < children.length; i++) {
                TreeItem item = new TreeItem(currentItem, 0);
                item.setText(children[i].getName().toString());
                item.setData(children[i]);
                completeTreeItem(item);
            }
        }
    }
    
    final class MyMenuDetectListener implements Listener {
        public void handleEvent (Event e) {
            Entry entry = getTheSelectedEntry();
            if(entry!=null){
                Description desc = getPropertyResolver().resolve(ExplorerConstants.MENU_PROPERTY, entry, getParentEntry());
                if(desc!=null){
                    ((MenuDescription)desc).setDescriptionType("popup");
                    Menu menu = (Menu) getDescriptionInterpreter().interprete(desc, getMenuItemTreeView());
                    if(menu!=null){
                        menu.setLocation (e.x, e.y);
                        menu.setVisible (true);
                        while (!menu.isDisposed () && menu.isVisible ()) {
                            if (!display_.readAndDispatch ()) display_.sleep ();
                        }
                        menu.dispose ();
                    }
                }
            }
            /*
            TreeItem currentItem = (TreeItem)e.item;
            if (currentItem != null) {
                System.out.println ("Menu detect");
            }
            */
        }
    }
    
    final class MyMouseDownListener implements Listener {
        public void handleEvent (Event e) {
            //Point point = new Point (e.x, e.y);
            //TreeItem item = getItem (point);
            //if (item != null) {
                //System.out.println ("Mouse down");
            //}
        }
    }

    final class MySelectionListener implements Listener {
        public void handleEvent (Event e) {
            Entry entry = getTheSelectedEntry(); 
            displayInfo(entry);
            refreshActionItem(entry);
        }
    }
    
    final class MyTreeListener implements TreeListener {
        
        /* (non-Javadoc)
         * @see org.eclipse.swt.events.TreeListener#treeCollapsed(org.eclipse.swt.events.TreeEvent)
         */
        public void treeCollapsed(TreeEvent e) {
            nodeStates_.put(pathToString((TreeItem)e.item), Boolean.FALSE);
        }
        
        /* (non-Javadoc)
         * @see org.eclipse.swt.events.TreeListener#treeExpanded(org.eclipse.swt.events.TreeEvent)
         */
        public void treeExpanded(TreeEvent e) {
            TreeItem item = (TreeItem)e.item;
            getTrace().debug("treeExpanded: " + item.getText());
            nodeStates_.put(pathToString(item), Boolean.TRUE);
            TreeItem[] children = item.getItems();
            for(int i = 0 ; i < children.length ; i++){
                TreeItem child = children[i];
                String childPath = pathToString(child);
                if(nodeStates_.containsKey(childPath) && ((Boolean)nodeStates_.get(childPath)).booleanValue()){
                    getTrace().debug("Expand " + child.getText());
                    TreeItem[] children2 = child.getItems();
                    for(int j=0;j<children2.length;j++){
                        children2[j].dispose();
                    }
                    child.setExpanded(true);
                    setRedraw(true);
                }
            }
            
        }
    }
    
    // ==================================================================
    //
    // Internal classes for Drag & Drop.
    //
    // ==================================================================

    
    protected class MyDragSourceListener implements DragSourceListener {

        protected Entry dragSourceEntry = null;        
        
        /* (non-Javadoc)
         * @see org.eclipse.swt.dnd.DragSourceListener#dragStart(org.eclipse.swt.dnd.DragSourceEvent)
         */
        public void dragStart(DragSourceEvent event) {
            //event.doit = false;
            dragSourceEntry = getTheSelectedEntry();
        }
                
        /* (non-Javadoc)
         * @see org.eclipse.swt.dnd.DragSourceListener#dragFinished(org.eclipse.swt.dnd.DragSourceEvent)
         */
        public void dragFinished(DragSourceEvent event) {
        }
        
        /* (non-Javadoc)
         * @see org.eclipse.swt.dnd.DragSourceListener#dragSetData(org.eclipse.swt.dnd.DragSourceEvent)
         */
        public void dragSetData(DragSourceEvent event) {
            event.data = dragSourceEntry;
        }
        
    }
    
    protected class MyDropTargetAdapter extends DropTargetAdapter {

        protected Entry dragEntry = null;

        protected TreeItem oldItem_ = null;
        
        /** The currentAction_ */
        protected boolean actionChanged_ = false;        
        
        protected String getDropActionName(int dropAction){
            if(dropAction==DND.DROP_MOVE)
                return DnDDescription.MOVE_ACTION;
            else if(dropAction==DND.DROP_COPY)
                return DnDDescription.COPY_ACTION;
            else if(dropAction==DND.DROP_LINK)
                return DnDDescription.LINK_ACTION;
            else 
                return "";
        }         

        protected String computeActionDesc(Entry entry, String dropAction){
            DnDDescriptions desc = (DnDDescriptions)getPropertyResolver().resolve(ExplorerConstants.DND_PROPERTY, entry);
            if(desc!=null){
                DnDDescription dndDesc = desc.getDropAction(dropAction);
                if(dndDesc !=null){
                    return dndDesc.getLabel();
                }
            }
            return "";
        }
                
        /* (non-Javadoc)
         * @see org.eclipse.swt.dnd.DropTargetListener#dragEnter(org.eclipse.swt.dnd.DropTargetEvent)
         */
        public void dragEnter(DropTargetEvent event) {
            super.dragEnter(event);
            dragEntry = getTheSelectedEntry();
        }
        
        /* (non-Javadoc)
         * @see org.eclipse.swt.dnd.DropTargetListener#dragLeave(org.eclipse.swt.dnd.DropTargetEvent)
         */
        public void dragLeave(DropTargetEvent event) {
            super.dragLeave(event);
        }
        
        /* (non-Javadoc)
         * @see org.eclipse.swt.dnd.DropTargetListener#dragOperationChanged(org.eclipse.swt.dnd.DropTargetEvent)
         */
        public void dragOperationChanged(DropTargetEvent event) {
            super.dragOperationChanged(event);
            actionChanged_ = true;
        }
        
        /* (non-Javadoc)
         * @see org.eclipse.swt.dnd.DropTargetListener#dragOver(org.eclipse.swt.dnd.DropTargetEvent)
         */
        public void dragOver(DropTargetEvent event) {
            event.feedback = DND.FEEDBACK_EXPAND | DND.FEEDBACK_SCROLL | DND.FEEDBACK_SELECT;
            TreeItem currentItem = (TreeItem)event.item;
            
            if(currentItem==null){
                oldItem_ = null;
                if(getTextComponent()!=null){
                    getTextComponent().setText("");
                }
            } else if(oldItem_ == null || (currentItem != null && !currentItem.equals(oldItem_)) || actionChanged_){
                actionChanged_ = false;
                oldItem_ = currentItem;
                String value = computeActionDesc((Entry)currentItem.getData(), getDropActionName(event.detail));
                if(getTextComponent()!=null){
                    getTextComponent().setText(value);
                }
            }
        }
        
        /* (non-Javadoc)
         * @see org.eclipse.swt.dnd.DropTargetListener#drop(org.eclipse.swt.dnd.DropTargetEvent)
         */
        public void drop(DropTargetEvent event) {
            super.drop(event);
            if (event.data == null) {
                event.detail = DND.DROP_NONE;
                return;
            }
            
            if(getTextComponent()!=null){
                getTextComponent().setText("");
            }
            
            //Entry dropEntry = getSelectedEntry();
            if(event.item!=null){
                TreeItem item = (TreeItem)event.item;
                
                Entry dropEntry = (Entry)item.getData();
                
                DnDDescriptions desc = (DnDDescriptions)getPropertyResolver().resolve(ExplorerConstants.DND_PROPERTY, dropEntry);
                if(desc!=null){
                    DnDDescription dndDesc = desc.getDropAction(getDropActionName(event.detail));
                    if(dndDesc !=null){
                        DropAction dropAction = (DropAction)getDescriptionInterpreter().interprete(dndDesc.getCodeDescription(),null);
                        if(dropAction!=null && dragEntry!=null){
                            try{
                                dropAction.execute(new DefaultDropTreeView((Tree)DynamicTree.this, dropEntry, getParentEntry(), dragEntry));
                            }catch(Exception e){
                                getTrace().warn(getClass().getName() + ", " + e.getClass().getName() + ": "+ e.getMessage());
                                MessageBox mb = new MessageBox(DynamicTree.shell_, SWT.ICON_ERROR | SWT.OK);
                                mb.setMessage(e.getClass().getName() + ":\n" + e.getMessage());
                                mb.setText("Exception (" + event.widget + ")");
                                mb.open();
                            }
                            DynamicTree.this.refreshAllPathes();
                        }
                    }
                }                
                
            }
            
        }
        
        /* (non-Javadoc)
         * @see org.eclipse.swt.dnd.DropTargetListener#dropAccept(org.eclipse.swt.dnd.DropTargetEvent)
         */
        public void dropAccept(DropTargetEvent event) {
            super.dropAccept(event);
        }
}
}
