/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.core.menu;

import org.objectweb.util.explorer.core.menu.api.AcceleratorDescription;
import org.objectweb.util.explorer.core.menu.lib.BasicAcceleratorDescription;

import junit.framework.Assert;
import junit.framework.TestCase;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class AcceleratorDescriptionTest 
     extends TestCase 
{


    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    protected AcceleratorDescription charOnly_,ctrlChar_,ctrlShiftChar_,all_;
    
    protected char theChar_ = 'A', otherChar_ = 'B';
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal method.
    //
    // ==================================================================

    /*
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        charOnly_ = new BasicAcceleratorDescription();
        charOnly_.setAcceleratorCharacter(theChar_);
        ctrlChar_ = new BasicAcceleratorDescription();
        ctrlChar_.setCtrlKey(true);
        ctrlChar_.setAcceleratorCharacter(theChar_);
        ctrlShiftChar_ = new BasicAcceleratorDescription();
        ctrlShiftChar_.setCtrlKey(true);
        ctrlShiftChar_.setShiftKey(true);
        ctrlShiftChar_.setAcceleratorCharacter(theChar_);
        all_ = new BasicAcceleratorDescription();
        all_.setCtrlKey(true);
        all_.setShiftKey(true);
        all_.setAltKey(true);
        all_.setMetaKey(true);
        all_.setAcceleratorCharacter(theChar_);
    }
    
    // ==================================================================
    //
    // Public methods. 
    //
    // ==================================================================    
    
    /**
     * Checks if two definitions of AcceleratorDefinition are considered as equal.
     */
    public void testEqualsMethod(){
        AcceleratorDescription expected = null;
        Assert.assertNotSame(expected,charOnly_);
        Assert.assertEquals(ctrlChar_,ctrlChar_);
        expected = new BasicAcceleratorDescription();
        expected.setAcceleratorCharacter(otherChar_);
        Assert.assertNotSame(expected,charOnly_);
        expected.setAcceleratorCharacter(theChar_);
        Assert.assertEquals(expected,charOnly_);
        expected.setCtrlKey(true);
        Assert.assertEquals(expected,ctrlChar_);
        Assert.assertNotSame(expected,ctrlShiftChar_);
        expected.setShiftKey(true);
        Assert.assertEquals(expected,ctrlShiftChar_);
        expected.setAltKey(true); 
        expected.setMetaKey(true); 
        Assert.assertNotSame(expected,ctrlChar_);
        Assert.assertEquals(expected,all_); 
    }
    
    /**
     * Checks the String representation of a
     */
    public void testToStringMethod(){
        String expected = "BasicAcceleratorDescription[value=" + theChar_ + "]";
        Assert.assertEquals(expected,charOnly_.toString());
        expected="BasicAcceleratorDescription[value=ctrl-" + theChar_ + "]";
        Assert.assertEquals(expected,ctrlChar_.toString());
        expected="BasicAcceleratorDescription[value=ctrl-shift-" + theChar_ + "]";
        Assert.assertEquals(expected,ctrlShiftChar_.toString());
        expected="BasicAcceleratorDescription[value=ctrl-alt-shift-meta-" + theChar_ + "]";
        Assert.assertEquals(expected,all_.toString());
    }

}

