/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.core.role;

import org.objectweb.util.explorer.ExplorerUtils;
import org.objectweb.util.explorer.core.role.api.Role;
import org.objectweb.util.explorer.core.role.api.RoleDescription;
import org.objectweb.util.explorer.core.role.lib.DefaultRoleDescription;
import org.objectweb.util.explorer.core.role.lib.RoleProperty;

import junit.framework.Assert;
import junit.framework.TestCase;

/**
 * 
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class RolePropertyTest 
     extends TestCase 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    /** The RoleDescription object to use to test the RoleProperty component. */
    protected RoleDescription r0,r0bis,r1,r2,r3,r4,r5,r6,r7,r8,r9,r10,r11,r12;
    
    protected RoleProperty roleProperty_ = null;
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Internal method.
    //
    // ==================================================================
    
    protected RoleDescription createRoleDesc(String id, String[] inheritedRoles, boolean concrete){
        RoleDescription r = new DefaultRoleDescription();
        r.setId(id);
        r.setConcrete(concrete);
        r.setInheritance(inheritedRoles);
        return r;
    }
    
    protected RoleDescription createRoleDesc(String id, String[] inheritedRoles){
        return createRoleDesc(id, inheritedRoles, true);
    }
      
    protected void feedRoleDesc(RoleDescription[] roles){
        for (int i = 0; i < roles.length; i++) {
            roleProperty_.feed(roles[i]);
        }
    }
    
    protected void checkTreeInheritance(String[] roleIds, String[] expectedRolesIds){
        Role[] expectedRoles = roleProperty_.getRoles(expectedRolesIds);
        Role[] resultRoles = roleProperty_.getRoles(roleProperty_.getInheritedRoleIds(roleIds));
        String result = ExplorerUtils.arrayToString(resultRoles);
        String expected = ExplorerUtils.arrayToString(expectedRoles);
        Assert.assertEquals(expected,result);
    }
    
    /*
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        r0 = createRoleDesc("0",new String[]{"1"});  
        r1 = createRoleDesc("1",new String[]{"3"});
        r2 = createRoleDesc("2",new String[]{"4","5"});
        r3 = createRoleDesc("3",new String[]{"6","10"});
        r4 = createRoleDesc("4",new String[]{"7"});
        r5 = createRoleDesc("5",new String[]{"8","9"});
        r6 = createRoleDesc("6",new String[]{});
        r7 = createRoleDesc("7",new String[]{"10","11"});
        r8 = createRoleDesc("8",null);
        r9 = createRoleDesc("9",new String[]{"12"});
        r10 = createRoleDesc("10",new String[]{});
        r11 = createRoleDesc("11",new String[]{});
        r12 = createRoleDesc("12",null);
        r0bis = createRoleDesc("0",new String[]{"1","4","2","1"});
        roleProperty_ = new RoleProperty();
    }    
        
    // ==================================================================
    //
    // Public methods for ... interface.
    //
    // ==================================================================    

    /*
     * Checks the behavior of the RoleProperty component.
     */
    public void testConcreteRole() {
        feedRoleDesc(new RoleDescription[]{r0,r0bis});
        Role r = roleProperty_.getRole("0");
        Assert.assertTrue(r.isConcrete());
    }    
            
    /*
     * Checks the behavior of the RoleProperty component.
     */
    public void testMultipleDefinition() {
        feedRoleDesc(new RoleDescription[]{r0,r0bis,r1,r2,r4});
        checkTreeInheritance(new String[]{"0"},new String[]{"0","1","4","2"});    
    }

    /*
     * Checks the behavior of the RoleProperty component. 
     */
    public void testInheritanceTree() {
        feedRoleDesc(new RoleDescription[]{r0,r0bis,r1,r2,r3,r4,r5,r6,r7,r8,r9,r10,r11,r12});
        
        // Checks for {r0}
        checkTreeInheritance(new String[]{"0"}, new String[]{"0","1","4","2","3","7","5","6","10","11","8","9","12"});
        
        // Checks for {r1,r5}
        // Limit: The corresponding algorithm doesn't know the level of each role it treats
        checkTreeInheritance(new String[]{"1","5"}, new String[]{"1","5","3","8","9","6","10","12"});        
                
    }    
    
    /*
     * Checks the behavior of the RoleProperty component.
     */
    public void testInfiniteLoop() {
        RoleDescription role0 = createRoleDesc("0",new String[]{"1"});
        RoleDescription role1 = createRoleDesc("1",new String[]{"3"});
        RoleDescription role2 = createRoleDesc("2",new String[]{"1"});
        RoleDescription role3 = createRoleDesc("3",new String[]{"2"});
        feedRoleDesc(new RoleDescription[]{role0,role1,role2,role3});
        checkTreeInheritance(new String[]{"0"}, new String[]{"0","1","3","2"});
        checkTreeInheritance(new String[]{"1"}, new String[]{"1","3","2"});   
    }
        
}