/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): .

--------------------------------------------------------------------
$Id: OptionContext.java,v 1.1 2004/05/19 15:58:29 rouvoy Exp $
====================================================================*/

package org.objectweb.util.launcher.option;


import java.util.Vector;

import org.objectweb.util.cmdline.api.Iterator;
import org.objectweb.util.cmdline.lib.DefaultOptionArgument;

import org.objectweb.util.launcher.CommandJava;
import org.objectweb.util.launcher.CommandFactory;

import org.objectweb.util.launcher.parser.ContextDescription;
import org.objectweb.util.launcher.parser.Repository;


/**
 * Definition of the --runid context.<BR>
 * <p>
 * This context specify the run identifier to use for starting the 
 * application.
 * </p>
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 * @version 0.1
 */
public class OptionContext
     extends DefaultOptionArgument
  implements OptionLauncher
{
    /** Short Tag Description */
    public final static String shortTag = "-ctx" ;
    /** Long Tag Description */
    public final static String longTag = "--context" ;

    /** List of contexts to load */
    protected Vector contexts_ ;

    /**
     * Default Constructor
     */
    public OptionContext(){
        super(new String[]{shortTag, longTag},
              "<ContextID>",
              "The name of a context",
              "");
        contexts_=new Vector();
    }


    /**
     * Loader of context description.
     * 
     * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
     * @version 0.1
     */
    protected class ContextBuilder 
    {
        /** Description of the context to build */
        protected String opt ;

        /**
         * Constructor.
         * @param arg description of the context to build.
         */
        public ContextBuilder(String arg) {
            this.opt = arg ;
        }
        
        /**
         * Complete the command java by setting the identifier of
         * the run to use
         *
         * @param cmd - the command to complete
         */
        public void complete(Repository repository) {
            ContextDescription desc = repository.getDescription(opt);
            CommandFactory.instance().getDefaultClasspath().addAll(desc.getClasses());
            CommandFactory.instance().getDefaultArguments().addAll(desc.getArguments());
            CommandFactory.instance().getDefaultProperties().putAll(desc.getProperties());
        }
    }

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.Context
    //
    // ==================================================================

    /**
     * Consumes command line arguments from an iterator.
     *
     * @param iterator The command line argument iterator.
     */
    public void consume(Iterator iterator) {
        setArgument(consumeArgument(iterator));
        contexts_.addElement(new ContextBuilder(getArgument()));
    }
    
    /**
     * Complete the command java by setting the identifier of
     * the run to use
     *
     * @param cmd - the command to complete
     */
    public CommandJava[] complete(Repository repository) {
        java.util.Iterator contexts = contexts_.iterator();
        Vector list = new Vector();
        while (contexts.hasNext())
            ((ContextBuilder)contexts.next()).complete(repository);           
        return new CommandJava[0];
    }

    /**
     * Creates a commandline representation of the context.
     * @param value value of the context.
     * @return commandline representation of the context.
     */
    public static String create(String opt) {
        return shortTag+" "+opt+" " ; 
    }
}
