@echo off
rem # ====================================================================
rem #
rem # OpenCCM: The Open CORBA Component Model Platform
rem # Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
rem # Contact: openccm@objectweb.org
rem #
rem # This library is free software; you can redistribute it and/or
rem # modify it under the terms of the GNU Lesser General Public
rem # License as published by the Free Software Foundation; either
rem # version 2.1 of the License, or any later version.
rem #
rem # This library is distributed in the hope that it will be useful,
rem # but WITHOUT ANY WARRANTY; without even the implied warranty of
rem # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
rem # Lesser General Public License for more details.
rem #
rem # You should have received a copy of the GNU Lesser General Public
rem # License along with this library; if not, write to the Free Software
rem # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
rem # USA
rem #
rem # Initial developer(s): Philippe Merle, Mathieu Vadet.
rem # Contributor(s): Christophe Demarey, Jerome Offroy,
rem #                 Christophe Contreras, Romain Rouvoy.
rem #
rem # ====================================================================
rem #
rem #  Start a Java Application from an xml configuration file.
rem #
rem #  Parameters :
rem #  - %1 represents launcher mode.
rem #  - %2 is the file where standard output is redirected. (ex: cmd.output, cmd.ior)
rem #  - %3 is a file containing the Processus ID.
rem #  
rem #  Variables (must be setted before calling this script) :
rem #  - LAUNCHER_JVM_ARGS represents jvm parameters.
rem #  - LAUNCHER_ARGS represents application parameters.
rem #
rem #  If %1 = "exec", simply run the application.
rem #  If %1 = "fork", run the application as a server.
rem #  If %1 = "fork_err", run the application as a server
rem #                       and redirect output on error output.
rem #
rem # ====================================================================
rem # $Id: launcher.bat,v 1.6 2004/10/18 16:20:03 contrera Exp $
rem # ====================================================================
rem # setlocal

set CLASSPATH=@LAUNCHER_CLASSPATH@

set LAUNCHER_JVM_PROPS="-DOW_CONFIGDIR=%OW_CONFIGDIR% -DOW_HOMEDIR=%OW_HOMEDIR%" %LAUNCHER_JVM_ARGS%
set JVM=java %LAUNCHER_JVM_PROPS% @OW_LAUNCHER_CLASSNAME@
set LAUNCHER_OUTPUT_FILE=%2
set LAUNCHER_OUTPUT_FILE=%LAUNCHER_OUTPUT_FILE:"=%
set LAUNCHER_PID_FILE=%3
set LAUNCHER_PID_FILE=%LAUNCHER_PID_FILE:"=%

if "%1"=="exec" goto exec
if "%1"=="fork" goto fork
if "%1"=="fork_err" goto fork_err
goto :eof

:exec
%JVM% %LAUNCHER_ARGS%
goto :eof

:fork
if exist "%LAUNCHER_OUTPUT_FILE%.tmp" del "%LAUNCHER_OUTPUT_FILE%.tmp"
exec "%LAUNCHER_PID_FILE%" %JVM% %LAUNCHER_ARGS% > "%LAUNCHER_OUTPUT_FILE%.tmp"
goto :wait

:fork_err
if exist "%LAUNCHER_OUTPUT_FILE%.tmp" del "%LAUNCHER_OUTPUT_FILE%.tmp"
exec "%LAUNCHER_PID_FILE%" %JVM% %LAUNCHER_ARGS% 2> "%LAUNCHER_OUTPUT_FILE%.tmp"

:wait
set READY=
if not exist "%LAUNCHER_OUTPUT_FILE%.tmp" goto :wait
copy "%LAUNCHER_OUTPUT_FILE%.tmp" "%LAUNCHER_OUTPUT_FILE%" > nul
for /f "usebackq" %%i in ("%LAUNCHER_OUTPUT_FILE%") do set READY=ok
if not defined READY goto :wait
