/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Areski Flissi.

====================================================================*/

package org.objectweb.ccm.CORBA;

/**
 * This class encapsulates the JacORB CORBA::ORB singleton object.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @version 0.1
 */

public abstract class TheORB extends TheORBSingleton {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    private TheORB() {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Remove ORB arguments.
     *
     * @param args The command-line arguments.
     *  
     * @return The command line arguments without JacORB options.
     */
    protected static String[]
    remove_ORB_arguments(String[] args) {
        int nb = 0;

        // Copy arguments without ORB ones.
        String[] tmp = new String[args.length];
        
        for (int i = 0; i < args.length; i++) {
            
            if (args[i].startsWith("-ORBInitRef")) {
                i++;
                            
            } else if (!args[i].startsWith("-ORB")) {
                tmp[nb] = args[i];
                nb++;
            }
        }

        // Construct the returned array.
        args = new String[nb];
        for (int i = 0; i < nb; i++) {
            args[i] = tmp[i];
        }

        // Return arguments without ORB ones.
        return args;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Initialize the JacORB CORBA::ORB singleton object.
     *
     * @param args The command line arguments.
     *
     * @return The command line arguments without JacORB options.
     */
    public static String[]
    initialize(String[] args) {
        
        for (int i = 0; i < args.length; i++) {
            if (args[i].equals("-ORBInitRef")) {
                i++;
                // Manage CORBA initial references provided as 
                // Multicast group address (e.g. 224.0.0.100)
                
                String arg = args[i];
                int index = arg.indexOf('=');
                String location = arg.substring(index + 1);
                String ior;

                if (location.startsWith("mcast:")) {

                    // Store the IOR obtained from the MULTICAST server
                    // in the command line arguments.
                    args[i] =
                        arg.substring(0, index)
                            + '='
                            + TheORBSingleton.request_ior(location);
                }
            }
        }
        
        // Define system properties.
        java.util.Properties properties = System.getProperties();
        properties.put("org.omg.CORBA.ORBClass", "org.jacorb.orb.ORB");
        properties.put(
            "org.omg.CORBA.ORBSingletonClass",
            "org.jacorb.orb.ORBSingleton");

        properties.put("jacorb.suppress_no_props_warning", "on");
        properties.put("jacorb.orb.print_version", "off");
        properties.put("jacorb.verbosity", "0");

        // Initialize the CORBA::ORB.
        setORB(org.omg.CORBA.ORB.init(args, properties));

        return remove_ORB_arguments(args);
    }

    /**
     * Destroy the CORBA::ORB singleton object.
     */
    public static void destroy() {
        getORB().destroy();
    }
}
