/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.coach.actor.componentBrowser;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import java.util.EventObject;
import org.omg.CORBA.*;
import org.coach.util.IorPrinter;

public class ComponentBrowser extends JFrame {
    private TreePath selection;
    private ComponentModel model;
    private final org.omg.CORBA.ORB orb;

    public ComponentBrowser(Component parent, org.omg.CORBA.ORB orb, String name, org.omg.CORBA.Object obj) {
        this.orb = orb;
        model = new ComponentModel(new ComponentNode(name, obj));
        ComponentNode.setOrb(orb);

        JTree tree = new JTree(model);
        JScrollPane scrollPane = new JScrollPane(tree);

        getContentPane().add(scrollPane, BorderLayout.CENTER);

        final ComponentControlPanel p = new ComponentControlPanel(model, orb);
        getContentPane().add(p, BorderLayout.SOUTH);
        
        tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);

        tree.addTreeExpansionListener(new TreeExpansionListener() {
            public void treeCollapsed(TreeExpansionEvent e) {
            }
                                                        
            public void treeExpanded(TreeExpansionEvent e) {
                TreePath path = e.getPath();
                ComponentNode node = (ComponentNode)path.getLastPathComponent();

                node.explore(true);
                model.nodeStructureChanged(node);
            }
        });
        tree.addTreeSelectionListener(new TreeSelectionListener() {
            public void valueChanged(TreeSelectionEvent e) {
                TreePath selection = e.getNewLeadSelectionPath();
                model.setSelection(selection);
                p.iorTextArea.setText("");
                p.hostTextField.setText("     ");
                p.portTextField.setText("     ");
                p.typeTextField.setText("     ");
                p.pingLabel.setText("         ");
                if(selection != null) {
                    ComponentNode node = (ComponentNode)selection.getLastPathComponent();
                    if (node.isLeaf()) {
                        String ior = node.getIor();
                        p.iorTextArea.setText(ior);
                        p.iorTextArea.setCaretPosition(0);

                        IorPrinter iorPrinter = new IorPrinter(ior);
                        p.hostTextField.setText(iorPrinter.getHost());
                        p.portTextField.setText(iorPrinter.getPort());
                        p.typeTextField.setText(iorPrinter.getTypeId());
                        p.hostTextField.setCaretPosition(0);
                        p.portTextField.setCaretPosition(0);
                        p.typeTextField.setCaretPosition(0);
                        p.repaint();
                    }
                }
            }
        });

        setTitle(name);
        while(!(parent instanceof JFrame || parent instanceof Frame)) {
            parent = parent.getParent();
        }
        Point pp = parent.getLocationOnScreen();      
        setLocation(pp.x + 10, pp.y + 10);
        setVisible(true);

        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        pack();
    }
}
