package org.coach.tracing.server.viewer;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.util.*;
import org.coach.idltree.*;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import org.coach.tracing.server.EventRecord;

public class EventBrowser extends JFrame  {  
    JTabbedPane propertyBrowser = new JTabbedPane();
    JPanel params_panel = new JPanel();
    BorderLayout borderLayout1 = new BorderLayout();
    DefaultTableColumnModel dtcm = new DefaultTableColumnModel();
    PropertyTableModel dataModel;
    TableColumn tc1 = new TableColumn(0);
    TableColumn tc2 = new TableColumn(1);
    JScrollPane property_scroll = new JScrollPane();
    JScrollPane params_scroll = new JScrollPane();
    JTable property_table = null; 
    EventRecord er; 

    public EventBrowser(EventRecord r) {
        er = r;
        dataModel = new PropertyTableModel(r);

        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void jbInit() throws Exception {
        property_table = new JTable(dataModel);
        dtcm.addColumn(tc1);
        dtcm.addColumn(tc2);
        tc1.setHeaderValue("Name");
        tc2.setHeaderValue("Value");
        tc1.setMaxWidth(120);
        property_table.setColumnModel(dtcm);
        PropertyTableRenderer mtr = new PropertyTableRenderer(dataModel);
        property_table.setDefaultRenderer(property_table.getColumnClass(1), mtr);
        property_table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        property_table.getSelectionModel().setSelectionInterval(0, 0);
        this.getContentPane().add(propertyBrowser, BorderLayout.CENTER);
        propertyBrowser.addTab("Properties", property_scroll);

        try {           
            JTree typeTree = new JTree();
            IdlNode params = er.getParameters();
            if (params != null) {
                final IdlModel model = new IdlModel(params);
                typeTree.setModel(model);
                typeTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
                typeTree.addTreeSelectionListener(new TreeSelectionListener() {
                    public void valueChanged(TreeSelectionEvent e) {
                        TreePath selection = e.getNewLeadSelectionPath();
                        model.setSelection(selection);
                    }
                });
                params_scroll.getViewport().add(typeTree, null);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        propertyBrowser.addTab("Parameters, Results, Exceptions", params_scroll);
        property_scroll.getViewport().add(property_table, null);
        propertyBrowser.setSelectedComponent(property_scroll);
    }
}

class PropertyTableModel extends AbstractTableModel  {
    private static String[] fields = {
        "EventCounter",
        "TrailLabel",
        "MessageId",
        "ObjectName",
        "ObjectType",
        "ComponentName",      
        "ComponentType",
        "ContainerName",
        "ContainerType",
        "ProcessId",
        "HostName",
        "IpAddress",
        "ThreadId",
        "TrailId",
        "Operation",
        "InteractionPoint",
        "TimeStamp"
    };
    private Object[] values = new Object[fields.length];
        
    public PropertyTableModel(EventRecord me) {
    	int i = 0;
    	Long key = me.getIdentityKey();
        values[i++] = "" + me.getEventCounter();
        values[i++] = me.getTrailLabel();
        values[i++] = me.getMessageId();
        values[i++] = IdentityNode.getObjectName(key);
        values[i++] = IdentityNode.getObjectType(key);
        values[i++] = IdentityNode.getCmpName(key);
        values[i++] = IdentityNode.getCmpType(key);
        values[i++] = IdentityNode.getCntName(key);
        values[i++] = IdentityNode.getCntType(key);
        values[i++] = IdentityNode.getProcessId(key);
        values[i++] = IdentityNode.getNodeName(key);
        values[i++] = IdentityNode.getNodeIp(key); 
        values[i++] = me.getThreadId();   
        values[i++] = me.getTrailId();
        values[i++] = me.getOpName();
        values[i++] = me.getInteractionPointName();
        values[i++] = me.getTimeStamp();
    }

    public int getColumnCount() {
        return 2;
    }

    public int getRowCount() {
        return fields.length;
    }

    public Object getValueAt(int x, int y) {
        if (y == 0) {
            return fields[x];
        } else {
            return values[x];
        }
    }
    
    public Class getColumnClass(int columnIndex) {
        if (columnIndex == 1) {
            return Object.class;
        } else {
            return String.class;
        }
    }
}

class PropertyTableRenderer extends DefaultTableCellRenderer  {
    PropertyTableModel model = null;

    public PropertyTableRenderer(PropertyTableModel tm) {
        super();
        model = tm;
    }

    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
        Component comp = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
        ToolTipManager.sharedInstance().registerComponent((JComponent)comp);
        return comp;
    }
}

