package org.coach.tracing.server.viewer;

import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import org.coach.tracing.api.InteractionPoint; // from tracing.idl
import org.coach.tracing.server.EventRecord;

public class EventGui extends JComponent  {
    private final JPopupMenu popup;
    private static Diagram view;
    private static ScaleManager sm;
    public final static int min_hsize = 10;
    public final static int vspace_between = 2;
    public final static int circle_diam = 10;
    public final static Color normal_event_color = Color.white;
    public final static Color oneway_event_color = Color.green;
    public final static Color exception_event_color = Color.red;
    public final static Color normal_border_color = Color.black;
    public final static Color trail_border_color = Color.magenta;

    private int position;
    private Long eventKey;
    private Dimension text_dimension;
    private String opName;
    private String objectName;
    private boolean isOneway;
    private boolean isException;
    private boolean isReturn;
    
    private static Stack pool = new Stack();
           
    public EventGui(Diagram v) {
        super();
        if (view == null) {
            view = v;
            sm = ScaleManager.getScaleManager();
        }       
        popup = new JPopupMenu();
        JMenuItem menu1 = new JMenuItem("Colour causality chain for this Event");
        menu1.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                colour_actionPerformed(e);
            }
        });
        popup.add(menu1);

        menu1 = new JMenuItem("Browse Event");
        menu1.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                browse_actionPerformed(e);
            }
        });
        popup.add(menu1);

        addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent e) {
                if (e.isPopupTrigger()) {
                    popup.show(e.getComponent(), e.getX(), e.getY());
                }
            }
            public void mouseReleased(java.awt.event.MouseEvent e) {
                if (e.isPopupTrigger()) {
                    popup.show(e.getComponent(), e.getX(), e.getY());
                }
            }
        });
    }
 
    public static EventGui getEventGui(EventRecord e) {
        EventGui m = null;
        if (pool.isEmpty()) {
            m = new EventGui(view);
        } else {
            m = (EventGui)pool.pop();
        }
        m.setEvent(e);
        return m;
    }
    
    public static void releaseEventGui(EventGui m) {
        pool.push(m);
    }
       
    private void setEvent(EventRecord e) {
        eventKey = e.key();
        opName = e.getOpName();
        objectName = IdentityNode.getObjectName(e.getIdentityKey());
        isException = e.isException();
        isOneway = e.isOneway();
        isReturn = e.isReturn();

        float scale = sm.getScale();
        text_dimension = sm.getStringDimension(""/*data_link.get("cflow_hop"))*/
                                                                                , this);
        text_dimension.width += 1;
        text_dimension.height += 1;
        setSize(Math.max(text_dimension.width, (int)(min_hsize * scale)) + 1, (int)(text_dimension.height + ((vspace_between + circle_diam) * scale)));

        ToolTipManager.sharedInstance().registerComponent(this);
        setToolTipText(opName);
    }

    public boolean isReturn() {
        return isReturn;
    }

    public boolean isException() {
        return isException;
    }       

    public boolean isOneway() {
        return isOneway;
    }
        
    public java.awt.Color getColor() {
        return view.getColor(eventKey);
    }
    
    public String getOpName() {
        return opName;
    }

    public String getObjectName() {
        return objectName;
    }
        
    public void setPosition(int p) {
        position = p;
    }

    public int getPosition() {
        return position;
    }
    
    public Point getCenter() {
        float scale = sm.getScale();
        return new Point((int)(getX() + (getWidth() / 2)), (int)(getY() + getHeight() - ((circle_diam * scale) / 2)));
    }

    public int getRadius() {
        float scale = sm.getScale();
        return (int)(scale * circle_diam / 2);
    }

    public void paint(Graphics g) {
        //paint the event
        super.paint(g);
        float scale = sm.getScale();
        Graphics2D g2 = (Graphics2D)g;
        g2.setRenderingHints(sm.getHints());
        g2.setStroke(sm.getStroke());
        g2.setFont(sm.getFont());
        //draw the filled oval

        if (isException) {
            g2.setColor(exception_event_color);
        } else if (isOneway) {
            g2.setColor(oneway_event_color);
        } else {
            g2.setColor(normal_event_color);
        }
        g2.fillOval((int)((getWidth() - (circle_diam * scale)) / 2), (int)(text_dimension.height + (vspace_between * scale)), (int)(circle_diam * scale), (int)(circle_diam * scale));
        //set dashed stroke if the message is a return one
        BasicStroke s = sm.getStrokeCircle();

        if (isReturn) {
            s = sm.getStrokeCircleDashed();
        }
        g2.setStroke(s);
        if (getColor() != null) {
            g2.setColor(getColor());
        } else {
            g2.setColor(normal_border_color);
        }
        g2.draw(new Ellipse2D.Float((int)((s.getLineWidth() + getWidth() - (circle_diam * scale)) / 2), (int)(text_dimension.height + (vspace_between * scale) + (s.getLineWidth() / 2)), (int)(circle_diam * scale) - (int)(s.getLineWidth()), (int)(circle_diam * scale) - (int)(s.getLineWidth())));
    }

    public Long getEventKey() {
        return eventKey;
    }

    public void colour_actionPerformed(ActionEvent e) {
        view.colour_causality_chain(eventKey);
    }

    public void browse_actionPerformed(ActionEvent e) {
        view.createEventBrowser(eventKey, getLocationOnScreen());
    }
}
