/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;

/**
 * Implementation of the CORBA::AliasDef interface.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.5
 */

public class AliasDef_impl
     extends Contained_impl
  implements AliasDefOperations
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The original type of the alias. */
    protected IDLType_ref original_type_def_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * TODO.
     */
    public
    AliasDef_impl(IFR ifr,
                  Container_impl container)
    {
        // Calls the Contained_impl contructor.
        super(ifr, container);

        // Sets the tie servant object.
        setServant(new AliasDefPOATie(this));

        // Creates the original_type_def reference.
        original_type_def_ = new IDLType_ref(this);
    }

    // ==================================================================
    //
    // Internal methods for the inherited IRObject_impl class.
    //
    // ==================================================================

    /**
     **  Warning : this operation must be implemented by subclasses.
     **  It allows us to avoid some casts while creating ::CORBA::TypeCode
     **/
    protected TypeCode
    recursiveType(org.objectweb.ccm.util.Vector idSeq)
    {
        try
        {
            Contained_impl cont = (Contained_impl)original_type_def_.getBaseImpl();
            int idx = idSeq.indexOf(cont.id());
            if (idx!=-1)
            {
                // check if the contained can be a recursive type
                int dk = cont.def_kind().value();
                if ((dk!=DefinitionKind._dk_Struct) &&
                    (dk!=DefinitionKind._dk_Exception) &&
                    (dk!=DefinitionKind._dk_Union) &&
                    (dk!=DefinitionKind._dk_Value) &&
                    (dk!=DefinitionKind._dk_Alias))
                    throw exceptionIllegalRecursion(id(), cont.id());
            }
        }
        catch (java.lang.ClassCastException ex)
        {
        }

        TypeCode tc = original_type_def_.recursiveType(idSeq);

        // delegate to the TypeCodeFactory.
        return getIFR().getTCF().create_alias_tc(id(), name(), tc);
    }

    // ==================================================================
    //
    // Internal methods for managing dependencies.
    //
    // ==================================================================

    /**
     * Cuts dependencies to other objects.
     */
    protected void
    cutDependencies()
    {
        // Cuts the dependency.
        original_type_def_.cutDependency();

        // Calls the Contained_impl cutDependencies operation.
        super.cutDependencies();
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
        Obtain its CORBA::AliasDef object reference.
     **/
    public AliasDef
    asAliasDef()
    {
        return AliasDefHelper.narrow(asObject());
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IRObject/def_kind:1.0
     **/
    public DefinitionKind
    def_kind()
    {
        return DefinitionKind.dk_Alias;
    }

    // ==================================================================
    //
    // For CORBA::IDLType interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/IDLType/type:1.0
     **/
    public TypeCode
    type()
    {
        // check for recursion.
        org.objectweb.ccm.util.Vector idSeq = new org.objectweb.ccm.util.Vector();
        return recursiveType(idSeq);
    }

    // ==================================================================
    //
    // For CORBA::AliasDef interface.
    //
    // ==================================================================

    /**
        IDL:omg.org/CORBA/AliasDef/original_type_def:1.0
     **/
    public IDLType
    original_type_def()
    {
        // delegate to the IDLType reference.
        return original_type_def_.asIDLType();
    }

    public void
    original_type_def(IDLType val)
    {
        // try to cast val to a local object
        // and delegate to the IDLType reference.
        original_type_def_.setDependency( castToLocal(val) );
    }
}
