/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/



package org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.installer;

// Import all the packages required
import java.io.InputStreamReader;
import org.objectweb.ccm.descriptor.ZipEntryRetriever;
import org.objectweb.ccm.descriptor.componentassembly.beans.*;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.InitializationError;
import org.objectweb.ccm.descriptor.softpkg.*;
import org.objectweb.ccm.descriptor.softpkg.beans.*;

/**
 *
 * ComponentFileDeployer is used to read xml file information.
 *
 * @author <a href="mailto:Frederic.Briclet@lifl.fr">Briclet Frederic</a>
 *
 * @version 0.1
 */
public class ComponentfileDeployer 
       extends ComponentfileDeployerContext 
    {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /*Prefix for the outputed message*/
    private String prefix="ComponentFileDeployer >> ";

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================
  

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================


    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Initialize the imbricated softpkg tree precised in xml file
     * @throws InitializationError if a problem occured during initialization
     */
    public void 
    initializeSoftpkgTree() 
    throws InitializationError 
    {
        try {
            ZipEntryRetriever zer =
                getFileinarchiveDeployer().getInnerArchive();
            
            
            String  csdname ;
            if(getFileinarchiveDeployer().getFileinarchive().getName()!=null
            	&&getFileinarchiveDeployer().getFileinarchive().getName().endsWith(".csd"))
            	
            	csdname=getFileinarchiveDeployer().getFileinarchive().getName();
            else csdname= zer.getZipEntriesEndingBy(".csd")[0];
            

            //Take only the first file which ending by .csd an exception 
            // must be thrown later

            InputStreamReader reader =
                new InputStreamReader(
                    zer.getZipEntryISIgnoringDelimiterType(csdname));
            getRootDeployerContext().connectOpenReader(reader);
            SoftpkgBean soft = SoftpkgBeanImpl.unmarshalBean(reader, true);
            connectSoftpkg(soft);
            //Browse all the implementation to store the reference of Java implementation
            for (java.util.Iterator i = soft.getImplementationList().iterator();
                i.hasNext();
                ) {
                ImplementationBeanImpl ii = (ImplementationBeanImpl) i.next();
                if (((Programminglanguage) ii
                    .getProgramminglanguageList()
                    .get(0))
                    .getName()
                    .equalsIgnoreCase("java")) {
                    connectImplementation(ii);
                }
            }

        } catch (Exception e) {            
            throw new InitializationError(
                this,
                e,
                "Fatal error:\n"
                    + prefix
                    + "Cannot open CSD file ["
                    + getFileinarchiveDeployer().getFileinarchive().getName()
                    + "] in the current ComponentAssembly archive.\n"
                    + getStringifiedDeployerDescription());
        }

        getLifeCycleManager().stepEndedSuccessFully();
    }
}
