/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.installer;

import org.objectweb.ccm.descriptor.componentassembly.beans.DestinationBean;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.*;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.util.FindbyDeployer;
import org.objectweb.ccm.descriptor.softpkg.ccm.ImplementationDeployer;
import org.objectweb.ccm.descriptor.softpkg.ccm.SoftpkgDeployer;

/**
 * The destination deployer context defines all the connection requires 
 * to install a home on a target host.
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class DestinationDeployerContext 
       extends ChildDeployerContext 
    {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    private DestinationBean destination;
    private FindbyDeployer findbyDeployer;
    private NodeDeployer nodeDeployer;
    private SoftpkgDeployer softpkgDeployer;
    private ImplementationDeployer implementationDeployer;
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    // ==================================================================
    //
    //  Public accessors methods.
    //
    // ==================================================================    
    /**
     * @return
     */
    public DestinationBean 
    getDestination() 
    {
        return destination;
    }

    /**
     * @param bean
     */
    public void 
    connectDestination(DestinationBean bean) 
    {
        destination = bean;
        connectDeployerDescription(bean);
    }

    /**
     * @return
     */
    public FindbyDeployer 
    getFindbyDeployer() 
    {
        return findbyDeployer;
    }

    /**
     * @param deployer
     */
    public void 
    connectFindbyDeployer(FindbyDeployer deployer) 
    {
        findbyDeployer = deployer;
    }

    /**
     * @return
     */
    public NodeDeployer 
    getNodeDeployer() 
    {
        return nodeDeployer;
    }

    /**
     * @param deployer
     */
    public void 
    connectNodeDeployer(NodeDeployer deployer) 
    {
        nodeDeployer = deployer;
    }

    /**
     * @return
     */
    public ImplementationDeployer 
    getImplementationDeployer() 
    {
        return implementationDeployer;
    }

    /**
     * @return
     */
    public SoftpkgDeployer 
    getSoftpkgDeployer() 
    {
        return softpkgDeployer;
    }

    /**
     * @param deployer
     */
    public void 
    connectImplementationDeployer(ImplementationDeployer deployer) 
    {
        implementationDeployer = deployer;
    }

    /**
     * @param deployer
     */
    public void 
    connectSoftpkgDeployer(SoftpkgDeployer deployer) 
    {
        softpkgDeployer = deployer;
    }
    
    /**
     * This initialization method is used to check that all the services and 
     * information required to deploy are presents. If all is OK the deployer 
     * is put in inactive deployer state waiting for the deployment processing.
     * 
     * @exception InitializationError is thrown if a problem is detected 
     */  
    public void initialize() 
    throws InitializationError 
    {
        if(getDestination()==null)
               getErrorManager()
              .submitException(new InitializationError(this,"Connection on destination bean is required"+
                                                       "check the handler code it must specify this connection"));
        //TODO: add detsination format verification                                
        getLifeCycleManager().stepEndedSuccessFully();
    }



}
