/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): Philippe Merle.

====================================================================*/
package org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.installer;

// Local Interface Import Statements
import org.objectweb.ccm.descriptor.componentassembly.beans.DestinationBeanImpl;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.*;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.util.FindbyDeployer;

/**
 * This handler is used to produce DestinationDeployer from an
 * destination bean. The implementation use the current 
 * DestinationDeployerfactory to found the right destination 
 * deployer to use.
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class DestinationDeployerHandler
     extends HandlerBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    private DestinationDeployerBase _deploy;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================        

    /**
     * Produce the destinationdeployer deployer binded
     * to the given @param cons.
     * 
     * @param cons The destination bean 
     * @return The deployer produced in initialize state
     * @throws InitializationError Thrown if a problem is detected
     */  
    public Object 
    getInstance(DestinationBeanImpl cons)
    throws InitializationError 
    {
        if (_deploy == null) 
        {
            if (getRootDeployerContext()
                .getDestinationDeployerFactory()== null)
                throw new InitializationError(_deploy, null, 
                            "Cannot found a valid Destination Deployer, no DestinationDelpoyerFactory connected");

/* Code related to the 2nd IST COACH project redefinition
   of the destination element done.
*/

            try 
            {

                // Controls if children were set.
                if(   (cons.getNode()==null)
                   && (   (cons.getInstallation()==null)
                       || (cons.getActivation()==null)
                      )
                  )
                {
                    throw new InitializationError(_deploy,
                                                  null,
                                                  "The <node> child or the nodes <installation> and <activation>" +
                                                  " must be present in the <destination>\n" +
                                                  "Check the <destination> Description:\n" +
                                                  getStringifiedDescription(cons));
                }

                String deployType = null;
                if(cons.getNode() != null)
                {
                    deployType = NodeDeployer.getNodeManagerType();
                }
                else
                {
                    deployType = cons.getActivation().getType().trim();
                }

                _deploy = getRootDeployerContext()
                          .getDestinationDeployerFactory()
                          .getDestinationDeployer(deployType);
            }
            catch (UnknownDestinationDeployer e) 
            {
                throw new InitializationError(_deploy, e, e.getMessage());
            }

            _deploy.connectDestination(cons);

            applyCommonConfig(_deploy);

            if(cons.getNode() != null)
            {
                _deploy.connectNodeDeployer((NodeDeployer)getDeployer(cons.getNode()));
            }

            if (cons.getActivation() != null && cons.getActivation().getFindby() != null)
            {
                _deploy.connectFindbyDeployer(
                    (FindbyDeployer)getDeployer(cons.getActivation().getFindby())
                );
            }

            _deploy.initialize();

        }
        return (_deploy);
    }

}
