/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.installer;

import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;

import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.FatalDeploymentException;
import org.objectweb.ccm.descriptor.softpkg.ccm.CodeInstallationFailureException;
import org.objectweb.ccm.descriptor.softpkg.ccm.DependencyResolutionFailureException;
import org.omg.Components.CCMHome;
import org.omg.Components.ConfigValue;
import org.omg.Components.Deployment.ComponentInstallation;
import org.omg.Components.Deployment.ComponentInstallationHelper;
import org.omg.Components.Deployment.ComponentServer;
import org.omg.Components.Deployment.Container;
import org.omg.Components.Deployment.InstallationFailure;
import org.omg.Components.Deployment.InvalidLocation;
import org.omg.Components.Deployment.ServerActivator;
import org.omg.Components.Deployment.ServerActivatorHelper;

/**
 * The ECMDestinationDeployer is the DestinationDeployer able to correctly install
 * a home on ECM destination host.  
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class ECMDestinationDeployer 
       extends DestinationDeployerBase 
    {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    private ServerActivator sa;
    private ComponentInstallation ci;
    private ComponentServer cs;
    private Hashtable containerTable;
    private java.util.List config_values;
    private NodeDependencyVisitorDeployer depVisitor;
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================
    public ECMDestinationDeployer() 
    {
        containerTable = new Hashtable();
        config_values = new java.util.LinkedList();
        depVisitor = new NodeDependencyVisitorDeployer();
    }
    // ==================================================================
    //
    // Internals methods.
    //
    // ==================================================================
    /**
     * Internal method to found the ServerActivtor denoted by the component 
     * assembly descriptor.
     * @return the ServerActivator found
     */
    private ServerActivator 
    getSA() 
    {
        if (sa == null)
            try {
                sa =
                    ServerActivatorHelper.narrow(
                        getFindbyDeployer().resolveReference());
            } catch (Exception e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        return sa;
    }
    /**
     * Internal method to found the ComponentInstallation denoted by the component 
     * assembly descriptor.
     * @return the ComponentInstallation found
     */
    private ComponentInstallation 
    getCI() 
    {
        if (ci == null) {
            String ci_name ="ExtCI"+
                getFindbyDeployer()
                    .getFindby()
                    .getNamingservice()
                    .getName().substring(2);



            try {
                ci = ComponentInstallationHelper.narrow(getRef(ci_name));
            } catch (Exception e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
        return ci;
    }
    /**
     * Internal method to start a HTTP server to upload the full component archive.
     * 
     * @param input The input stream on the file to upload
     * @param name The string identifier of the ressource
     * @return the string URL identifier used to identifies the ressource
     * @throws java.io.IOException
     */
    private String 
    startHttpServer(InputStream input, String name)
    throws java.io.IOException 
    {
        (new Thread(new org
            .objectweb
            .ccm
            .deploytool
            .MicroServerHttp(
                getRootDeployerContext().getServerSocket(),
                input,
                name)))
            .start();
        return "http://"
            + java.net.InetAddress.getLocalHost().getHostAddress()
            + ':'
            + getRootDeployerContext().getServerSocket().getLocalPort()
            + '/'
            + name;
    }
    /**
     * Internal method to install a home.
     * 
     * @param csConfig
     * @param contConfig
     * @param homeConfig
     * @param implID
     * @param entryPoint
     * @return
     * @throws FatalDeploymentException
     */
    private CCMHome 
    installCCMHome(
        ConfigValue[] csConfig,
        ConfigValue[] contConfig,
        ConfigValue[] homeConfig,
        String implID,
        String entryPoint)
    throws FatalDeploymentException {
        CCMHome home = null;
        try {
            if (cs == null)
                cs = getSA().create_component_server(csConfig);
            Container cont = cs.create_container(contConfig);
            home = cont.install_home(implID, entryPoint, homeConfig);
            containerTable.put(home, cont);
        } catch (Exception e) {
            e.printStackTrace();
            throw new FatalDeploymentException(e);
        }
        return home;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ================================================================== 
    /**
     * Make the target destination deployer process all the dependencies
     * of the implementation.
     * 
     * @throws FatalDeploymentException if a problem occured
     *  during processing of dependencies
     */
    public void 
    resolveDependency() 
    throws FatalDeploymentException 
    {
        try {
            /*config_values.addAll(Arrays.asList(getSoftpkgDeployer()
                                                .processDependencies
                                                    (getComponentInstallation())));*/
            getSoftpkgDeployer().visitDependencyDeployers(depVisitor);
            /*  List l=Arrays.asList(getImplementationDeployer()
                                   .install(getComponentInstallation()));
               
               config_values.addAll(l);*/
            getImplementationDeployer().visitDependencyDeployers(depVisitor);
            getImplementationDeployer().visitCodeDeployers(depVisitor);

            try {
                String url =
                    startHttpServer(
                        getSoftpkgDeployer().getComponentArchiveStream(),
                        getSoftpkgDeployer().getSoftpkg().getName() + ".car");
                getComponentInstallation().install(
                    getImplementationDeployer().getImplementation().getId(),
                    url);

            } catch (IOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
                throw new FatalDeploymentException(
                    e,
                    "Cannot install implementation in target node");
            } catch (InvalidLocation e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
                throw new FatalDeploymentException(
                    e,
                    "Cannot install implementation in target node");
            } catch (InstallationFailure e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
                throw new FatalDeploymentException(
                    e,
                    "Cannot install implementation in target node");
            }

        } catch (DependencyResolutionFailureException e) {
            getErrorManager().submitException(e);
        } catch (CodeInstallationFailureException e) {
            e.printStackTrace();
            getErrorManager().submitException(e);
        }
    }

    /**
     * InstallHome install the home on the precised destination according
     * to the host requirements, such as dependencies and event factory.
     * 
     * @return The home installed
     * @throws FatalDeploymentException
     */
    public CCMHome 
    installHome() 
    throws FatalDeploymentException 
    {

        ConfigValue[] config_home =
        /*(ConfigValue[])
           config_values.toArray(new ConfigValue[config_values.size()]);*/
        depVisitor.getConfigValue();

        return installCCMHome(
            new ConfigValue[0],
            new ConfigValue[0],
            config_home,
            getImplementationDeployer().getImplementation().getId(),
            depVisitor.getEntryPoint());
    }

    /**
     * Return the component installation of the target host used
     * to install the home.
     * 
     * @return The component installation of the target host
     * @throws FatalDeploymentException if a fatal problem occured during installation
     */ 
    public ComponentInstallation 
    getComponentInstallation()
    throws FatalDeploymentException 
    {
        return getCI();
    }

    /**
     * Remove a previously installed CCMHome
     * 
     * @param home the home to remove
     * @throws FatalDeploymentException if a fatal problem occured during uninstallation
     */
    public void 
    removeHome(CCMHome home)
    throws FatalDeploymentException 
    {
        try {
            Container cont = (Container) containerTable.remove(home);
            if (cont == null)
                return;
            if (cont.get_homes().length == 0)
                cont.remove();
            cont.remove_home(home);

        } catch (Exception e) {
            e.printStackTrace();
            throw new FatalDeploymentException(e);
        }
    }

}
