/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): Philippe Merle.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.installer;

import org.objectweb.ccm.descriptor.componentassembly.beans.NodeBean;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.InitializationError;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.ChildDeployerContext;

/**
 * The nodedeployer context defines all the connection requires for
 * search and resolve a node using DCIInformation service.
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class NodeDeployerContext 
       extends ChildDeployerContext 
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    private NodeBean nodeBean;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public accessors methods.
    //
    // ==================================================================

    /**
     * @return
     */    
    public NodeBean 
    getNode() 
    {
        return nodeBean;
    }

    /**
     * @param bean
     */
    public void 
    connectNode(NodeBean bean) 
    {
        nodeBean = bean;
        connectDeployerDescription(bean);
    }
    
    /**
     * This initialization method is used to check that all the services and 
     * information required to deploy are presents. If all is OK the deployer 
     * is put in inactive deployer state waiting for the deployment processing.
     * 
     * @exception InitializationError is thrown if a problem is detected 
     */     
    public void initialize() throws InitializationError {
        if (getNode() == null)
            getErrorManager().submitException(
                new InitializationError(
                    this,
                    "Connection on node bean is required"
                        + "check the handler code it must specify this connection"));
        if (getNode().getName() == null || getNode().getName().length() == 0)
            getErrorManager().submitException(
                new InitializationError(
                    this,
                    " The name attribute of node element must be fullfilled"));
        //ajouter verif sur le formatage de la destination                                 
        getLifeCycleManager().stepEndedSuccessFully();
    }

}
