/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.ccm.descriptor.softpkg.ccm;

import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.ChildDeployerContext;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.InitializationError;
import org.objectweb.ccm.descriptor.corbacomponent.ccm.CorbacomponentDeployer;
import org.objectweb.ccm.descriptor.corbacomponent.beans.CorbacomponentBean;
import org.objectweb.ccm.descriptor.softpkg.beans.DescriptorBean;

/**
 * The descriptor deployer context defines all the connection requires for
 * descriptor information reading.
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public abstract class DescriptorDeployerContext 
       extends ChildDeployerContext 
    {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    private DescriptorBean descriptor;
    private FileinarchiveDeployer fileinarchiveDeployer;
    private CorbacomponentDeployer corbaComponentDeployer;
    private CorbacomponentBean corbacomponent;
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================    
    /**
     * @return
     */
    public DescriptorBean 
    getDescriptor() 
    {
        return descriptor;
    }

    /**
     * @return
     */
    public FileinarchiveDeployer 
    getFileinarchiveDeployer() 
    {
        return fileinarchiveDeployer;
    }

    /**
     * @param bean
     */
    public void 
    connectDescriptor(DescriptorBean bean) 
    {
        descriptor = bean;
        connectDeployerDescription(bean);
    }

    /**
     * @param deployer
     */
    public void 
    connectFileinarchiveDeployer(FileinarchiveDeployer deployer) 
    {
        fileinarchiveDeployer = deployer;
    }

    /**
     * @return
     */
    public CorbacomponentDeployer 
    getCorbaComponentDeployer() 
    {
        return corbaComponentDeployer;
    }

    /**
     * @param deployer
     */
    public void 
    connectCorbaComponentDeployer(CorbacomponentDeployer deployer) 
    {
        corbaComponentDeployer = deployer;
    }

    /**
     * @return
     */
    public CorbacomponentBean 
    getCorbacomponent() 
    {
        return corbacomponent;
    }

    /**
     * @param bean
     */
    public void 
    connectCorbacomponent(CorbacomponentBean bean) 
    {
        corbacomponent = bean;
    }
    /**
     * Initialization method to load the sub corbacomponent tree
     * 
     * @throws CorbaComponentFileLoadingFailureException if the file 
     * cannot be load
     */    
    public  abstract void 
    initializeCorbaComponentTree()
    throws CorbaComponentFileLoadingFailureException;
    
    /**
     * This initialization method is used to check that all the services and 
     * information required to deploy are presents. If all is OK the deployer 
     * is put in inactive deployer state waiting for the deployment processing.
     * 
     * @exception InitializationError is thrown if a problem is detected 
     */
    public void initialize() throws InitializationError {
        if(getDescriptor()==null)
                 getErrorManager()
                 .submitException(new InitializationError(this,"Connection on descriptor bean is required"+
                                                          "check the handler code it must specify this connection"));
       initializeCorbaComponentTree();
       getLifeCycleManager().stepEndedSuccessFully();
    }



}
