/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.ccm.descriptor.softpkg.ccm;

import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.HandlerBase;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.InitializationError;
import org.objectweb.ccm.descriptor.corbacomponent.ccm.CorbacomponentDeployer;
import org.objectweb.ccm.descriptor.softpkg.beans.DescriptorBeanImpl;

/**
 * This handler is used to produce descriptor deployer from an
 * descriptor bean.
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class DescriptorDeployerHandler 
       extends HandlerBase 
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================    
    private DescriptorDeployer _deploy;
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================
    /**
     * Produce the descriptor deployer binded
     * to the given @param comp.
     * 
     * @param comp The descriptor bean 
     * @return the deployer produced in initialize state
     * @throws InitializationError thrown if a problem is detected
     */  
    public Object 
    getInstance(DescriptorBeanImpl comp)
    throws InitializationError 
    {
        if (_deploy == null) {
            _deploy = new DescriptorDeployer();
            _deploy.connectDescriptor(comp);
            applyCommonConfig(_deploy);

            if (comp.getFileinarchive() != null)
                _deploy.connectFileinarchiveDeployer(
                    (FileinarchiveDeployer) getDeployer(comp
                        .getFileinarchive()));
            try {
                _deploy.initialize();
                if (_deploy.getCorbacomponent() != null)
                    _deploy.connectCorbaComponentDeployer(
                        (CorbacomponentDeployer) getDeployer(_deploy
                            .getCorbacomponent()));
            } catch (CorbaComponentFileLoadingFailureException e) {
                _deploy.getErrorManager().submitException(e);
            }

        }
        return (_deploy);
    }
}
