/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey, Philippe Merle.
Contributor(s): .

====================================================================*/

package org.objectweb.corba.command.lib;

// Package dependencies.
import org.objectweb.corba.command.api.CommandOnAST;
import org.objectweb.util.cmdline.api.CommandLine;

/**
 * Abstract base class for all commands working with the OpenCCM Abstract Syntax Tree.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 1.2
 */

public abstract class CommandOnASTBase
              extends CommandOnIR3Base
           implements CommandOnAST
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The Abstract Syntax Tree. */
    private org.objectweb.corba.ast.api.AST ast_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the command line manager.
     *
     * @param commandLine The command line manager.
     */
    public
    CommandOnASTBase(CommandLine commandLine)
    {
        // Calls the CommandOnIR3Base constructor.
        super(commandLine);

        // Inits internal state.
        ast_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cmdline.api.Application
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.Application
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.CommandOnIR3
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.CommandOnAST
    //
    // ==================================================================

    /**
     * Inits the OpenCCM Abstract Syntax Tree.
     *
     * @return True if ok, else false.
     */
    public boolean
    initAST()
    {
        // Checks availability of the OpenCCM Interface Repository.
        if(!checkComponentRepository())
            return false;

        // Creates the OpenCCM Abstract Syntax Tree.
        ast_ = new org.objectweb.corba.ast.lib.Repository(getComponentRepository());

        return true;
    }

    /**
     * Obtains the OpenCCM Abstract Syntax Tree.
     *
     * @return The OpenCCM Abstract Syntax Tree.
     */
    public org.objectweb.corba.ast.api.AST
    getAST()
    {
        return ast_;
    }

    // ==================================================================
    //
    // Other public methods.
    //
    // ==================================================================
}
