/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.corba.generator.psdl.lib;

/** To access AST. */
import org.objectweb.corba.ast.api.AbstractStorageHomeDecl;
import org.objectweb.corba.ast.api.AbstractStorageTypeDecl;
import org.objectweb.corba.ast.api.StorageHomeKeyDecl;
import org.objectweb.corba.ast.api.StorageHomeFactoryDecl;
import org.objectweb.corba.ast.api.PsdlOperationDecl;
import org.objectweb.corba.ast.api.Declaration;
import org.objectweb.corba.ast.api.DeclarationKind;

/** To access Java AST. */
import org.objectweb.corba.generator.java.ast.api.*;
import org.objectweb.corba.generator.java.ast.lib.*;

/** Others. */
import java.util.Vector;
import org.objectweb.corba.generator.common.lib.GenerationException;


/**
 * This class generates Abstract Storage Home Mapping.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</A>
 */

public class AbstractStorageHomeMapping
  implements org.objectweb.corba.generator.psdl.api.JavaMappingObject
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The Abstract Storage Home declaration. */
    private AbstractStorageHomeDecl ash_;

    // ===========================================================
    //
    // Constructors.
    //
    // ===========================================================

    /**
     * The default constructor.
     *
     * @param ash - The Abstract Storage Home to map.
     */

    public AbstractStorageHomeMapping(AbstractStorageHomeDecl ash)
    {
        // Init internal state
        ash_ = ash;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Map an Abstract Storage Home definition into a java Interface.
     *
     * @param generator - The PSDL to Java generator.
     */
    public void
    toJava(org.objectweb.corba.generator.psdl.api.PSDL2JavaGenerator generator)
    throws GenerationException
    {
        InterfaceObject itf = null;
        MethodObject method = null;
        AbstractStorageHomeDecl[] inherited = null;
        Vector vect = null;
        PsdlOperationDecl[] operations = null;
        AbstractStorageTypeDecl ast = null;
        StorageHomeKeyDecl[] keys = null;
        StorageHomeFactoryDecl[] factories = null;
        Declaration decl = null;

        itf = new InterfaceObjectImpl(ash_.getName());
        itf.addComment(" Abstract Storage Home "+ash_.getName());
        itf.addComment("");
        itf.addComment(" @author OpenCCM PSDL Compiler");
        itf.setModifier(ModifierKindImpl.mk_public);
        itf.setPackage(generator.getTranslator().getPackage(ash_));

        // Add Inherited Abstract Storage Home by ash
        inherited = ash_.getAbstractStorageHomeList().getAbstractStorageHomes();
        for (int i=0; i<inherited.length; i++)
            itf.addInheritedObject(generator.getTranslator().getAbsoluteName(inherited[i]));

        if (inherited.length < 1)
            itf.addInheritedObject("org.omg.CosPersistentState.StorageHomeBase");

        generator.getJavaRepository().addObject(itf);

        // Get the managed Abstract Storage Type
        ast = ash_.getAbstractStorageType();

        // Map Factories
        vect = generator.getDeclarations(ash_, DeclarationKind.dk_storage_home_factory);
        factories = (StorageHomeFactoryDecl[])vect.toArray(new StorageHomeFactoryDecl[0]);
        for (int i=0; i<factories.length; i++)
        {
            org.objectweb.corba.generator.psdl.api.PSDLFactoryMapping factory
                = new PSDLFactoryMapping(factories[i]);
            factory.toJava(generator, itf, ast, null);
        }

        // Map Keys
        vect = generator.getDeclarations(ash_, DeclarationKind.dk_storage_home_key);
        keys = (StorageHomeKeyDecl[])vect.toArray(new StorageHomeKeyDecl[0]);
        for (int i=0; i<keys.length; i++)
        {
            org.objectweb.corba.generator.psdl.api.PSDLKeyMapping key
                = new PSDLKeyMapping(keys[i]);
            key.toJava(generator, itf, ast);
        }

         // Map Operations
        vect = generator.getDeclarations(ash_, DeclarationKind.dk_psdl_operation);
        operations = (PsdlOperationDecl[])vect.toArray(new PsdlOperationDecl[0]);
        for (int i=0; i<operations.length; i++)
        {
            org.objectweb.corba.generator.psdl.api.PSDLOperationMapping op
                = new PSDLOperationMapping(operations[i]);
            op.toJava(generator.getTranslator(), itf);
        }
   }

}
