/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Offroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.trace.PI;


/**
 * @author Jerome Offroy
 *
 *class used to read CORBA::ServerRequestInfo
 *
 */
//TODO change logLevel to a more configurable way to get CORBA information about current call
public class ServerRequestInfoReader extends RequestInfoReader {

    /**
     * constant defining the more verbose server Loglevel
     */
    final static private short logLevelServerStart = 2;

    /**
     * Constructor 
     * @param logLevel
     **/
    public ServerRequestInfoReader(short logLevel) {
        super(logLevel);
    }

    /************************************************************************
     * Public Method 
     ***********************************************************************/
    /**
     * Method displayServerRequestInfoMONOLOG.
     * @param info
     *  @return String 
     **/
    public String displayServerRequestInfoMonolog(org.omg.PortableInterceptor.ServerRequestInfo info) {
            return displayServerRequestInfoMonolog(info,true);
    }
        
    /**
     * Method displayServerRequestInfoMONOLOG.
	 * @param info
	 * @param withException
	 * @return String
	 */
	public String displayServerRequestInfoMonolog(org.omg.PortableInterceptor.ServerRequestInfo info, boolean withException) {
        String res = "";
        res = res + displayRequestInfo(info, logLevel) ;
        if (withException) res = res + displayException(info);
        if (logLevel > logLevelServerStart) {
            res = res + displayObjectID(info) + displayAdapterID(info);
        }
        res = res + displayMostDerivedInterface(info);

        return res;
    }

    /************************************************************************
     * Protected Method 
     ***********************************************************************/

    /**
     * Display PortableInterceptor::ServerRequestInfo ObjectID
     * @param info
     * @return String 
     **/
    protected String displayObjectID(org.omg.PortableInterceptor.ServerRequestInfo info) {
        String res = "";
        try {
            byte[] octets = info.object_id();
            res = res + "object id = ";
            for (int i = 0; i < octets.length; i++)
                res = res + getOctet(octets[i]);
        } catch (org.omg.CORBA.BAD_INV_ORDER ex) {
            // res = res + "Operation not supported in this context";
        }
        return res;
    }

    /**
     * Display PortableInterceptor::ServerRequestInfo AdapterID
     * @param info
     * @return String 
     **/
    protected String displayAdapterID(org.omg.PortableInterceptor.ServerRequestInfo info) {
        String res = "";
        try {
            byte[] octets = info.adapter_id();
            res = res + "adapter id = ";
            for (int i = 0; i < octets.length; i++)
                res = res + getOctet(octets[i]);
        } catch (org.omg.CORBA.BAD_INV_ORDER ex) {
            //res = res + "Operation not supported in this context";
        }
        return res;
    }

    /**
     * Display PortableInterceptor::ServerRequestInfo MostDerivedInterface
     * @param info
     * @return String 
     **/
    protected String displayMostDerivedInterface(org.omg.PortableInterceptor.ServerRequestInfo info) {
        String res = "";
        try {
            String tmdi = info.target_most_derived_interface();
            res = res + "\ttarget most derived interface = " + tmdi;
        } catch (org.omg.CORBA.BAD_INV_ORDER ex) {
            // Operation not supported in this context
        }
        return res;
    }

    /************************************************************************
     * Abstract implementation  Method 
     ***********************************************************************/

    protected String displayMore(org.omg.PortableInterceptor.RequestInfo info) {
        return displayServerRequestInfoMonolog((org.omg.PortableInterceptor.ServerRequestInfo) info);
    }

    protected String getLabelException() {
        return "sending exception = ";
    }

    protected org.omg.CORBA.Any requestOpException(org.omg.PortableInterceptor.RequestInfo info) {
        return ((org.omg.PortableInterceptor.ServerRequestInfo) info).sending_exception();
    }
}