# ====================================================================
#
# OpenCCM: The Open CORBA Component Model Platform
# Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
# Contact: openccm@objectweb.org
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
# Initial developer(s): Pierre Carpentier.
# Contributor(s): ______________________________________.
#
# ====================================================================
#
#  This class parses the model to check if it is valid, ie it
#  respects all the UML Profile for CORBA 2 and CORBA 3 constraints.
#
# ====================================================================

import ConstraintException;
import UMLOperations;

class ModelChecker
{

    # =================================================================
    # Empty constructor for the ModelChecker.
    # =================================================================
    proc __ModelChecker__(self)
    {
        self.umlOperations = UMLOperations.UMLOperations();
    }


    # =================================================================
    # Checks if the model is correct.
    # umlPackage - The UML Package
    # =================================================================
    proc checkModel(self)
    {
        JAVA.declare("org.objectweb.corba.uml.lib");
        self.umlPackage = org.objectweb.corba.uml.lib.TheUMLPackage.getUMLPackage();
        self.umlOperations.setUMLPackage(self.umlPackage);
        exceptions = self.umlPackage.getCommon_Behavior().getUmlException().refAllOfType().toArray();
        classes = self.umlPackage.getCore().getUmlClass().refAllOfType().toArray();
        associations = self.umlPackage.getCore().getAssociation().refAllOfType().toArray();
        try {
            self.check(exceptions);
            self.check(classes);
            self.check(associations);
        } catch (ConstraintException.ConstraintException ce) {
            ce.show();
            throw ce;
        }
    }


    # =================================================================
    # Checks if an element of the model is correct.
    # modelElements - The element
    # =================================================================
    proc check (self, modelElements)
    {
        for element in modelElements {
            stereotypes = element.getStereotype().toArray();
            for stereotype in stereotypes {
                eval("self.check" + stereotype.getName() + "(element);");
            }
        }
    }

}
