/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.openccm.OpenCCM_DCI.cif;

import org.objectweb.openccm.OpenCCM_DCI.*;
import org.objectweb.ccm.CORBA.*;

/**
 * This is the implementation of the AssemblyFactory facet provided by 
 * AssemblyFactoryManager component.
 *
 * This class inherits from the skeleton
 * generated by the OpenCCM's CIF to Java mapping generator.
 *
 * @author <a href="mailto:Frederic.Briclet@lifl.fr">Briclet Frederic</A>
 */
public class AssemblyFactorySegImpl 
  extends  org.objectweb.openccm.OpenCCM_DCI.
  AssemblyFactoryManagerSessionComposition.AssemblyFactorySeg
{
    // ==================================================================
    //
    // Internal state
    //
    // ==================================================================
    org.objectweb.openccm.OpenCCM_DCI.AssemblyFactoryManagerCCM AFM;
 

    // ==================================================================
    //
    // Internal method
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================
    public AssemblyFactorySegImpl(){
        super();                 
    }
    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    public void 
    setup()
    {
        AFM=(org.objectweb.openccm.OpenCCM_DCI.AssemblyFactoryManagerCCM)
            get_context(); 
    }
    
    // ==================================================================
    //
    // AssemblyFactory methods.
    //
    // ==================================================================

    /**
     * This method create an assembly taking in parameter a 
     * assembly file location.
     * @param assembly_loc The assembly location
     */
    public org.omg.Components.Cookie 
    create_assembly(String assembly_loc)
    throws org.omg.Components.Deployment.InvalidLocation,
           org.omg.Components.CreateFailure
    {       
        try{
            //instantiate the assembly manager
            org.objectweb.openccm.OpenCCM_DCI.AssemblyManager am=
                AFM.get_connection_assembly_manager_home()
                .create_assembly_manager(assembly_loc);
          
            //connect the homefinder service
            am.connect_home_finder(AFM.get_connection_home_finder());
            //connect the HomeRegistration interface            
            am.connect_home_registration(AFM.get_connection_home_registration());
            //connect the nameservice service
            am.connect_naming_service(AFM.get_connection_naming_service());
            //connect the trading service
            //am.connect_trader_service(AFM.get_connection_trader_service());
            am.connect_dci_information(AFM.get_connection_dci_information());
            //connect the AssemblyManagerRegistration interface
            am.connect_assembly_manager_registration(AFM.provide_assembly_manager_registration());   
            //configure complete the AssemblyManager
            // This one will register itself with AssemblyManagerRegistration
            // and will be connect to AssemblyFactoryManager
            am.configuration_complete();
            //Retrieve all connection on AssemblyManager
            org.objectweb.openccm.OpenCCM_DCI.
                AssemblyFactoryManagerPackage.assembly_managersConnection 
                connect[]=AFM.get_connections_assembly_managers();
            // Compare AssemblyManager ref to retrieve connection cookie
            for(int i=0;i<connect.length;i++){
                if(connect[i].objref._is_equivalent(am))
                    return connect[i].ck;
            }
            
        }
        catch(Exception e){
            e.printStackTrace();
            System.err.println(e.getMessage());
            System.err.println("Cannot create the assembly");        
      }        
        throw new org.omg.Components.CreateFailure();
    }   

    
    /**
     * org.omg.Components.Deployment.Assembly lookup method
     * implementation. Returned the assembly binded to the 
     * cookie
     * @param c The assembly cookie 
     * @return  The assembly found
     */
    public org.omg.Components.Deployment.Assembly 
    lookup(org.omg.Components.Cookie c)
    throws org.omg.Components.Deployment.InvalidAssembly
    {
        //Retrieve all the AssemblyManagerconnection
        org.objectweb.openccm.OpenCCM_DCI.
            AssemblyFactoryManagerPackage.assembly_managersConnection 
            connect[]=AFM.get_connections_assembly_managers();
        //Compare all cookie to find the right assembly
        for(int i=0;i<connect.length;i++)
            {
                if(connect[i].ck.equals(c))
                    return connect[i].objref.provide_assembly();
            }
        //The assembly was not found due to a bad cookie
        throw new org.omg.Components.Deployment.InvalidAssembly();        
    }
    
    /**
     * org.omg.Components.Deployment.Assembly destroy(cookie c) method
     * implementation. Destroy the assembly binded to the given in 
     * parameter cookie.
     * @param c The Assembly cookie
     */      
    public  void
    destroy(org.omg.Components.Cookie c)
    throws org.omg.Components.Deployment.InvalidAssembly,
           org.omg.Components.RemoveFailure
    {
        //Retrieve all AssemblyManager connections.
        org.objectweb.openccm.OpenCCM_DCI.
            AssemblyFactoryManagerPackage.assembly_managersConnection 
            connect[]=AFM.get_connections_assembly_managers();
        //Compare all connection cookie to found the right
        // AssemblyManager and remove it.
        for(int i=0;i<connect.length;i++){
            if(connect[i].ck.equals(c))
              connect[i].objref.remove();
        }   
    }  


    // ==================================================================
    //
    // AssemblyFactoryExt methods.
    //
    // ==================================================================

    
    /**
     * This method create an assembly taking in parameter a 
     * the XML code to process.
     * @param xml the assembly descriptor XML code. 
     */
    public org.omg.Components.Cookie    
    create_assembly_XML(java.lang.String xml)
    throws org.omg.Components.CreateFailure
    {

        try{
            // create the AssemblyManager
            org.objectweb.openccm.OpenCCM_DCI.AssemblyManager am=
                AFM.get_connection_assembly_manager_home()
                .create_assembly_manager_xml(xml);
                        //connect the homefinder service
            am.connect_home_finder(AFM.get_connection_home_finder());
            //connect the HomeRegistration interface            
            am.connect_home_registration(AFM.get_connection_home_registration());
            //connect the nameservice service
            am.connect_naming_service(AFM.get_connection_naming_service());
            //connect the trading service
            //am.connect_trader_service(AFM.get_connection_trader_service());
            am.connect_dci_information(AFM.get_connection_dci_information());
            //connect the AssemblyManagerRegistration interface
            am.connect_assembly_manager_registration(AFM.provide_assembly_manager_registration());   
            //configure complete the AssemblyManager
            // This one will register itself with AssemblyManagerRegistration
            // and will be connect to AssemblyFactoryManager
            am.configuration_complete();
            //Retrieve all connection on AssemblyManager
            org.objectweb.openccm.OpenCCM_DCI.
                AssemblyFactoryManagerPackage.assembly_managersConnection 
                connect[]=AFM.get_connections_assembly_managers();
            // Compare AssemblyManager ref to retrieve connection cookie
            for(int i=0;i<connect.length;i++){
                if(connect[i].objref._is_equivalent(am))
                    return connect[i].ck;
            }
           
        }
        catch(Exception e){
            e.printStackTrace();
            System.err.println(e.getMessage());
            System.err.println("Cannot create the assembly");        
        }
        
        throw new org.omg.Components.CreateFailure();
    }
    
    /**
     * This method allow to retrieve the AssemblyManager binded 
     * with the given in parameter cookie.
     * @param c The AssemblyManager cookie
     * @return  The AssemblyManager found.
     */
    public org.objectweb.openccm.OpenCCM_DCI.AssemblyManager
    lookup_assembly_manager(org.omg.Components.Cookie c)
    throws org.omg.Components.Deployment.InvalidAssembly
    {
        //Retrieve all the AssemblyManagerConnections
        org.objectweb.openccm.OpenCCM_DCI.
            AssemblyFactoryManagerPackage.assembly_managersConnection 
            connect[]=AFM.get_connections_assembly_managers();
        //Compare all cookie to found the right AssemblyManager
        for(int i=0;i<connect.length;i++){
            if(connect[i].ck.equals(c))
                return (org.objectweb.openccm.OpenCCM_DCI.AssemblyManager)
                    connect[i].objref;
        }      
        //The AssemblyManager was not found due to a bad cookie.
        throw new org.omg.Components.Deployment.InvalidAssembly();
    }
    
    /**
     * Destroy the current assembly factory manager including 
     * all the created assembly managers 
     */
     public void
     destroy_assembly_factory()
     throws org.omg.Components.RemoveFailure
     {
        //Retrieve all AssemblyManager connections.
        org.objectweb.openccm.OpenCCM_DCI.
            AssemblyFactoryManagerPackage.assembly_managersConnection 
            connect[]=AFM.get_connections_assembly_managers();
        //Compare all connection cookie to found the right
        // AssemblyManager and remove it.
        for(int i=0;i<connect.length;i++){
              connect[i].objref.remove();
        }   	
     }
}
