/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/



package org.objectweb.openccm.OpenCCM_DCI.cif;

//Package dependencies
import java.net.URL;

import org.objectweb.openccm.OpenCCM_DCI.*;
import org.objectweb.ccm.Deployment.AssemblyBase;
import org.objectweb.ccm.Deployment.AssemblyConfigurator;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.root.*;
import org.objectweb.ccm.descriptor.componentassembly.beans.*;
import org.objectweb.ccm.descriptor.*;
import org.omg.Components.Deployment.Assembly;

/**
 * AssemblyImpl is the top level implementation of the 
 * assembly behaviour. In the DCI implementation the assembly interface
 * is provided by a AssemblyManager component. So many connection of 
 * this component are used for the initialization of the deployment 
 * process.
 *
 * @author <a href="mailto:frederic.briclet@lifl.fr">Briclet Frederic</a>
 *
 * @version 0.1
 */
public class AssemblySegImpl 
    extends  org.objectweb.openccm.OpenCCM_DCI.
             AssemblyManagerSessionComposition.AssemblySeg
    implements AssemblyConfigurator
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    // xml handler path if 
    private String xmlFilePath;
    
    // Console for emittion of message
    private org.objectweb.util.cmdline.api.Console console;
    // State of the assembly
    private AssemblyBase etat;
    
    // The assembly manager component
    private AssemblyManager am;   

    private java.util.zip.ZipFile zipFile;
    private java.io.File tmpZipFile;
    private java.io.InputStreamReader reader;  
    //Root  of xml tree cad representation
    private ComponentassemblyBean assemblyObject;
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
       // ==================================================================
    //
    // Internal class.
    //
    // ==================================================================
    public ComponentassemblyDeployerHandler
    configure(ComponentassemblyDeployerHandler context){
        context.connectHomefinder( get_context().get_connection_home_finder()) ;
        context.connectHomeregistration(get_context().get_connection_home_registration());
        context.connectNamingcontext(get_context().get_connection_naming_service());
        context.connectDciinfo(get_context().get_connection_dci_information());
        context.connectAssemblyManager(am);
        context.connectZipEntryRetriever(new ZipEntryRetriever(zipFile));
        return context;                      
    }
 
   public Assembly getAssembly(){
       return 
       ((org.objectweb.openccm
        .OpenCCM_DCI.AssemblyManagerCCM)get_context())
        .provide_assembly();
   }
    
   public void removeAssembly(){
          
   }
   
   public void 
   deleteTemporaryFile()
   {
    
    try{
        reader.close();
        zipFile.close();
    }catch(Exception e)
    {
        //TODO correct the lose of temporary file
        //System.err.println("ZipFile cannot be closed");
    }

      if(!tmpZipFile.delete())
             tmpZipFile.deleteOnExit();
           
   }
   
   public ComponentassemblyBean getComponentassemblyTree()    
      {
          try{
          // get the ORB
         org.omg.CORBA.ORB orb
             = org.objectweb.ccm.CORBA.TheORB.getORB();
         //console.message("Got the ORB");

         // inits OpenCCM runtime
         org.objectweb.ccm.Components.Runtime.init (orb);
         //console.message("OpenCCM runtime is ready");

         java.io.InputStreamReader reader ;
         
          java.net.URL url= new URL(xmlFilePath);          
          tmpZipFile =java.io.File.createTempFile(System.currentTimeMillis()+"","aar");
          java.io.FileOutputStream fos= new java.io.FileOutputStream(tmpZipFile);

    
           java.io.InputStream bis=url.openStream();
   
           byte [] byteArray= new byte[32000];
           int readed=readed=bis.read(byteArray,0,32000);                        
    
           System.err.print("Downloading "+xmlFilePath+" [");
           while(readed>0)                      
               {   
                   fos.write(byteArray,0,readed);
                   readed=bis.read(byteArray,0,32000);
                   System.err.print("*");
               }
           System.err.println("]");
           fos.flush();
           fos.close();
    
           zipFile= 
               new java.util.zip.ZipFile(tmpZipFile);
                                   
       
        
         for(java.util.Enumeration entries=zipFile.entries();entries.hasMoreElements();)
             {
        
                 java.util.zip.ZipEntry ze=(java.util.zip.ZipEntry)entries.nextElement();
                 //System.out.println("Filename:"+ze.getName());
                 if(ze.getName().endsWith(".cad"))
                     {
                         System.err.println("AssemblyManager>>File ["+ze.getName()+"] is used for deployment");
                         reader=new java.io.InputStreamReader
                             (zipFile.getInputStream(ze));
                        
                        assemblyObject= org.objectweb.ccm.descriptor
                             .componentassembly.beans.ComponentassemblyBeanImpl
                             .unmarshalBean(reader,true);
                        return assemblyObject;     
                     }                        
             }
        }
        catch(Exception e){
                e.printStackTrace(); 
        }
        return assemblyObject;
      }
    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================


    /**
     * Method to setup the path to the XML file to process
     */     
    public void 
    setupXMLFilePath(String xmlFilePath)
    {        
        this.xmlFilePath=xmlFilePath;    
        etat=new AssemblyBase(this);
    }
    
    public void
    setupAssemblyManager(org.objectweb.openccm.OpenCCM_DCI.AssemblyManager am)
    {
        this.am=am;
    }   
        
    public void registerAssembly(){
        etat.registerAssembly();
    }
    
    public void unregisterAssembly(){
        try{
            zipFile.close();
            tmpZipFile.delete();
        }
        catch(Exception e)
        {
            e.printStackTrace();
            //Not a problem tmp file will be removed by the OS
        }

        etat.unregisterAssembly();
    }

    // ==================================================================
    //
    // Assembly interface methods    
    //
    // ==================================================================

    /*Delegation operation to the state assembly*/
    public void 
    build() 
    throws org.omg.Components.CreateFailure
    {
        etat.build();    
    }
    
    public void 
    tear_down() 
    throws org.omg.Components.RemoveFailure 
    {
        etat.tear_down();    
    }
    
    
    public org.omg.Components.Deployment.AssemblyState 
    get_state()
    {
        return etat.get_state();
    }
    
    public String 
    getName()
    {
        try{
            return assemblyObject.getId();
        }
        catch(Exception e){
            e.printStackTrace();
            return "AssemblyManager";
        }
    }
    
}
