/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  Philippe Merle.

====================================================================*/
package org.objectweb.openccm.browser.CORBA;

/** To use the ORB class */
import org.omg.CORBA.ORB;

/** The administration console's API imports */
import org.objectweb.util.browser.core.naming.DecoderHandler;

/**
 * This class allows to decode a CORBA stub and 
 * provide the associated object.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class CorbaIORDecoderHandler extends DecoderHandler {

    /** To object's class concern by the handler */
    protected final Class class_ = org.omg.CORBA.Object.class;

    /** The current ORB */
    protected ORB orb_ = null;

    /**
     * Default Constructor
     */
    public CorbaIORDecoderHandler() {
        orb_ = org.objectweb.ccm.CORBA.TheORB.getORB();
    }

    /**
     * Uses to decode an object, for example if it's a CORBA Object
     * @param node The object to decode
     * @return the decoded object
     */
    public Object decode(Object node) {
        //
        // Related to Bug #300847.
        // Contributor: Philippe Merle
        //
        // Deal with null CORBA Object References.
        //
        if(node == null)
            return new String("NULL CORBA OBJECT REFERENCE");

        if (class_.isAssignableFrom(node.getClass())) {
            try {
                TypageCORBA typeCORBA = new TypageCORBA((org.omg.CORBA.Object) node, orb_);
                Class[] argsType = new Class[] { org.omg.CORBA.Object.class };
                Object[] args = new Object[] { node };
                Class c = typeCORBA.getJavaHelperClass();
                if (c != null) {
                    Object o = c.getMethod("narrow", argsType).invoke(null, args);
                    if (!node.getClass().getName().equals(o.getClass().getName()))
                        return o;
                }
            } catch (NoSuchMethodException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            } catch (java.lang.reflect.InvocationTargetException e) {
                e.printStackTrace();
            } catch (ObjectToStringNotSupportedException e) {
                // Ignore
            }
        } else if (next_ != null) {
            return next_.decode(node);
        }
        return node;
    }

}
