/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.browser.Components;

/** The consoles's imports */
import org.objectweb.openccm.browser.CORBA.TypageCORBA;
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.api.Context;
import org.objectweb.util.browser.api.Wrapper;
import org.objectweb.util.browser.core.naming.DefaultEntry;
import org.objectweb.util.browser.core.naming.DefaultName;
import org.omg.Components.PublisherDescription;
import org.omg.Components.SubscriberDescription;

/**
 * This context displays the available connections of the given publisher.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class PublisherContext 
    implements Context, Wrapper {

    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /** The PublisherContainer which contains the PublisherDescription to browse */
    protected PublisherContainer publisher_ = null;
    
    // ==================================================================
    //
    // No constructor.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    protected String getName(org.omg.CORBA.Object o) {
        TypageCORBA tc = new TypageCORBA(o, org.objectweb.ccm.CORBA.TheORB.getORB());
        return tc.getTypeID();
    }

    // ==================================================================
    //
    // Public methods for interface Context.
    //
    // ==================================================================

    public Entry[] getEntries() {
        PublisherDescription p = (PublisherDescription) publisher_.getPublisher();
        SubscriberDescription[] sd = p.consumers;
        Entry[] values = new Entry[sd.length];
        for (int i = 0; i < sd.length; i++) {
            SubscriberContainer sc = new SubscriberContainer();
            sc.setSubscriber_(sd[i]);
            sc.setPublisherContainer_(publisher_);
            values[i] = new DefaultEntry(sc, new DefaultName("Connection (" + i + ")"), this);
        }
        return values;
    }

    // ==================================================================
    //
    // Additional public methods for Wrapper interface.
    //
    // ==================================================================

    /**
     * Fix the wrapped object
     * @param object The object to wrap
     */
    public void setWrapped(Object object) {
        publisher_ = (PublisherContainer) object;
    }

    /**
     * Gets the wrapped object
     * @return The wrapped object
     */
    public Object getWrapped() {
        return publisher_;
    }

}
