/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/

package org.objectweb.openccm.browser.DCI;

/** The Browser's imports */
import java.util.Vector;

import org.objectweb.util.browser.api.TreeView;
import org.objectweb.util.browser.api.Table;
import org.objectweb.util.browser.core.naming.DefaultName;
import org.objectweb.util.browser.core.naming.DefaultEntry;

/** The CCM's imports */
import org.ist.coach.DCI.DCIManager;
import org.ist.coach.DCI.ExtHomeFinder;
import org.ist.coach.DCI.HomeInfo;

/**
 * Displays the homes for the given DCIManager.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class DCIAvailableHomesTable
  implements Table 
{

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================
    
    //==================================================================
    //
    // No constructor.
    //
    //==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ================================================================== 
        
    //==================================================================
    //
    // Public methods for Table interface.
    //
    //==================================================================

    /**
     * Overriding methods
     * @see org.objectweb.util.browser.api.Table#getHeaders()
     */
    public String[] getHeaders(TreeView treeView) {
        return new String[]{"Homes"};
    }
    
    /**
     * Overriding methods
     * @see org.objectweb.util.browser.api.Table#getRows(org.objectweb.util.browser.api.TreeView)
     */
    public Object[][] getRows(TreeView treeView) {
        DCIManager DCIManager_ = (DCIManager)treeView.getSelectedObject();
        ExtHomeFinder homeFinder = DCIManager_.provide_home_finder();
        HomeInfo[] homes = homeFinder.get_all_homes();

        Object[][] contenu = new Object[homes.length][1];
        for (int i = 0; i < homes.length; i++) {
            contenu[i][0] = new DefaultEntry(homes[i].home_ref, new DefaultName(homes[i].uid));
        }

        return contenu;
    }
}
