/*===========================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jrme Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/
package org.objectweb.openccm.browser.DCI;

import org.objectweb.util.browser.api.DropTreeView;
import org.objectweb.util.browser.api.DropAction;
import org.objectweb.util.browser.gui.api.DialogAction;
import org.objectweb.util.browser.gui.api.DialogBox;
import org.objectweb.util.browser.gui.lib.DefaultDialogBox;
import org.objectweb.util.browser.gui.lib.LabelBox;

import org.omg.Components.CCMHome;
import org.omg.Components.HomeRegistration;

/**
 * This action registers a <code>CCMHome</code> into the given <code>HomeRegistration</code>.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class RegisterHomeAction 
    implements DropAction, DialogAction {

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    protected HomeRegistration home_registration_;

    protected LabelBox name_;

    protected CCMHome home_ = null;

    //==================================================================
    //
    // No constructor.
    // 
    //==================================================================

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================

    //==================================================================
    //
    // Public methods for DropAction interface.
    //
    //==================================================================

    public void execute(DropTreeView e) throws Exception {
        home_registration_ = (HomeRegistration)e.getSelectedObject();
        if(CCMHome.class.isAssignableFrom(e.getDragSourceObject().getClass())){
            home_ = (CCMHome)e.getDragSourceObject();
            String defaultName = e.getDragSourceEntry().getName().toString();
            DialogBox dialog = new DefaultDialogBox("Register an home");      
            name_ = new LabelBox("Home name", defaultName);
            dialog.addElementBox(name_);
            dialog.setValidateAction(this);
            dialog.show();
        } else {
            throw new Exception("CCMHome expected !");
        }
    }

    //==================================================================
    //
    // Public methods for DialogAction interface.
    //
    //==================================================================

    /**
     * Executes an action
     */
    public void executeAction() throws Exception {
        // The InterfaceRepository must be started for obtaining this data.
        String component_repid = "ir3_not_started";
        String home_repid = "ir3_not_started";
//         try{
//             IRObject component_def = home_.get_component_def();
//             IRObject home_def = home_.get_home_def();
//         } catch(Exception e) {
//             // Ignore
//         }
        home_registration_.register_home(home_, home_repid, component_repid, name_.getLabel());
    }

}
