/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.browser.menu;

/** The java API's imports */
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.SwingConstants;

import java.awt.Component;
import java.awt.Dimension;

/**
 * This class represents the panel which allows to specify a label.
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class LabelBox extends Box {

    /** The name of the entry */
    protected JTextField name_;

    /** The label of the entry */
    protected JLabel fieldLabel_;

    /** 
     * Default constructor
     */
    public LabelBox(String label, String initValue) {
        super(BoxLayout.X_AXIS);
        add(Box.createHorizontalGlue());
        fieldLabel_ = new JLabel(label, SwingConstants.RIGHT);
        fieldLabel_.setAlignmentX(Component.RIGHT_ALIGNMENT);
        fieldLabel_.setAlignmentY(Component.CENTER_ALIGNMENT);
        add(fieldLabel_);
        add(Box.createHorizontalStrut(5));
        name_ = new JTextField(initValue);
        name_.setPreferredSize(new Dimension(225, 20));
        name_.setMaximumSize(new Dimension(225, 20));
        add(name_);
    }

    /** 
     * Default constructor
     */
    public LabelBox(String label) {
        this(label, "");
    }

    public void addActionListener(java.awt.event.ActionListener listener) {
        name_.addActionListener(listener);
    }

    /** Returns the specify label */
    public String getLabel() {
        return name_.getText();
    }

    /**
     * Sets the tool tip text on each JComponent of the box.
     *
     * @param text The tool tip text to set.
     */
    public void setToolTipText(String text) {
        name_.setToolTipText(text);
        fieldLabel_.setToolTipText(text);
    }

    /**
     * Gets the tool tip text set on each JComponent of the box.
     *
     * @return The tool tip text used.
     */
    public String getToolTipText() {
        return name_.getToolTipText();
    }
}
