/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.browser.menu;

/** The administration console's imports */
import org.objectweb.util.browser.core.xmlparser.XMLParser;
import org.objectweb.util.browser.core.xmlparser.BrowserXMLParser;
import org.objectweb.util.browser.core.api.BrowserProperty;
import org.objectweb.util.browser.core.common.DynamicTree;

/** The java API's imports */
import java.io.File;
import java.awt.Component;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;

/**
 * This class represents the action which loads a new configuration file
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class LoadConfigurationFileAction extends AbstractMenuAction {

    /** The file chooser that allow to find the file to load */
    protected JFileChooser fileChooser_;

    /** The parent */
    protected Component parent_;

    /**
     * The default constructor
     */
    public LoadConfigurationFileAction(String nom, ImageIcon image, String desc, Integer mnemonic, DynamicTree tree, Component parent) {
        super(nom, image, desc, mnemonic, tree);
        fileChooser_ = JFileChooserSingleton.getInstance(OpenCCMBrowserConstants.XML_FILE_CHOOSER, "Loads a new configuration file ...", JFileChooser.FILES_ONLY);
        parent_ = parent;
    }

    /**
     * Shows a JFileChooser component and tries to load the new configuration file
     */
    public void actionPerformed(java.awt.event.ActionEvent ae) {
        int returnVal = fileChooser_.showOpenDialog(parent_);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File f = fileChooser_.getSelectedFile();
            tree_.addBrowserProperty(new String[]{f.getName()});
            tree_.refreshAll();
        }
    }

}
