/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Contreras
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.openccm.deployment.gui;


/**
 * This is the Singleton register for scanning Ids in the GUI
 *
 * @author <a href="mailto:Christophe.Contreras@lifl.fr">Christophe Contreras</a>
 *
 */
public class CCMIdRefsSingleton
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** the instance of the registry */
    private static CCMIdRefsSingleton instance = null;

    /** the assemblies list */
    private java.util.List assemblies_list ;

    /** the packages list */
    private java.util.List packages_list ;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     *  The default constructor.
     */
    public
    CCMIdRefsSingleton()
    {
        // inits the lists
        assemblies_list = new java.util.LinkedList();
        packages_list = new java.util.LinkedList();
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * adds an assembly to list
     *
     * @param Componentassembly ass an instance of CAD
     */
    public void
    addAssembly(org.objectweb.ccm.descriptor.componentassembly.beans.ComponentassemblyBean ass)
    {
        this.assemblies_list.add(ass);
    }

    /**
     * adds a package to list
     *
     * @param Softpkg softpkg an instance of CSD
     */
    public void
    addPackage(org.objectweb.ccm.descriptor.softpkg.beans.SoftpkgBean softpkg)
    {
        this.packages_list.add(softpkg);
    }

    /**
     * returns the instances list
     *
     * @return List of instances referenced in this panel
     */
    public java.util.List
    getComponentInstList()
    {
        java.util.List componentinst_list = new java.util.LinkedList();

        for (java.util.Iterator k = assemblies_list.iterator() ; k.hasNext() ; )
        {
            // the instance is an assembly
            org.objectweb.ccm.descriptor.componentassembly.beans.ComponentassemblyBean
             assembly_instance
             = (org.objectweb.ccm.descriptor.componentassembly.beans.ComponentassemblyBean)
                k.next();


            java.util.List homeplacements
             = new java.util.LinkedList();

            homeplacements.addAll(
                assembly_instance.getPartitioning().getHomeplacementList()
                );

            java.util.List processcollocations
             = assembly_instance.getPartitioning().getProcesscollocationList();

            for (java.util.Iterator i = processcollocations.iterator() ; i.hasNext() ; )
            {
                org.objectweb.ccm.descriptor.componentassembly.beans.ProcesscollocationBean
                 element
                 = (org.objectweb.ccm.descriptor.componentassembly.beans.ProcesscollocationBean)
                    i.next();

                homeplacements.addAll(
                    element.getHomeplacementList()
                    );
            }

            java.util.List hostcollocations
             = assembly_instance.getPartitioning().getHostcollocationList();

            for (java.util.Iterator i = hostcollocations.iterator() ; i.hasNext() ; )
            {
                org.objectweb.ccm.descriptor.componentassembly.beans.HostcollocationBean
                 element
                 = (org.objectweb.ccm.descriptor.componentassembly.beans.HostcollocationBean)
                    i.next();

                processcollocations
                 = element.getProcesscollocationList();

                for (java.util.Iterator j = processcollocations.iterator() ; j.hasNext() ; )
                {
                    org.objectweb.ccm.descriptor.componentassembly.beans.ProcesscollocationBean
                     subelement
                     = (org.objectweb.ccm.descriptor.componentassembly.beans.ProcesscollocationBean)
                        j.next();

                    homeplacements.addAll(
                        subelement.getHomeplacementList()
                        );
                }

                homeplacements.addAll(
                    element.getHomeplacementList()
                    );
            }

            for (java.util.Iterator i = homeplacements.iterator() ; i.hasNext() ; )
            {
                org.objectweb.ccm.descriptor.componentassembly.beans.HomeplacementBean
                 element
                 = (org.objectweb.ccm.descriptor.componentassembly.beans.HomeplacementBean)
                    i.next();

                for (
                    java.util.Iterator j = element.getComponentinstantiationList().iterator()
                    ; j.hasNext()
                    ; )
                {
                    org.objectweb.ccm.descriptor.componentassembly.beans.ComponentinstantiationBean
                     instance_element
                     = (org.objectweb.ccm.descriptor.componentassembly.beans.ComponentinstantiationBean)
                        j.next();

                    componentinst_list.add(
                        instance_element.getId()
                        );
                }
            }
        }

        return componentinst_list;
    }

    /**
     * returns the implementations list
     *
     * @return List of implementations referenced in this panel
     */
    public java.util.List
    getComponentImplList()
    {
        java.util.List componentimpl_list = new java.util.LinkedList();

        for (java.util.Iterator i = packages_list.iterator() ; i.hasNext() ; )
        {
            // this instance is a software package descriptor
            org.objectweb.ccm.descriptor.softpkg.beans.SoftpkgBean
             software_instance
             = (org.objectweb.ccm.descriptor.softpkg.beans.SoftpkgBean)
                i.next();

            // gets the componentimplem list of this assembly
            java.util.List componentimplems
             = software_instance.getImplementationList();

            // adds the ids for each element
            for (java.util.Iterator j = componentimplems.iterator() ; j.hasNext() ; )
            {
                org.objectweb.ccm.descriptor.softpkg.beans.ImplementationBean
                 element
                 = (org.objectweb.ccm.descriptor.softpkg.beans.ImplementationBean)
                    j.next();

                // adds its id to this list
                componentimpl_list.add(element.getId());
            }
        }

        return componentimpl_list;
    }

    /**
     * returns the files list
     *
     * @return List of componentfiles referenced in this panel
     */
    public java.util.List
    getComponentFilesList()
    {
        java.util.List componentfiles_list = new java.util.LinkedList();

        for (java.util.Iterator i = assemblies_list.iterator() ; i.hasNext() ; )
        {
            // the instance is an assembly
            org.objectweb.ccm.descriptor.componentassembly.beans.ComponentassemblyBean
             assembly_instance
             = (org.objectweb.ccm.descriptor.componentassembly.beans.ComponentassemblyBean)
                i.next();

            // gets the componentfiles list of this assembly
            java.util.List componentfiles
             = assembly_instance.getComponentfiles().getComponentfileList();

            // adds the ids for each element
            for (java.util.Iterator j = componentfiles.iterator() ; j.hasNext() ; )
            {
                org.objectweb.ccm.descriptor.componentassembly.beans.ComponentfileBean
                 element
                 = (org.objectweb.ccm.descriptor.componentassembly.beans.ComponentfileBean)
                    j.next();

                // adds its id to this list
                componentfiles_list.add(element.getId());
            }
        }

        return componentfiles_list;
    }

    // ==================================================================
    //
    // Static methods.
    //
    // ==================================================================

    /**
     * get the instance
     *
     */
    public static CCMIdRefsSingleton
    getInstance()
    {
        if (instance == null)
        {
            instance = new CCMIdRefsSingleton();
        }

        return instance;
    }
}
