/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.services.pss.jdo.lib;

//Package dependencies
import javax.jdo.PersistenceManagerFactory;
import javax.jdo.PersistenceManager;

/**
 * This is a class to connect a jdo implementation.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class Connector
     extends org.objectweb.openccm.services.pss.common.lib.ConnectorBase
  implements org.objectweb.openccm.services.pss.jdo.api.Connector
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public Connector()
    {
        super("org.objectweb.openccm.services.pss.JDO");
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for org.omg.CosPersistentState.Connector
    //
    // ==================================================================

    /**
     * Create a basic session.
     */
    public org.omg.CosPersistentState.Session
    create_basic_session(short access_mode,
                         org.omg.CosPersistentState.Parameter[] additional_parameters)
    {
        String pmf_class = null,
               db_url = null,
               db_driver = null,
               user_name = null,
               password = null;
        org.omg.CosPersistentState.Parameter param = null;

        if (additional_parameters.length < 3)
            return null;

        for (int i=0; i<additional_parameters.length; i++)
        {
            param = additional_parameters[i];

            if (param.name.compareTo(Connector.PM_FACTORY_KEY) == 0)
            {
                // Get the Persistence Manager factory class
                pmf_class = param.val.extract_string();
            }
            else if (param.name.compareTo(Connector.DB_URL_KEY) == 0)
            {
                // Get the DataBase URL
                db_url = param.val.extract_string();
            }
            else if (param.name.compareTo(Connector.DB_DRIVER_KEY) == 0)
            {
                // Get the DataBase Driver
                db_driver = param.val.extract_string();
            }
            else if (param.name.compareTo(Connector.USER_NAME_KEY) == 0)
            {
                // Get the User name
                user_name = param.val.extract_string();
            }
            else if (param.name.compareTo(Connector.PASSWORD_KEY) == 0)
            {
                // Get the User password
                password = param.val.extract_string();
            }
       }

        // Create and set up a persistence manager factory
      try {
            PersistenceManagerFactory pmf =
                (PersistenceManagerFactory) Class.forName(pmf_class).newInstance();
            pmf.setConnectionURL(db_url);
            pmf.setConnectionDriverName(db_driver);
            if (user_name != null)
                pmf.setConnectionUserName(user_name);
            if (password != null)
                pmf.setConnectionPassword(password);

            PersistenceManager pm = pmf.getPersistenceManager();

            return new org.objectweb.openccm.services.pss.jdo.lib.Session(get_free_id(), pm);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Create a transactional session.
     */
    public org.omg.CosPersistentState.TransactionalSession
    create_transactional_session(short access_mode,
                                 short default_isolation_level,
                                 org.omg.CosPersistentState.EndOfAssociationCallback callback,
                                 org.omg.CosPersistentState.Parameter[] additional_parameters)
    {
        // TO DO
        return null;
    }

    /**
     * Create a session pool.
     */
    public org.omg.CosPersistentState.SessionPool
    create_session_pool(short access_mode,
                        short tx_policy,
                        org.omg.CosPersistentState.Parameter[] additional_parameters)
    {
        // TO DO
        return null;
    }

    /**
     * Get the current session.
     */
    public org.omg.CosPersistentState.TransactionalSession
    current_session()
    {
        // TO DO
        return null;
    }

    /**
     * Get sessions.
     */
    public org.omg.CosPersistentState.TransactionalSession[]
    sessions(org.omg.CosTransactions.Coordinator transaction)
    {
        // TO DO
        return null;
    }


    // ==================================================================
    //
    // Methods for org.objectweb.openccm.services.pss.api.Connector
    //
    // ==================================================================

}
