/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Sylvain Leblanc_______________________.

====================================================================*/

package org.objectweb.openccm.services.trading;

import org.omg.CORBA.TCKind;
import org.omg.DynamicAny.DynSequence;
import org.omg.DynamicAny.DynSequenceHelper;
import org.objectweb.ccm.CORBA.TypeCodeUtils;

/**
 * This class provides trader specific CORBA::TypeCode util functions.
 *
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 * @version 0.1
 */
public abstract class TraderTypeCodeUtils {

    private TraderTypeCodeUtils() {
    }

    /**
     * Retrieve the TCKind object relative to a typeCodeId.
     *
     * @param typeCodeId The type code kind identifier as a string.
     * This method provides all primitive type code from IDL keywords
     * and also sequences of primitives type codes.
     *
     * @return The equivalent <code>org.omg.CORBA.TypeCode</code> or
     * <code>null</code> if the <code>typeCodeId</code> doesn't exist.
     */
    public static org.omg.CORBA.TypeCode 
    getTypeCode(String typeCodeId) {
        org.omg.CORBA.ORB orb = org.objectweb.ccm.CORBA.TheORB.getORB();
        if (typeCodeId.equals("boolean"))
            return orb.get_primitive_tc(TCKind.tk_boolean);
        if (typeCodeId.equals("char"))
            return orb.get_primitive_tc(TCKind.tk_char);
        if (typeCodeId.equals("wchar"))
            return orb.get_primitive_tc(TCKind.tk_wchar);
        if (typeCodeId.equals("double"))
            return orb.get_primitive_tc(TCKind.tk_double);
        if (typeCodeId.equals("float"))
            return orb.get_primitive_tc(TCKind.tk_float);
        if (typeCodeId.equals("short"))
            return orb.get_primitive_tc(TCKind.tk_short);
        if (typeCodeId.equals("unsigned short"))
            return orb.get_primitive_tc(TCKind.tk_ushort);
        if (typeCodeId.equals("long"))
            return orb.get_primitive_tc(TCKind.tk_long);
        if (typeCodeId.equals("unsigned long"))
            return orb.get_primitive_tc(TCKind.tk_ulong);
        if (typeCodeId.equals("longlong"))
            return orb.get_primitive_tc(TCKind.tk_longlong);
        if (typeCodeId.equals("unsigned longlong"))
            return orb.get_primitive_tc(TCKind.tk_ulonglong);
        if (typeCodeId.equals("longdouble"))
            return orb.get_primitive_tc(TCKind.tk_longdouble);
        if (typeCodeId.equals("octet"))
            return orb.get_primitive_tc(TCKind.tk_octet);
        if (typeCodeId.equals("string"))
            return orb.get_primitive_tc(TCKind.tk_string);
        if (typeCodeId.equals("wstring"))
            return orb.get_primitive_tc(TCKind.tk_wstring);
        if (typeCodeId.startsWith("sequence")) {
            int idx = typeCodeId.indexOf("<");        
            String content_type = typeCodeId.substring(idx+1, typeCodeId.length()-1);
            org.omg.CORBA.TypeCode content_tc = getTypeCode(content_type);
            switch (content_tc.kind().value()) {
//                 case org.omg.CORBA.TCKind._tk_boolean : { return org.omg.CORBA.BooleanSeqHelper.type(); }
//                 case org.omg.CORBA.TCKind._tk_char : { return org.omg.CORBA.CharSeqHelper.type(); }
//                 case org.omg.CORBA.TCKind._tk_wchar : { return org.omg.CORBA.WCharSeqHelper.type(); }
//                 case org.omg.CORBA.TCKind._tk_double : { return org.omg.CORBA.DoubleSeqHelper.type(); }
//                 case org.omg.CORBA.TCKind._tk_float : { return org.omg.CORBA.FloatSeqHelper.type(); }
//                 case org.omg.CORBA.TCKind._tk_short : { return org.omg.CORBA.ShortSeqHelper.type(); }
//                 case org.omg.CORBA.TCKind._tk_ushort : { return org.omg.CORBA.UShortSeqHelper.type(); }
//                 case org.omg.CORBA.TCKind._tk_long : { return org.omg.CORBA.LongSeqHelper.type(); }
//                 case org.omg.CORBA.TCKind._tk_ulong : { return org.omg.CORBA.ULongSeqHelper.type(); }
//                 case org.omg.CORBA.TCKind._tk_longlong : { return org.omg.CORBA.LongLongSeqHelper.type(); }
//                 case org.omg.CORBA.TCKind._tk_ulonglong : { return org.omg.CORBA.ULongLongSeqHelper.type(); }
//                     // case org.omg.CORBA.TCKind._tk_longdouble : { return org.omg.CORBA.LongDoubleSeqHelper.type(); }
//                 case org.omg.CORBA.TCKind._tk_octet : { return org.omg.CORBA.OctetSeqHelper.type(); }
//                 case org.omg.CORBA.TCKind._tk_string : { return org.omg.CORBA.StringSeqHelper.type(); }
//                 case org.omg.CORBA.TCKind._tk_wstring : { return org.omg.CORBA.WStringSeqHelper.type(); }
                default : return orb.create_sequence_tc(0, content_tc);
            }            
        }
        
        return null ;	// unmanaged type code identifier
    }

    /**
     * Insert a value in an <code>org.omg.CORBA.Any</code>.</br>
     * 
     * Issue: How an array of values can be represented ??
     * Are values separated by "," ???</br>
     *      
     * Here we make the assumption of values inserted
     * in "()" and separeted by "," e.g. (10, 5, 54,
     * 67) is a sequence of long, ("test", "string") a
     * sequence of strings and ('a', 'b', 'c') a
     * sequence of characters.</br>
     *
     * @param tc The type code of the value.
     * @param value The value image as a string.
     *
     * @return The filled CORBA::Any or <code>null</code> if 
     * a the TypeCode is not managed or if the value has an 
     * incorrect format.
     */
    public static org.omg.CORBA.Any 
    insertInAny(org.omg.CORBA.TypeCode orig_tc, String value) 
    {
        org.omg.CORBA.Any the_any = org.objectweb.ccm.CORBA.TheORB.getORB().create_any();
        org.omg.CORBA.TypeCode tc = orig_tc;
        while (tc.kind() == TCKind.tk_alias) {
            try {
                tc = tc.content_type();
            } catch (org.omg.CORBA.TypeCodePackage.BadKind ex) {}
        }
        int kind = tc.kind().value();
        try {
            switch (kind) {
                case TCKind._tk_short : {
                    short v = Short.parseShort(value);
                    the_any.insert_short(v);
                    break;
                }
                case TCKind._tk_ushort : {
                    short v = Short.parseShort(value);
                    the_any.insert_ushort(v);
                    break;
                }
                case TCKind._tk_long : {
                    int v = Integer.parseInt(value);
                    the_any.insert_long(v);
                    break;
                }
                case TCKind._tk_ulong : {
                    int v = Integer.parseInt(value);
                    the_any.insert_ulong(v);
                    break;
                }
                case TCKind._tk_longlong : {
                    long v = Long.parseLong(value);
                    the_any.insert_longlong(v);
                    break;
                }
                case TCKind._tk_ulonglong : {
                    long v = Long.parseLong(value);
                    the_any.insert_ulonglong(v);
                    break;
                }
                case TCKind._tk_float : {
                    float v = Float.parseFloat(value);
                    the_any.insert_float(v);
                    break;
                }
                case TCKind._tk_double : {
                    double v = Double.parseDouble(value);
                    the_any.insert_double(v);
                    break;
                }
                case TCKind._tk_wchar : {
                    char v = value.charAt(0);
                    the_any.insert_wchar(v);
                    break;
                }
                case TCKind._tk_char : {
                    char v = value.charAt(0);
                    the_any.insert_char(v);
                    break;
                }
                case TCKind._tk_wstring : {
                    the_any.insert_wstring(value);
                    break;
                }
                case TCKind._tk_string : {
                    the_any.insert_string(value);
                    break;
                }
                case TCKind._tk_octet : {
                    byte v = Byte.parseByte(value);
                    the_any.insert_octet(v);
                    break;
                }
                case TCKind._tk_boolean : {
                    boolean v = new Boolean(value).booleanValue();
                    the_any.insert_boolean(v);
                    break;
                }
                case TCKind._tk_alias : {
                    return insertInAny(TypeCodeUtils.getOrigType(tc), value);
                }
                case TCKind._tk_sequence : {
                    // How an array of values can be represented ??
                    // Are values separated by "," ???
                    
                    // Here we make the assumption of values inserted
                    // in "()" and separeted by "," e.g. (10, 5, 54,
                    // 67) is a sequence of long, ("test", "string") a
                    // sequence of strings and ('a', 'b', 'c') a
                    // sequence of characters..

                    // creates a tokenizer on the seq values
                    String content = value.substring(value.indexOf('(')+1, value.indexOf(')'));
                    java.util.StringTokenizer st = new java.util.StringTokenizer(content, ",");
                    
                    try {
                        
                        // Creates a new dyn sequence
                        org.omg.DynamicAny.DynAny dyn_any = org.objectweb.ccm.CORBA.TheDynamicAnyFactory.getFactory().create_dyn_any_from_type_code(tc);                    
                        org.omg.DynamicAny.DynSequence dyn_seq = DynSequenceHelper.narrow(dyn_any);                    

                        // constructing the contained values sequence and
                        // init the dyn sequence size
                        org.omg.CORBA.Any[] values = new org.omg.CORBA.Any[st.countTokens()];
                        dyn_seq.set_length(st.countTokens());

                        // resolving the contained type 
                        org.omg.CORBA.TypeCode content_tc = null;                    
                        try {
                            content_tc = tc.content_type();
                        } catch (org.omg.CORBA.TypeCodePackage.BadKind ex) {
                            System.err.println("in a content catch");
                            // cannot happened here
                        }

                        // feeding the contained values sequence
                        switch (content_tc.kind().value()) {
                            case TCKind._tk_short : {
                                int i=0;
                                while (st.hasMoreTokens()) {
                                    values[i] = org.objectweb.ccm.CORBA.TheORB.getORB().create_any();
                                    values[i].insert_short(Short.parseShort(st.nextToken().trim()));
                                    i++;
                                }                            
                                break;
                            }
                            case TCKind._tk_ushort : {
                                int i=0;
                                while (st.hasMoreTokens()) {
                                    values[i] = org.objectweb.ccm.CORBA.TheORB.getORB().create_any();
                                    values[i].insert_ushort(Short.parseShort(st.nextToken().trim()));
                                    i++;
                                }
                                break;
                            }
                            case TCKind._tk_long : {
                                int i=0;
                                while (st.hasMoreTokens()) {
                                    values[i] = org.objectweb.ccm.CORBA.TheORB.getORB().create_any();
                                    values[i].insert_long(Integer.parseInt(st.nextToken().trim()));
                                    i++;
                                }
                                break;
                            }
                            case TCKind._tk_ulong : {
                                int i=0;
                                while (st.hasMoreTokens()) {
                                    values[i] = org.objectweb.ccm.CORBA.TheORB.getORB().create_any();
                                    values[i].insert_ulong(Integer.parseInt(st.nextToken().trim()));
                                    i++;
                                }
                                break;
                            }
                            case TCKind._tk_longlong : {
                                int i=0;
                                while (st.hasMoreTokens()) {
                                    values[i] = org.objectweb.ccm.CORBA.TheORB.getORB().create_any();
                                    values[i].insert_longlong(Long.parseLong(st.nextToken().trim()));
                                    i++;
                                }
                                break;
                            }
                            case TCKind._tk_ulonglong : {
                                int i=0;
                                while (st.hasMoreTokens()) { 
                                   values[i] = org.objectweb.ccm.CORBA.TheORB.getORB().create_any();
                                    values[i].insert_ulonglong(Long.parseLong(st.nextToken().trim()));
                                    i++;
                                }
                                break;
                            }
                            case TCKind._tk_float : {
                                int i=0;
                                while (st.hasMoreTokens()) {
                                    values[i] = org.objectweb.ccm.CORBA.TheORB.getORB().create_any();
                                    values[i].insert_float(Float.parseFloat(st.nextToken().trim()));
                                    i++;
                                }
                                break;
                            }
                            case TCKind._tk_double : {
                                int i=0;
                                while (st.hasMoreTokens()) {
                                    values[i] = org.objectweb.ccm.CORBA.TheORB.getORB().create_any();
                                    values[i].insert_double(Double.parseDouble(st.nextToken().trim()));
                                    i++;
                                }
                                break;
                            }
                            case TCKind._tk_wchar : {
                                int i=0;
                                while (st.hasMoreTokens()) {
                                    String val = st.nextToken().trim();
                                    // tests if packed as a char
                                    if ( ! val.startsWith("'") || 
                                         ! val.endsWith("'") || 
                                         val.length() != 3 ) 
                                        return null;
                                    // gets the middle char
                                    values[i] = org.objectweb.ccm.CORBA.TheORB.getORB().create_any();
                                    values[i].insert_wchar(val.charAt(1));
                                    i++;
                                }
                                break;
                            }
                            case TCKind._tk_char : {
                                int i=0;
                                while (st.hasMoreTokens()) {
                                    String val = st.nextToken().trim();
                                    // tests if packed as a char
                                    if ( ! val.startsWith("'") || 
                                         ! val.endsWith("'") || 
                                         val.length() != 3 ) 
                                        return null;
                                    // gets the middle char
                                    values[i] = org.objectweb.ccm.CORBA.TheORB.getORB().create_any();
                                    values[i].insert_char(val.charAt(1));
                                    i++;
                                }
                                break;
                            }
                            case TCKind._tk_wstring :  {
                                int i=0;
                                while (st.hasMoreTokens()) {
                                    String val = st.nextToken().trim();
                                    // tests if packed as a string
                                    if ( ! val.startsWith("\"") || 
                                         ! val.endsWith("\"") ) 
                                        return null;
                                    values[i] = org.objectweb.ccm.CORBA.TheORB.getORB().create_any();
                                    values[i].insert_wstring(val.substring(1, val.length()-1));
                                    i++;
                                }
                                break;
                            }
                            case TCKind._tk_string : {
                                int i=0;
                                while (st.hasMoreTokens()) {
                                    String val = st.nextToken().trim();
                                    // tests if packed as a string
                                    if ( ! val.startsWith("\"") || 
                                         ! val.endsWith("\"") ) 
                                        return null;
                                    values[i] = org.objectweb.ccm.CORBA.TheORB.getORB().create_any();
                                    values[i].insert_string(val.substring(1, val.length()-1));
                                    i++;
                                }
                                break;
                            }
                            case TCKind._tk_octet : {
                                int i=0;
                                while (st.hasMoreTokens()) {
                                    values[i] = org.objectweb.ccm.CORBA.TheORB.getORB().create_any();
                                    values[i].insert_octet(Byte.parseByte(st.nextToken().trim()));
                                    i++;
                                }
                                break;
                            }
                            case TCKind._tk_boolean : {
                                int i=0;
                                while (st.hasMoreTokens()) {
                                    values[i] = org.objectweb.ccm.CORBA.TheORB.getORB().create_any();
                                    values[i].insert_boolean(Boolean.valueOf(st.nextToken().trim()).booleanValue());
                                    i++;
                                }
                                break;
                            }
                        }
                        dyn_seq.set_elements(values);
                        return dyn_seq.to_any();
                    } catch (org.omg.DynamicAny.DynAnyFactoryPackage.InconsistentTypeCode ex) {
                    } catch (org.omg.DynamicAny.DynAnyPackage.InvalidValue ex) {
                    } catch (org.omg.DynamicAny.DynAnyPackage.TypeMismatch ex) {
                    }
                    return null;
                }
                default : return null;
            }
            return the_any;
        } catch (NumberFormatException ex) {            
            throw new org.objectweb.util.api.ExceptionWrapper(ex, "cannot insert in any");
        }
    }
//     /**
//      * Insert a value in an <code>org.omg.CORBA.Any</code>.</br>
//      * 
//      * Issue: How an array of values can be represented ??
//      * Are values separated by "," ???</br>
//      *      
//      * Here we make the assumption of values inserted
//      * in "()" and separeted by "," e.g. (10, 5, 54,
//      * 67) is a sequence of long, ("test", "string") a
//      * sequence of strings and ('a', 'b', 'c') a
//      * sequence of characters.</br>
//      *
//      * @param tc The type code of the value.
//      * @param value The value image as a string.
//      *
//      * @return The filled CORBA::Any or <code>null</code> if 
//      * a the TypeCode is not managed or if the value has an 
//      * incorrect format.
//      */
//     public static org.omg.CORBA.Any 
//     insertInAny(org.omg.CORBA.TypeCode tc, String value) 
//     {
//         org.omg.CORBA.Any the_any = org.objectweb.ccm.CORBA.TheORB.getORB().create_any();
//         int kind = tc.kind().value();
//         try {
//             switch (kind) {
//                 case TCKind._tk_short : {
//                     short v = Short.parseShort(value);
//                     the_any.insert_short(v);
//                     break;
//                 }
//                 case TCKind._tk_ushort : {
//                     short v = Short.parseShort(value);
//                     the_any.insert_ushort(v);
//                     break;
//                 }
//                 case TCKind._tk_long : {
//                     int v = Integer.parseInt(value);
//                     the_any.insert_long(v);
//                     break;
//                 }
//                 case TCKind._tk_ulong : {
//                     int v = Integer.parseInt(value);
//                     the_any.insert_ulong(v);
//                     break;
//                 }
//                 case TCKind._tk_longlong : {
//                     long v = Long.parseLong(value);
//                     the_any.insert_longlong(v);
//                     break;
//                 }
//                 case TCKind._tk_ulonglong : {
//                     long v = Long.parseLong(value);
//                     the_any.insert_ulonglong(v);
//                     break;
//                 }
//                 case TCKind._tk_float : {
//                     float v = Float.parseFloat(value);
//                     the_any.insert_float(v);
//                     break;
//                 }
//                 case TCKind._tk_double : {
//                     double v = Double.parseDouble(value);
//                     the_any.insert_double(v);
//                     break;
//                 }
//                 case TCKind._tk_wchar : {
//                     char v = value.charAt(0);
//                     the_any.insert_wchar(v);
//                     break;
//                 }
//                 case TCKind._tk_char : {
//                     char v = value.charAt(0);
//                     the_any.insert_char(v);
//                     break;
//                 }
//                 case TCKind._tk_wstring : {
//                     the_any.insert_wstring(value);
//                     break;
//                 }
//                 case TCKind._tk_string : {
//                     the_any.insert_string(value);
//                     break;
//                 }
//                 case TCKind._tk_octet : {
//                     byte v = Byte.parseByte(value);
//                     the_any.insert_octet(v);
//                     break;
//                 }
//                 case TCKind._tk_boolean : {
//                     boolean v = new Boolean(value).booleanValue();
//                     the_any.insert_boolean(v);
//                     break;
//                 }
//                 case TCKind._tk_alias : {
//                     return insertInAny(TypeCodeUtils.getOrigType(tc), value);
//                 }
//                 case TCKind._tk_sequence : {
//                     // How an array of values can be represented ??
//                     // Are values separated by "," ???
                    
//                     // Here we make the assumption of values inserted
//                     // in "()" and separeted by "," e.g. (10, 5, 54,
//                     // 67) is a sequence of long, ("test", "string") a
//                     // sequence of strings and ('a', 'b', 'c') a
//                     // sequence of characters..
//                     String content = value.substring(value.indexOf('(')+1, value.indexOf(')'));
//                     java.util.StringTokenizer st = new java.util.StringTokenizer(content, ",");

//                     org.omg.CORBA.TypeCode content_tc = null;
//                     try {
//                         content_tc = tc.content_type();
//                     } catch (org.omg.CORBA.TypeCodePackage.BadKind ex) {
//                         // cannot happened here
//                     }

//                     switch (content_tc.kind().value()) {
//                         case TCKind._tk_short : {
//                             short[] arr = new short[st.countTokens()];
//                             int i=0;
//                             while (st.hasMoreTokens()) {
//                                 arr[i]= Short.parseShort(st.nextToken().trim());
//                                 i++;
//                             }
//                             org.omg.CORBA.ShortSeqHelper.insert(the_any, arr);
//                               break;
//                         }
//                         case TCKind._tk_ushort : {
//                             short[] arr = new short[st.countTokens()];
//                             int i=0;
//                             while (st.hasMoreTokens()) {
//                                 arr[i]= Short.parseShort(st.nextToken().trim());
//                                 i++;
//                             }
//                             org.omg.CORBA.UShortSeqHelper.insert(the_any, arr);
//                             break;
//                         }
//                         case TCKind._tk_long : {
//                             int[] arr = new int[st.countTokens()];
//                             int i=0;
//                             while (st.hasMoreTokens()) {
//                                 arr[i]= Integer.parseInt(st.nextToken().trim());
//                                 i++;
//                             }
//                             org.omg.CORBA.LongSeqHelper.insert(the_any, arr);
//                             break;
//                         }
//                         case TCKind._tk_ulong : {
//                             int[] arr = new int[st.countTokens()];
//                             int i=0;
//                             while (st.hasMoreTokens()) {
//                                 arr[i]= Integer.parseInt(st.nextToken().trim());
//                                 i++;
//                             }
//                             org.omg.CORBA.ULongSeqHelper.insert(the_any, arr);
//                             break;
//                         }
//                         case TCKind._tk_longlong : {
//                             long[] arr = new long[st.countTokens()];
//                             int i=0;
//                             while (st.hasMoreTokens()) {
//                                 arr[i]= Long.parseLong(st.nextToken().trim());
//                                 i++;
//                             }
//                             org.omg.CORBA.LongLongSeqHelper.insert(the_any, arr);
//                             break;
//                         }
//                         case TCKind._tk_ulonglong : {
//                             long[] arr = new long[st.countTokens()];
//                             int i=0;
//                             while (st.hasMoreTokens()) {
//                                 arr[i]= Long.parseLong(st.nextToken().trim());
//                                 i++;
//                             }
//                             org.omg.CORBA.ULongLongSeqHelper.insert(the_any, arr);
//                             break;
//                         }
//                         case TCKind._tk_float : {
//                             float[] arr = new float[st.countTokens()];
//                             int i=0;
//                             while (st.hasMoreTokens()) {
//                                 arr[i]= Float.parseFloat(st.nextToken().trim());
//                                 i++;
//                             }
//                             org.omg.CORBA.FloatSeqHelper.insert(the_any, arr);
//                             break;
//                         }
//                         case TCKind._tk_double : {
//                             double[] arr = new double[st.countTokens()];
//                             int i=0;
//                             while (st.hasMoreTokens()) {
//                                 arr[i]= Double.parseDouble(st.nextToken().trim());
//                                 i++;
//                             }
//                             org.omg.CORBA.DoubleSeqHelper.insert(the_any, arr);
//                             break;
//                         }
//                         case TCKind._tk_wchar : {
//                             char[] arr = new char[st.countTokens()];
//                             int i=0;
//                             while (st.hasMoreTokens()) {
//                                 String val = st.nextToken().trim();
//                                 // tests if packed as a char
//                                 if ( ! val.startsWith("'") || 
//                                      ! val.endsWith("'") || 
//                                      val.length() != 3 ) 
//                                     return null;
//                                 // gets the middle char
//                                 arr[i]= val.charAt(1);
//                                 i++;
//                             }
//                             org.omg.CORBA.WCharSeqHelper.insert(the_any, arr);
//                             break;
//                         }
//                         case TCKind._tk_char : {
//                             char[] arr = new char[st.countTokens()];
//                             int i=0;
//                             while (st.hasMoreTokens()) {
//                                 String val = st.nextToken().trim();
//                                 // tests if packed as a char
//                                 if ( ! val.startsWith("'") || 
//                                      ! val.endsWith("'") || 
//                                      val.length() != 3 ) 
//                                     return null;
//                                 // gets the middle char
//                                 arr[i]= val.charAt(1);
//                                 i++;
//                             }
//                             org.omg.CORBA.CharSeqHelper.insert(the_any, arr);
//                             break;
//                         }
//                         case TCKind._tk_wstring :  {
//                             String[] arr = new String[st.countTokens()];
//                             int i=0;
//                             while (st.hasMoreTokens()) {
//                                 String val = st.nextToken().trim();
//                                 // tests if packed as a string
//                                 if ( ! val.startsWith("\"") || 
//                                      ! val.endsWith("\"") ) 
//                                     return null;
//                                 arr[i]= val.substring(1, val.length()-1);
//                                 i++;
//                             }
//                             org.omg.CORBA.WStringSeqHelper.insert(the_any, arr);
//                             break;
//                         }
//                         case TCKind._tk_string : {
//                             String[] arr = new String[st.countTokens()];
//                             int i=0;
//                             while (st.hasMoreTokens()) {
//                                 String val = st.nextToken().trim();
//                                 // tests if packed as a string
//                                 if ( ! val.startsWith("\"") || 
//                                      ! val.endsWith("\"") ) 
//                                     return null;
//                                 arr[i]= val.substring(1, val.length()-1);
//                                 i++;
//                             }
//                             org.omg.CORBA.StringSeqHelper.insert(the_any, arr);
//                             break;
//                         }
//                         case TCKind._tk_octet : {
//                             byte[] arr = new byte[st.countTokens()];
//                             int i=0;
//                             while (st.hasMoreTokens()) {
//                                 arr[i]= Byte.parseByte(st.nextToken().trim());
//                                 i++;
//                             }
//                             org.omg.CORBA.OctetSeqHelper.insert(the_any, arr);
//                             break;
//                         }
//                         case TCKind._tk_boolean : {
//                             boolean[] arr = new boolean[st.countTokens()];
//                             int i=0;
//                             while (st.hasMoreTokens()) {
//                                 arr[i]= Boolean.valueOf(st.nextToken().trim()).booleanValue();
//                                 i++;
//                             }
//                             org.omg.CORBA.BooleanSeqHelper.insert(the_any, arr);
//                             break;
//                         }
//                     }
//                     return the_any;
//                 }
//                 default : return null;
//             }
//             return the_any;
//         } catch (NumberFormatException ex) {            
//             throw new org.objectweb.util.api.ExceptionWrapper(ex, "cannot insert in any");
//         }
//     }
}
