/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.api;

/** The console's imports */
import org.objectweb.util.browser.naming.api.Context;

/** The Java API's imports*/
import java.io.File;

/**
 * Interface which allows to configure the dynamic tree
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public interface TreeConfiguration {

    /** 
     * Creates a new browser property with the given fileName.
     * @param files The properties files to parse and to apply
     */
    public void setNewBrowserProperty(File[] files);

    /** 
     * Sets a new browser property.
     * @param properties The properties to apply
     */
    public void setNewBrowserProperty(BrowserProperty properties);

    /**
     * Gives the current BrowserProperty. 
     * @return The current BrowserProperty
     */
    public BrowserProperty getBrowserProperty();

    /**
     * Creates a new context property with the given fileName
     * @param file The properties files to parse and to apply
     */
    public void setNewContextProperty(File file);

    /**
     * Sets a new context property
     * @param properties The properties to apply 
     */
    public void setNewContextProperty(ContextProperty properties);

    /**
     * Gives the current ContextProperty.
     * @return The current ContextProperty
     */
    public ContextProperty getContextProperty();

    /**
     * Adds an object into the initial context.
     *  
     * @param id Name of the object
     * @param object The object to add
     */
    public void addToInitialContext(String id, Object object);

    /**
     * Adds an object into the initial context and expands the ndoe until a specific level if this one is a context.
     *  
     * @param id Name of the object
     * @param object The object to add
     * @param level The level (>0 for a specific level or <0 for all levels)
     */
    public void addToInitialContext(String id, Object object, int level);

    /**
     * Gives the initial context 
     * @return The initial context
     */
    public Context getInitialContext();

    /**
     * Returns true if this initial context contains an entry for the specified id.
     * @param id Key whose presence in this initial context is to be tested.
     * @return True if this initial context contains an entry for the specified id.
     */
    public boolean containsEntry(String id);

    /**
     * Fixes the view panel for the tree (Where the associated panels will be displayed).
     * @param viewPanel The panel to use to display associated panels
     */
    public void setViewPanel(ViewPanel viewPanel);

}
