/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.naming.wrappers.common;

/** To use the NameFactory interface */
import org.objectweb.util.browser.naming.api.NameFactory;

/** To use the Name interface. */
import org.objectweb.util.browser.naming.api.Name;

/**
 * Default basic implementation of the NameFactory interface.
 *
 * This factory creates DefaultName instances.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @version 0.1
 */
public class DefaultNameFactory 
    implements NameFactory {
	
    // ==================================================================
    //
    // No internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // No internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for interface NameFactory.
    //
    // ==================================================================

    /**
     * Creates a new {@link Name Name} with an identifier and no subname.
     *
     * The identifier and the subname of the created {@link Name Name}
     * are set respectively to the given id and the null value.
     *
     * @param id The identifier of the {@link Name Name} to create.
     *
     * @return The created {@link Name Name}.
     *
     * @postcondition
     *           return.getOWId() == id
     *  and return.getOWSubName() == null;
     */
    public Name newOWName(String id) {
        return new DefaultName(id);
    }

    /**
     * Creates a new {@link Name Name} with an identifier and a subname.
     *
     * The identifier and the subname of the created {@link Name Name}
     * are set respectively to the given id and subname.
     *
     * @param id The identifier of the {@link Name Name} to create.
     * @param subname The subname of the {@link Name Name} to create.
     *
     * @return The created {@link Name Name}.
     *
     * @postcondition
     *           return.getOWId() == id
     *  and return.getOWSubName() == subname;
     */
    public Name newOWName(String id, Name subname) {
        return new DefaultName(id, subname);
    }
}
