/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2003 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.xmlreader.decoder;

/** The console's imports */
import org.objectweb.util.browser.api.ContextProperty;
import org.objectweb.util.browser.gui.core.ClassResolver;
import org.objectweb.util.browser.contextConfig.Context;
import org.objectweb.util.browser.contextConfig.ContextUnmarshaller;
import org.objectweb.util.browser.contextConfig.Decoders;
import org.objectweb.util.browser.contextConfig.Decoder;
import org.objectweb.util.browser.naming.api.ContextContainer;
import org.objectweb.util.browser.naming.api.DecoderFactory;
import org.objectweb.util.browser.naming.wrappers.common.DefaultContextContainer;
import org.objectweb.util.browser.naming.wrappers.common.DefaultDecoderFactory;

/** The java API's imports */
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.List;
import java.util.Iterator;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class ContextXMLDecoder implements XMLDecoder, ContextProperty {

    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /** The Decoder corresponding to the configuration of the context */
    protected org.objectweb.util.browser.naming.api.Decoder decoder_;

    // ==================================================================
    //
    // Protected methods.
    //
    // ==================================================================

    /**
     * Builds the chain of decoder corresponding to the given decoder list
     * @param decoders The decoders node to interpret
     */
    protected void buildDecoder(Decoders decoders) {
        if (decoders != null) {
            ContextContainer context = new DefaultContextContainer();
            List decoderList = decoders.getDecoderList();
            Iterator it = decoderList.iterator();
            while (it.hasNext()) {
                Decoder decoderNode = (Decoder) it.next();
                String className = decoderNode.getJavaClass();
                try {
                    Class c = ClassResolver.resolve(className);
                    context.addEntry(className, c);
                } catch (ClassNotFoundException e) {
                    System.out.println("Error : " + className + " : Class not found !");
                }
            }
            DecoderFactory decoderFactory = new DefaultDecoderFactory();
            decoder_ = decoderFactory.create(context);
        }
    }

    // ==================================================================
    //
    // Public methods of XMLDecoder interface.
    //
    // ==================================================================

    /* 
     * @see org.objectweb.kilim.browser.xmlreader.decoder.BrowserConfigXMLDecoder#getDecoder()
     */
    public org.objectweb.util.browser.naming.api.Decoder getDecoder() {
        return decoder_;
    }

    /* 
     * @see org.objectweb.kilim.browser.xmlreader.decoder.XMLDecoder#setPropertyFile(java.io.File)
     */
    public void setPropertyFile(File path) {
        if (path != null && !path.isDirectory()) {
            try {
                Context context = ContextUnmarshaller.unmarshal(new FileInputStream(path));
                if (context != null) {
                    buildDecoder(context.getDecoders());
                }
            } catch (FileNotFoundException e) {
                System.out.println("Error : " + path + " : File not found !");
            } catch (IOException e) {
                System.out.println("Error : " + path + " : I/O Exception !");
            }
        }
    }

    /**
     * Fixes an array of files (or directories) which contains the properties
     * @param files The path containing the XML file(s) containing the configuration 
     * we want to abtain under a Context view (Each value of the array can be a single file or a directory)
     */
    public void setPropertyFile(File[] files) {
        for (int i = 0; i < files.length; i++)
            setPropertyFile(files[i]);
    }

}
