/*====================================================================

ObjectWeb Util CommandLine Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): .

====================================================================*/

package org.objectweb.util.cmdline.lib;

// Package dependencies.
import org.objectweb.util.cmdline.api.CommandLine;
import org.objectweb.util.cmdline.api.CommandLineHolder;

/**
 * Default implementation of the CommandLineHolder interface.
 *
 * This provides the following properties:
 *
 * - CommandLine: The associated command line manager.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public class DefaultCommandLineHolder
  implements org.objectweb.util.cmdline.api.CommandLineHolder
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** To store the command line manager. */
    private CommandLine commandLine_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /** The default constructor. */
    public
    DefaultCommandLineHolder()
    {
        // Inits internal state.
        commandLine_ = null;
    }

    /**
     * The constructor with the command line manager.
     *
     * @param commandLine The command line manager.
     */
    public
    DefaultCommandLineHolder(CommandLine commandLine)
    {
        // Inits internal state.
        commandLine_ = commandLine;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cmdline.api.CommandLineHolder
    //
    // ==================================================================

    /**
     * Obtains the command line manager.
     *
     * @return The command line manager.
     */
    public CommandLine
    getCommandLine()
    {
        return commandLine_;
    }

    /**
     * Sets the command line manager.
     *
     * @param cl The command line manager.
     */
    public void
    setCommandLine(CommandLine cl)
    {
        commandLine_ = cl;
    }
}
