/*====================================================================

ObjectWeb Util Preprocessor Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.cpp.lib;

// Package dependencies.
import java.util.Hashtable;
import java.util.Vector;
import java.io.File;
import java.io.OutputStream;
import java.io.FileOutputStream;

/**
 * This provides a preprocessor written in Java.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public class PreprocessorJPP
       extends PreprocessorBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    PreprocessorJPP()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cpp.api.Preprocessor
    //
    // ==================================================================

    /**
     * Preprocesses a file.
     *
     * @param fileName The name of the file to preprocess.
     * @param tmpFile The file containing the output file.
     *
     * @return True if OK.
     */
    public boolean
    preprocess(String fileName,
               java.io.File tmpFile)
    {
        FileOutputStream output = null;
        try
        {
            // Creates an output stream on this temp file.
            output = new FileOutputStream(tmpFile);
        }
        // Wraps any java.io.IOException exception.
        catch(java.io.IOException exc)
        {
            throw new org.objectweb.util.api.ExceptionWrapper(exc);
        }

        boolean status = false;
        try
        {
            // Preprocesses.
            status = preprocess(fileName, output);
	}
        finally
        {
            // Always closes the stream. 
            IOHelper.close(output);
        }

        return status;
    }

    /**
     * Preprocesses a file.
     *
     * @param fileName The name of the file to preprocess.
     * @param output The output stream.
     */
    public boolean
    preprocess(String fileName,
               java.io.OutputStream output)
    {
        try
        {
            // Creates the preprocessor.
            JPP jpp = new JPP(fileName, output,
                              ((Vector)includeDirectories_.clone()),
                              ((Hashtable)macros_.clone()));
            // Preprocesses.
            return jpp.preprocess();
        }
        catch(java.io.FileNotFoundException exc)
        {
            getConsole().error(fileName + ": No such file or directory");
            return false;
        }
        // Wraps any java.io.IOException exception.
        catch(java.io.IOException exc)
        {
            throw new org.objectweb.util.api.ExceptionWrapper(exc);
        }
    }
}
