/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Contreras
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.openccm.deployment.gui;

import org.objectweb.openccm.descriptor.softpkg.beans.SoftpkgBean;
import org.objectweb.openccm.descriptor.softpkg.beans.SoftpkgBeanImpl;

/**
 * This is the action class for opening a CSD from an assembly
 *
 */
public class ComponentfileEditActionListener
  implements org.objectweb.util.browser.api.MenuItem
{
    /**
     *  Status of the menu item
     *
     * @param treeView to obtain the activated node
     */
    public int
    getStatus(org.objectweb.util.browser.api.TreeView treeView)
    {
        return org.objectweb.util.browser.api.MenuItem.ENABLED_STATUS;
    }

    public void
    actionPerformed (org.objectweb.util.browser.api.MenuItemTreeView the_event)
    {
        // retrieve the XML element from which comes the call
        org.objectweb.openccm.descriptor.componentassembly.beans.ComponentfileBean csd_file
         = (org.objectweb.openccm.descriptor.componentassembly.beans.ComponentfileBean)
           the_event.getSelectedObject();

        // gets the calling tree
        org.objectweb.util.browser.core.common.DynamicTree calling_tree
         = (org.objectweb.util.browser.core.common.DynamicTree)
           the_event.getTree();

        //gets the frame
        org.objectweb.apollon.gui.ApollonFrame calling_frame
         = (org.objectweb.apollon.gui.ApollonFrame)
            calling_tree.getRootPane().getParent();

        // The found path for this component (CSD or CAR)
        String softpkg_path = null;

        // Handling of eventual CSD file pointed by a <fileinarchive>
        if (csd_file.getFileinarchive() != null)
        {
            // in case the CSD is in a CAR file we open the CAR
            if (csd_file.getFileinarchive().getLink() != null)
            {
                // checks if the given href is relative
                if( csd_file.getFileinarchive().getLink()
                    .getHref().indexOf(":")==-1)
                {
                    softpkg_path
                     = csd_file.getFileinarchive().getLink()
                         .getHref();
                }
                else
                {
                    System.err.println(
                        "*** remote files access not implemented");
                }
            }
            else
            {
                // the CSD file is directly in the AAR archive
                softpkg_path
                 = csd_file.getFileinarchive().getName();
            }
        }
        
        if (csd_file.getLink() != null )
        {
            // checks if the given href is relative
            if( csd_file.getLink().getHref().indexOf(":")==-1)
            {
                softpkg_path
                 = csd_file.getLink().getHref();
            }
            else
            {
                System.err.println(
                    "*** remote files access not implemented");
            }
        }

        java.io.File softpkg = new java.io.File(softpkg_path);
        // file is new
        if ( softpkg!=null && !softpkg.exists())
        {
            SoftpkgBean softpkg_instance
             = new SoftpkgBeanImpl();

            softpkg_instance.setDocType("softpkg","-//OMG//DTD CORBA Software Descriptor 3.0//EN","http://openccm.objectweb.org/dtd/ccm/softpkg.dtd");

            try {
                softpkg_instance.marshalBean(
                    new java.io.File(softpkg_path)
                    );
            } catch (Exception exception) {
                exception.printStackTrace();
            }
        }
        else
        {
            // file exists
        }

        FileOperations.open_file(
            new java.io.File(softpkg_path)
            , calling_frame
            );
    }
}
