/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Contreras
Contributor(s):

====================================================================*/

package org.objectweb.openccm.descriptor.softpkg.gui;

import org.objectweb.openccm.descriptor.softpkg.beans.*;
import org.objectweb.openccm.descriptor.softpkg.*;

/**
 * This is an extension to the Apollon generated framework around softpkg DTD
 *  it looks for subelements that could contain files to archive
 *  and calls their own zipper extensions to do the same job
 *  any found file name is sent to a zip factory
 *
 */
public class SoftpkgZipper
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     *  The constructor taking an event as parameter
     */
    public void
    zipCall(
        SoftpkgBean softpkg
        , org.objectweb.openccm.deployment.gui.ZipFactory zip_factory
        )
    {
        // retrieves the elements that may contain files to zip and call their extension
        java.util.List subelement_list;

        // dependencies
        subelement_list
         = softpkg.getDependencyList();

        // goes through each elements and call its zip method
        for (java.util.Iterator i = subelement_list.iterator() ; i.hasNext() ; )
        {
            DependencyBean sub_element
             = (DependencyBean) i.next();

            DependencyZipper sub_zipper
             = (DependencyZipper) sub_element
                .getExtensionManager()
                .getExtensionByName("gui.Zipper")
                .getInstance();

            sub_zipper.zipCall(sub_element, zip_factory);
        }

        // CCD descriptors
        subelement_list
         = softpkg.getDescriptorList();
        // goes through each elements and call its zip method
        for (java.util.Iterator i = subelement_list.iterator() ; i.hasNext() ; )
        {
            DescriptorBean sub_element
             = (DescriptorBean) i.next();

            DescriptorZipper sub_zipper
             = (DescriptorZipper) sub_element
                .getExtensionManager()
                .getExtensionByName("gui.Zipper")
                .getInstance();

            sub_zipper.zipCall(sub_element, zip_factory);
        }

        // IDL files
        subelement_list
         = softpkg.getIdlList();
        // goes through each elements and call its zip method
        for (java.util.Iterator i = subelement_list.iterator() ; i.hasNext() ; )
        {
            IdlBean sub_element
             = (IdlBean) i.next();

            IdlZipper sub_zipper
             = (IdlZipper) sub_element
                .getExtensionManager()
                .getExtensionByName("gui.Zipper")
                .getInstance();

            sub_zipper.zipCall(sub_element, zip_factory);
        }

        // Properties Files
        subelement_list
         = softpkg.getPropertyfileList();
        // goes through each elements and call its zip method
        for (java.util.Iterator i = subelement_list.iterator() ; i.hasNext() ; )
        {
            PropertyfileBean sub_element
             = (PropertyfileBean) i.next();

            PropertyfileZipper sub_zipper
             = (PropertyfileZipper) sub_element
                .getExtensionManager()
                .getExtensionByName("gui.Zipper")
                .getInstance();

            sub_zipper.zipCall(sub_element, zip_factory);
        }

        // implementations
        subelement_list
         = softpkg.getImplementationList();
        // goes through each elements and call its zip method
        for (java.util.Iterator i = subelement_list.iterator() ; i.hasNext() ; )
        {
            ImplementationBean sub_element
             = (ImplementationBean) i.next();

            ImplementationZipper sub_zipper
             = (ImplementationZipper) sub_element
                .getExtensionManager()
                .getExtensionByName("gui.Zipper")
                .getInstance();

            sub_zipper.zipCall(sub_element, zip_factory);
        }
    }
}
