/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet, Christophe Demarey.

====================================================================*/

package org.objectweb.openccm.ir3;

// Package dependencies
import org.omg.CORBA.*;

/** 
 * Implementation of the CORBA::NativeDef interface.
 *
 * @author <a=href="Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a=href="Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class NativeDef_impl
       extends Contained_impl
       implements NativeDefOperations
{
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * TODO
     */
    public
    NativeDef_impl(IFR ifr,
                   Container_impl container)
    {
        // call the ContainedTypedefDef_impl contructor.
        super(ifr, container);

        // set the tie servant object.
        setServant(new NativeDefPOATie(this));
    }

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for the inherited IRObject_impl class.
    //
    // ==================================================================

    /** 
     * Warning : this operation must be implemented by subclasses.
     * It allows us to avoid some casts while creating ::CORBA::TypeCode
     */
    protected TypeCode
    recursiveType(java.util.List idSeq)
    {
        return type();
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /** 
     * Obtain its CORBA::NativeDef object reference.
     */
    public NativeDef
    asNativeDef()
    {
        return NativeDefHelper.narrow(asObject());
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /** 
     * IDL:omg.org/CORBA/IRObject/def_kind:1.0
     */
    public DefinitionKind
    def_kind()
    {
        return DefinitionKind.dk_Native;
    }

    // ==================================================================
    //
    // For CORBA::IDLType interface.
    //
    // ==================================================================

    /** 
     * IDL:omg.org/CORBA/IDLType/type:1.0
     */
    public TypeCode
    type()
    {
        // delegate to the TypeCodeFactory.
        return getIFR().getTCF().create_native_tc(id(), name());
    }

    // ==================================================================
    //
    // For CORBA::NativeDef interface.
    //
    // ==================================================================
}
