/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.openccm.task;

// Package dependencies
import java.util.Iterator;
import java.util.Map;


/**
 * This task generates :
 * - local OMG IDL interfaces mapping for OMG CIDL definitions
 * - Java component and home executor skeletons implementing previous interfaces
 * - Java class dependencies files for each CIDL composition
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 * 
 * @version 0.1
 */
public class CIDLtoCIF
     extends CompilerBase
  implements Runnable 
{
    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /** The output file name. */
    private String dest_file_;

    /** List of prefix to ignore. */
    private java.util.List no_prefix_;

    /** List of user files to include. */
    private java.util.List user_includes_;

    /** The output directory for Java Mapping. */
    private String output_directory_;

    /** The output directory for Dependencies. */
    private String dependencies_directory_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     * 
     * @param project_properties - Properties of the Ant Project.
     */
    public CIDLtoCIF(Map project_properties)
    {
        // Call CompilerBase constructor
        super(project_properties);
        
        // Init internal states
        dest_file_ = "generated_cif.idl";
        user_includes_ = new java.util.ArrayList();
        no_prefix_ = new java.util.ArrayList();
        output_directory_ = "generated"; 
        dependencies_directory_ = "dependencies";
    }
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
    /**
     * Configure the task.
     */
    public void
    configure()
    {
        Iterator it = null;
        
        // Call the CompilerBase configure method
        super.configure();
        
        setXmlFile( project_properties_.getProperty("OpenCCM_HOMEDIR") + "/xml/launcher/CIDLtoCIF.xml" );
        addProperty( "OpenCCM_CONFIG_DIR",
                     project_properties_.getProperty("OpenCCM_CONFIG_DIR") );
        // Add destination file
        addArgument("-o");
        addArgument(dest_file_);
        // Add user include declarations
        for (it = user_includes_.iterator(); it.hasNext(); )
        {
            addArgument("-ipath");
            addArgument( (String) it.next() );
        }
        // Add user include declarations
        for (it = no_prefix_.iterator(); it.hasNext(); )
        {
            addArgument("--noprefix");
            addArgument( (String) it.next() );
        }
        // Add the output directory
        addArgument("-d");
        addArgument(output_directory_);
        // Add the dependencies directory
        addArgument("-dep");
        addArgument(dependencies_directory_);
    }

    /**
     * Set Java mapping output directory.
     * 
     * @param outputdir - The output directory name.
     */
    public void
    setDestfile(String filename)
    {
        dest_file_ = filename;
    }
    
    /**
     * Set Java mapping output directory.
     * 
     * @param outputdir - The output directory name.
     */
    public void
    setDestdir(String destdir)
    {
        output_directory_ = destdir;
    }
    
    /**
     * Set Java mapping output directory.
     * 
     * @param outputdir - The output directory name.
     */
    public void
    setDependenciesdir(String depdir)
    {
        dependencies_directory_ = depdir;
    }
    
    /**
     * Add a user include declaration in the generated file.
     * 
     * @param str - The declaration name to include.
     */
    public void
    addConfiguredUserinclude(StringType str)
    {
        user_includes_.add(str.getName()); 
    }

    /**
     * Add a prefix to ignore in dependencies.
     * 
     * @param str - The declaration name to include.
     */
    public void
    addConfiguredIgnoreprefix(StringType str)
    {
        no_prefix_.add(str.getName()); 
    }
}
