/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.openccm.task;

// Packaga dependencies
import java.io.File;
import java.util.Map;

/**
 * This task run the OpenCCM Interface Repository
 * and feeds IFR and Components idl files.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 * 
 * @version 0.1
 */
public class IR3Start
     extends LauncherApplication
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The file to write IOR in. */
    private String ior_file_;

    /** To disable OMG IDL2.x mappings. */
    private boolean no_mappings_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    
    /**
     * The default constructor.
     * 
     * @param project_properties - Properties of the Ant Project.
     */
    public IR3Start(Map project_properties)
    {
        // Call Launcher constructor
        super(project_properties);
        
        // Init internal states
        ior_file_ = project_properties_.getProperty("OpenCCM_CONFIG_DIR") + "/IR3.IOR";
        no_mappings_ = false;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
    /**
     * Configure the task.
     */
    public void
    configure()
    {
        String idl_dir = null;

        addProperty( "OpenCCM_CONFIG_DIR",
                     project_properties_.getProperty("OpenCCM_CONFIG_DIR") );
        addArgument("--ior-file");
        addArgument(ior_file_);
        if (no_mappings_)
        {
            addArgument("--no-mappings");
        }

        setXmlFile( project_properties_.getProperty("OpenCCM_HOMEDIR") + "/xml/launcher/IR3Server.xml" );
    }
    
    /**
     * Callback method to insert code after
     * application launched.
     */
    public void
    post_exec()
    {
        // Wait for the IR3 IOR
        File file = new File(ior_file_);
        while ( (!file.exists()) ) 
        {
            // Wait for this file
        }
    }

    /**
     * Stop the launcher application process.
     */
    public void
    stop()
    {
        super.stop();
        
        // Remove the IOR file
        File file = new File(ior_file_);
        file.delete();
    }

    /**
     * Set the name of the file to write IOR in.
     *
     * @param ior_file - The IOR file name.
     */
    public void
    setIorfile(String ior_file)
    {
        this.ior_file_ = ior_file;
    }

    /**
     * To disable OMG IDL2.x mappings.
     *
     * @param b - If true, disables OMG IDL2.x mappings.
     */
    public void
    setNo_mappings(boolean b)
    {
        this.no_mappings_ = b;
    }
}
