/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.openccm.task;

// Package dependencies
import java.io.File;
import java.util.Map;


/**
 * This task generates the Java container code associated to an OpenCCM Interface Repository
 * object and its children objects.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 * 
 * @version 0.1
 */
public class IR3toJava
     extends LauncherApplication
  implements Runnable 
{
    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /** The output directory name. */
    private File dest_dir_;

    /** The scope to generate. */
    private String scope_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     * 
     * @param project_properties - Properties of the Ant Project.
     */
    public IR3toJava(Map project_properties)
    {
        // Call CompilerBase constructor
        super(project_properties);
        
        // Init internal states
        dest_dir_ = null;
        scope_ = null; 
    }
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
    /**
     * Configure the task.
     */
    public void
    configure()
    {
        setXmlFile( project_properties_.getProperty("OpenCCM_HOMEDIR") + "/xml/launcher/IR3toJava.xml" );
        addProperty( "OpenCCM_CONFIG_DIR",
                     project_properties_.getProperty("OpenCCM_CONFIG_DIR") );
        // Set working directory
        setWorkingDirectory(dest_dir_);
        // Add target scope
        addArgument(scope_);
    }

    /**
     * Set Java mapping output directory.
     * 
     * @param outputdir - The output directory name.
     */
    public void
    setDestdir(String filename)
    {
        dest_dir_ = new java.io.File(filename);
    }
    
    /**
     * Set the scope to generate.
     * 
     * @param scope - The scope to generate.
     */
    public void
    setScope(String scope)
    {
        scope_ = scope;
    }
}
