/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.pss.demo1;

/**
 * This Server :
 * - Initialize the ORB and retrieve the PSS
 * - Register Storage Types and Storage Homes in the PSS
 * - Create a new session from demo properties
 * - Instantiate a new Form, activate it and waiting for requests.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.2
 */
public class Server
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * The application entry point.
     * 
     * @param args - Application arguments.
     */
    public static void main( String [] args )
    {
        /* ===== Step 1 : Init the ORB ===== */
        org.objectweb.openccm.corba.TheORB.initialize(args);

        /* ===== Step 2 : Create the connector and register storage factories ===== */
        org.omg.CosPersistentState.ConnectorRegistry connectorRegistry = null;
        connectorRegistry = org.omg.CosPersistentState.ConnectorRegistryHelper.narrow(
                org.objectweb.openccm.corba.TheORB.resolve_initial_reference("PSS") );
        org.objectweb.openccm.pss.runtime.common.api.Connector connector = null;
        connector = org.objectweb.openccm.pss.runtime.common.lib.ConnectorFactory.create();
        try
        {
            connector.register_storage_home_factory("PSDL:openccm.objectweb.org/pss/demo1/SH_Person:1.0",
                                                    Class.forName("org.objectweb.openccm.pss.demo1.SH_Person") );
            connector.register_storage_object_factory("PSDL:openccm.objectweb.org/pss/demo1/ST_Person:1.0",
                                                      Class.forName("org.objectweb.openccm.pss.demo1.PersonBaseImpl") );
            connector.register_storage_home_factory("PSDL:openccm.objectweb.org/pss/demo1/SH_Address:1.0",
                                                    Class.forName("org.objectweb.openccm.pss.demo1.SH_Address") );
            connector.register_storage_object_factory("PSDL:openccm.objectweb.org/pss/demo1/ST_Address:1.0",
                                                      Class.forName("org.objectweb.openccm.pss.demo1.ST_Address") );
        }catch(ClassNotFoundException e){
            e.printStackTrace();
        }
        connectorRegistry.register_connector(connector);


        /* ===== Step 3 : Retrieve the connector and create a session (catalog) ===== */
        // This code should be simplified outside this demo.
        // The connector name must be known by PSS user.
        try
        {
            connector = (org.objectweb.openccm.pss.runtime.common.api.Connector)
                            connectorRegistry.find_connector("org.objectweb.openccm.pss.runtime.JDO");
        }catch(org.omg.CosPersistentState.NotFound e1){
            try 
            {
                connector = (org.objectweb.openccm.pss.runtime.common.api.Connector)
                                 connectorRegistry.find_connector("org.objectweb.openccm.pss.runtime.Hibernate");
            }catch(org.omg.CosPersistentState.NotFound e2){
                e2.printStackTrace();
            }
        }

        org.omg.CosPersistentState.Session mySession = null;
        try {
            org.omg.CosPersistentState.Parameter[] parameters = null;
            org.omg.CosPersistentState.Parameter param = null;
            org.omg.CORBA.Any a = null;

            // Init parameters
            parameters = new org.omg.CosPersistentState.Parameter[1];
            
            a = org.objectweb.openccm.corba.TheORB.getORB().create_any();
            a.insert_string( "demo1.properties" );
            param = new org.omg.CosPersistentState.Parameter("CONF_FILE", a);
            parameters[0] = param;

            // create the session
            mySession = connector.create_basic_session( org.omg.CosPersistentState.READ_WRITE.value,
                                                        parameters
                                                      );
            ((org.objectweb.openccm.pss.runtime.common.api.CatalogBase)mySession).set_connector(connector);
        } catch (Exception e) {
            e.printStackTrace();
        }


        /* ===== Step 4 : Create the Corba Form Object and register it in the NS  ===== */

        // Obtain the ORB.
        org.omg.CORBA.ORB orb = org.objectweb.openccm.corba.TheORB.getORB();

        // Obtain the Name Service.
        System.out.println("Obtaining the Name Service...");
        org.omg.CORBA.Object obj = null;
        try {
            obj = orb.resolve_initial_references("NameService");
        } catch(org.omg.CORBA.ORBPackage.InvalidName ex) {
            ex.printStackTrace();
        }
        org.omg.CosNaming.NamingContext nc = org.omg.CosNaming.NamingContextHelper.narrow(obj);

        org.omg.CORBA.Object objPoa = null;
        org.omg.PortableServer.POA rootPOA = null;

        try {
            objPoa = orb.resolve_initial_references( "RootPOA" );
        } catch(org.omg.CORBA.ORBPackage.InvalidName ex) {
            ex.printStackTrace();
        }
        rootPOA = org.omg.PortableServer.POAHelper.narrow( objPoa );
        org.omg.PortableServer.POAManager manager = rootPOA.the_POAManager();

        // Create the CORBA object
        FormImpl form = new FormImpl( mySession );
        obj = form._this( orb );

        // Bind the pss::demo1::form in ths NS
        org.omg.CosNaming.NameComponent[] name = new org.omg.CosNaming.NameComponent[1];
        name[0] = new org.omg.CosNaming.NameComponent();
        name[0].id = "My_PSS_demo1_form";
        name[0].kind = "";

        try {
            nc.rebind(name, obj);
        } catch (org.omg.CosNaming.NamingContextPackage.NotFound err) {
            System.out.println("Object not found ! ");
        } catch (org.omg.CosNaming.NamingContextPackage.CannotProceed err) {
            System.out.println("CannotProceed exception !");
        } catch (org.omg.CosNaming.NamingContextPackage.InvalidName err) {
            System.out.println("InvalidName exception !");
        }

        // Wait for requests
        System.out.println("The server is ready!");
        try {
            manager.activate();
        } catch(org.omg.PortableServer.POAManagerPackage.AdapterInactive ex) {
            ex.printStackTrace();
        }
        orb.run();
    }
}
