/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.openccm.pss.generator.common.api;

//Package dependencies
import org.objectweb.openccm.ast.api.StorageTypeDecl;
import org.objectweb.openccm.generator.common.api.GeneratorBase;
import org.objectweb.openccm.generator.common.lib.GenerationException;

/**
 * Interface to get Mapping from PSDL Storage Types.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</A>
 * 
 * @version 0.1
 */

public interface StorageTypeMapping
{
    /** 
     * Set the Storage Type Declaration to map. 
     *
     * @param st - The Storage Type Declaration to map.
     */
     public void
     setStorageType(StorageTypeDecl st);

    /**
     * Get the list of all state members that will be mapped.
     *
     * @param generator - A base generator to get declarationss.
     *
     * @return The list of all state members.
     **/
    public java.util.List
    getAllStates(GeneratorBase generator);

    /**
     * Get the list of all state members that will be mapped to native types.
     *
     * @param generator - A base generator to get declarationss.
     *
     * @return The list of all state members mapped to native types.
     **/
    public java.util.List
    getStatesToNative(GeneratorBase generator);

    /**
     * Get the list of all state members that will be mapped to an IOR.
     *
     * @param generator - A base generator to get declarationss.
     *
     * @return The list of all state members mapped to an IOR.
     **/
    public java.util.List
    getStatesToIOR(GeneratorBase generator);

    /**
     * Get the list of all state members that will be mapped to a byte array.
     *
     * @param generator - A base generator to get declarationss.
     *
     * @return The list of all state members mapped to a byte array.
     **/
    public java.util.List
    getStatesToCodec(GeneratorBase generator);

    /**
     * Get the list of all state members that will be mapped to a byte array.
     *
     * @param generator - A base generator to get declarationss.
     *
     * @return The list of all state members mapped to a byte array.
     **/
    public java.util.List
    getStatesToClass(GeneratorBase generator);

    /**
     * Generate a java persistent capable class for this storage type.
     *
     * @param storage_object - If true, generates mapping for a storage object.
     * @param generator - The PSDL to Java generator.
     */
    public void
    toJava(boolean storage_object,
           PSDL2JavaGenerator generator)
    throws GenerationException;
}
