/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.pss.generator.common.lib;

// Package dependencies
/** To access AST. */
import org.objectweb.openccm.ast.api.AbstractStorageTypeDecl;
import org.objectweb.openccm.ast.api.StorageTypeDecl;
import org.objectweb.openccm.ast.api.AbstractStorageHomeDecl;
import org.objectweb.openccm.ast.api.StorageHomeDecl;
import org.objectweb.openccm.ast.api.DeclarationKind;
import org.objectweb.openccm.ast.api.Scope;
import org.objectweb.openccm.ast.api.AST;
/** To access Java AST. */
import org.objectweb.openccm.generator.java.core.lib.*;
import org.objectweb.openccm.generator.java.ast.api.*;
import org.objectweb.openccm.generator.java.ast.lib.*;
/** Others. */
import org.objectweb.openccm.generator.common.lib.GenerationException;
import org.objectweb.openccm.generator.translator.idl2java.api.PSDL_JavaTranslator;
import org.objectweb.openccm.pss.generator.hibernate.api.HibernateXmlGenerator;
import org.objectweb.openccm.pss.generator.jdo.api.JdoXmlGenerator;

import java.util.List;


/**
 * This class generates PSDL to Java Mappings.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 * 
 * @version 0.1
 */

public class PSDL2JavaGenerator
     extends org.objectweb.openccm.generator.common.lib.GeneratorBase
  implements org.objectweb.openccm.pss.generator.common.api.PSDL2JavaGenerator
{

    // ==================================================================
    //
    // Constants.
    //
    // ==================================================================

    /** Constant to refer a JDO Back-end */
    private static final int JDO_BACKEND = 0;

    /** Constant to refer an Hibernate Back-end */
    private static final int HIBERNATE_BACKEND = 1;

    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /** The java repository */
    private Repository repository_;

    /** The backend to used for mapping */
    private int backend_;

    /** Utility class to convert types in Java */
    public PSDL_JavaTranslator translator_;
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     *
     * @param ast - The Abstract Syntax Tree.
     */

    public PSDL2JavaGenerator(AST ast)
    {
        super(ast);
        // Init internal state
        repository_ = null;
        translator_ = new org.objectweb.openccm.generator.translator.idl2java.lib.PSDL_JavaTranslator();
        backend_ = JDO_BACKEND;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Useful methods.
    //
    // ==================================================================


    // ==================================================================
    //
    // Others methods.
    //
    // ==================================================================

    /**
     * Get the generator translator utility class.
     *
     * @return The translator.
     **/
    public PSDL_JavaTranslator
    getTranslator()
    {
        return translator_;
    }

    /**
     * Get the java Repository.
     *
     * @return The java repository.
     **/
    public Repository
    getJavaRepository()
    {
        return repository_;
    }


    /**
     * Generate java files for the PSDL JAVA Mappings.
     *
     * @param base_dir - The base directory where files will be generated.
     * @param scope - The scope to map.
     * @param user_properties_file - The user properties file name.
     * @param backend - The target persistent framework.
     **/
    public void
    psdl_to_java(String base_dir,
                 Scope scope,
                 String user_properties_file,
                 String backend)
    throws GenerationException
    {
        java.io.File out_dir = null;
        repository_ = new RepositoryImpl();
        List vect = null;
        AbstractStorageTypeDecl[] ast_array = null;
        StorageTypeDecl[] st_array = null;
        AbstractStorageHomeDecl[] ash_array = null;
        StorageHomeDecl[] sh_array = null;

        // Check and create the output directory
        try{
            out_dir = org.objectweb.openccm.generator.common.lib.FileManager.mkdir(base_dir);
        }catch(GenerationException ex){
            System.err.println(ex.getMessage());
            return;
        }
        
        // Fix persistent framework properties
        System.getProperties().put("openccm.pss.pf", backend);
        if (backend.compareTo("jdo") == 0)
        {
            backend_ = JDO_BACKEND;
        }
        else if (backend.compareTo("hibernate") == 0)
        {
            backend_ = HIBERNATE_BACKEND;
        }


        /** Get all Abstract Storage Types contained in the file scope and map **/
        vect = getAllDeclarations(scope, DeclarationKind.dk_abstract_storage_type);
        ast_array = (AbstractStorageTypeDecl[])vect.toArray(new AbstractStorageTypeDecl[0]);
        for (int i=0; i<ast_array.length; i++)
        {
            org.objectweb.openccm.pss.generator.common.api.AbstractStorageTypeMapping ast = null;
            
            ast = (org.objectweb.openccm.pss.generator.common.api.AbstractStorageTypeMapping)
                            MappingObjectFactory.getClass("AbstractStorageTypeMapping");
            ast.setAbstractStorageType(ast_array[i]);
            ast.toJava(this);
        }

        /** Get all Abstract Storage Homes contained in the file scope and map **/
        vect = getAllDeclarations(scope, DeclarationKind.dk_abstract_storage_home);
        ash_array = (AbstractStorageHomeDecl[])vect.toArray(new AbstractStorageHomeDecl[0]);
        for (int i=0; i<ash_array.length; i++)
        {
            org.objectweb.openccm.pss.generator.common.api.AbstractStorageHomeMapping ash = null;
            
            ash = (org.objectweb.openccm.pss.generator.common.api.AbstractStorageHomeMapping)
                            MappingObjectFactory.getClass("AbstractStorageHomeMapping");
            ash.setAbstractStorageHome(ash_array[i]);
            ash.toJava(this);
        }

        /** Get all Storage Types contained in the file scope and map **/
        vect = getAllDeclarations(scope, DeclarationKind.dk_storage_type);
        st_array = (StorageTypeDecl[])vect.toArray(new StorageTypeDecl[0]);
        for (int i=0; i<st_array.length; i++)
        {
            org.objectweb.openccm.pss.generator.common.api.StorageTypeMapping st = null;

            st = (org.objectweb.openccm.pss.generator.common.api.StorageTypeMapping)
                            MappingObjectFactory.getClass("StorageTypeMapping");
            st.setStorageType(st_array[i]);
            st.toJava(true, this);
        }
        // Generate associated xml file
        if (backend_ == JDO_BACKEND)
        {
            JdoXmlGenerator jdo_xml_gen = null;
            
            jdo_xml_gen = new org.objectweb.openccm.pss.generator.jdo.lib.JdoXmlGenerator(this);
            jdo_xml_gen.initialize("psdl_java");
            jdo_xml_gen.to_jdo_xml(base_dir, vect, user_properties_file);
        }
        else
        {
            HibernateXmlGenerator hibernate_xml_gen = null;
            
            hibernate_xml_gen = new org.objectweb.openccm.pss.generator.hibernate.lib.HibernateXmlGenerator(this);
            hibernate_xml_gen.initialize("psdl_java");
            hibernate_xml_gen.to_hibernate_xml(base_dir, vect, user_properties_file);
        }

        /** Get all Storage Homes contained in the file scope and map **/
        vect = getAllDeclarations(scope, DeclarationKind.dk_storage_home);
        sh_array = (StorageHomeDecl[])vect.toArray(new StorageHomeDecl[0]);
        for (int i=0; i<sh_array.length; i++)
        {
            org.objectweb.openccm.pss.generator.common.api.StorageHomeMapping sh = null;

            sh = (org.objectweb.openccm.pss.generator.common.api.StorageHomeMapping)
                            MappingObjectFactory.getClass("StorageHomeMapping");
            sh.setStorageHome(sh_array[i]);
            sh.toJava(this);
        }

        // Generate Java Files
        org.objectweb.openccm.generator.java.core.lib.JavaGenerator java_gen = new JavaGenerator();
        java.util.List v = new java.util.ArrayList();
        v.add("org/objectweb/openccm/pss/generator/common/psdl2java_common.vm");
        if (backend_ == JDO_BACKEND)
        {
            v.add("org/objectweb/openccm/pss/generator/jdo/psdl2java_jdo.vm");
        }
        else
        {            
            v.add("org/objectweb/openccm/pss/generator/hibernate/psdl2java_hibernate.vm");
        }
        java_gen.initialize("PSDL2JAVA", v);
        java_gen.generate(repository_, base_dir);
    }
}
