/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.openccm.pss.generator.jdo.lib;

// Package dependencies
/** To access Java AST. */
import org.objectweb.openccm.generator.java.ast.api.*;
import org.objectweb.openccm.generator.java.ast.lib.*;
/** Others. */
import org.objectweb.openccm.generator.translator.idl2java.api.PSDL_JavaTranslator;
import org.objectweb.openccm.pss.generator.common.lib.PSDLTools;


/**
 * This class generates the mapping for a State Member.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 * 
 * @version 0.1
 */
public class StateMemberMapping
     extends org.objectweb.openccm.pss.generator.common.lib.StateMemberMapping
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public StateMemberMapping()
    {
        super();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Map a Storage Type State Member.
     *
     * @param translator - A Java utility class to translate types.
     * @param obj - The StorageType mapping object.
     * @param finalAccessor - True if accessors must be final.
     */
    public void
    toJavaAccessors( PSDL_JavaTranslator translator,
                     InterfaceObject obj,
                     boolean finalAccessor )
    {
        MethodObject method = null;
        ParameterObject param = null;

        if (PSDLTools.isAbstractStorageTypeReference(state_.getType()))
        {
            // Add the read only accessor method returning an incarnation
            method = new MethodObjectImpl();
            method.addComment("Get the "+state_.getName()+" value.");
            method.addComment(" ");
            method.addComment("@return The "+state_.getName()+" value.");
            method.setName( state_.getName() );
            method.setReturnType( translator.RefToJava(state_.getType()) );
            method.setFinal( finalAccessor );
            method.getImpl().setMacro("REF_GET_METHOD");
            method.getImpl().addContextValue("st_type", translator.RefToJava(state_.getType()) );
            method.getImpl().addContextValue("state", state_.getName());
            obj.addMethod(method);

            // Add the read only accessor method returning a reference (pid)
            method = new MethodObjectImpl();
            method.addComment("Get the "+state_.getName()+" value.");
            method.addComment(" ");
            method.addComment("@param yr - Use to have a different signature.");
            method.addComment(" ");
            method.addComment("@return The "+state_.getName()+" value.");
            method.setName( state_.getName() );
            method.setReturnType( translator.toJava(state_.getType()) );
            method.setFinal( finalAccessor );
            param = new ParameterObjectImpl();
            param.setName("yr");
            param.setType("org.omg.CosPersistentState.YieldRef");
            method.addParameter(param);
            method.getImpl().setMacro("GET_METHOD");
            method.getImpl().addContextValue("var", mapStateName());
            obj.addMethod(method);

            if (!state_.isReadonly())
            {
                // Add the modifier method with an incarnation parameter
                method = new MethodObjectImpl();
                method.addComment("Set the "+state_.getName()+" value.");
                method.addComment(" ");
                method.addComment("@param new_one - The value to set.");
                method.setName( state_.getName() );
                method.setReturnType("void");
                method.setFinal( finalAccessor );
                param = new ParameterObjectImpl();
                param.setName("new_one");
                param.setType( translator.RefToJava(state_.getType()) );
                method.addParameter(param);
                method.getImpl().setMacro("REF_SET_METHOD");
                method.getImpl().addContextValue("var", mapStateName());
                method.getImpl().addContextValue("value", "new_one");
                obj.addMethod(method);

                // Add the modifier accessor method with a reference parameter (pid)
                method = new MethodObjectImpl();
                method.addComment("Set the "+state_.getName()+" value.");
                method.addComment(" ");
                method.addComment("@param new_one_ref - The value reference to set.");
                method.setName( state_.getName() );
                method.setReturnType("void");
                method.setFinal( finalAccessor );
                param = new ParameterObjectImpl();
                param.setName("new_one_ref");
                param.setType( translator.toJava(state_.getType()) );
                method.addParameter(param);
                method.getImpl().setMacro("SET_METHOD");
                method.getImpl().addContextValue("var", mapStateName());
                method.getImpl().addContextValue("value", "new_one_ref");
                obj.addMethod(method);
            }
        }
        else
        {
            // Add the read only accessor method
            method = new MethodObjectImpl();
            method.addComment("Get the "+state_.getName()+" value.");
            method.addComment(" ");
            method.addComment("@return The "+state_.getName()+" value.");
            method.setName( state_.getName() );
            method.setReturnType( translator.toJava(state_.getType()) );
            method.setFinal( finalAccessor );
            method.getImpl().setMacro("GET_METHOD");
            method.getImpl().addContextValue("var", mapStateName());
            obj.addMethod(method);

            if (!state_.isReadonly())
            {
                // Add the modifier method
                method = new MethodObjectImpl();
                method.addComment("Set the "+state_.getName()+" value.");
                method.addComment(" ");
                method.addComment("@param new_one - The value to set.");
                method.setName( state_.getName() );
                method.setReturnType("void");
                method.setFinal( finalAccessor );
                param = new ParameterObjectImpl();
                param.setName("new_one");
                param.setType( translator.toJava(state_.getType()) );
                method.addParameter(param);
                method.getImpl().setMacro("SET_METHOD");
                method.getImpl().addContextValue("var", mapStateName());
                method.getImpl().addContextValue("value", "new_one");
                obj.addMethod(method);

                if ( !PSDLTools.isImmutable(state_.getType())
                     && !PSDLTools.isAbstractStorageType(state_.getType()) )
                {
                    // Add the read / update accessor method
                    method = new MethodObjectImpl();
                    method.addComment("Update the "+state_.getName()+" value.");
                    method.addComment(" ");
                    method.addComment("@param fu - ForUpdate.");
                    method.setName( state_.getName() );
                    method.setReturnType( translator.toJava(state_.getType()) );
                    method.setFinal( finalAccessor );
                    param = new ParameterObjectImpl();
                    param.setName("fu");
                    param.setType("org.omg.CosPersistentState.ForUpdate");
                    method.addParameter(param);
                    method.getImpl().setMacro("GET_UPDATE_METHOD");
                    method.getImpl().addContextValue("var", mapStateName());
                    obj.addMethod(method);
                }
            }
        }
    }

}
