# ====================================================================
#
# OpenCCM: The Open CORBA Component Model Platform
# Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
# Contact: openccm@objectweb.org
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
# Initial developer(s): Pierre Carpentier.
# Contributor(s): Philippe Merle.
#
# --------------------------------------------------------------------
# $Id: ConstraintException.cs,v 1.2 2004/04/20 13:17:50 carpentier Exp $
# ====================================================================
#
#  This class represents the exception thrown when a constraint
#  in not respected. This exception contains the stereotype which
#  doesn't respect the constraint, the reference of it in the
#  specification and the error message.
#
# ====================================================================

class ConstraintException
{

    # =================================================================
    # Constructor for a ConstraintException.
    # stereotype - The stereotype which doesn't respect the constraint
    # cause - The name of the element which causes the exception
    # specification - The specification which defines the constraint
    # section - The corresponding section of the specification
    # message - The error message
    # =================================================================
    proc __ConstraintException__(self, stereotype, cause, specification, section, message)
    {
        self.stereotype = stereotype;
        self.cause = cause;
        self.specification = specification;
        self.section = section;
        self.message = message;
    }


    # =================================================================
    # Transforms the attributes to a formatted String
    # for the presentation.
    # =================================================================
    proc toString(self)
    {
        formatString = self.stereotype;
        if (self.specification.length > 0) {
            formatString = formatString + " - " + self.specification;
        }
        if (self.section.length > 0) {
            formatString = formatString + " [" + self.section + "]";
        }
        formatString = formatString + "\n" + self.message + " (" + self.cause + ")";
        return formatString;
    }


    # =================================================================
    # Shows a comprehensive message.
    # =================================================================
    proc show(self)
    {
        println(self.toString());
    }

}
