/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Pierre Carpentier.
Contributor(s): Philippe Merle.

---------------------------------------------------------------------
$Id: UML2CCM.java,v 1.1 2004/05/26 11:25:34 carpentier Exp $
====================================================================*/

package org.objectweb.openccm.uml.api;

/**
 * Interface of the uml2ccm command.
 *
 * @author <a href="mailto:Pierre.Carpentier@lifl.fr">Pierre Carpentier</a>
 *
 * @version 1.1
 */

public interface UML2CCM
         extends org.objectweb.openccm.command.api.Compiler
{

    /**
     * Inits the UML repository.
     *
     * @return TRUE if the UML repository is initialized.
     */
    public boolean
    initUMLRepository();


    /**
     * Parses the UML XMI file for reading the UML Package.
     *
     * @param filename The filename of the UML XMI file.
     *
     * @throws org.xml.sax.SAXException If any parse errors occur.
     * @throws javax.xml.parsers.ParserConfigurationException If a DocumentBuilder cannot be
     *        created which satisfies the configuration requested.
     * @throws java.io.IOException If any IO errors occur.
     */
    public void
    runTransformation(String filename)
    throws
        org.xml.sax.SAXException,
        javax.xml.parsers.ParserConfigurationException,
        java.io.IOException;
    
    
    /**
     * Sets the monolithic flag to indicate if the Java implementation
     * will be the monolithic or CIF version.
     * 
     * @param monolithic TRUE if the Java implementation should be
     * the monolithic version.
     */
    public void
	setMonolithic(boolean monolithic);
    
    
    /**
     * Sets the output directory for the generated files.
     * 
     * @param directory The output directory for the generated files.
     */
    public void
	setOutputDirectory(String directory);

}
