/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Pierre Carpentier.
Contributor(s): Philippe Merle.

---------------------------------------------------------------------
$Id: OptionOutputDirectory.java,v 1.1 2004/05/26 11:25:34 carpentier Exp $
====================================================================*/

package org.objectweb.openccm.uml.lib;

import org.objectweb.util.cmdline.lib.DefaultOptionBase;

/**
 * The '-d' or '--outputdir' option to set the output
 * directory for the generated files.
 *
 * @author <a href="mailto:Pierre.Carpentier@lifl.fr">Pierre Carpentier</a>
 *
 * @version 0.1
 */

public class OptionOutputDirectory
     extends DefaultOptionBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /** The default constructor. */
    public
    OptionOutputDirectory(org.objectweb.openccm.uml.api.UML2CCM generator)
    {
        super(new String[] { "-d", "--outputdir"},
              new String[0],
              "Set the output directory for the generated files ('generated' by default)");
        this.generator = generator;
        setArguments(new String[] {"directory"});
        setMandatory(false);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    /** The generator of the Java Implementation. */
    org.objectweb.openccm.uml.api.UML2CCM generator;

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.Option
    //
    // ==================================================================

    /**
     * Consumes command line arguments from an iterator.
     *
     * @param iterator The command line argument iterator.
     */
    public void
    consume(org.objectweb.util.cmdline.api.Iterator iterator)
    {
    	String directory = consumeArgument(iterator);
        generator.setOutputDirectory(directory);
    }

    // ==================================================================
    //
    // Other public methods.
    //
    // ==================================================================
}
