/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Pierre Carpentier.
Contributor(s): Philippe Merle.

---------------------------------------------------------------------
$Id: MappingRuleEngineWithCollection.java,v 1.1 2004/05/26 11:25:35 carpentier Exp $
====================================================================*/

package org.objectweb.openccm.uml.transformation.engine;

import ispuml.mdaTransformation.EngineContext;
import ispuml.mdaTransformation.MappingRuleEngine;
import ispuml.mdaTransformation.RuleContext;
import ispuml.mdaTransformation.TransformationException;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

/**
 * Extension of the MdaTransf engine in order to take in account the
 * transformation of collections.
 */
public class MappingRuleEngineWithCollection extends MappingRuleEngine {

    /**
     * Apply the transformation rules to each object of the collection.
     * @param bean The bean to transform
     * @param request The current request context
     * @param rulesetName The name of the ruleset to use. If null, use the default ruleset
     * @param ruleName The name of the rule to use. If null, use a matching rule.
     * @return Object A list of result.
     */
    protected Object transformCollection(Object bean, RuleContext request, String rulesetName, String ruleName)
        throws TransformationException {
        if (!(bean instanceof Collection))
            throw new TransformationException("Object to transform should denote a Collection.");
        
        if (request.getCurrentRuleContext().getRule() instanceof ispuml.mdaTransformation.rules.xml.MultiParamXmlRule) {
            ispuml.mdaTransformation.rules.xml.MultiParamXmlRule rule = (ispuml.mdaTransformation.rules.xml.MultiParamXmlRule)request.getCurrentRuleContext().getRule();
            ispuml.mdaTransformation.rules.Parameters parameters = rule.getSrcParameterNames();
            if (parameters != null && parameters.size() > 1) {
                return transform(bean, request, rulesetName, ruleName);
            }
        }
        
        Collection collection = (Collection) bean;
        Iterator iter = collection.iterator();
        List results = new ArrayList();
        while (iter.hasNext()) {
            Object curBean = iter.next();
            if (curBean != null) {
                if (Collection.class.isAssignableFrom(curBean.getClass())) {
                    List list = (List) transformCollection(curBean, request, rulesetName, ruleName);
                    results.addAll(list);
                } else {
                    Object res = transform(curBean, request, rulesetName, ruleName);
                    if (res != null)
                        results.add(res);
                }
            } // end if
        } // end loop

        return results;
    }

    /* (non-Javadoc)
     * @see ispuml.mdaTransformation.MappingRuleEngine#getContext()
     */
    public EngineContext getContext() {
        EngineContext context = super.getContext();
        if (context.getAttribute("CR") == null)
            context.putAttribute("CR", "\n");
        return context;
    }

}
