/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Pierre Carpentier.
Contributor(s): Philippe Merle.

---------------------------------------------------------------------
$Id: GenerateMacro.java,v 1.1 2004/05/26 11:25:35 carpentier Exp $
====================================================================*/

package org.objectweb.openccm.uml.transformation.rules.codeGenerator;

import java.io.Writer;

import org.apache.velocity.app.Velocity;

import ispuml.mdaTransformation.RuleContext;
import ispuml.mdaTransformation.TransformationException;
import ispuml.mdaTransformation.rules.codeGenerator.GenerateAction;
import ispuml.mdaTransformation.rules.codeGenerator.VelocityContext;
import ispuml.mdaTransformation.rules.codeGenerator.VelocityTemplateDirectives;

/**
 * This action allows to call a Velocity macro (in template) to generate a file.
 * The template must be in the classpath.
 */
public class GenerateMacro extends GenerateAction {

    /**
     * The Velocimacro to call.
     */
    protected String macro;

    /**
     * Gets the Velocimacro name.
     * @return The Velocimacro name.
     */
    public String getMacro() {
        return macro;
    }

    /**
     * Sets the Velocimacro name.
     * @param string The name.
     */
    public void setMacro(String macro) {
        this.macro = macro;
    }

    /**
     * Calling to the Velocity macro.
     * @param bean The current bean to generate (with Velocity Template).
     * @param request The context.
     * @return Object The bean.
     * @throws TransformationException
     */
    public Object execute(Object bean, RuleContext request) throws TransformationException {
        // Get or create the writer.
        // Also remember if we should close the writer at the end.
        boolean shouldCloseWriter = false;

        // First Check if there is a writer in the context
        Writer writer = (Writer) request.getAttribute(WRITER_ATTRIBUTE_NAME);
        // Create the writer if none is found or if the create flag is set
        if (alwaysCreateWriter || writer == null) {
            writer = createWriter(bean, request);
            request.putAttribute(WRITER_ATTRIBUTE_NAME, writer);
            // We should close the writer if it is created from a filename.
            shouldCloseWriter = (filename != null);
        } // end if

        // Generate output from the template
        try {
            VelocityContext context = new VelocityTemplateDirectives(bean, request);
            if (template != null) {
                if (macro != null) {
                    // Velocity properties to remove the log file and to search the templates in the classpath.
                    org.apache.velocity.app.VelocityEngine engine_ = new org.apache.velocity.app.VelocityEngine();
                	engine_.setProperty(org.apache.velocity.app.VelocityEngine.RUNTIME_LOG_LOGSYSTEM_CLASS,
                                        "org.apache.velocity.runtime.log.NullLogSystem" );
                	engine_.setProperty(org.apache.velocity.app.VelocityEngine.RESOURCE_LOADER, "classpath");
                    engine_.setProperty("classpath." + org.apache.velocity.app.VelocityEngine.RESOURCE_LOADER + ".class",
                                        org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader.class.getName());
                    engine_.setProperty(org.apache.velocity.app.VelocityEngine.VM_LIBRARY, template);
                    engine_.init();
                    boolean res = engine_.invokeVelocimacro(macro, template, new String[0], context, writer);
                } else {
                    org.apache.velocity.runtime.RuntimeSingleton.setProperty(org.apache.velocity.app.VelocityEngine.RUNTIME_LOG_LOGSYSTEM_CLASS,
                                                                             "org.apache.velocity.runtime.log.NullLogSystem" );
                    Velocity.init();
                    Velocity.mergeTemplate(template, "ISO-8859-1", context, writer);
                }
            } else if (templateString != null) {
                Velocity.evaluate(context, writer, "template", templateString);
            } // end if

            // Close the writer if we have created it.
            if (shouldCloseWriter)
                writer.close();
        } catch (java.io.IOException ex) {
            throw new TransformationException("Can't close writer.", ex);
        } catch (Exception ex) {
            throw new TransformationException("Template exception.", ex);
        }

        return bean;
    }

}
