/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Pierre Carpentier.
Contributor(s): Philippe Merle.

---------------------------------------------------------------------
$Id: EnumDataTypeCondition.java,v 1.1 2004/05/26 11:25:35 carpentier Exp $
====================================================================*/

package org.objectweb.openccm.uml.transformation.rules.xml;

import javax.jmi.reflect.RefPackage;

import ispuml.mdaTransformation.RuleContext;
import ispuml.mdaTransformation.TransformationException;
import ispuml.mdaTransformation.model.SimpleModelContext;
import ispuml.mdaTransformation.model.jmi.JmiModelUtil;
import ispuml.mdaTransformation.rules.xml.DataTypeCondition;

/**
 * Condition for a DataType representing an enumeration.
 * 
 * @author Pierre Carpentier
 */
public class EnumDataTypeCondition extends DataTypeCondition {

    /** The associated model. */
    private String model;
    
    /** The type of the enumeration. */
    private String type;
    

    /**
     * Sets the associated model.
     * @param string The String representing the uri of the model.
     */
    public void setModel(String model) {
        this.model = model;
    }

    /**
     * Sets the type of the enumeration.
     * @param string The String representing the name of the enumeration.
     */
    public void setType(String type) {
        this.type = type;
    }

    /**
     * Creates the object representing the enumeration with the correct value.
     * @param object The source bean.
     * @param context The rule context.
     */
    protected Object executeCreate(Object object, RuleContext context) throws TransformationException {
        SimpleModelContext modelContext = (SimpleModelContext)context.getEngineContext().getModel(model);
        JmiModelUtil modelUtil = (JmiModelUtil)modelContext.getModelUtil();
        RefPackage extent = modelUtil.getOutermostRefPackage();
        
        String[] names = type.split( "\\." );
        int i=0;
        for(i=0; i<names.length-1; i++ ) {
            extent = (RefPackage)extent.refPackage( names[i] );
        }
        
        EnumerationRule rule = (EnumerationRule)context.getCurrentRuleContext().getRule();
        return extent.refGetEnum(type, rule.getEquivalentLabel(object));
    }

}
