/*====================================================================

ObjectWeb Util CommandLine Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.cmdline.lib;

// Package dependencies.
import org.objectweb.util.cmdline.api.OptionApplication;
import org.objectweb.util.cmdline.api.Application;

/**
 * Default implementation of the
 * the command line --version option.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public class DefaultOptionVersion
     extends DefaultOptionBase
  implements OptionApplication
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The associated application. */
    Application application_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the associated application.
     *
     * @param application The associated application.
     */
    public
    DefaultOptionVersion(Application application)
    {
        // Calls the DefaultOptionBase constructor.
        super(new String[] { "--version"},
              new String[0],
              "Output version information and exit");

	// Inits internal state.
        application_ = application;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.Option
    //
    // ==================================================================

    /**
     * Consumes command line arguments from an iterator.
     *
     * @param iterator The command line argument iterator.
     *
     * @return true if the option has consumed arguments.
     */
    public void
    consume(org.objectweb.util.cmdline.api.Iterator iterator)
    {
        // Obtains the standard output stream of the application console. 
        java.io.PrintStream output = application_.getConsole().getOutputStream();

        // Obtains the application version information.
        String[] versionInformation = application_.getVersionInformation();

        // Outputs the application version information.
        for(int i=0; i<versionInformation.length; i++)
            output.println(versionInformation[i]);

        // Then exits.
        System.exit(0);
    }

    // ==================================================================
    //
    //  Public methods for interface org.objectweb.util.cmdline.api.ApplicationHolder
    //
    // ==================================================================

    /**
     * Obtains the associated application.
     *
     * @return The associated application.
     */
    public Application
    getApplication()
    {
        return application_;
    }

    /**
     * Sets the associated application.
     *
     * @param console The associated application.
     */
    public void
    setApplication(Application application)
    {
        application_ = application;
    }

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.OptionApplication
    //
    // ==================================================================

    // ==================================================================
    //
    // Other public methods.
    //
    // ==================================================================
}
